﻿/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+

JQuery  widget for slick grid data viewer
Created By      : Rony for Assyst International Pvt. Ltd.
Reviewed By     : 
Created Date    : 21-July-2015
Version         : 1.0.1
Project         : PanBI
USAGE:  

Option:
        
Public Functions:
    
+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/
(function ($) {

    //Create widget for slick grid data viewer object
    $.widget('Assyst.DataViewer', {
        //Default values for options
        options: {
            schema: undefined,
            dataURL: undefined,
            records: undefined,
            query: undefined,
            childGrid: true,
            updateSchemaURL: undefined,
            getSchema: undefined,
            MaxRowCount: undefined,
            enableJsonp: true,
            grid: {
                enable: true,
                height: "400px",
                enableCellNavigation: false,
                enableColumnReorder: false,
                forceFitColumns: true,
                leaveSpaceForNewRows: true,
                explicitInitialization: false,
                syncColumnCellResize: true,
                hiddenColumns: []
                //autoHeight: true
            },
            paging: {
                enable: true,
                panel: 'bottom',
                showItemCount: true,
                pageSize: 1000,
                pageSizes: [5, 10, 20, 50, 100, 250, 500, 1000]
            },
            onRecordLoad: undefined,
            buttons: []
        },
        rendered: false,
        grid: undefined,
        fields: undefined, //[{id: xfield, name: xfield, field: xfield}]
        pagedList: {
            pageNumber: 1,
            pageSize: 10,
            pageCount: 1,
            itemCount: 0,
            isFirstPage: false,
            isLastPage: false,
            hasNextPage: false,
            hasPreviousPage: false,
            set: function (recordcount) {
                var pageCount = Math.floor(recordcount / this.pageSize);
                if (recordcount % this.pageSize != 0) {
                    ++pageCount;
                }
                if (pageCount == 0) pageCount = 1;
                if (this.pageNumber > pageCount) this.pageNumber = pageCount;
                this.itemCount = recordcount;
                this.pageCount = pageCount;
                this.isFirstPage = this.pageNumber == 1;
                this.isLastPage = this.pageNumber == this.pageCount;
                this.hasPreviousPage = this.pageNumber > 1;
                this.hasNextPage = this.pageNumber < this.pageCount;
            }
        },

        _init: function () {
            try {
                var self = this;
                var $element = $(self.element);
                var $block = $('<div class="jtable-main-container ui-corner-all"></div>');
                var $pagingBlock = $('<div class="jtable-bottom-panel ui-corner-bottom"><div class="button-container"></div><div class="paging-container"></div></div>');
                var windowidth = $(window).height() - 140 + "px";
                var $gridBlock = $('<div class="grid-container"></div>').css("height", windowidth);
                if (self.options.paging.panel == 'top') {
                    $block.append($pagingBlock);
                    $block.append($gridBlock);
                }
                else {
                    $block.append($gridBlock);
                    $block.append($pagingBlock);
                }
                $element.empty().append($block);
                this.rendered = !this.options.grid.explicitInitialization;
                this._createButtons();
                this.render("load");
            }
            catch (e) {
                console.log(e);
            }
        },

        _create: function () {
            try {
                if (this.options.childGrid == false) {
                    this._personalize();
                    this.options.paging.pageSize = this.options.MaxRowCount; /*Fixed by setting the deafult page size value as maximum row count-Nikhil.*/
                    this._initOptions();
                }
            }
            catch (e) {
                console.log(e);
            }
        },

        _initOptions: function () {
            var self = this;
            var pageSize = self._tryParseInt(self.options.paging.pageSize, self.options.MaxRowCount)/*replaced hard coded value to maximum row count.-Nikhil*/
            self.pagedList.pageSize = $.inArray(pageSize, self.options.paging.pageSize) > -1 ? pageSize : self.options.MaxRowCount; /*assigning array of maximum row counts-Nikhil.*/
            self.fields = getColumns(self.options.schema);
            function getColumns(fields) {
                var columnList = [];
                var isArray = $.isArray(fields);
                $.each(fields, function (key, value) {
                    var field = isArray ? value.id : key;
                    if ($.inArray(field, self.options.grid.hiddenColumns) > -1) return;
                    var hastotal;

                    if (value.format) {
                        if (value.format.category == "Currency")
                            hastotal = true;
                        else
                            hastotal = false;
                    }
                    else
                        hastotal = false;

                    var column = {
                        id: field,
                        name: value.title,
                        field: field,
                        type: value.dataType,
                        decimalPoint: value.decimalPoint || 2,
                        toolTip: value.description || value.title,
                        sortable: false,
                        minWidth: 80,
                        hasTotal: hastotal,
                        format: value.format,
                        header: value.formula ? {
                            buttons: [{
                                cssClass: "icon-highlight-off",
                                command: "toggle-highlight",
                                tooltip: "Derived column"
                            }]
                        } : {},
                        isHidden: (value.isHidden == "true" || value.isHidden == true) ? "Y" : "N",
                        formatter: slickFormatter,
                        cssClass: getClass(value.dataType),
                        headerCssClass: getClass(value.dataType)
                    };
                    columnList.push(column);
                });
                // Modified by Assyst (Rony at 07-Feb-2014)    
                // For text align based on data type
                /******************************************/
                function getClass(type) {
                    var className = '';
                    switch (type) {
                        case 'number':
                        case 'numeric':
                        case 'integer':
                            className = 'align-right'
                            break;
                        case 'date':
                        case 'datetime':
                        case 'date-time':
                        case 'timestamp':
                        case 'geo':
                            className = 'align-center'
                            break;
                        default:
                            className = 'align-left'
                            break;
                    }
                    return className;
                }
                /***************End*********************/
                return columnList;
            };
        },

        render: function (event) {
            var self = this;
            var param;
            if (self.options.query) {
                param = $.extend(true, {}, JSON.parse(self.options.query));
                if (param.aggregate.length == 0)
                    param.fields = [];
            }
            else param = null;

            var postdata = {
                filterCriteria: param == null ? null : JSON.stringify(param),
                startIndex: ((self.pagedList.pageNumber - 1) * self.pagedList.pageSize),
                pageSize: parseInt(self.pagedList.pageSize.MaxRowCount) + 1, /*setting the page size as maximum row count value + 1*/
                sorting: null
            }
            
            // (ATP/48)changed to ajax call to resolve url length issue..           
            self._ajaxcall(
                self.options.dataURL,
                postdata,
                true, //asynchronous
                function (data) { //success
                    if (data != null) {
                        if (typeof data.Records == 'string') { data.Records = JSON.parse(data.Records); }
                        data.TotalRecordCount = self._tryParseInt(data.Records.length, 0);
                        if (self._tryParseInt(self.options.MaxRowCount.MaxRowCount, 0) < self._tryParseInt(data.TotalRecordCount, 0)) {
                            if (self.options.MaxRowCount != undefined) {
                                /*Show notification if the row count is larger than settings count*/
                                data.Records = ShowNotificationAndSliceData(_app.MaxRecordCount, data.Records, self.options.MaxRowCount.show);
                                data.TotalRecordCount = self._tryParseInt(data.TotalRecordCount - 1, 0);
                            }
                        }
                        self.pagedList.set(data.TotalRecordCount);
                        TotalRecordCount = data.TotalRecordCount;
                        self.options.records = data.Records;
                        if (self.options.paging.enable) self._createPageNavigator();
                        if (self.options.grid.enable) self._createGrid(data.Records);

                        var pagingState = {
                            pageSize: self.pagedList.pageSize,
                            pagedList: {
                                pageNumber: self.pagedList.pageNumber
                            }
                        };

                        if (typeof self.options.onRecordLoad == "function") self.options.onRecordLoad(event, data, pagingState);
                    }
                    else {
                        if (typeof self.options.onRecordLoad == "function") self.options.onRecordLoad(event, data, pagingState);
                    }
                },
                function (p_request, p_status, p_error) {
                    if (window.console && window.console.log) console.log((p_error.message ? p_error.message : p_error));
                });
        },

        show: function () {
            // If the div is hidden, SlickGrid will calculate wrongly some
            // sizes so we must render it explicitly when the view is visible
            if (!this.rendered) {
                if (!this.grid) {
                    this.render();
                }
                this.grid.init();
                var totalsPlugin = new TotalsPlugin($.getScrollbarWidth());
                this.grid.registerPlugin(totalsPlugin);
                this.rendered = true;
            }
        },
        resizeCanvas: function () {
            this.grid.resizeCanvas();
            var totalsPlugin = new TotalsPlugin($.getScrollbarWidth());
            this.grid.registerPlugin(totalsPlugin);
        },
        redrow: function () {
            // If the div is hidden, SlickGrid will calculate wrongly some
            // sizes so we must render it explicitly when the view is visible
            if (!this.grid) {
                return;
            }
            this.render();
        },

        _personalize: function () {
            var self = this;
            var $element = $(self.element);

            $element.on('click', 'button.page-navigator', function (event) {
                self.pagedList.pageNumber = self._tryParseInt($(this).attr("pageindex"), 1);
                self.options.grid.explicitInitialization = false;
                self.render("paging");
            });

            $element.on('change', 'select.jtable-pagination-dropdown', function (event) {
                self.pagedList.pageNumber = self._tryParseInt($(this).val(), 1);
                self.options.grid.explicitInitialization = false;
                self.render("paging");
            });

            $element.on('change', 'select.jtable-page-size-dropdown', function (event) {
                self.pagedList.pageSize = self._tryParseInt($(this).val(), 5);
                if (((self.pagedList.pageNumber - 1) * self.pagedList.pageSize) > self.pagedList.itemCount) self.pagedList.pageNumber = 1;
                self.options.grid.explicitInitialization = false;
                self.render("paging");
            });

            $element.on('click', 'span.cust-button', function (event) {

                var buttonId = $(this).attr("id");
                var elements = $.grep(self.options.buttons, function (element, index) {
                    return element.id == buttonId;
                });

                if (elements.length == 0) return false;
                var button = elements[0];
                if (typeof button.click == "function") button.click(event);
            });
        },

        _createButtons: function (data) {
            var self = this;
            var $container = $(self.element).find("div.button-container");
            $container.empty();
            var template = '{{#buttons}}<div class="dropdown cust-button {{cssClass}}" title="{{title}}" id="{{id}}">' +
                              '<button class="btn btn-primary dropdown-toggle" type="button" data-toggle="dropdown">Save<span class="caret"></span></button>' +
                              '<div class="dropdown-menu" aria-labelledby="dropdownMenu1">' +
                              '<div class="dropdown-item dashlet" href="#">Save As</div>' +
                              '<div class="dropdown-item dashletSave {{^buttonShow}}disableClass{{/buttonShow}}" href="#">Save</div>' +
                              '</div>' +
                            '</div>{{/buttons}}';

            var tmplData = { buttons: self.options.buttons };
            var buttonsHtml = Mustache.render(template, tmplData);
            $container.html(buttonsHtml);
        },

        _createGrid: function (data) {
            var self = this;
            var $container = $(self.element).find("div.grid-container");
            $container.empty();
            for (var i = 0; i < self.fields.length; ++i) {
                if (self.fields[i].formula) {
                    var temp = self.fields[i];
                    self.fields.splice(self.fields[i].formula.position, 0, temp);
                    self.fields.splice(i + 1, 1);
                }
            }
            var items = $.grep(self.fields, function (value, i) {
                return value.isHidden == 'N';
            });
            var sql = 'SELECT ';
            //console.log('Underscore Start ' + new Date().getSeconds());
            _.each(self.fields, function (val, i) {
                if (val.formula) {
                    sql += '(' + val.formula.text + ') AS [' + val.id + "],";
                }
                else {
                    sql += '[' + val.id + '],';
                }
            });
            //console.log('End ' + new Date().getSeconds());

            sql = sql.substring(0, sql.length - 1);
            data = alasql(sql + ' FROM ? \ ', [data]);
            self.grid = new Slick.Grid($container, new TotalsDataView(data, items), items, self.options.grid);
            self.grid.resizeCanvas();
            var headerButtonsPlugin = new Slick.Plugins.HeaderButtons();
            headerButtonsPlugin.onCommand.subscribe(function (e, args) {
                var column = args.column;
                var button = args.button;
                var command = args.command;
                var position, postionId;
                var expBuilderFormat = $.map(self.fields, function (value, i) {
                    if (value.isHidden == 'N' && value.type == "numeric") {
                        if (value.id != column.id) return { DisplayText: value.name, Value: value.id, format: value.formula };
                        else {
                            position = i;
                            postionId = value.id;
                        }
                    }
                });
                _.each(self.fields, function (val, i) {
                    if (val.id == column.id)
                        self.fields.splice(i, 1);
                });

                $(e.target).append('<div class="preview1"></div>');
                $('.preview1').ExpressionBuilder({
                    Variables: expBuilderFormat,
                    createName: {
                        show: true,
                        label: column.name
                    },
                    FormulaFn: column.formula.label,
                    FormulaFieldEquation: column.formula.label,
                    dialog: { title: column.name }
                }).click();

                $('#ebDialogue_div').on('ebComplete', function (event, equationData) {
                    $('.spnformulaGroup').attr('dt-value', JSON.stringify(equationData));
                    $.extend(true, equationData, { position: position, postionId: postionId });
                    self.createNewSchema(equationData);
                });
            });
            self.grid.registerPlugin(headerButtonsPlugin);
            //self.contextMenu();
            var totalsPlugin = new TotalsPlugin($.getScrollbarWidth());
            self.grid.registerPlugin(totalsPlugin);
        },

        contextMenu: function () {
            var self = this;
            var items = {};
            items["name"] = "Show/Hide Fields";
            items["icon"] = "";
            _.each(self.fields, function (val, i) {
                var prow = {};
                prow[val.id] = {
                    name: val.name,
                    icon: "dropdownIconsel",
                    callback: function (key, options) {
                        if (options.$selected.hasClass('cm-icon-dropdownIconsel')) {
                            options.$selected.removeClass('cm-icon-dropdownIconsel').addClass('cm-icon-dropdownIconUnsel');
                            var selected = _.findWhere(self.fields, { id: key });
                            selected.isHidden = 'Y';
                        }
                        else {
                            options.$selected.removeClass('cm-icon-dropdownIconUnsel').addClass('cm-icon-dropdownIconsel');
                            var selected = _.findWhere(self.fields, { id: key });
                            selected.isHidden = 'N';
                        }
                        self._createGrid(self.options.records);
                    }
                };
                items["items"] = $.extend(true, items["items"], prow);
            });

            $.contextMenu({
                selector: '#gridViewer .slick-header-column',
                callback: function (key, options) {
                },
                items: {
                    "addFields": {
                        name: "Add Field",
                        icon: "",
                        callback: function (key, opt) {
                            var position = $(this).attr('title');
                            var postionId = $(this).attr('title');
                            var items = $.map(self.fields, function (value, i) {
                                position = value.name == position ? i + 1 : position;
                                postionId = value.name == postionId ? value.id : postionId;
                                if (value.isHidden == 'N' && value.type == "numeric") {
                                    return { DisplayText: value.name, Value: value.id, format: value.formula };
                                }
                            });
                            $(opt.$selected).append('<div class="preview"></div>');
                            $('.preview').ExpressionBuilder({
                                Variables: items,
                                createName: {
                                    show: true,
                                    label: ""
                                },
                                dialog: { title: $('.txtFieldName').val() }
                            });
                            $('.preview').click();
                            $('#ebDialogue_div').on('ebComplete', function (event, equationData) {
                                $('.spnformulaGroup').attr('dt-value', JSON.stringify(equationData));
                                $.extend(true, equationData, { position: position, postionId: postionId });
                                self.createNewSchema(equationData);
                            });
                        }
                    },
                    "showHideFields": items,
                    "Delete": {
                        name: "Delete",
                        icon: "",
                        disabled: getMenuStatus,
                        callback: function (key, opt) {
                            var name = $(this).find('.slick-column-name').text(); // the name of the field which we selected to delete
                            var item = _.findWhere(self.fields, { name: name }); //selecting the particular item which is going to delete from the grid
                            var position = self.fields.getIndexBy("name", name); //positon of the selected field
                            self.fields.splice(position, 1); //removing the selected item from schema of fields
                            self._createGrid(self.options.records); // creating grid with respect to the schema
                            var schema;
                            //getting the schema from server to save the new schema to it
                            $.ajax({
                                url: self.options.getSchema,
                                type: 'POST',
                                dataType: 'json',
                                contentType: "application/json; charset=utf-8",
                                async: false,
                                success: function (data) {
                                    if (data && typeof data.Record == "string") data = JSON.parse(data.Record);
                                    schema = data;
                                },
                                error: function () {
                                }
                            });
                            //deleting the selected field schema
                            _.each(schema[0], function (value, index) {
                                if (index == item.id) {
                                    delete schema[0][index];
                                }
                            });
                            var postdata = { schema: JSON.stringify(schema) };
                            self.saveSchema(postdata, 'The selected field has been deleted.');
                        }
                    }
                }
            });

            function getMenuStatus(key, options, x) {
                var div = options.$trigger;
                if (div.find('.slick-header-button').length == 0) {
                    return true;
                }
                else return false;
            }
        },

        createNewSchema: function (equationData) {
            var self = this;
            if (equationData.name != "") {
                var id = equationData.name.replace(/ /g, '');

                var newSchema = {
                    id: id,
                    name: equationData.name,
                    field: id,
                    type: "numeric",
                    decimalPoint: 2,
                    toolTip: equationData.name,
                    sortable: false,
                    minWidth: 80,
                    formula: equationData,
                    formatter: slickFormatter,
                    hasTotal: true,
                    header: {
                        buttons: [{
                            cssClass: "icon-highlight-off",
                            command: "toggle-highlight",
                            tooltip: "Derived column"
                        }]
                    },
                    isHidden: "N",
                    cssClass: "align-right",
                    headerCssClass: "align-right"
                };
                if (equationData.position)
                    self.fields.splice(equationData.position, 0, newSchema);
                self._createGrid(self.options.records);
                //self.manipulateSchema(newSchema);
            }
            else {
                _app.notify({
                    title: 'Save Successfully',
                    type: 'error',
                    text: "Please select a name",
                    delay: 3000
                });
            }
        },

        manipulateSchema: function (newSchema) {
            var self = this;
            var schema;
            $.ajax({
                url: self.options.getSchema,
                type: 'POST',
                dataType: 'json',
                contentType: "application/json; charset=utf-8",
                async: false,
                success: function (data) {
                    if (data && typeof data.Record == "string") data = JSON.parse(data.Record);
                    schema = data;
                },
                error: function () {
                }
            });
            _.each(schema[0], function (val, i) {
                if (newSchema.formula.postionId == i) {
                    schema[0][newSchema.id] = {
                        "title": newSchema.name,
                        "type": "textbox",
                        "dataType": newSchema.type,
                        "dateFormat": "",
                        "formula": newSchema.formula,
                        "required": "false",
                        "options": "",
                        "align": "left",
                        "defaultValue": "",
                        "decimalPoint": 2,
                        "isHidden": "N",
                        "isFilterable": "Y",
                        "isAxis": "Y",
                        "isGeo": "N",
                        "isKPI": "N",
                        "isTimeSeries": "N",
                        "aggType": "none",
                        "fieldType": "term",
                        "filter": false,
                        "aggregate": false,
                        "metrics": false
                    }
                }
            });
            var postdata = { schema: JSON.stringify(schema) };
            self.saveSchema(postdata, 'Formula field has been created.');
        },

        saveSchema: function (schema, message) {
            var self = this;
            self._ajaxcall(
                self.options.updateSchemaURL,
                schema,
                true, //asynchronous
                function (data) { //success
                    _app.notify({
                        title: 'Save Successfully',
                        type: 'success',
                        text: message,
                        delay: 3000
                    });
                }
            );
        },

        _createPageNavigator: function () {
            var self = this;
            var $container = $(self.element).find("div.paging-container");
            var template = '<span class="jtable-page-list">' +
            '{{#pageInfo}}' +
                '<span class="jtable-page-info">' +
                    '<span class="_lngTrans_Translatable">Showing first</span>' +
                    '<span>&nbsp;{{itemCount}}&nbsp;</span>' +
                    '<span class="_lngTrans_Translatable">records</span>' +
                '</span>' +
            '{{/pageInfo}}';

            var pages = [];
            var pageSize = [];
            var pagedList = self.pagedList;

            $.each(self.options.paging.pageSizes, function (idx, size) {
                pageSize.push({ value: size, selected: (size == pagedList.pageSize) });
            });

            var start = 1;
            var max = pagedList.pageCount;
            if (pagedList.pageCount > 100) {
                if ((pagedList.pageNumber + 100) >= pagedList.pageCount) {
                    start = pagedList.pageCount - 100;
                    max = pagedList.pageCount
                }
                else if (pagedList.pageNumber > 50) {
                    start = pagedList.pageNumber - 50;
                    max = start + 100;
                }
                else max = 100;
            }


            for (idx = start; idx <= max; idx++) {
                pages.push({ value: idx, selected: (idx == pagedList.pageNumber) });
            }

            var pageConfig = {
                pageInfo: (self.options.paging.showItemCount && pagedList.itemCount > 0),
                from: (pagedList.isFirstPage ? 1 : ((pagedList.pageNumber - 1) * pagedList.pageSize) + 1),
                to: (pagedList.isLastPage ? pagedList.itemCount : (pagedList.pageNumber * pagedList.pageSize)),
                prevPageIndex: (pagedList.pageNumber - 1),
                nextPageIndex: (pagedList.pageNumber + 1),
                pages: pages,
                pageSize: pageSize
            };
            var tmplData = $.extend(true, {}, pagedList, pageConfig);
            var pagingHtml = Mustache.render(template, tmplData);
            $container.empty().html(pagingHtml);
        },

        /* Parse to int value.
        *************************************************************************/
        _tryParseInt: function (value, defaultValue) {
            var retValue = defaultValue;
            var value = $.trim(value);
            if (value.length > 0 && !isNaN(value)) {
                retValue = parseInt(value);
            }
            return retValue;
        },

        /* Performs an AJAX call to specified URL.
        *************************************************************************/
        _ajaxcall: function (url, postData, async, success, error) {
            $.ajax({
                url: url,
                type: 'POST',
                dataType: 'jsonp',
                data: postData,
                async: async,
                success: function (data) {
                    if (data && typeof data == "string") data = JSON.parse(data);
                    success(data);
                },
                error: function () {
                    error();
                }
            });
        },

        /* Performs an JSONP call to specified URL.
        *************************************************************************/
        _jsonpcall: function (url, postData, async, success, error) {
            $.jsonp({
                url: '/BI/DataViewer/Preview', //url,
                callbackParameter: "callback",
                data: postData,
                cache: false,
                success: function (data) {
                    if (data && typeof data == "string") data = JSON.parse(data);
                    success(data);
                },
                error: function (xOptions, textStatus) {
                    error();
                }
            });
        }
    });
})(jQuery);

Array.prototype.getIndexBy = function (name, value) {
    for (var i = 0; i < this.length; i++) {
        if (this[i][name] == value) {
            return i;
        }
    }
    return -1;
}