﻿/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   
JQuery  widget for mis report viewer creation
Created By      : Rony T S for Assyst International Pvt. Ltd.
Created Date    : 17-March-2014
Version         : 1.0.0
Project         : PanAgro

+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/
//"use strict";        // ECMA 5 onwards.. recommended as it avoid silly mistakes..
var TotalRecordCount = 0;
(function ($) {    
    $.widget('assyst.reclineviewer', {
        /*
        Report viewer support three options
        1) url => here we can specify the  initial schema url of master report
        2) title => we can set the default title for the report
        3) search => for bind the advance search and set the initial parameter, it should be updated when the search criteria has changed
        4) GIS => has license for Map view, value: True/False
        5) viewer => viewer style, value: any of { "MultiView", "Grid", "Graph", "Map" };
        */
        options: {
            misID: undefined,
            dashletID: undefined,
            title: undefined,
            userRights: "",
            requestPage:undefined,
            MaxRowCount:undefined,
            minLegend: 5,
            catalog:undefined,
            source: {
                schema: undefined,
                data: undefined,
                schemaApi: undefined,
                dataApi: undefined
            },
            search: {
                show: true,
                controller: undefined,
                parameters: [],
                query: []
            },
            viewer: {
                view: "All",
                activeView: "Pivot",
                GIS: "True",
                visual: visual.Data
            },
            dashlet: {
                config: undefined,
                titles: undefined,
                name: "MIS",
                description: "Description",
                tag: "Tag",
                category: "Category"
            },
            exportURL: '/BI/MISViewer/Export/',
            saveDashletURL: '/BI/MISViewer/SaveDashlet/',
            updateDashletURL: '/BI/MISViewer/UpdateDashlet/',
            removeDashletURL: '/BI/MISViewer/RemoveDashlet/',
            saveConfigURL: '/BI/DataViewer/UpdateSchema/',
            gotoAnalyticsURL: undefined
        },
        schema: undefined,
        viewContainer: undefined,
        viewer: {
            _view: Viewer.All,
            _default: Viewer.All,
            get: function () {
                return this._view;
            },
            set: function (view) {
                switch (view.toUpperCase()) {
                    case "GRID":
                        this._view = Viewer.Grid;
                        break;
                    case "GRAPH":
                        this._view = Viewer.Graph;
                        break;
                    case "MAP":
                        this._view = Viewer.Map;
                        break;
                    case "TIMESERIES":
                        this._view = Viewer.Timeseries;
                        break;
                    case "PIVOT":
                        this._view = Viewer.Pivot;
                        break;
                    case "STATE":
                        break;
                    default:
                        this._view = Viewer.All;
                        break;
                }
            },
            getDefault: function () {
                return this._default;
            },
            setDefault: function (view) {
                switch (view.toUpperCase()) {
                    case "GRID":
                        this._default = Viewer.Grid;
                        break;
                    case "GRAPH":
                        this._default = Viewer.Graph;
                        break;
                    case "MAP":
                        this._default = Viewer.Map;
                        break;
                    case "PIVOT":
                        this._default = Viewer.Pivot;
                        break;
                    case "TIMESERIES":
                        this._default = Viewer.Timeseries;
                        break;
                    case "STATE":
                        break;
                    default:
                        this._default = Viewer.All;
                        break;
                }
            }
        },

        /*
        Constructor for report viewer
        _render method called for create the viewer 
        _eventManager for bind the event handling functions
        */
        _create: function () {
            try {
                this._initOptions();
                this._createViewer();
            }
            catch (e) {
                console.log(e);
            }
        },

        _initOptions: function () {
            var self = this;
            self.options.search.query = $.parseJSON(htmlDecode(self.options.search.query)) || [];
            self.options.dashlet.config = self.options.dashlet.config || undefined;
            self.options.dashlet.titles = $.parseJSON(htmlDecode(self.options.dashlet.titles)) || [];
            if (self.options.viewer.view) self.viewer.set(self.options.viewer.view);
            if (self.options.viewer.activeView) self.viewer.setDefault(self.options.viewer.activeView);
        },

        _createViewer: function () {
            var self = this;
            var l_container = self.element;
            if (self.options.source.schemaApi) {
                self._jsonpcall(self.options.source.schemaApi, 'GET', null, false, function (jsondata) {
                    bindSchema(jsondata);
                });
            }
            else {
                bindSchema(self.options.source.schema);
            }

            function bindSchema(jsondata) {
                if (typeof jsondata == "undefined") return;
                self.schema = jsondata;
                self._render();
            }
        },

        /*
        _render method for create the viewer 
        */
        _render: function () {
            var self = this;
            var l_container = self.element;
            l_container.empty();
            var l_tabReport = self._getUniqueIdentity("tabReport_");
            var l_mastertab = $("<div class='mis-container' id='" + l_tabReport + "'></div>");
            /*
            code for initialize the tab controller by adding a tab for master report
            before declare a div as tab, it should exist at least one tab div. 
            At the time of declaration we can also specify the template of tab li, 
            In our tabs we specified a custom template with a remove button for tab close
            */
            if (self.schema.title) self.options.title = self.schema.title;
            if (typeof self.schema.misSchema != "undefined" && self.options.dashlet.config) {
                var misViews = self.schema.misSchema.views.view;
                misViews = _.extendOptions(misViews, self.options.dashlet.config);
                /*$.each(self.options.dashlet.config, function (key, value) {
                    self.viewer.setDefault(key); 
                });*/
            }

            if (self.options.viewer.visual == visual.Dashboard) {
                //Check whetehr the data is schema or not
                if (self.options.dashlet.config) {
                    $.each(self.options.dashlet.config, function (key, value) { self.viewer.set(key); });
                }
                else if (typeof self.schema.misSchema != "undefined" && $.isArray(self.schema.misSchema)) {
                    var misViews = self.schema.misSchema[0].views;
                    $.each(misViews, function (key, value) {
                        if (value.show) { self.viewer.set(key); return false; }
                    });
                }
            }
            l_container.append(l_mastertab);

            //function call for bind the recline with div container by featching data from initial url
            self._loadData(self.schema, l_mastertab, self.options.title, self.viewer.get(), ($.trim(self.options.userRights).length > 0));
        },


        /*
        method for ditach the viewer widget with element
        */
        destroy: function () {
            // the DOM element associated with this instance
            var element = this.element;
            // the index, or location of this instance in the instances array
            var position = $.inArray(element, $.ui.dialog.instances);
            // if this instance was found, splice it off
            if (position > -1) { $.ui.dialog.instances.splice(position, 1); }
            this.element.removeClass('assyst.reclineviewer').enableSelection();
            // if using jQuery UI 1.8.x
            $.Widget.prototype.destroy.call(this);
            // if using jQuery UI 1.9.x
            //this._destroy();
        },

        _loadData: function (p_jsondata, p_container, p_title, p_viewer, p_saveAsDashlet) {
            var self = this;
            if (typeof p_jsondata === 'undefined') return;
            var l_recline_container = $("<div id='" + self._getUniqueIdentity("recline_") + "' class='recline_Tab'></div>").appendTo(p_container);
            //Check whetehr the data is schema or not
            if (typeof p_jsondata.misSchema != "undefined") {
                var misSchema = p_jsondata.misSchema;
                //Rest call for get the data
                var criteria = getCriteria(misSchema.views, self.options.search.query);

                if (self.options.source.data) {
                    misSchema["data"] = self.options.source.data;
                    var l_subviewer = $("<div id='" + self._getUniqueIdentity("viewer_") + "' class='viewerTab'></div>").appendTo(l_recline_container);
                    self._reclineinit(l_subviewer, misSchema, p_viewer, p_title, p_saveAsDashlet, undefined, criteria);
                }
                else {
                    var dataApi = self.options.source.dataApi || misSchema.dataApi;
                    self._jsonpcall(dataApi, 'GET', criteria, true, function (coredata) {
                        if (typeof coredata === 'undefined') return;                       
                        if (!coredata) return;
                        if (typeof coredata.Records == 'string') { coredata.Records = JSON.parse(coredata.Records); }
                        if (parseInt(self.options.MaxRowCount) < coredata.Records.length) {
                            if($('.dashboardDiv').length == 0) {
                                coredata.Records = coredata.Records.slice(0, -1);
                                var pivot_check = "Pivot"
                                if(self.options.viewer.activeView.toLowerCase() != pivot_check.toLowerCase()){
                                    _app.notify({
                                        title: 'Maximum Records',
                                        type: 'notice',
                                        text: 'Maximum records exceeded, please apply further filters for accurate result.',
                                        delay: 3000
                                    });
                                }
                            }
                        }

                        misSchema["data"] = coredata.Records;
                        misSchema["totalRecords"] = coredata.TotalRecordCount;
                        TotalRecordCount = coredata.TotalRecordCount;

                        var l_subviewer = $("<div id='" + self._getUniqueIdentity("viewer_") + "' class='viewerTab'></div>").appendTo(l_recline_container);
                        self._reclineinit(l_subviewer, misSchema, p_viewer, p_title, p_saveAsDashlet, dataApi, criteria);
                    });
                }
            }
            else {
                var data = ($.isArray(p_jsondata.data) ? p_jsondata.data : ($.isArray(p_jsondata) ? p_jsondata : []));
                var hiddenColumn = ['_dataImportId', '_catalogIdId', '_connectionId', 'geo'];
                var jsondata = {};
                var jsonfields = [];
                if (data.length > 0) {
                    $.each(data[0], function (key, value) {
                        jsonfields.push({
                            id: key,
                            label: key,
                            type: ((value) ? typeof value : 'string'),
                            isHidden: (($.inArray(key, hiddenColumn) > -1) ? 'Y' : 'N')
                        });
                    });
                }
                jsondata["data"] = data;
                jsondata["fields"] = jsonfields;
                jsondata["views"] = self.options.dashlet.config;
                self._reclineinit(l_recline_container, jsondata, p_viewer, p_title, p_saveAsDashlet);
            }

            function getCriteria(views, query) {
                var criteria = { displayFields: [], filter: [], aggregate: [] };
                if (query) criteria = $.extend(true, {}, criteria, query);

                var pagingState = { pageSize: 1000, pagedList: { pageNumber: 1} };
                if (views.state) {
                    if (views.state.pagingState) pagingState = $.extend(true, {}, pagingState, views.state.pagingState);
                    if (views.state.timeState && views.state.timeState.field && views.state.timeState.start && views.state.timeState.end) {
                        var field = _.findWhere(criteria.filter, { fieldName: views.state.timeState.field });
                        if (field) {
                            field.operator = "Between";
                            field.type = "date";
                            field.value1 = moment(views.state.timeState.start).format('YYYY-MM-DD');
                            field.value2 = moment(views.state.timeState.end).format('YYYY-MM-DD');
                        }
                        else {
                            var timeparmas = {
                                id: "",
                                fieldName: views.state.timeState.field,
                                displayName: "Period",
                                operator: "Between",
                                type: "date",
                                value1: moment(views.state.timeState.start).format('YYYY-MM-DD'),
                                value2: moment(views.state.timeState.end).format('YYYY-MM-DD')
                            };
                            criteria.filter.push(timeparmas);
                        }
                    }
                }
                var parameter = {
                    filterCriteria: (criteria) ? JSON.stringify(criteria) : null,
                    startIndex: ((pagingState.pagedList.pageNumber - 1) * pagingState.pageSize),
                    pageSize: parseInt(_app.MaxRecordCount) + 1,
                    sorting: null
                }
                return null;
            }
        },


        /*
        Method for bind the container and data with recline viewer
        */
        _reclineinit: function (p_container, p_data, p_viewer, p_title, p_saveAsDashlet, p_dataApi, p_parameter) {
            // Bind with recline js
            var self = this;
            var buttons = {
                saveAsDashlet: p_saveAsDashlet || false,
                save: false, // p_saveAsDashlet || false
                gotoAnalytics: self.options.gotoAnalyticsURL ? true : false
            };
            if (self.options.viewer.visual == visual.Dashlet) {
                $.extend(buttons, {
                    save: false,
                    export: true,
                    navigation: false,
                    pagination: true,
                    search: false,
                    gotoAnalytics: true
                });
            }

            self.viewContainer = p_container;
            $(p_container).reclineInit({
                title: p_title,
                views: p_data.views.view,
                requestPage:self.options.requestPage,
                viewer: (p_viewer || self.viewer.get()),
                catalog:self.options.catalog,
                defaultViewer: (p_viewer || self.viewer.getDefault()),
                GIS: self.options.viewer.GIS === 'True',
                minLegend: self.options.minLegend,
                source: {
                    records: p_data.data,
                    fields: p_data.fields,
                    totalRecords: p_data.totalRecords
                },
                buttons: buttons,
                dashlet: self.options.dashlet,
                export: {
                    title: p_title,
                    criteria: getCriteria(),
                    type: "PDF",
                    pageSize: "A4",
                    promptDialog: true,
                    url: self.options.exportURL
                },

                onGotoAnalytics: self.options.gotoAnalyticsURL ? function () { window.location = self.options.gotoAnalyticsURL; } : undefined,

                onSaveAsDashlet: function (jsonData, isDashboard, title, saveOption,obj) {
                    var data = {
                        misID: /*(self.options.viewer.visual == visual.Dashlet) ?*/ self.options.dashletID,// : self.options.misID,
                        isDashboard: isDashboard,
                        dashletGroup: JSON.stringify(jsonData),
                        title: title
                    };
                    var url;
                    if(self.options.viewer.visual == visual.DashletOnView) {                        
                        if(saveOption == "Save" && self.options.catalog.CatalogTypeId == _app.catalogType.Dashlet) {
                            url = self.options.updateDashletURL;
                        }
                        else {
                            url = self.options.saveDashletURL;
                        }
                    }                    
                    else if(self.options.viewer.visual == visual.Dashlet) {
                        //Checking the dashlet saving is update or save
                        if(saveOption == "Save As") {
                            url = self.options.saveDashletURL;
                        }
                        else if(saveOption == "Save") {
                            url = self.options.updateDashletURL;
                        }
                    }
                    else {
                        url = self.options.saveDashletURL;
                    }
                    self.saveDashlet(url,data,obj);
                },

                onSaveConfig: function (jsonData) {
                    var data = {
                        misID: self.options.misID,
                        config: JSON.stringify(jsonData)
                    };
                    self._ajaxcall(self.options.saveConfigURL, 'POST', data, false, function (result) {
                        if (result.Message)
                        $("#dialog-confir").html(result.Message);
                        $("#dialog-confir").data('object', self).dialog({
                            resizable: false,
                            modal: true,
                            height: "180",
                            width: "auto",
                            draggable: false,
                            close: function () { $(this).dialog('destroy'); },
                            dialogClass: "message info-dialog custom-style no-titlebar",
                            buttons: {
                                Ok: function () {
                                    $(this).dialog('destroy');
                                }
                            }
                        });
                    });
                },

                onRemoveDashlet: function (item) {
                    var data = {
                        misID: item.value,
                        name: item.text
                    };
                    self._ajaxcall(self.options.removeDashletURL, 'POST', data, false, function (result) {
                        $("#dialog-confir").html(result.Message);
                        $("#dialog-confir").data('object', self).dialog({
                            resizable: false,
                            modal: true,
                            height: "180",
                            width: "auto",
                            draggable: false,
                            dialogClass: "message info-dialog custom-style no-titlebar",
                            buttons: {
                                Ok: function () {
                                    $(this).dialog('destroy');
                                }

                            }
                        });
                    });
                },

                onDateRangeChange: function (timeseries, picker, callback) {
                    if (typeof p_dataApi) {
                        var query = $.parseJSON(p_parameter.filterCriteria);
                        var field = _.findWhere(query.filter, { lookupName: 'timeseries' });
                        if (field) {
                            field.fieldName = timeseries.group;
                            field.operator = "Between";
                            field.type = "date";
                            field.value1 = moment(picker.startDate).format('YYYY-MM-DD');
                            field.value2 = moment(picker.endDate).format('YYYY-MM-DD');
                        }
                        else {
                            var timeparmas = {
                                displayName: "Period",
                                fieldName: timeseries.group,
                                filterType: "required",
                                lookupName: 'timeseries',
                                multiValue: "false",
                                operator: "Between",
                                type: "date",
                                value1: moment(picker.startDate).format('YYYY-MM-DD'),
                                value2: moment(picker.endDate).format('YYYY-MM-DD')
                            };
                            query.filter.push(timeparmas);
                        }

                        p_parameter["filterCriteria"] = JSON.stringify(query);
                        self._jsonpcall(p_dataApi, 'GET', p_parameter, false, callback);
                    }
                },

                onPagingChange: function (pagingState, callback) {
                    if (typeof p_dataApi) {
                        p_parameter["startIndex"] = ((pagingState.pagedList.pageNumber - 1) * pagingState.pageSize);
                        p_parameter["pageSize"] = pagingState.pageSize;
                        self._jsonpcall(p_dataApi, 'GET', p_parameter, false, callback);
                    }
                }
            });

            function getCriteria() {
                var criteriaList = [];
                return criteriaList;
            }
        },

        saveDashlet: function (url, data, obj) {
            var self = this;
            self._ajaxcall(url, 'POST', data, false, function (result) {
                if(result.Result != "ERROR") {
                    //To Load Saved dashlet after saving Dashlet
                    window.history.pushState("object or string", "Title", window.location.pathname+"?misID="+result.CatalogId+"&fromCatalogViewer="+result.fromCatalog);
                    self.options.catalog.CatalogId = result.CatalogId;
                    $('.titleName').text(result.CatalogName);
                    self.options.viewer.visual = "Dashlet";
                    self.options.misID = result.CatalogId;
                    self.options.dashletID = result.CatalogId;
                    if (self.options.source.schemaApi)
                    self.options.source.schemaApi = (self.options.source.schemaApi).split("?misID=")[0]+"?misID="+result.CatalogId;
                    var newDashletList = {};
                    if((self.options.catalog.dashletList).length==0)  newDashletList = {CatalogId:result.CatalogId,Name:result.CatalogName,selected:true,DashletType:$('.apple_btn.active').text()};
                    else {
                        $.each(self.options.catalog.dashletList,function(i,val){
                            if(val.CatalogId != result.CatalogId){
                                val.selected = false;
                                newDashletList = {CatalogId:result.CatalogId,Name:result.CatalogName,selected:true,DashletType:$('.apple_btn.active').text()};
                            }
                        });
                    }
                    if(!$.isEmptyObject(newDashletList))self.options.catalog.dashletList.push(newDashletList);
                    var pivot = _.findWhere(obj.pageViews,{id:"pivot"});
                    pivot.view.state.set({'Pivot':obj.model.queryState.get('Pivot')});
                    $(".pivotedDiv").height($(window).height()-160);
                    obj.reDraw(result,self.options.catalog.dashletList);
                    _app.notify({
                            title: 'Success',
                            type: 'Success',
                            text: 'Catalog saved successfully.',
                            delay: 3000
                    });
                }
                if(result.Result!="OK")
                {
                    //To Display validation messages while saving Dashlets
                    $("#dialog-confir").html(result.Message);
                    $("#dialog-confir").data('object', self).dialog({
                            resizable: false,
                            modal: true,
                            height: "180",
                            width: "auto",
                            draggable: false,
                            close: function () { $(this).dialog('destroy'); },
                                dialogClass: "message success-dialog custom-style no-titlebar",
                        buttons: {
                            Ok: function () {
                                $(this).dialog('destroy');
                            }
                        }
                    });
                }
            });
        },

        //Ajax Call - Communicate with server

        _ajaxcall: function (url, type, postData, async, success) {
            $.ajax({
                url: url,
                type: type || 'GET',
                dataType: 'json',
                data: postData,
                async: async,
                success: success,
                error: function (p_request, p_status, p_error) {
                    if (window.console && window.console.log) console.log((p_error.message ? p_error.message : p_error))
                }
            });
        },
        _jsonpcall: function (url, type, postData, async, success) {
            $.jsonp({
                url: url,
                callbackParameter: "callback",
                data: postData,
                cache: false,
                success: function (data) {
                    if (data && typeof data == "string") data = JSON.parse(data);
                    success(data)
                },
                error: function (xOptions, textStatus) {
                    if (window.console && window.console.log) console.log(textStatus)
                }
            });
        },
        /*
        Method for create a unique id
        */
        _getUniqueIdentity: function (stratwith) {
            return String(stratwith) + Math.floor(Math.random() * 99999);
        },

        SaveReclineConfig: function (misID) {
            var self = this;
            if (misID) self.options.misID = misID;
            $(self.viewContainer).reclineInit("SaveReclineConfig");
        },

        RestoreData: function (data, pagingState) {
            var self = this;
            $(self.viewContainer).reclineInit("RestoreData", data, pagingState);
        },

        GetActiveViewConfig: function (data) {
            var self = this;
            return $(self.viewContainer).reclineInit("GetActiveViewConfig");
        },

        GetReclineConfig: function (data) {
            var self = this;
            return $(self.viewContainer).reclineInit("GetReclineConfig");
        }
    });
})($);
