﻿/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   
JQuery  widget for mis report viewer creation
Created By      : Rony T S for Assyst International Pvt. Ltd.
Created Date    : 31-July-2015
Version         : 1.0.0
Project         : PanBI

+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/
//"use strict";        // ECMA 5 onwards.. recommended as it avoid silly mistakes..

var Viewer = { All: 0, Grid: 1, Graph: 2, Map: 3, Timeseries: 4 };
var visual = { Data: "Data", Dashlet: "Dashlet", Dashboard: "Dashboard", DashletOnView: "DashletOnView" };

(function ($) {
    $.widget('assyst.reclineviewer_notinuse', {
        /*
        Report viewer support three options
        1) url => here we can specify the  initial schema url of master report
        2) title => we can set the default title for the report
        3) search => for bind the advance search and set the initial parameter, it should be updated when the search criteria has changed
        4) GIS => has license for Map view, value: True/False
        5) viewer => viewer style, value: any of { "MultiView", "Grid", "Graph", "Map" };
        */
        options: {
            misID: undefined,
            title: undefined,
            userRights: "",

            source: {
                restapiMode: true,
                schema: undefined,
                data: undefined,
                timeState: { start: moment("01-Jan-2014"), end: moment(new Date()), interval: 2 }
            },
            search: {
                show: true,
                controller: undefined,
                parameters: [],
                query: []
            },
            viewer: {
                view: "All",
                activeView: "Graph",
                GIS: "True",
                visual: visual.Data
            },
            dashlet: {
                config: undefined,
                titles: undefined,
                name: "MIS",
                description: "Description",
                tag: "Tag",
                category: "Category"
            },
            exportURL: '/BI/MISViewer/Export/',
            saveDashletURL: '/BI/MISViewer/SaveDashlet/',
            updateDashletURL: '/BI/MISViewer/UpdateDashlet/',
            removeDashletURL: '/BI/MISViewer/RemoveDashlet/',
            saveConfigURL: '/BI/DataViewer/UpdateSchema/',
            gotoAnalyticsURL: undefined
        },
        schema: undefined,
        viewContainer: undefined,
        viewer: {
            _view: Viewer.All,
            _default: Viewer.All,
            get: function () {
                return this._view;
            },
            set: function (view) {
                switch (view.toUpperCase()) {
                    case "GRID":
                        this._view = Viewer.Grid;
                        break;
                    case "GRAPH":
                        this._view = Viewer.Graph;
                        break;
                    case "MAP":
                        this._view = Viewer.Map;
                        break;
                    case "TIMESERIES":
                        this._view = Viewer.Timeseries;
                        break;
                    case "STATE":
                        break;
                    default:
                        this._view = Viewer.All;
                        break;
                }
            },
            getDefault: function () {
                return this._default;
            },
            setDefault: function (view) {
                switch (view.toUpperCase()) {
                    case "GRID":
                        this._default = Viewer.Grid;
                        break;
                    case "GRAPH":
                        this._default = Viewer.Graph;
                        break;
                    case "MAP":
                        this._default = Viewer.Map;
                        break;
                    case "TIMESERIES":
                        this._default = Viewer.Timeseries;
                        break;
                    case "STATE":
                        break;
                    default:
                        this._default = Viewer.All;
                        break;
                }
            }
        },

        /*
        Constructor for report viewer
        _render method called for create the viewer 
        _eventManager for bind the event handling functions
        */
        _create: function () {
            try {
                this._initOptions();
                this._createViewer();
            }
            catch (e) {
                console.log(e);
            }
        },

        _initOptions: function () {
            var self = this;
            self.options.search.query = $.parseJSON(htmlDecode(self.options.search.query)) || [];
            self.options.dashlet.config = $.parseJSON(htmlDecode(self.options.dashlet.config)) || undefined;
            self.options.dashlet.titles = $.parseJSON(htmlDecode(self.options.dashlet.titles)) || [];
            if (self.options.viewer.view) self.viewer.set(self.options.viewer.view);
            if (self.options.viewer.activeView) self.viewer.setDefault(self.options.viewer.activeView);
        },

        _createViewer: function () {
            var self = this;
            var l_container = self.element;
            if (self.options.source.restapiMode) {
                self._jsonpcall(self.options.source.schema, 'GET', null, false, function (jsondata) {
                    bindSchema(jsondata);
                });
            }
            else {
                bindSchema(self.options.source.schema);
            }

            function bindSchema(jsondata) {
                if (typeof jsondata == "undefined") return;
                //if (typeof jsondata == "string") jsondata = JSON.parse(jsondata);

                self.schema = jsondata;
                self._render();
            }
        },


        /*
        _render method for create the viewer 
        */
        _render: function () {
            var self = this;
            var l_container = self.element;
            var l_tabReport = self._getUniqueIdentity("tabReport_");
            var l_mastertab = $("<div class='mis-container' id='" + l_tabReport + "'></div>");
            /*
            code for initialize the tab controller by adding a tab for master report
            before declare a div as tab, it should exist at least one tab div. 
            At the time of declaration we can also specify the template of tab li, 
            In our tabs we specified a custom template with a remove button for tab close
            */
            if (self.schema.title) self.options.title = self.schema.title;

            if (typeof self.schema.misSchema != "undefined" && $.isArray(self.schema.misSchema) && self.options.dashlet.config) {
                var misViews = self.schema.misSchema[0].views;
                //misViews = $.extend(true, misViews, self.options.dashlet.config);
                misViews = _.extendOptions(misViews, self.options.dashlet.config);
                //self.schema.misSchema[0].views = _.defaults({}, self.options.dashlet.config, misViews);
                $.each(self.options.dashlet.config, function (key, value) { self.viewer.setDefault(key); });
                //misViews = self.options.dashlet.config;
            }

            if (self.options.viewer.visual == visual.Dashboard) {
                /*var appendDiv = "<div class='corpDiv'></div>";
                $('body').last().append(appendDiv);
                $('.corpDiv').append(l_container);
                $('.corpDiv').siblings().hide();*/

                //Check whetehr the data is schema or not
                if (self.options.dashlet.config) {
                    $.each(self.options.dashlet.config, function (key, value) { self.viewer.set(key); });
                }
                else if (typeof self.schema.misSchema != "undefined" && $.isArray(self.schema.misSchema)) {
                    var misViews = self.schema.misSchema[0].views;
                    $.each(misViews, function (key, value) {
                        if (value.show) { self.viewer.set(key); return false; }
                    });
                }
            }
            l_container.append(l_mastertab);

            //function call for bind the recline with div container by featching data from initial url
            self._loadData(self.schema, l_mastertab, self.options.title, self.viewer.get(), ($.trim(self.options.userRights).length > 0));
        },

        _renderRecline: function (options) {
        },

        /*
        method for ditach the viewer widget with element
        */
        destroy: function () {
            // the DOM element associated with this instance
            var element = this.element;
            // the index, or location of this instance in the instances array
            var position = $.inArray(element, $.ui.dialog.instances);
            // if this instance was found, splice it off
            if (position > -1) { $.ui.dialog.instances.splice(position, 1); }
            this.element.removeClass('assyst.reportviewer').enableSelection();
            // if using jQuery UI 1.8.x
            $.Widget.prototype.destroy.call(this);
            // if using jQuery UI 1.9.x
            //this._destroy();
        },

        _loadData: function (p_jsondata, p_container, p_title, p_viewer, p_saveAsDashlet) {

            var self = this;
            if (typeof p_jsondata === 'undefined') return;
            var l_recline_container = $("<div id='" + self._getUniqueIdentity("recline_") + "'></div>").appendTo(p_container);

            //Check whetehr the data is schema or not
            if (typeof p_jsondata.misSchema != "undefined") {

                //Schema may have more tha one report
                $.each(p_jsondata.misSchema, function (key, value) {
                    //Rest call for get the data
                    var criteria = getCriteria(value.views, self.options.search.query);
                    var parameter = (criteria) ? { filterCriteria: JSON.stringify(criteria)} : null;

                    if (self.options.source.restapiMode) {
                        if ($.trim(value.dataUrl).length == 0) value.dataUrl = self.options.source.data;
                        self._jsonpcall(value.dataUrl, 'GET', parameter, true, function (coredata) {

                            if (typeof coredata === 'undefined') return;
                            //if (typeof coredata == "string") coredata = JSON.parse(coredata);

                            if (!coredata) return; value["data"] = coredata.data || coredata;

                            var l_subviewer = $("<div id='" + self._getUniqueIdentity("viewer_") + "'></div>").appendTo(l_recline_container);

                            //bind the data and container with recline
                            self._reclineinit(l_subviewer, value, p_viewer, p_title, p_saveAsDashlet, value.dataUrl, criteria);
                        });
                    }
                    else {
                        if (!self.options.source.data) return;
                        value["data"] = self.options.source.data;

                        var l_subviewer = $("<div id='" + self._getUniqueIdentity("viewer_") + "'></div>").appendTo(l_recline_container);
                        //bind the data and container with recline
                        self._reclineinit(l_subviewer, value, p_viewer, p_title, p_saveAsDashlet, value.dataUrl, criteria);
                    }
                });
            }
            else {
                var data = ($.isArray(p_jsondata.data) ? p_jsondata.data : ($.isArray(p_jsondata) ? p_jsondata : []));
                var hiddenColumn = ['_dataImportId', '_catalogIdId', '_connectionId', 'geo'];
                var jsondata = {};
                var jsonfields = [];
                if (data.length > 0) {
                    $.each(data[0], function (key, value) {
                        jsonfields.push({
                            id: key,
                            label: key,
                            type: ((value) ? typeof value : 'string'),
                            isHidden: (($.inArray(key, hiddenColumn) > -1) ? 'Y' : 'N')
                        });
                    });
                }
                jsondata["data"] = data;
                jsondata["fields"] = jsonfields;
                jsondata["views"] = self.options.dashlet.config;
                self._reclineinit(l_recline_container, jsondata, p_viewer, p_title, p_saveAsDashlet);
            }

            function getCriteria(views, query) {

                var criteria = { displayFields: [], filter: [], aggregate: [] };
                //Old Advance search json fix
                if (query && $.isArray(query)) criteria = $.extend(true, {}, { displayFields: [], filter: query, aggregate: [] });

                if (views.state && views.state.field && views.state.start && views.state.end) {

                    var field = _.findWhere(criteria.filter, { fieldName: views.state.field });
                    if (field) {
                        lookupName: 'timeseries';
                        field.operator = "Between";
                        field.type = "date";
                        field.value1 = moment(views.state.start).format('YYYY-MM-DD');
                        field.value2 = moment(views.state.end).format('YYYY-MM-DD');
                    }
                    else {
                        var timeparmas = {
                            displayName: "Period",
                            fieldName: views.state.field,
                            filterType: "required",
                            lookupName: 'timeseries',
                            multiValue: "false",
                            operator: "Between",
                            type: "date",
                            value1: moment(views.state.start).format('YYYY-MM-DD'),
                            value2: moment(views.state.end).format('YYYY-MM-DD')
                        };
                        criteria.filter.push(timeparmas);
                    }
                }
                return criteria;
            }
        },


        /*
        Method for bind the container and data with recline viewer
        */
        _reclineinit: function (p_container, p_data, p_viewer, p_title, p_saveAsDashlet, p_dataUrl, p_parameter) {
            // Bind with recline js
            var self = this;

            var buttons = {
                saveAsDashlet: p_saveAsDashlet || false,
                save: p_saveAsDashlet || false,
                gotoAnalytics: self.options.gotoAnalyticsURL ? true : false
            };
            if (self.options.viewer.visual == visual.Dashlet) {
                $.extend(buttons, {
                    save: false,
                    export: false,
                    navigation: false,
                    pagination: true,
                    search: false,
                    gotoAnalytics: false
                });
            }

            self.viewContainer = p_container;
            self._renderRecline({
                title: p_title,
                views: p_data.views,
                viewer: (p_viewer || self.viewer.get()),
                defaultViewer: (p_viewer || self.viewer.getDefault()),
                GIS: self.options.viewer.GIS === 'True',
                source: {
                    url: p_dataUrl,
                    query: p_parameter,
                    records: p_data.data,
                    fields: p_data.fields,
                    timeState: self.options.source.timeState // $.extend({}, self.options.timeState, (p_data.views.sate ? (p_data.views.sate.timeState || {}) : {}))
                },
                buttons: buttons,
                dashlet: self.options.dashlet,
                export: {
                    title: p_title,
                    criteria: getCriteria(),
                    type: "PDF",
                    pageSize: "A4",
                    promptDialog: true,
                    url: self.options.exportURL
                },

                onGotoAnalytics: self.options.gotoAnalyticsURL ? function () { window.location = self.options.gotoAnalyticsURL; } : undefined,
                onSaveAsDashlet: function (jsonData, isDashboard, title) {
                    var data = {
                        misID: self.options.misID,
                        isDashboard: isDashboard,
                        dashletGroup: JSON.stringify(jsonData),
                        title: title
                    };
                    var url = (self.options.viewer.visual == visual.Dashlet) ? self.options.updateDashletURL : self.options.saveDashletURL;
                    self._ajaxcall(url, 'POST', data, false, function (result) {
                        if (result.Message) alert(result.Message);
                    });
                },
                onSaveConfig: function (jsonData) {
                    var data = {
                        misID: self.options.misID,
                        config: JSON.stringify(jsonData)
                    };
                    self._ajaxcall(self.options.saveConfigURL, 'POST', data, false, function (result) {
                        if (result.Message) ShowValidationMessages(result.Message);
                    });
                },
                onRemoveDashlet: function (item) {
                    var data = {
                        misID: item.value,
                        name: item.text
                    };
                    self._ajaxcall(self.options.removeDashletURL, 'POST', data, false, function (result) {
                        if (result.Message) alert(result.Message);
                    });
                },
                onDateRangeChange: function (timeseries, picker, callback) {
                    if (typeof p_dataUrl) {

                        //Old Advance search json fix
                        if (p_parameter && $.isArray(p_parameter)) p_parameter = $.extend(true, {}, { displayFields: [], filter: p_parameter, aggregate: [] });
                        p_parameter = $.extend({ displayFields: [], filter: [], aggregate: [] }, p_parameter);

                        var field = _.findWhere(p_parameter.filter, { lookupName: 'timeseries' });
                        if (field) {
                            field.fieldName = timeseries.group;
                            field.operator = "Between";
                            field.type = "date";
                            field.value1 = moment(picker.startDate).format('YYYY-MM-DD');
                            field.value2 = moment(picker.endDate).format('YYYY-MM-DD');
                        }
                        else {
                            var timeparmas = {
                                displayName: "Period",
                                fieldName: timeseries.group,
                                filterType: "required",
                                lookupName: 'timeseries',
                                multiValue: "false",
                                operator: "Between",
                                type: "date",
                                value1: moment(picker.startDate).format('YYYY-MM-DD'),
                                value2: moment(picker.endDate).format('YYYY-MM-DD')
                            };
                            p_parameter.filter.push(timeparmas);
                        }

                        var parameter = (p_parameter) ? { filterCriteria: JSON.stringify(p_parameter)} : null;
                        self._jsonpcall(p_dataUrl, 'GET', parameter, false, callback);
                    }
                },
                onPagingChange: function (pagingState, callback) {
                    if (typeof p_dataUrl) {

                        //Old Advance search json fix
                        if (p_parameter && $.isArray(p_parameter)) p_parameter = $.extend(true, {}, { displayFields: [], filter: p_parameter, aggregate: [] });
                        p_parameter = $.extend({ displayFields: [], filter: [], aggregate: [] }, p_parameter);

                        var postdata = {
                            filterCriteria: JSON.stringify(p_parameter),
                            startIndex: ((pagingState.pagedList.pageNumber - 1) * pagingState.pageSize),
                            pageSize: pagingState.pageSize,
                            sorting: null
                        }
                        self._jsonpcall(p_dataUrl, 'GET', postdata, false, callback);
                    }
                }
            });

            function getCriteria() {
                var criteriaList = [];
                /*_.each(self.options.search.query, function (data) {
                var criteria = {
                fieldName: data.fieldName,
                operator: data.operator,
                value1: data.type == "lookup" ? data.lookupName : data.value1,
                value2: data.value2,
                type: data.type == "lookup" ? "string" : data.type,
                filterType: data.filterType,
                displayName: data.displayName,
                enabletimeformat: "false",
                lookupName: null
                };
                criteriaList.push(criteria);
                });*/
                return criteriaList;
            }

        },

        //Ajax Call - Communicate with server

        _ajaxcall: function (url, type, postData, async, success) {
            $.ajax({
                url: url,
                type: type || 'GET',
                dataType: 'json',
                data: postData,
                async: async,
                success: success,
                error: function (p_request, p_status, p_error) {
                    if (window.console && window.console.log) console.log((p_error.message ? p_error.message : p_error))
                }
            });
        },
        _jsonpcall: function (url, type, postData, async, success) {
            $.jsonp({
                url: url,
                callbackParameter: "callback",
                data: postData,
                cache: false,
                success: function (data) {
                    if (data && typeof data == "string") data = JSON.parse(data);
                    success(data)
                },
                error: function (xOptions, textStatus) {
                    if (window.console && window.console.log) console.log(textStatus)
                }
            });
        },
        /*
        Method for create a unique id
        */
        _getUniqueIdentity: function (stratwith) {
            return String(stratwith) + Math.floor(Math.random() * 99999);
        },

        saveReclineConfig: function (misID) {
            var self = this;
            if (misID) self.options.misID = misID;
            $(self.viewContainer).reclineInit("saveReclineConfig");
        },

        RestoreData: function (data) {
            var self = this;
            $(self.viewContainer).reclineInit("RestoreData", data);
        }
    });

    $.widget('assyst.reclineviewer', $.assyst.reclineviewer, {

        recloptions: {
            title: undefined,
            GIS: true,
            viewer: Viewer.All,
            defaultViewer: Viewer.All,
            source: {
                url: undefined,
                query: undefined,
                records: undefined,
                fields: undefined,
                filters: undefined,
                axisFields: undefined,
                geoFields: undefined,
                timeSeries: undefined
            },
            views: {
                grid: {
                    show: false,
                    label: 'Grid',
                    state: {
                        gridOptions: {
                            editable: false,
                            enabledAddRow: true,
                            enableCellNavigation: true
                        },
                        hiddenColumns: undefined,
                        columnsWidth: undefined
                    }
                },
                graph: {
                    show: false,
                    label: 'Chart',
                    state: {
                        series: [],
                        graphTypes: [],
                        graphType: "multiBar",
                        group: undefined,
                        series: [],
                        showData: false
                    }
                },
                map: {
                    show: false,
                    label: 'Map',
                    state: {
                        heatField: undefined,
                        type: "Point"
                    }
                },
                timeseries: {
                    show: false,
                    label: 'Timeseries',
                    state: {
                        series: [],
                        graphTypes: [],
                        graphType: "line",
                        group: undefined,
                        compareField: undefined,
                        multipleCompare: "true",
                        series: []
                    }
                },
                state: {
                    timeState: {
                        enable: false,
                        datePicker: true,
                        start: undefined, //moment("01-Jan-2014")
                        end: undefined, //moment(newDate())
                        interval: 2,
                        field: undefined
                    },
                    queryState: {
                        size: 500,
                        from: 0,
                        sort: undefined
                    }
                }
            },
            export: {
                title: "MIS", //Give required title dynamically here
                criteria: undefined, //Structure [{ "fieldName": "StartDate", "operator": "==", "value1": "01-08-2013", "value2": null, "type": "dateTime", "filterType": "mandatory", "displayName": "StartDate", "enabletimeformat": "false", "lookupName": null }, { "fieldName": "EndtDate", "operator": "==", "value1": "31-08-2013", "value2": null, "type": "dateTime", "filterType": "mandatory", "displayName": "EndDate", "enabletimeformat": "false", "lookupName": null }, { "fieldName": "Estate", "operator": "in", "value1": "COTTABETTA", "value2": null, "type": "string", "filterType": "mandatory", "displayName": "Estate", "enabletimeformat": "false", "lookupName": null }, { "fieldName": "Division", "operator": "0", "value1": "", "value2": null, "type": "string", "filterType": "default", "displayName": "Division", "enabletimeformat": "false", "lookupName": null }, { "fieldName": "Gang", "operator": "0", "value1": "", "value2": null, "type": "string", "filterType": "default", "displayName": "Gang", "enabletimeformat": "false", "lookupName": null }]
                type: "PDF", //Default export type(optional)
                pageSize: "A4", //Default page size(optional)
                promptDialog: true, // If set to false, will be exported in default type and page size
                url: undefined // Url for document down load
            },
            dashlet: {
                name: "MIS",
                description: "Description",
                tag: "Tag",
                category: "Category"
            },
            buttons: {
                save: true,
                export: true,
                saveAsDashlet: true,
                navigation: true,
                pagination: true,
                search: true
            }
        },
        _reclineView: undefined,
        _renderRecline: function (options) {
            $.extend(true, this.recloptions, options);

            this._create_Structure();
            if (g_LangTranslator) {
                g_LangTranslator.ApplyTranslation(this.element.id);
            }
        },

        _create_Structure: function () {
            var self = this;
            window.dataExplorer = null;
            window.explorerDiv = this.element;

            var state = recline.View.parseQueryString(decodeURIComponent(window.location.search));
            if (state) {
                _.each(state, function (value, key) {
                    try {
                        value = JSON.parse(value);
                    } catch (e) { }
                    state[key] = value;
                });
            } else {
                state.url = 'demo';
            }

            var dataset = null;
            if (state.dataset || state.url) {
                var datasetInfo = _.extend({
                    url: state.url,
                    backend: state.backend
                },
                    state.dataset
                );
                dataset = new recline.Model.Dataset(datasetInfo);
            } else {
                if (self.recloptions.source.fields != undefined) {

                    // Code for initialize the time state selector
                    if (self.recloptions.views.timeseries && self.recloptions.views.timeseries.show && self.recloptions.views.timeseries.state.group && _.findWhere(self.recloptions.source.fields, { id: self.recloptions.views.timeseries.state.group })) {
                        self.recloptions.views.state.timeState.enable = true;
                        //self.recloptions.views.state.timeState.datePicker = self.recloptions.views.timeseries.datePicker;

                        var field = self.recloptions.views.timeseries.state.group;
                        if (!self.recloptions.views.state.timeState.field) self.recloptions.views.state.timeState.field = field;
                        if (!self.recloptions.views.state.queryState.sort) self.recloptions.views.state.queryState.sort = [{ field: field, order: "asc"}];

                        var groupDates = $.map(self.recloptions.source.records, function (value) { return moment(value[field]); });
                        if (groupDates && groupDates.length > 0) {
                            if (!self.recloptions.views.state.timeState.start) self.recloptions.views.state.timeState.start = Math.min.apply(this, groupDates);
                            if (!self.recloptions.views.state.timeState.end) self.recloptions.views.state.timeState.end = Math.max.apply(this, groupDates);
                        }
                    }

                    dataset = new recline.Model.Dataset({
                        records: self.recloptions.source.records,
                        fields: self.recloptions.source.fields,
                        filters: self.recloptions.source.filters,
                        axisFields: self.recloptions.source.axisFields,
                        geoFields: self.recloptions.source.geoFields,
                        timeSeries: self.recloptions.source.timeSeries,
                        timeState: self.recloptions.views.state.timeState,
                        queryInit: self.recloptions.views.state.queryState
                    });
                }
                else {
                    dataset = new recline.Model.Dataset({
                        records: self.recloptions.source.records
                    });
                }
            }
            switch (self.recloptions.defaultViewer) {
                case Viewer.Grid:
                    state["currentView"] = "grid";
                    break;
                case Viewer.Graph:
                    state["currentView"] = "graph";
                    break;
                case Viewer.Map:
                    state["currentView"] = "map";
                    break;
                case Viewer.Timeseries:
                    state["currentView"] = "timeseries";
                    break;
                default:
                    state["currentView"] = "all";
                    break;
            }

            switch (self.recloptions.viewer) {
                case Viewer.Grid:
                    self._create_GridExplorer(dataset);
                    break;
                case Viewer.Graph:
                    self._create_GraphExplorer(dataset);
                    break;
                case Viewer.Map:
                    self._create_MapExplorer(dataset);
                    break;
                case Viewer.Timeseries:
                    self._create_TimeseriesExplorer(dataset);
                    break;
                default:
                    self._create_ViewExplorer(dataset, state);
                    break;
            }
        },

        _create_GridExplorer: function (dataset) {
            var self = this;
            var continer = self.element;
            var grid = new recline.View.SlickGrid({
                model: dataset,
                state: self.recloptions.views.graph.state,
                el: continer
            });
            grid.visible = true;
            grid.render();
        },

        _create_GraphExplorer: function (dataset) {
            var self = this;
            var continer = self.element;
            var graph = new recline.View.Graph({
                model: dataset,
                state: self.recloptions.views.graph.state,
                el: continer
            });
            graph.render();
            graph.redraw();
        },

        _create_MapExplorer: function (dataset) {
            var self = this;
            var continer = self.element;
            self.recloptions.views.map.state["autoZoom"] = true;
            var map = new recline.View.Map({
                model: dataset,
                state: self.recloptions.views.map.state,
                el: continer
            });
            map.render();
            map.redraw();
        },

        _create_TimeseriesExplorer: function (dataset) {
            var self = this;
            var continer = self.element;
            var timeseries = new recline.View.Timeseries({
                model: dataset,
                state: self.recloptions.views.timeseries.state,
                el: continer
            });
            timeseries.render();
            timeseries.redraw();
        },

        _create_ViewExplorer: function (dataset, state) {
            var self = this;
            $el = $('<div />');
            $el.appendTo(window.explorerDiv);

            var reload = false;
            if (window.dataExplorer) {
                window.dataExplorer.remove();
                reload = true;
            }
            var views = [];
            if (self.recloptions.views.graph.show) {
                self.recloptions.views.graph.state.showData = true;
                var graph = {
                    id: 'graph',
                    label: self.recloptions.views.graph.label,
                    view: new recline.View.Graph({
                        model: dataset,
                        state: self.recloptions.views.graph.state
                    })
                };
                views.push(graph);
            }
            if (self.recloptions.views.grid.show) {
                var grid = {
                    id: 'grid',
                    label: self.recloptions.views.grid.label,
                    view: new recline.View.SlickGrid({
                        model: dataset,
                        state: self.recloptions.views.grid.state
                    })
                };
                views.push(grid);
            }
            if (self.recloptions.views.map.show && self.recloptions.GIS) {
                var map = {
                    id: 'map',
                    label: self.recloptions.views.map.label,
                    view: new recline.View.Map({
                        model: dataset,
                        state: self.recloptions.views.map.state
                    })
                };
                views.push(map);
            }
            if (self.recloptions.views.timeseries.show) {
                var timeseries = {
                    id: 'timeseries',
                    label: self.recloptions.views.timeseries.label,
                    view: new recline.View.Timeseries({
                        model: dataset,
                        state: self.recloptions.views.timeseries.state
                    })
                };
                views.push(timeseries);
            }
            _reclineView = new recline.View.MultiView({
                title: self.recloptions.title,
                model: dataset,
                el: $el,
                state: state,
                views: views,
                source: {
                    url: self.recloptions.source.url,
                    query: self.recloptions.source.query
                },
                buttons: self.recloptions.buttons,
                dashlet: self.recloptions.dashlet,
                export: self.recloptions.export,
                onDateRangeChange: self.recloptions.onDateRangeChange,
                onSaveAsDashlet: self.recloptions.onSaveAsDashlet,
                onRemoveDashlet: self.recloptions.onRemoveDashlet,
                onGotoAnalytics: self.recloptions.onGotoAnalytics,
                onSaveConfig: self.recloptions.onSaveConfig
            });
            window.dataExplorer = _reclineView;

            if ($("#editor-auto-zoom")) $("#editor-auto-zoom", self.element).trigger("click");
            if ($(".leaflet-control-layers-base input:radio").length > 0) $(".leaflet-control-layers-base input:radio")[0].checked = true;
            $(".menu-right div").removeClass("btn-group");

        },
        saveReclineConfig: function () {
            if (_reclineView) _reclineView.onSaveconfigClick();
        },
        RestoreData: function (data) {
            if (_reclineView) _reclineView.model.restore(data);
        }
    });
})($);