/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   
JQuery Immediately-Invoked Function for recline modifications
Created By      : Rony T S for Assyst International Pvt. Ltd.
Created Date    : 20-Feb-2014
Version         : 1.0.2
Project         : PanAgro

+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/
this.recline = this.recline || {};
this.recline.Backend = this.recline.Backend || {};
this.recline.Backend.Memory = this.recline.Backend.Memory || {};

(function (my) {
    "use strict";
    my.__type__ = 'memory';

    // private data - use either jQuery or Underscore Deferred depending on what is available
    var Deferred = (typeof jQuery !== "undefined" && jQuery.Deferred) || _.Deferred;

    // ## Data Wrapper
    //
    // Turn a simple array of JS objects into a mini data-store with
    // functionality like querying, faceting, updating (by ID) and deleting (by
    // ID).
    //
    // @param records list of hashes for each record/row in the data ({key:
    // value, key: value})
    // @param fields (optional) list of field hashes (each hash defining a field
    // as per recline.Model.Field). If fields not specified they will be taken
    // from the data.
    my.Store = function (records, fields, filters, axisFields, geoFields, kpiFields, timeSeries) {
        var self = this;
        this.records = records;

        // backwards compatability (in v0.5 records was named data)
        this.data = this.records;

        if (fields) {
            this.fields = fields;
        } else if (records) {
            this.fields = _.map(records[0], function (value, key) {
                return {
                    id: key,
                    type: 'string'
                };
            });
        }

        // Modified by Assyst (Rony at 03-Feb-2014)
        // For limit the fileds of recline  combo
        /******************************************/

        //this.filters = filters;
        if (filters) {
            this.filters = filters;
        } else if (fields) {
            var filterField = $.grep(this.fields, function (el, i) {
                return el.isFilterable === "Y";
            });
            this.filters = filterField.length > 0 ? filterField : this.fields;
        }
        //this.axisFields = axisFields;
        if (axisFields) {
            this.axisFields = axisFields;
        } else if (fields) {
            var axisField = $.grep(this.fields, function (el, i) {
                return el.isAxis === "Y";
            });
            this.axisFields = axisField.length > 0 ? axisField : this.fields;
        }

        //this.axisFields = axisFields;
        if (timeSeries) {
            this.timeSeries = timeSeries;
        } else if (fields) {
            var tSeries = $.grep(this.fields, function (el, i) {
                return el.isTimeSeries === "Y";
            });
            this.timeSeries = tSeries.length > 0 ? tSeries : this.fields;
        }

        //this.geoFields = geoFields;
        if (geoFields) {
            this.geoFields = geoFields;
        } else if (fields) {
            var geoField = $.grep(this.fields, function (el, i) {
                return el.isGeo === "Y";
            });
            this.geoFields = geoField.length > 0 ? geoField : this.fields;
        }

        //this.kpiFields = kpiFields;
        if (kpiFields) {
            this.kpiFields = kpiFields;
        } else if (fields) {
            var kpiField = $.grep(this.fields, function (el, i) {
                return el.isKPI === "Y";
            });
            this.kpiFields = kpiField.length > 0 ? kpiField : this.fields;
        }
        /*****************END**********************/

        this.update = function (doc) {
            _.each(self.records, function (internalDoc, idx) {
                if (doc.id === internalDoc.id) {
                    self.records[idx] = doc;
                }
            });
        };

        this.remove = function (doc) {
            var newdocs = _.reject(self.records, function (internalDoc) {
                return (doc.id === internalDoc.id);
            });
            this.records = newdocs;
        };

        this.restore = function (recordset) {
            this.records = recordset;
        };

        this.save = function (changes, dataset) {
            var self = this;
            var dfd = new Deferred();
            // TODO _.each(changes.creates) { ... }
            _.each(changes.updates, function (record) {
                self.update(record);
            });
            _.each(changes.deletes, function (record) {
                self.remove(record);
            });
            dfd.resolve();
            return dfd.promise();
        },

            this.query = function (queryObj) {
                var dfd = new Deferred();
                var numRows = queryObj.size || this.records.length;
                var start = queryObj.from || 0;
                var results = this.records;

                results = this._applyFilters(results, queryObj);
                results = this._applyFreeTextQuery(results, queryObj);

                // TODO: this is not complete sorting!
                // What's wrong is we sort on the *last* entry in the sort list if there are multiple sort criteria
                _.each(queryObj.sort, function (sortObj) {
                    var fieldName = sortObj.field;
                    results = _.sortBy(results, function (doc) {
                        var _out = doc[fieldName];
                        return _out;
                    });
                    if (sortObj.order == 'desc') {
                        results.reverse();
                    }
                });
                var facets = this.computeFacets(results, queryObj);
                var out = {
                    total: results.length,
                    hits: results.slice(start, start + numRows),
                    facets: facets
                };
                dfd.resolve(out);
                return dfd.promise();
            };

        // in place filtering
        this._applyFilters = function (results, queryObj) {
            var filters = queryObj.filters;
            // register filters
            var filterFunctions = {
                term: term,
                range: range,
                geo_distance: geo_distance
            };

            var dataParsers = {
                integer: function (e) {
                    return parseFloat(e, 10);
                },
                float: function (e) {
                    return parseFloat(e, 10);
                },
                number: function (e) {
                    return parseFloat(e, 10);
                },
                numeric: function (e) {
                    return parseFloat(e, 10);
                },
                string: function (e) {
                    return e.toString();
                },
                date: function (e) {
                    return moment(e).valueOf();
                },
                datetime: function (e) {
                    return new Date(e).valueOf();
                }
            };

            var keyedFields = {};
            _.each(self.fields, function (field) {
                keyedFields[field.id] = field;
            });

            function getDataParser(filter) {
                var fieldType = keyedFields[filter.fieldName].type || 'string';
                return dataParsers[fieldType];
            }

            // filter records
            return _.filter(results, function (record) {
                var passes = _.map(filters, function (filter) {
                    var fun = filter.operator == "Between" ? "range" : "term";
                    return filterFunctions[fun](record, filter);
                });
                // return only these records that pass all filters
                return _.all(passes, _.identity);
            });

            if (!String.prototype.startsWith) {
                String.prototype.startsWith = function (searchString, position) {
                    position = position || 0;
                    return this.indexOf(searchString, position) === position;
                };
            }

            function term(record, filter) {
                var queryField = filter["fieldName"].toString();
                var queryData;         
		debugger;       
                if(filter["value1"].indexOf("|,")){
                    filter["value1"] = filter["value1"].replace("|,", ",");
                }               
                if(filter["value1"].indexOf(" ,") != -1){
                    queryData = filter["value1"].split(' ,');
                }
                else {
                    queryData = [filter["value1"]];
                }

                var daysAgo=filter["value1"];
                var queryFormat = record[queryField];
                queryFormat=$.isNumeric(queryFormat)?parseFloat(queryFormat).toFixed(2):queryFormat;
                var rtnValue = false;

               /* var format =[];
                format= $.map(self.fields,function(val,i){
                    if(filter.fieldName == val.id)
                        return val.format;
                });*/
                  var format=_.findWhere(self.fields, { id: filter.fieldName }).format;
                _.each(queryData, function (text, i) {
                    //Formatting values in QueryData
                    if(filter.type =="date-time"||filter.type =="date"){
                        queryData[i]=moment(text, "YYYY-MM-DDTHH:mm:ss").format(format.format);
                       
                    }
                });
                //Formatting QueryFormat Value
                if(filter.type =="date-time"||filter.type =="date"){
                       queryFormat=moment(queryFormat, "YYYY-MM-DDTHH:mm:ss").format(format.format);
                       
                 }
                $.each(queryData, function (i,text) {
                    text = $.isNumeric(text)?parseFloat(text):text;                   
                    queryFormat = $.isNumeric(queryFormat)?parseFloat(queryFormat):queryFormat;
                   
                    if(text == "null" || text == "''" || text == " "){
                        if(queryFormat == null) {
                            rtnValue = true;
                            return false;
                        }
                    }
                    else {
                    var format=_.findWhere(self.fields, { id: filter.fieldName }).format;
                         switch(filter.operator) {
                            case "In":
                                if(queryFormat === ($.isNumeric(text) ? text : text.replace('|,',',')) && filter.type !="month" ) { //text.replace('|,',',')
                                    rtnValue = true;
                                    return false;
                                }
                                else if(filter.type =="month" && getMonthAsNumeric(queryFormat) == getMonthAsNumeric($.isNumeric(text)?text:text.replace('|,',','))) {
                                    rtnValue = true;
                                    return false;
                                }
                                else if(queryFormat == text){
                                    rtnValue = true;
                                     return false;
                                }
                            break;

                            case "NotIn":                           
                                if( filter.type !="month" && queryFormat != text){// ($.isNumeric(text)?text:text.replace('|,',','))) {                               
                                    rtnValue = true;
                                }
                                else if(filter.type =="month" && getMonthAsNumeric(queryFormat) != getMonthAsNumeric(queryData) ){
                                    rtnValue = true;
                                }
                                else
                                {
                                rtnValue = false;
                                return false;
                                }
                                                              
                            break;

                            case "Like":
                                queryFormat = $.isNumeric(queryFormat)?queryFormat.toString():queryFormat;
                                text = $.isNumeric(text)?text.toString():text;                              
                                if(queryFormat.toLowerCase().indexOf($.trim(text.toLowerCase()))>=0) {
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case "NotLike":
                                 queryFormat = $.isNumeric(queryFormat)?queryFormat.toString():queryFormat;
                                text = $.isNumeric(text)?text.toString():text; 
                                for(var i=0;i<queryData.length;i++) {
                                     if(queryFormat.toLowerCase().indexOf($.trim(text.toLowerCase()))<0) {
                                        rtnValue = true;                                                                                                                   
                                    } else {
                                        rtnValue = false;
                                        break;
                                    }
                                }                                
                            break;

                            case "=":
                                if( parseFloat(queryFormat) === parseFloat(text) || queryFormat === text) {
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case "!=":
                                if($.inArray( queryFormat.toString(), queryData )==-1 && filter.type =="number"){
                                    rtnValue = true;
                                    return false;
                                }
                                else if($.inArray( queryFormat.toString(), queryData )==-1 && (filter.type =="date"||filter.type =="date-time")){
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case ">":
                                if(parseFloat(queryFormat) > parseFloat(text) && filter.type =="number"){
                                    rtnValue = true;
                                    return false;
                                }
                                else if(filter.type =="date"||filter.type =="date-time")
                                {
                                    if( moment(queryFormat, format.format).format("YYYY-MM-DDTHH:mm:ss") > moment(text, format.format).format("YYYY-MM-DDTHH:mm:ss")  ) {
                                        rtnValue = true;
                                        return false;
                                    }
                                }
                                else if(filter.type =="month" && getMonthAsNumeric(queryFormat) > getMonthAsNumeric(queryData) )
                                {
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case "<":
                                if(parseFloat(queryFormat) < parseFloat(text) && filter.type =="number"){
                                    rtnValue = true;
                                    return false;
                                }
                                else if(filter.type =="date"||filter.type =="date-time")
                                {
                                    if( moment(queryFormat, format.format).format("YYYY-MM-DDTHH:mm:ss") < moment(text, format.format).format("YYYY-MM-DDTHH:mm:ss")) {
                                        rtnValue = true;
                                        return false;
                                    }
                                }
                                 else if(filter.type =="month" && getMonthAsNumeric(queryFormat) < getMonthAsNumeric(queryData) )
                                {
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case ">=":
                                if(parseFloat(queryFormat) >= parseFloat(text) && filter.type =="number"){
                                    rtnValue = true;
                                    return false;
                                }
                                else if(filter.type =="date"||filter.type =="date-time")
                                {
                                    if( moment(queryFormat, format.format).format("YYYY-MM-DDTHH:mm:ss") >= moment(text, format.format).format("YYYY-MM-DDTHH:mm:ss")) {
                                        rtnValue = true;
                                        return false;
                                    }
                                }
                                 else if(filter.type =="month" && getMonthAsNumeric(queryFormat) >= getMonthAsNumeric(queryData) )
                                {
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case "<=":
                                if(parseFloat(queryFormat) <= parseFloat(text) && filter.type =="number"){
                                    rtnValue = true;
                                    return false;
                                }
                               else if(filter.type =="date"||filter.type =="date-time")
                                {
                                    if( moment(queryFormat, format.format).format("YYYY-MM-DDTHH:mm:ss") <= moment(text, format.format).format("YYYY-MM-DDTHH:mm:ss")) {
                                        rtnValue = true;
                                        return false;
                                    }
                                }
                                 else if(filter.type =="month" && getMonthAsNumeric(queryFormat) <= getMonthAsNumeric(queryData) )
                                {
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case "between":
                                if(filter.type =="month" && getMonthAsNumeric(queryFormat) <= getMonthAsNumeric(queryData) )
                                {
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case "Month To Date":
                                var date = new Date();
                                var firstDay = new Date(date.getFullYear(), date.getMonth(), 1);
                                if(   new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(firstDay)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(date))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "Year To Date":
                                var date = new Date();
                                var yearStart = '01/01/'+new Date().getFullYear()
                                if(   new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(yearStart)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(date))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "Quarter To Date":
                                 var date = new Date();
                                 var quarterFrom=getQuarterFirstDay(date);
                                  if(   new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(quarterFrom)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(date))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "Today":
                                var date = new Date();
                                  if(   new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) == new Date($.datepicker.formatDate('MM/dd/yy', new Date(date)))) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "Yesterday":
                                var today = new Date();
                                var yesterday = new Date(today);
                                 if(   new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) == new Date($.datepicker.formatDate('MM/dd/yy', new Date(yesterday.setDate(today.getDate() - 1))))) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "This Week":
                                var date = new Date;
                                var first = date.getDate() - date.getDay(); // First day is the day of the month - the day of the week
                                var firstday = new Date(date.setDate(first)).toUTCString();
                                  if(   new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(firstday)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(date))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "Prior Week":
                                var date = new Date();
                                var first = date.getDate() - date.getDay();
                                var firstday = new Date(date.setDate(first)).toUTCString();
                                var lastWeekFirstDay = new Date(firstday);
                                lastWeekFirstDay.setDate(lastWeekFirstDay.getDate() - 7);
                                var lastWeekLastDay =new Date(lastWeekFirstDay.getDate() + 7);
                                if( new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(lastWeekFirstDay)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(lastWeekLastDay))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "Prior 2 Weeks":
                                var date = new Date();
                                var first = date.getDate() - date.getDay();
                                var firstday = new Date(date.setDate(first)).toUTCString();
                                var lastWeekFirstDay = new Date(firstday);
                                lastWeekFirstDay.setDate(lastWeekFirstDay.getDate() - 14);
                                var lastWeekLastDay =new Date(lastWeekFirstDay.getDate() + 14);
                                if( new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(lastWeekFirstDay)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(lastWeekLastDay))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "This Month":
                                var date = new Date();
                                var thisMonthFirstDate = new Date(date.getFullYear(), (date.getMonth()) % 12, 1);
                                     if(   new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(thisMonthFirstDate)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(date))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "Prior Month":
                                var date = new Date();
                                var prevMonthFirstDate = new Date(date.getFullYear(), (date.getMonth() - 1 + 12) % 12, 1);
                                var prevMonthLastDate = new Date(date.getFullYear(), date.getMonth(), 0);
                                    if(   new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(prevMonthFirstDate)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(prevMonthLastDate))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "This Quarter":
                                 var date = new Date();
                                 var quarterFrom=getQuarterFirstDay(date);
                                 var endDate=Date(date.getFullYear(),new Date(quarterFrom).getMonth()+3, 0);
                                  if( new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(quarterFrom)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(endDate))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "Prior Quarter":
                                 var date = new Date();
                                 var quarter = Math.floor((date.getMonth() / 3));    
                                 var firstDate = new Date(date.getFullYear(), quarter * 3 - 3, 1);
                                 var lastDate= new Date(firstDate.getFullYear(), firstDate.getMonth() + 3, 0);
                                  if( new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(firstDate)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(lastDate))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                            case "This Year":
                                var date=new Date();
                                var FirstDay = new Date(date.getFullYear(), 0, 1);
                                var LastDay = new Date(date.getFullYear(),12 , 0);
                                 if( new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(FirstDay)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(LastDay))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                            break;

                            case "Prior Year":
                                 var date=new Date();
                                 var FirstDay = new Date(date.getFullYear()-1, 0, 1);
                                 var LastDay = new Date(date.getFullYear()-1,12 , 0);
                                 if( new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(FirstDay)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date(LastDay))) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;

                             case "Days Ago":
                                var date = new Date();
                                var daysAgoDate = date.setDate(date.getDate() - daysAgo);
                                if( new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) >= new Date($.datepicker.formatDate('MM/dd/yy', new Date(daysAgoDate)))&& new Date($.datepicker.formatDate('MM/dd/yy', new Date(queryFormat))) <= new Date($.datepicker.formatDate('MM/dd/yy', new Date())) ) {
                                    rtnValue = true;
                                    return false;
                                }
                             break;
                        }
                    }
                });
                return rtnValue;
            }

            //get numeric month from month name
            function getMonthAsNumeric(month) {
                var date = new Date();
                if (month == "This Month")
                    return date.getMonth() + 1;
                else
                    return new Date(Date.parse(month + " 1, 2016")).getMonth() + 1
            }

            //get first day of the quarter
            function getQuarterFirstDay(d) {
                var m = d.getMonth() - d.getMonth() % 3;
                return moment(new Date(d.getFullYear(), m, 1)).format('MM/DD/YYYY');
            }

            function range(record, filter) {
                /*function to filter using range operator "between"*/
                if(filter.type == "date" || filter.type == "date-time")
                {   
                    //Method to find in between values in date fields
                    var format=_.findWhere(self.fields, { id: filter.fieldName }).format;
                    var filterStart = (filter.type == "date" || filter.type == "date-time") ? (moment(filter["value1"], "YYYY-MM-DDTHH:mm:ss").format(format.format)) : (filter["value1"] || filter["value1"]);
                    var filterStop = (filter.type == "date" || filter.type == "date-time") ? (moment(filter["value2"], "YYYY-MM-DDTHH:mm:ss").format(format.format)) : (filter["value2"] || filter["value2"]);
                    var filterDate = (filter.type == "date" || filter.type == "date-time") ? (moment(record[filter.fieldName], "YYYY-MM-DDTHH:mm:ss").format(format.format)) : (record[filter.fieldName] || record[filter.fieldName]);
                    var dateFrom = moment(filterStart,format.format).format("YYYY/MM/DD");
                    var dateTo = moment(filterStop,format.format).format("YYYY/MM/DD");
                    var dateCheck = moment(filterDate,format.format).format("YYYY/MM/DD");
                    var date1 = dateFrom.split("/");
                    var date2 = dateTo.split("/");
                    var checkDate = dateCheck.split("/");
                    var from = new Date(date1[1]+"/"+date1[2]+"/"+date1[0]); 
                    var to   = new Date(date2[1]+"/"+date2[2]+"/"+date2[0]);
                    var check = new Date(checkDate[1]+"/"+checkDate[2]+"/"+checkDate[0]);
                    if(check >= from && check <= to)
                    {
                        return true;
                    }
                    else
                        return false;
                }
                else
                {
                    var filterStart = (filter.type == "date" || filter.type == "date-time") ? ($.datepicker.formatDate('MM/dd/yy', new Date(filter["value1"])) || $.datepicker.formatDate('MM dd, yy', new Date(filter["value1"]))) : (filter["value1"] || filter["value1"]);
                    var filterStop = (filter.type == "date" || filter.type == "date-time") ? ($.datepicker.formatDate('MM/dd/yy', new Date(filter["value2"])) || $.datepicker.formatDate('MM dd, yy', new Date(filter["value2"]))) : (filter["value2"] || filter["value2"]);
                    var startNull = (_.isUndefined(filterStart) || filterStart === null || filterStart === '');
                    var stopNull = (_.isUndefined(filterStop) || filterStop === null || filterStop === '');
                    var parse = getDataParser(filter);
                    var value = filter.type == "month" ? getMonthAsNumeric(record[filter.fieldName]) : ((filter.type == "date" || filter.type == "date-time") ? parse($.datepicker.formatDate('MM/dd/yy', new Date(record[filter.fieldName]))) : (filter.type == "month" ? record[filter.fieldName] : parse(record[filter.fieldName])));
                    var start = filter.type == "month" ? (filterStart == "This Month" ? new Date().getMonth() + 1 : getMonthAsNumeric(filterStart)) : (parse(startNull ? '' : filterStart));
                    var stop = filter.type == "month" ? (filterStop == "This Month" ? new Date().getMonth() + 1 : getMonthAsNumeric(filterStop)) : (parse(stopNull ? '' : filterStop));

                    // if at least one end of range is set do not allow '' to get through
                    // note that for strings '' <= {any-character} e.g. '' <= 'a'
                    if ((!startNull || !stopNull) && value === '') {
                        return false;
                    }
                    return ((startNull || value >= start) && (stopNull || value <= stop));
                }
            }

            function geo_distance() {
                // TODO code here
            }
        };

        // we OR across fields but AND across terms in query string
        this._applyFreeTextQuery = function (results, queryObj) {
            if (queryObj.q) {
                var terms = queryObj.q.split(',');
                var patterns = _.map(terms, function (term) {
                    return new RegExp($.trim(term.toLowerCase()));
                });
                results = _.filter(results, function (rawdoc) {
                    var matches = false;
                    _.each(patterns, function (pattern) {
                        var foundmatch = false;
                        _.each(self.fields, function (field) {

                            var value = rawdoc[field.id];
                            if ((value !== null) && (value !== undefined)) {
                                value = value.toString();
                            } else {
                                // value can be null (apparently in some cases)
                                value = '';
                            }

                            // TODO regexes?
                            foundmatch = foundmatch || (pattern.test(value.toLowerCase()));
                            // TODO: early out (once we are true should break to spare unnecessary testing)
                            // if (foundmatch) return true;
                        });
                        matches = matches || foundmatch;
                        // TODO: early out (once false should break to spare unnecessary testing)
                        // if (!matches) return false;
                    });
                    return matches;
                });
            }
            return results;
        };

        this.computeFacets = function (records, queryObj) {
            var facetResults = {};
            if (!queryObj.facets) {
                return facetResults;
            }
            _.each(queryObj.facets, function (query, facetId) {
                // TODO: remove dependency on recline.Model
                facetResults[facetId] = new recline.Model.Facet({
                    id: facetId
                }).toJSON();
                facetResults[facetId].termsall = {};
            });
            // faceting
            _.each(records, function (doc) {
                _.each(queryObj.facets, function (query, facetId) {
                    var fieldId = query.terms.field;
                    var val = doc[fieldId];
                    var tmp = facetResults[facetId];
                    if (val) {
                        tmp.termsall[val] = tmp.termsall[val] ? tmp.termsall[val] + 1 : 1;
                    } else {
                        tmp.missing = tmp.missing + 1;
                    }
                });
            });
            _.each(queryObj.facets, function (query, facetId) {
                var tmp = facetResults[facetId];
                var terms = _.map(tmp.termsall, function (count, term) {
                    return {
                        term: term,
                        count: count
                    };
                });
                tmp.terms = _.sortBy(terms, function (item) {
                    // want descending order
                    return -item.count;
                });
                tmp.terms = tmp.terms.slice(0, 10);
            });
            return facetResults;
        };
    };

} (this.recline.Backend.Memory));

/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/

this.recline = this.recline || {};
this.recline.Model = this.recline.Model || {};

(function (my) {
    "use strict";

    // use either jQuery or Underscore Deferred depending on what is available
    var Deferred = (typeof jQuery !== "undefined" && jQuery.Deferred) || _.Deferred;

    // ## <a id="dataset">Dataset</a>
    my.Dataset = Backbone.Model.extend({
        constructor: function Dataset() {
            Backbone.Model.prototype.constructor.apply(this, arguments);
        },
        // ### initialize
        initialize: function () {
            var self = this;
            _.bindAll(this, 'query');
            this.backend = null;
            if (this.get('backend')) {
                this.backend = this._backendFromString(this.get('backend'));
            } else { // try to guess backend ...
                if (this.get('records')) {
                    this.backend = recline.Backend.Memory;
                }
            }

            this.fields = new my.FieldList();
            this.records = new my.RecordList();

            // Modified by Assyst (Rony at 03-Feb-2014)    
            // For limit the fileds of recline  combo
            /******************************************/

            this.filters = new my.FilterList();
            this.axisFields = new my.AxisList();
            this.timeSeries = new my.timeSeriesList();
            this.geoFields = new my.GeoList();
            this.kpiFields = new my.KpiList();
            this.timeState = new my.timeConfig();
            this.pagingState = new my.pagingConfig();
            /**************END*************************/
            this._changes = {
                deletes: [],
                updates: [],
                creates: []
            };
            this.facets = new my.FacetList();
            this.recordCount = null;
            this.queryInit = undefined;
            this.queryState = new my.Query();
            this.queryState.bind('change facet:add', function () {
                self.query(); // We want to call query() without any arguments.
            });
            // store is what we query and save against
            // store will either be the backend or be a memory store if Backend fetch
            // tells us to use memory store
            this._store = this.backend;
            if (this.backend == recline.Backend.Memory) {
                this.fetch();
            }
        },

        jsonData: {
            summarize: function (data, group, series, fields) {
                var result = [];

                if ($.isArray(group)) group = group[0];
                if ($.inArray(group, series) > -1) return result;

                data = $.makeArray(data);
                series = $.makeArray(series);

                var headers = _.uniq(_.pluck(data, group)).sort();

                var seriesType = $.grep(fields, function (val, i) {
                    return val.id == group;
                });

                $.each(headers, function (hidex, header) {
                    var row = {},
                        prow = {};
                    prow[group] = header;
                    if (seriesType[0].type == "date" || seriesType[0].type == "date-time") {
                        if (moment(header).format(seriesType[0].format.format) != "Invalid date") {
                            header = moment(header).format(seriesType[0].format.format);
                        }
                    }
                    row[group] = header;
                    var gpData = _.where(data, prow);
                    $.each(series, function (cidex, column) {
                        var values = _.pluck(gpData, column);
                        var total = _.reduce(values, function (memo, num) {
                            return parseFloat(memo) + parseFloat(num)
                        });
                        row[column] = parseFloat(total || 0);
                    });
                    result.push(row);
                });
                return result;
            },

            transpose: function (data, group, series, fields) {
                var result = [];

                if ($.isArray(group)) group = group[0];
                if ($.inArray(group, series) > -1) return result;

                data = $.makeArray(data);
                series = $.makeArray(series);

                data = this.summarize(data, group, series, fields);
                var headers = _.pluck(data, group);
                $.each(series, function (index, column) {
                    var values = _.pluck(data, column);
                    var row = _.object(headers, values);
                    row[group] = column;
                    result.push(row);
                });
                return result;
            },

            pivot: function (data, rowfield, colField, valField, decimalPoint, fields) {
                var pivotdata = [];
                decimalPoint = decimalPoint || 2;
                if (typeof data != "undefined" && data.length > 0) {
                    var result = pivot(data, [rowfield], [colField], {});
                    $.each(result.rowHeaders, function (ridx, row) {
                        var prow = {};
                        prow[rowfield] = row[0];
                        $.each(result.columnHeaders, function (cidx, column) {
                            var seriesType = $.grep(fields, function (val, i) {
                                return val.id == colField;
                            });
                            // date formatting for column field
                            if (seriesType[0].type == "date" || seriesType[0].type == "date-time") {
                                if(!moment(column[0], seriesType[0].format.format,true).isValid())
                                {
                                    if (moment(column[0]).format(seriesType[0].format.format) != "Invalid date") {
                                        column[0] = moment(column[0]).format(seriesType[0].format.format);
                                    }
                                }
                            }                           
                            var colH = column[0];
                            var items = result[ridx][cidx];
                            if (items) {
                                var values = _.pluck(items, valField);
                                var total = _.reduce(values, function (memo, num) {
                                    return parseFloat(memo) + parseFloat(num)
                                });
                                var format=_.findWhere(fields, {id: valField}).format;
                                if(format==null || format==undefined)
                                prow[colH] = (parseFloat(total || 0).toFixed(decimalPoint));
                                else
                                prow[colH] = parseFloat(total || 0).toFixed(decimalPoint);

                            } else {
                                prow[colH] = 0
                            }
                        });
                        pivotdata.push(prow);
                    });
                }
                return pivotdata;
            }
        },

        // ### fetch
        //
        // Retrieve dataset and (some) records from the backend.
        fetch: function () {
            var self = this;
            var dfd = new Deferred();

            if (this.backend !== recline.Backend.Memory) {
                this.backend.fetch(this.toJSON())
                    .done(handleResults)
                    .fail(function (args) {
                        dfd.reject(args);
                    });
            } else {

                // Modified by Assyst (Rony at 03-Feb-2014)    
                // For limit the fileds of recline  combo
                /******************************************/
                // special case where we have been given data directly
                handleResults({
                    records: this.get('records'),
                    fields: this.get('fields'),
                    filters: this.get('filters'),
                    axisFields: this.get('axisFields'),
                    geoFields: this.get("geoFields"),
                    kpiFields: this.get("kpiFields"),
                    timeSeries: this.get("timeSeries"),
                    timeState: this.get("timeState"),
                    queryInit: this.get("queryInit"),
                    pagingState: this.get("pagingState"),
                    totalRecords: this.get("totalRecords"),
                    useMemoryStore: true
                });
                /********************END*******************/
            }

            function handleResults(results) {
                // if explicitly given the fields
                // (e.g. var dataset = new Dataset({fields: fields, ...})
                // use that field info over anything we get back by parsing the data
                // (results.fields)
                var fields = self.get('fields') || results.fields;

                // Modified by Assyst (Rony at 03-Feb-2014)    
                // For limit the fileds of recline  combo
                /******************************************/
                var out = self._normalizeRecordsAndFields(results.records, fields, results.filters, results.axisFields, results.geoFields, results.kpiFields, results.timeSeries);
                if (results.useMemoryStore) {
                    self._store = new recline.Backend.Memory.Store(out.records, out.fields, out.filters, out.axisFields, out.geoFields, out.kpiFields, out.timeSeries);
                }

                self.set(results.metadata);
                self.fields.reset(out.fields);
                self.filters.reset(out.filters);
                self.axisFields.reset(out.axisFields);
                self.geoFields.reset(out.geoFields);
                self.kpiFields.reset(out.kpiFields);
                self.timeSeries.reset(out.timeSeries);
                self.timeState.reset(results.timeState);
                self.pagingState.reset(results.pagingState, results.totalRecords);
                /****************END***********************/

                self.query(results.queryInit)
                    .done(function () {
                        dfd.resolve(self);
                    })
                    .fail(function (args) {
                        dfd.reject(args);
                    });
            }
            return dfd.promise();
        },

        // ### _normalizeRecordsAndFields
        // 
        // Get a proper set of fields and records from incoming set of fields and records either of which may be null or arrays or objects
        //
        // e.g. fields = ['a', 'b', 'c'] and records = [ [1,2,3] ] =>
        // fields = [ {id: a}, {id: b}, {id: c}], records = [ {a: 1}, {b: 2}, {c: 3}]
        _normalizeRecordsAndFields: function (records, fields, filters, axisFields, geoFields, kpiFields, timeSeries) {

            // if no fields get them from records
            if (!fields && records && records.length > 0) {
                // records is array then fields is first row of records ...
                if (records[0] instanceof Array) {
                    fields = records[0];
                    records = records.slice(1);
                } else {
                    fields = _.map(_.keys(records[0]), function (key) {
                        return {
                            id: key,
                            label: key
                        };
                    });
                }
            }

            // Added by Assyst for a new key title
            // the value of key is assigned dynamicaly based on the hierarchy
            // that is child title will be parent-child label
            // this modification has suggested by rajan, 
            // since in some causes the childs of different parents have same name, it is very difficult to identify in filter drop down 
            _.each(fields, function (field, idx) {
                var parent = $.grep(fields, function (el, i) {
                    return $.inArray(field.id, el.children) > -1;
                })
                field["name"] = parent.length > 0 ? parent[0].label + ' - ' + field.label : field.label;
            });

            // fields is an array of strings (i.e. list of field headings/ids)
            if (fields && fields.length > 0 && (fields[0] === null || typeof (fields[0]) != 'object')) {
                // Rename duplicate fieldIds as each field name needs to be
                // unique.
                var seen = {};
                fields = _.map(fields, function (field, index) {
                    if (field === null) {
                        field = '';
                    } else {
                        field = field.toString();
                    }
                    // cannot use trim as not supported by IE7
                    var fieldId = field.replace(/^\s+|\s+$/g, '');
                    if (fieldId === '') {
                        fieldId = '_noname_';
                        field = fieldId;
                    }
                    while (fieldId in seen) {
                        seen[field] += 1;
                        fieldId = field + seen[field];
                    }
                    if (!(field in seen)) {
                        seen[field] = 0;
                    }
                    // TODO: decide whether to keep original name as label ...
                    // return { id: fieldId, label: field || fieldId }
                    return {
                        id: fieldId
                    };
                });
            }
            // records is provided as arrays so need to zip together with fields
            // NB: this requires you to have fields to match arrays
            if (records && records.length > 0 && records[0] instanceof Array) {
                records = _.map(records, function (doc) {
                    var tmp = {};
                    _.each(fields, function (field, idx) {
                        tmp[field.id] = doc[idx];
                    });
                    return tmp;
                });
            }


            // Modified by Assyst (Rony at 03-Feb-2014)    
            // For limit the fileds of recline  combo
            /******************************************/
            if (!filters && fields && fields.length > 0) {
                var filter = $.grep(fields, function (el, i) {
                    return el.isFilterable === "Y";
                });
                filters = filter.length > 0 ? filter : fields;
            }

            if (!axisFields && fields && fields.length > 0) {
                var axis = $.grep(fields, function (el, i) {
                    return el.isAxis === "Y";
                });
                axisFields = axis.length > 0 ? axis : fields;
            }

            if (!timeSeries && fields && fields.length > 0) {
                var tSeries = $.grep(fields, function (el, i) {
                    return el.isTimeSeries === "Y";
                });
                timeSeries = tSeries.length > 0 ? tSeries : fields;
            }

            if (!geoFields && fields && fields.length > 0) {
                var geo = $.grep(fields, function (el, i) {
                    return el.isGeo === "Y";
                });
                geoFields = geo.length > 0 ? geo : fields;
            }
            if (!kpiFields && fields && fields.length > 0) {
                var kpi = $.grep(fields, function (el, i) {
                    return el.isKPI === "Y";
                });

                var numericArray = ['number', 'numeric', 'integer', 'int', 'double', 'float'];
                if (kpi.length == 0) {
                    kpi = $.grep(fields, function (el, i) {
                        return $.inArray(el.type, numericArray) > -1;
                    });
                }
                kpiFields = kpi;
            }
            return {
                fields: fields,
                records: records,
                filters: filters,
                axisFields: axisFields,
                geoFields: geoFields,
                timeSeries: timeSeries,
                kpiFields: kpiFields
            };
            /********************END*******************/
        },

        save: function () {
            var self = this;
            // TODO: need to reset the changes ...
            return this._store.save(this._changes, this.toJSON());
        },

        restore: function (recordset, pagingState) {
            var self = this;
            this.trigger('query:start');
            var dfd = new Deferred();
            this._store.restore(recordset);
            var actualQuery = this.queryState.toJSON();

            this._store.query(actualQuery, this.toJSON())
                .done(function (queryResult) {
                    self._handleQueryResult(queryResult);
                    self.trigger('query:done');
                    dfd.resolve(self.records);
                })
                .fail(function (args) {
                    self.trigger('query:fail', args);
                    dfd.reject(args);
                });

            if (pagingState) this.pagingState.updatePaging(pagingState.pagedList, pagingState.pageSize);

            return dfd.promise();
        },

        // ### query
        //
        // AJAX method with promise API to get records from the backend.
        //
        // It will query based on current query state (given by this.queryState)
        // updated by queryObj (if provided).
        //
        // Resulting RecordList are used to reset this.records and are
        // also returned.
        query: function (queryObj) {
            var self = this;
            var dfd = new Deferred();
            this.trigger('query:start');

            if (queryObj) {
                this.queryState.set(queryObj, {
                    silent: true
                });
            }
            var actualQuery = this.queryState.toJSON();

            this._store.query(actualQuery, this.toJSON())
                .done(function (queryResult) {
                    self._handleQueryResult(queryResult);
                    self.trigger('query:done');
                    dfd.resolve(self.records);
                })
                .fail(function (args) {
                    self.trigger('query:fail', args);
                    dfd.reject(args);
                });
            return dfd.promise();
        },

        _handleQueryResult: function (queryResult) {
            var self = this;
            self.recordCount = queryResult.total;
            var docs = _.map(queryResult.hits, function (hit) {
                var _doc = new my.Record(hit);
                _doc.fields = self.fields;
                _doc.bind('change', function (doc) {
                    self._changes.updates.push(doc.toJSON());
                });
                _doc.bind('destroy', function (doc) {
                    self._changes.deletes.push(doc.toJSON());
                });
                return _doc;
            });
            self.records.reset(docs);
            if (queryResult.facets) {
                var facets = _.map(queryResult.facets, function (facetResult, facetId) {
                    facetResult.id = facetId;
                    return new my.Facet(facetResult);
                });
                self.facets.reset(facets);
            }
        },


        // Modified by Assyst (Rony at 03-Feb-2014)    
        // For limit the fileds of recline  combo
        /******************************************/
        toTemplateJSON: function () {
            var data = this.toJSON();
            data.recordCount = this.recordCount;
            data.fields = this.fields.toJSON();
            data.filters = this.filters.toJSON();
            data.axisFields = this.axisFields.toJSON();
            data.geoFields = this.geoFields.toJSON();
            data.kpiFields = this.kpiFields.toJSON();
            data.timeSeries = this.timeSeries.toJSON();
            return data;
        },
        /********************END*******************/


        // ### getFieldsSummary
        //
        // Get a summary for each field in the form of a `Facet`.
        // 
        // @return null as this is async function. Provides deferred/promise interface.
        getFieldsSummary: function () {
            var self = this;
            var query = new my.Query();
            query.set({
                size: 0
            });
            this.fields.each(function (field) {
                query.addFacet(field.id);
            });
            var dfd = new Deferred();
            this._store.query(query.toJSON(), this.toJSON()).done(function (queryResult) {
                if (queryResult.facets) {
                    _.each(queryResult.facets, function (facetResult, facetId) {
                        facetResult.id = facetId;
                        var facet = new my.Facet(facetResult);
                        // TODO: probably want replace rather than reset (i.e. just replace the facet with this id)
                        self.fields.get(facetId).facets.reset(facet);
                    });
                }
                dfd.resolve(queryResult);
            });
            return dfd.promise();
        },

        // Deprecated (as of v0.5) - use record.summary()
        recordSummary: function (record) {
            return record.summary();
        },

        // ### _backendFromString(backendString)
        //
        // Look up a backend module from a backend string (look in recline.Backend)
        _backendFromString: function (backendString) {
            var backend = null;
            if (recline && recline.Backend) {
                _.each(_.keys(recline.Backend), function (name) {
                    if (name.toLowerCase() === backendString.toLowerCase()) {
                        backend = recline.Backend[name];
                    }
                });
            }
            return backend;
        }
    });


    // ## <a id="record">A Record</a>
    // 
    // A single record (or row) in the dataset
    my.Record = Backbone.Model.extend({
        constructor: function Record() {
            Backbone.Model.prototype.constructor.apply(this, arguments);
        },

        // ### initialize
        // 
        // Create a Record
        //
        // You usually will not do this directly but will have records created by
        // Dataset e.g. in query method
        //
        // Certain methods require presence of a fields attribute (identical to that on Dataset)
        initialize: function () {
            _.bindAll(this, 'getFieldValue');
        },

        // ### getFieldValue
        //
        // For the provided Field get the corresponding rendered computed data value
        // for this record.
        //
        // NB: if field is undefined a default '' value will be returned
        getFieldValue: function (field) {
            var val = this.getFieldValueUnrendered(field);
            if (field && !_.isUndefined(field.renderer)) {
                val = field.renderer(val, field, this.toJSON());
            }
            return val;
        },

        // ### getFieldValueUnrendered
        //
        // For the provided Field get the corresponding computed data value
        // for this record.
        //
        // NB: if field is undefined a default '' value will be returned
        getFieldValueUnrendered: function (field) {
            if (!field) {
                return '';
            }
            var val = this.get(field.id);
            if (field.deriver) {
                val = field.deriver(val, field, this);
            }
            return val;
        },

        // ### summary
        //
        // Get a simple html summary of this record in form of key/value list
        summary: function (record) {
            var self = this;
            var html = '<div class="recline-record-summary">';
            this.fields.each(function (field) {
                if (field.id != 'id') {
                    html += '<div class="' + field.id + '"><strong>' + field.get('label') + '</strong>: ' + self.getFieldValue(field) + '</div>';
                }
            });
            html += '</div>';
            return html;
        },

        // Override Backbone save, fetch and destroy so they do nothing
        // Instead, Dataset object that created this Record should take care of
        // handling these changes (discovery will occur via event notifications)
        // WARNING: these will not persist *unless* you call save on Dataset
        fetch: function () { },
        save: function () { },
        destroy: function () {
            this.trigger('destroy', this);
        }
    });


    // ## A Backbone collection of Records
    my.RecordList = Backbone.Collection.extend({
        constructor: function RecordList() {
            Backbone.Collection.prototype.constructor.apply(this, arguments);
        },
        model: my.Record
    });


    // ## <a id="field">A Field (aka Column) on a Dataset</a>
    my.Field = Backbone.Model.extend({

        constructor: function Field() {
            Backbone.Model.prototype.constructor.apply(this, arguments);
        },
        // ### defaults - define default values
        //Assyst-notes
        defaults: {
            label: null,
            name: null,
            type: 'string',
            format: null,
            decimalPoint: 2,
            weightage: null,
            description: undefined,
            isFilterable: false,
            isAxis: false,
            isGeo: false,
            isHidden: false,
            isKPI: false,
            is_derived: false,
            is_subchild: false,
            children: undefined
        },
        // ### initialize
        //
        // @param {Object} data: standard Backbone model attributes
        //
        // @param {Object} options: renderer and/or deriver functions.
        initialize: function (data, options) {
            // if a hash not passed in the first argument throw error
            if ('0' in data) {
                throw new Error(textTranslation('Looks like you did not pass a proper hash with id to Field constructor'));
            }
            if (this.attributes.label === null) {
                this.set({
                    label: this.id
                });
            }
            if (this.attributes.type.toString() in this._typeMap) {
                this.attributes.type = this._typeMap[this.attributes.type.toString()];
            }
            //Assyst-notes
            if (this.attributes.isFilterable.toString() in this._boolMap) {
                this.attributes.isFilterable = this._boolMap[this.attributes.isFilterable.toString()];
            }
            if (this.attributes.isAxis.toString() in this._boolMap) {
                this.attributes.isAxis = this._boolMap[this.attributes.isAxis.toString()];
            }
            if (this.attributes.isGeo.toString() in this._boolMap) {
                this.attributes.isGeo = this._boolMap[this.attributes.isGeo.toString()];
            }
            if (this.attributes.isHidden.toString() in this._boolMap) {
                this.attributes.isHidden = this._boolMap[this.attributes.isHidden.toString()];
            }
            if (this.attributes.isKPI.toString() in this._boolMap) {
                this.attributes.isKPI = this._boolMap[this.attributes.isKPI.toString()];
            }

            if (options) {
                this.renderer = options.renderer;
                this.deriver = options.deriver;
            }
            if (!this.renderer) {
                this.renderer = this.defaultRenderers[this.get('type')];
            }
            this.facets = new my.FacetList();
        },
        _boolMap: {
            'Y': true,
            'N': false
        },
        _typeMap: {
            'text': 'string',
            'geo': 'geo_point',
            'double': 'number',
            'float': 'number',
            'numeric': 'number',
            'int': 'integer',
            'date': 'date-time',
            'datetime': 'date-time',
            'bool': 'boolean',
            'timestamp': 'date-time',
            'json': 'object',
            'tab': 'link-tab',
            'drilldown': 'link-drill-down',
            'link-drilldown': 'link-drill-down'
        },
        defaultRenderers: {
            object: function (val, field, doc) {
                return JSON.stringify(val);
            },
            geo_point: function (val, field, doc) {
                return val; //return JSON.stringify(val);
            },
            'number': function (val, field, doc) {
                var decimalPoint = field.get('decimalPoint');
                var format = field.get('format');
                if (format === 'percentage') {
                    return val + '%';
                }
                if (!Number.isNaN(parseFloat(val))) val = parseFloat(parseFloat(val).toFixed(decimalPoint)); // numberWithCommas(parseFloat(val).toFixed(decimalPoint));
                else if (val == "") {
                    val = "";
                } else if (val != "No Records") val = 0;
                return val;
            },
            'date-time': function (val, field, doc) {
                if (moment(val, "YYYY-MM-DDT00:00:00", true).isValid()) {
                    var out = moment(val, "YYYY-MM-DDTHH:mm:ss").format(field.attributes.format.format);
                    val = out;
                }
                return val;
            },
            'string': function (val, field, doc) {
                var format = field.get('format');
                if (format === 'markdown') {
                    if (typeof Showdown !== 'undefined') {
                        var showdown = new Showdown.converter();
                        out = showdown.makeHtml(val);
                        return out;
                    } else {
                        return val;
                    }
                } else if (format == 'plain') {
                    return val;
                } else {
                    // as this is the default and default type is string may get things
                    // here that are not actually strings
                    if (val && typeof val === 'string') {
                        val = val.replace(/(https?:\/\/[^ ]+)/g, '<a href="$1">$1</a>');
                    }
                    return val;
                }
            },
            'link': renderSubreport,
            'link-tab': renderSubreport,
            'link-drill-down': renderSubreport
        }
    });

    function renderSubreport(val, field, doc) {

        if (typeof field.attributes.link === 'undefined') {
            return val;
        } else {
            /* var params = {};  
            var innerText = val || "";   
            var className = val ? "lnkSpan" :  "imgSpan";
            var linkUrl = field.attributes.link; 
            var l_type =field.attributes.type || "link";
            var title = (linkUrl.title)?  Mustache.render(linkUrl.title, doc) : undefined;
            $.each(linkUrl.data, function(key, value){
            params[key]= doc[value] || value;
            });
            var jsonParams = JSON.stringify({ url: linkUrl.url, data: params, dataType :linkUrl.dataType, type: l_type, title: title});
            var key = (linkUrl.url + JSON.stringify(params)).replace(/[^a-z0-9:{}\s]/gi, '').replace(/[_\s]/g, '');

            var rendLink =  "<span class='subreport-link " +className +" _lngTrans_Translatable' key='"+ key +"'  goto ='"+jsonParams+"'>"+ innerText+ " </span>" ;            
            return rendLink;*/

            var innerText = val || "";
            var className = val ? "lnkSpan" : "imgSpan";

            field.attributes.link["type"] = field.attributes.type || "link";
            var jsonParams = Mustache.render(JSON.stringify(field.attributes.link), doc);

            var linkUrl = $.parseJSON(jsonParams);
            var key = (linkUrl.url + JSON.stringify(linkUrl.data)).replace(/[^a-z0-9:{}\s]/gi, '').replace(/[_\s]/g, '');

            var rendLink = "<span class='subreport-link " + className + " _lngTrans_Translatable' key='" + key + "'  goto ='" + jsonParams + "'>" + innerText + " </span>";
            return rendLink;
        }
    }
    my.FieldList = Backbone.Collection.extend({
        constructor: function FieldList() {
            Backbone.Collection.prototype.constructor.apply(this, arguments);
        },
        model: my.Field
    });

    // Modified by Assyst (Rony at 03-Feb-2014)    
    // For limit the fileds of recline  combo
    /******************************************/

    // ## <a id="field">A Field (aka Column) on a Dataset</a>
    my.Series = Backbone.Model.extend({
        constructor: function Series() {
            Backbone.Model.prototype.constructor.apply(this, arguments);
        },
        // ### defaults - define default values
        defaults: {
            label: null,
            type: 'string',
            format: null,
            is_derived: false
        },
        // ### initialize
        //
        // @param {Object} data: standard Backbone model attributes
        //
        // @param {Object} options: renderer and/or deriver functions.
        initialize: function (data, options) {
            // if a hash not passed in the first argument throw error
            if ('0' in data) {
                throw new Error(textTranslation('Looks like you did not pass a proper hash with id to Field constructor'));
            }
            if (this.attributes.label === null) {
                this.set({
                    label: this.id
                });
            }
            if (this.attributes.type.toLowerCase() in this._typeMap) {
                this.attributes.type = this._typeMap[this.attributes.type.toLowerCase()];
            }
            if (options) {
                this.renderer = options.renderer;
                this.deriver = options.deriver;
            }
            if (!this.renderer) {
                this.renderer = this.defaultRenderers[this.get('type')];
            }
            this.facets = new my.FacetList();
        },
        _typeMap: {
            'text': 'string',
            'geo': 'geo_point',
            'double': 'number',
            'float': 'number',
            'numeric': 'number',
            'int': 'integer',
            'datetime': 'date-time',
            'date': 'date-time',
            'bool': 'boolean',
            'timestamp': 'date-time',
            'json': 'object'
        },
        defaultRenderers: {
            object: function (val, field, doc) {
                return JSON.stringify(val);
            },
            geo_point: function (val, field, doc) {
                return val; // return JSON.stringify(val);
            },
            'number': function (val, field, doc) {
                var format = field.get('format');
                if (format === 'percentage') {
                    return val + '%';
                }
                if (!Number.isNaN(parseFloat(val))) val = parseFloat(parseFloat(val).toFixed(decimalPoint)); // numberWithCommas(parseFloat(val).toFixed(decimalPoint));
                else val = 0;
                return val;
            },
            'date-time': function (val, field, doc) {
                if (moment(val, "YYYY-MM-DDT00:00:00", true).isValid()) {
                    var out = moment(val, "YYYY-MM-DDTHH:mm:ss").format(dFormat);
                    val = out;
                }
                return val;
            },
            'string': function (val, field, doc) {
                var format = field.get('format');
                if (format === 'markdown') {
                    if (typeof Showdown !== 'undefined') {
                        var showdown = new Showdown.converter();
                        out = showdown.makeHtml(val);
                        return out;
                    } else {
                        return val;
                    }
                } else if (format == 'plain') {
                    return val;
                } else {
                    // as this is the default and default type is string may get things
                    // here that are not actually strings
                    if (val && typeof val === 'string') {
                        val = val.replace(/(https?:\/\/[^ ]+)/g, '<a href="$1">$1</a>');
                    }
                    return val;
                }
            }
        }
    });

    my.AxisList = Backbone.Collection.extend({
        constructor: function AxisList() {
            Backbone.Collection.prototype.constructor.apply(this, arguments);
        },
        model: my.Series
    });

    my.timeSeriesList = Backbone.Collection.extend({
        constructor: function timeSeriesList() {
            Backbone.Collection.prototype.constructor.apply(this, arguments);
        },
        model: my.Series
    });

    my.GeoList = Backbone.Collection.extend({
        constructor: function GeoList() {
            Backbone.Collection.prototype.constructor.apply(this, arguments);
        },
        model: my.Series
    });

    my.FilterList = Backbone.Collection.extend({
        constructor: function FilterList() {
            Backbone.Collection.prototype.constructor.apply(this, arguments);
        },
        model: my.Series
    });

    my.KpiList = Backbone.Collection.extend({
        constructor: function KpiList() {
            Backbone.Collection.prototype.constructor.apply(this, arguments);
        },
        model: my.Series
    });
    /********************END*******************/

    // ## <a id="query">Query</a>
    my.pagingConfig = Backbone.Model.extend({
        constructor: function pagingConfig() {
            Backbone.Model.prototype.constructor.apply(this, arguments);
        },
        defaults: function () {
            return {
                enable: true,
                showItemCount: true,
                pageSize: 1000,
                pageSizes: [5, 10, 20, 50, 100, 250, 500, 1000, 5000, 10000],
                pagedList: {
                    pageNumber: 1,
                    pageCount: 1,
                    itemCount: 0,
                    isFirstPage: false,
                    isLastPage: false,
                    hasNextPage: false,
                    hasPreviousPage: false
                }
            };
        },

        setPaging: function (recordcount) {
            this._setPaging(recordcount);
            this.trigger('change');
        },

        _setPaging: function (recordcount) {
            var pagedList = this.get('pagedList');
            var pageSize = this.get('pageSize');

            var pageCount = Math.floor(recordcount / pageSize);
            if (recordcount % pageSize != 0) {
                ++pageCount;
            }
            if (pageCount == 0) pageCount = 1;
            if (pagedList.pageNumber > pageCount) pagedList.pageNumber = pageCount;
            pagedList.itemCount = recordcount;
            pagedList.pageCount = pageCount;
            pagedList.isFirstPage = pagedList.pageNumber == 1;
            pagedList.isLastPage = pagedList.pageNumber == pagedList.pageCount;
            pagedList.hasPreviousPage = pagedList.pageNumber > 1;
            pagedList.hasNextPage = pagedList.pageNumber < pagedList.pageCount;
            this.set({
                pagedList: pagedList
            });
        },

        reset: function (config, recordcount) {
            var templates = {
                enable: this.get('enable'),
                showItemCount: this.get('showItemCount'),
                pageSize: this.get('pageSize'),
                pageSizes: this.get('pageSizes'),
                pagedList: this.get('pagedList')
            };
            if (config == undefined) config = templates;
            var ourconfig = JSON.parse(JSON.stringify(config));
            // not fully specified so use template and over-write
            ourconfig = _.defaults(ourconfig, templates);
            this.set({
                enable: (recordcount) ? ourconfig.enable : false,
                showItemCount: ourconfig.showItemCount,
                pageSize: ourconfig.pageSize,
                pageSizes: ourconfig.pageSizes,
                pagedList: ourconfig.pagedList
            });
            if (recordcount) this._setPaging(recordcount);
        },

        update: function (value) {
            this.set(value);
        },

        updatePaging: function (value, count) {
            var pagedList = this.get('pagedList');
            var pageSize = count || this.get('pageSize')
            $.extend(pagedList, value);
            this.set({
                pagedList: pagedList,
                pageSize: pageSize
            });
        }
    });



    // ## <a id="query">Query</a>
    my.timeConfig = Backbone.Model.extend({
        constructor: function timeConfig() {
            Backbone.Model.prototype.constructor.apply(this, arguments);
        },
        defaults: function () {
            return {
                enable: false,
                datePicker: true,
                field: undefined,
                start: moment().subtract(1, 'years'),
                end: moment(),
                interval: this._setDefaultInterval(moment().subtract(1, 'years'), moment()),
                intervals: [{
                    id: 3,
                    name: "Yearly"
                }, {
                    id: 2,
                    name: "Monthly"
                }, {
                    id: 1,
                    name: "Weekly"
                }, {
                    id: 0,
                    name: "Daily"
                }]
            };
        },

        _setDefaultInterval: function (start, end) {
            var intervals = {
                "Daily": 0,
                "Weekly": 1,
                "Monthly": 2,
                "Yearly": 3
            };
            var timeInterval = intervals.Daily;

            var startDate = new Date(start).getTime(),
                endDate = new Date(end).getTime();
            var datDiff = endDate - startDate;
            var years = Math.floor(datDiff / 31536000000);
            var months = Math.floor((datDiff % 31536000000) / 2628000000);
            var days = Math.floor(((datDiff % 31536000000) % 2628000000) / 86400000);
            var weeks = Math.floor(days / 7);
            if (years > 0) timeInterval = intervals.Yearly;
            else if (months > 0) timeInterval = intervals.Monthly;
            else if (weeks > 0) timeInterval = intervals.Weekly;
            else timeInterval = intervals.Daily;

            return timeInterval;
        },

        _getIntervals: function (start, end) {
            var self = this;
            var intervals = [];
            var defaultInterval = this._setDefaultInterval(start, end);
            switch (defaultInterval) {
                case 3:
                    intervals.push({
                        id: 3,
                        name: "Yearly"
                    });
                case 2:
                    intervals.push({
                        id: 2,
                        name: "Monthly"
                    });
                case 1:
                    intervals.push({
                        id: 1,
                        name: "Weekly"
                    });
                default:
                    intervals.push({
                        id: 0,
                        name: "Daily"
                    });
                    break;
            }
            return intervals;
        },

        reset: function (config) {
            var templates = {
                enable: this.get('enable'),
                datePicker: this.get('datePicker'),
                field: this.get('field'),
                start: this.get('start'),
                end: this.get('end'),
                interval: this.get('interval'),
                intervals: this.get('intervals')
            };
            if (config == undefined) config = templates;
            var ourconfig = JSON.parse(JSON.stringify(config));
            // not fully specified so use template and over-write
            ourconfig = _.defaults(ourconfig, templates);
            var defaultInterval = this._setDefaultInterval(ourconfig.start, ourconfig.end);
            ourconfig.interval = defaultInterval < (ourconfig.interval || -1) ? defaultInterval : ourconfig.interval;

            this.set({
                enable: ourconfig.enable,
                datePicker: ourconfig.datePicker,
                field: ourconfig.field,
                start: ourconfig.start,
                end: ourconfig.end,
                interval: ourconfig.interval,
                intervals: this._getIntervals(ourconfig.start, ourconfig.end)
            });
        },

        updateTime: function (config) {
            this.reset(config);
            this.trigger('change');
        },

        updateInterval: function (interval) {
            this.set({
                interval: interval
            });
            this.trigger('change');
        },
        update: function (value) {
            this.set(value);
        }
    });



    // ## <a id="query">Query</a>
    my.Query = Backbone.Model.extend({
        constructor: function Query() {
            Backbone.Model.prototype.constructor.apply(this, arguments);
        },
        defaults: function () {
            return {
                size: 10,
                from: 0,
                q: '',
                facets: {},
                filters: [],
                groupings: {}
            };
        },
        _filterTemplates: {
            term: {
                type: 'term',
                // TODO do we need this attribute here?
                field: '',
                term: ''
            },
            range: {
                type: 'range',
                start: '',
                stop: ''
            },
            geo_distance: {
                type: 'geo_distance',
                distance: 10,
                unit: 'km',
                point: {
                    lon: 0,
                    lat: 0
                }
            }
        },
        // ### addFilter(filter)
        //
        // Add a new filter specified by the filter hash and append to the list of filters
        //
        // @param filter an object specifying the filter - see _filterTemplates for examples. If only type is provided will generate a filter by cloning _filterTemplates
        addFilter: function (filter) {
            // crude deep copy
            var ourfilter = JSON.parse(JSON.stringify(filter));
            // not fully specified so use template and over-write
            if (_.keys(filter).length <= 3) {
                ourfilter = _.defaults(ourfilter, this._filterTemplates[filter.type]);
            }
            var filters = this.get('filters');
            filters.push(ourfilter);
            this.trigger('change:filters:new-blank');
        },
        updateFilter: function (index, value) { },
        // ### removeFilter
        //
        // Remove a filter from filters at index filterIndex
        removeFilter: function (filterIndex) {
            var filters = this.get('filters');
            filters.splice(filterIndex, 1);
            this.set({
                filters: filters
            });
            this.trigger('change');
        },
        // ### addFacet
        //
        // Add a Facet to this query
        //
        // See <http://www.elasticsearch.org/guide/reference/api/search/facets/>
        addFacet: function (fieldId) {
            var facets = this.get('facets');
            // Assume id and fieldId should be the same (TODO: this need not be true if we want to add two different type of facets on same field)
            if (_.contains(_.keys(facets), fieldId)) {
                return;
            }
            facets[fieldId] = {
                terms: {
                    field: fieldId
                }
            };
            this.set({
                facets: facets
            }, {
                silent: true
            });
            this.trigger('facet:add', this);
        },
        addHistogramFacet: function (fieldId) {
            var facets = this.get('facets');
            facets[fieldId] = {
                date_histogram: {
                    field: fieldId,
                    interval: 'day'
                }
            };
            this.set({
                facets: facets
            }, {
                silent: true
            });
            this.trigger('facet:add', this);
        }
    });


    // ## <a id="facet">A Facet (Result)</a>
    my.Facet = Backbone.Model.extend({
        constructor: function Facet() {
            Backbone.Model.prototype.constructor.apply(this, arguments);
        },
        defaults: function () {
            return {
                _type: 'terms',
                total: 0,
                other: 0,
                missing: 0,
                terms: []
            };
        }
    });

    // ## A Collection/List of Facets
    my.FacetList = Backbone.Collection.extend({
        constructor: function FacetList() {
            Backbone.Collection.prototype.constructor.apply(this, arguments);
        },
        model: my.Facet
    });

    // ## Object State
    //
    // Convenience Backbone model for storing (configuration) state of objects like Views.
    my.ObjectState = Backbone.Model.extend({});


    // ## Backbone.sync
    //
    // Override Backbone.sync to hand off to sync function in relevant backend
    Backbone.sync = function (method, model, options) {
        return model.backend.sync(method, model, options);
    };

} (this.recline.Model));

/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/

// Standard JS module setup
this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";
    // ## MultiView
    //
    // Manage multiple views together along with query editor etc. Usage:
    // 
    // <pre>
    // var myExplorer = new model.recline.MultiView({
    //   model: {{recline.Model.Dataset instance}}
    //   el: {{an existing dom element}}
    //   views: {{dataset views}}
    //   state: {{state configuration -- see below}}
    // });
    // </pre> 
    //
    // ### Parameters
    // 
    // **model**: (required) recline.model.Dataset instance.
    //
    // **el**: (required) DOM element to bind to. NB: the element already
    // being in the DOM is important for rendering of some subviews (e.g.
    // Graph).
    //
    // **views**: (optional) the dataset views (Grid, Graph etc) for
    // MultiView to show. This is an array of view hashes. If not provided
    // initialize with (recline.View.)Grid, Graph, and Map views (with obvious id
    // and labels!).
    //
    // <pre>
    // var views = [
    //   {
    //     id: 'grid', // used for routing
    //     label: 'Grid', // used for view switcher
    //     view: new recline.View.Grid({
    //       model: dataset
    //     })
    //   },
    //   {
    //     id: 'graph',
    //     label: 'Graph',
    //     view: new recline.View.Graph({
    //       model: dataset
    //     })
    //   }
    // ];
    // </pre>
    //
    // **sidebarViews**: (optional) the sidebar views (Filters, Fields) for
    // MultiView to show. This is an array of view hashes. If not provided
    // initialize with (recline.View.)FilterEditor and Fields views (with obvious 
    // id and labels!).
    //
    // <pre>
    // var sidebarViews = [
    //   {
    //     id: 'filterEditor', // used for routing
    //     label: 'Filters', // used for view switcher
    //     view: new recline.View.FielterEditor({
    //       model: dataset
    //     })
    //   },
    //   {
    //     id: 'fieldsView',
    //     label: 'Fields',
    //     view: new recline.View.Fields({
    //       model: dataset
    //     })
    //   }
    // ];
    // </pre>
    //
    // **state**: standard state config for this view. This state is slightly
    //  special as it includes config of many of the subviews.
    //
    // <pre>
    // state = {
    //     query: {dataset query state - see dataset.queryState object}
    //     view-{id1}: {view-state for this view}
    //     view-{id2}: {view-state for }
    //     ...
    //     // Explorer
    //     currentView: id of current view (defaults to first view if not specified)
    //     readOnly: (default: false) run in read-only mode
    // }
    // </pre>
    //
    // Note that at present we do *not* serialize information about the actual set
    // of views in use -- e.g. those specified by the views argument -- but instead 
    // expect either that the default views are fine or that the client to have
    // initialized the MultiView with the relevant views themselves.
    my.MultiView = Backbone.View.extend({
        template: ' \
    <div class="recline-data-explorer"> \
        <div class="alert-messages"></div> \
        <div class="header clearfix reclineHeader"> \
            <div class="nav-wrap" style="width:100%">\
                <div class="navigation"> \
                    <div class="btn-group" data-toggle="buttons-radio"> \
                        {{#views}}<a href="#{{id}}" data-view="{{id}}" class="apple_btn {{id}} _lngTrans_Translatable">{{label}}</a>{{/views}} \
                    </div> \
                </div> \
                <div class="menu-right"> \
                            {{#search}}<div class="query-editor-here" style="display:inline;float: left;"></div> \
                    <div class="btn-group" data-toggle="buttons-checkbox" style="float:left;"> \
                        {{#sidebarViews}}<a href="#" data-action="{{id}}" class="btn _lngTrans_Translatable">{{label}}</a>{{/sidebarViews}} \
                    </div> \
                {{/search}}\
        {{#save}}<span class="btn saveconfig _lngTrans_Translatable">Save Config</span>{{/save}}\
        {{#export}}<span class="btn exprt_btn _lngTrans_Translatable">Export</span>{{/export}}\
        {{#saveAsDashlet}}\
        <div class="dropdown">\
            <button class="btn btn-primary dropdown-toggle" type="button" data-toggle="dropdown">Save<span class="caret"></span></button>\
            <div class="dropdown-menu" aria-labelledby="dropdownMenu1">\
                <div class="dropdown-item dashlet" href="#">Save As</div>\
                {{#catalog}}<div class="dropdown-item dashletSave {{^buttonShow}}disableClass{{/buttonShow}}" href="#">Save</div>{{/catalog}}\
            </div>\
        </div>\
            <div id="saveDashlet" class="saveDashlet" title="Save As">\
            {{#dashletConfig}}\
                <div class="dash-row">\
                    <div class="dash-label"><label>Name :</label></div>\
                    <div class="dash-text"><input type="text" id="txtName" maxlength="50" class="col-md-12 form-control" value= "{{name}}"/></div>\
                </div>\
                <div class="dash-row">\
                    <div class="dash-label"><label>Description :</label></div>\
                    <div class="dash-text"><textarea id="txtDescription" class="col-md-12 form-control">{{description}}</textarea></div>\
                </div>\
                <div class="dash-row">\
                    <div class="dash-label"><label>Tags :</label></div>\
                    <div class="dash-text"><input type="text" id="txtTag" maxlength="50" class="col-md-12 form-control" value= "{{tag}}"/></div>\
                </div>\
                 <div class="dash-row">\
                    <div class="dash-label"><label>Category :</label></div>\
                    <div class="dash-text">\
                        <select id="ddlCatalogCategories"  class="col-md-12 form-control"> \
	                    {{#CatalogCategories}} \
	                        <option value="{{value}}" {{#selected}}selected{{/selected}}>{{text}}</option> \
	                    {{/CatalogCategories}} \
	                    </select> \
                    </div>\
                </div>\
            {{/dashletConfig}}\
            </div>\
        {{/saveAsDashlet}}\
      {{#export}}\
      <div class="exportPopup modalContent" title="Export To" style="display:none;" view="graph"><ul><li><select class="exportSelect">\
                <option>Excel</option>\
                <option>HTML</option>\
                <option>PDF</option>\
        </select></li><li class="exportGrid" style="float:left;"><input type="checkbox" /><label class="">Export All Columns</label></li><li class="exportGrid"><input type="checkbox" /><label class="_lngTrans_Translatable">Include Gridline</label></li></ul>\
        </div>\
      {{/export}}\
      </div> \
      <div class="menu-left"> \
        {{#timeseiesexist}} \
            <div id="rangeselector">\
	            <i class="glyphicon glyphicon-calendar fa fa-calendar"></i>\
	            <span></span> <b class="caret"></b>\
            </div>\
            <!-- <div class="timeInterval">\
	            <select> \
	            {{#intervals}} \
	                <option value="{{id}}" class="_lngTrans_Translatable {{name}}" {{#selected}}selected{{/selected}}>{{name}}</option> \
	            {{/intervals}} \
	            </select> \
            </div> -->\
        {{/timeseiesexist}} \
	  </div>\
      <div class="title-wrap">\
            <div class="pagination-result">\
            {{#pagination}}\
	            <div class="recline-results-info"> \
	            </div> \
            {{/pagination}}\
          </div>\
      </div>\
   {{#catalog}}{{#dashletList}}\
       <div class="dashletList">\
          <select class="drpDashList">{{#catalog}}{{^CatalogTypeId}}<option disabled selected>Select..</option>{{/CatalogTypeId}}{{/catalog}}\
              {{#Graph.length}}<optgroup label="Chart">{{#Graph}}<option value="{{CatalogId}}" {{#selected}}selected{{/selected}}>{{Name}}</option>{{/Graph}}</optgroup>{{/Graph.length}}\
              {{#Grid.length}}<optgroup label="Grid">{{#Grid}}<option value="{{CatalogId}}" {{#selected}}selected{{/selected}}>{{Name}}</option>{{/Grid}}</optgroup>{{/Grid.length}}\
              {{#Pivot.length}}<optgroup label="Pivot">{{#Pivot}}<option value="{{CatalogId}}" {{#selected}}selected{{/selected}}>{{Name}}</option>{{/Pivot}}</optgroup>{{/Pivot.length}}\
          </select>\
        </div>\
    {{/dashletList}}\
    <div class="commentShareDiv commentMainDiv"><span class="divCommentShareDiv" title="Comment"></span><div class="commentContainer"><span class="arrow-up"></span></div></div>\
    {{#isSharable}}<div class="commentShareDiv shareMainDiv"><span class="shareDiv" title="Share"></span><div class="shareContainer"></div></div>{{/isSharable}}{{/catalog}}\
    </div> \
    </div> \
  <div class="visuzlize-header" ><span class="visualize_btn _lngTrans_Translatable {{^gotoAnalytics}} fullWidth{{/gotoAnalytics}}" >Visualize</span>{{#gotoAnalytics}}<span class="analytic_btn _lngTrans_Translatable">Analyze</span>{{/gotoAnalytics}}</div> \
    <div class="visualize-buttons">\
        <div class="graph-editor visualize-button">Chart Editor</div> \
        <div class="timeseries-editor visualize-button">Timeseries Editor</div> \
    </div>\
    <div style="width:100%;">\
        {{#catalog}}{{#savedFilters}}{{#filterLength}}<div class="parentFilter"><div class="showFilters"> <span class="spnFilter"></span>\
            <div class="selectedFilters"><div class="filterLegendWrapper" title="Show Filters"><div class="filterLegend"></div></div><div class="selectedContentWrapper">{{#filter}}<div class="selectedContentwrap"><span class="hdFilter">{{title}}</span><span class="opFilter">({{operator}})</span><span class="contFilter">{{displayValue1}}{{displayValue2}}</span></div>{{/filter}}</div></div></div></div>{{/filterLength}}{{/savedFilters}}{{/catalog}}\
        <div class="data-view-sidebar"></div> \
        <div class="data-view-container-wrap">\
            <div class="data-view-container"></div>\
        </div>\
    </div> \
    </div> \
    </div> \
  ',
        events: {
            'click .menu-right a': '_onMenuClick',
            'click .navigation a': '_onSwitchView',
            'click .show-filter': '_onCustomSelectorClick',
            'click .exprt_btn': '_onExportClick',
            'click .analytic_btn': '_onGoToAnalyticClick',
            'click .dashlet': '_onDashletClick',
            'click .dashletSave': '_onDashletSaveClick',
            'click .saveconfig': 'onSaveconfigClick',
            'click .divCommentShareDiv': 'commentIconClick',
            'click .shareDiv': 'shareIconClick',
            'click .graph-editor, .timeseries-editor, .map-editor': '_showSideviewGraph',
            'click .visuzlize-header': '_hideSidebar',
            'click .filterLegend': '_hideSideFilters',
            'change .drpDashList': 'onChangeDashList'
        },

        initialize: function (options) {
            var self = this;
            this._setupState(options.state);
            // Hash of 'page' views (i.e. those for whole page) keyed by page name
            if (options.views) {
                this.pageViews = options.views;
            } else {
                this.pageViews = [{
                    id: 'grid',
                    label: 'Grid',
                    view: new my.SlickGrid({
                        model: this.model,
                        state: this.state.get('view-grid')
                    })
                }, {
                    id: 'graph',
                    label: 'Graph',
                    view: new my.Graph({
                        model: this.model,
                        state: this.state.get('view-graph')
                    })
                }, {
                    id: 'pivot',
                    label: 'Pivot',
                    view: new my.Pivot({
                        model: this.model,
                        state: this.state.get('view-pivot')
                    })
                }, {
                    id: 'map',
                    label: 'Map',
                    view: new my.Map({
                        model: this.model,
                        state: this.state.get('view-map')
                    })
                }, {
                    id: 'timeline',
                    label: 'Timeline',
                    view: new my.Timeline({
                        model: this.model,
                        state: this.state.get('view-timeline')
                    })
                }];
            }
            // Hashes of sidebar elements

            if (options.sidebarViews) {
                this.sidebarViews = options.sidebarViews;
            } else {
                this.sidebarViews = [{
                    id: 'OptionsView',
                    label: 'Options',
                    view: new my.Options({
                        model: this.model
                    })
                }, {
                    id: 'filterEditor',
                    label: 'Filters',
                    view: new my.FilterEditor({
                        model: this.model
                    })
                }, {
                    id: 'fieldsView',
                    label: 'Fields',
                    view: new my.Fields({
                        model: this.model
                    })
                }];
            }
            // these must be called after pageViews are created
            if (this.pageViews.length == 1 && this.pageViews[0].id == "grid")
                this.state.set('currentView', "grid");
            this.render();
            this._bindStateChanges();
            this._bindFlashNotifications();
            // now do updates based on state (need to come after render)
            if (this.state.get('readOnly')) {
                this.setReadOnly();
            }
            if (this.state.get('currentView')) {
                this.updateNav(this.state.get('currentView'));
                self.$el.parents('.mis-container').addClass('with-sidebar');
                if (this.state.get('currentView') == 'all') {
                    var heightPanel = $(".recline-data-explorer").height();
                    // $('#divSideBySideView').css('max-height', heightPanel+50);
                    //$('#divSideBySideView').css('height', heightPanel+50);
                }
            } else {
                //this.updateNav(this.pageViews[0].id);                
                this.updateNav('all');
            }
            this._showHideSidebar();

            this.listenTo(this.model, 'query:start', function () {
                self.notify({
                    loader: true,
                    persist: true
                });
            });
            this.listenTo(this.model, 'query:done', function () {
                self.clearNotifications();
                self.$el.find('.doc-count').text(self.model.recordCount || 'Unknown');
            });
            this.listenTo(this.model, 'query:fail', function (error) {
                self.clearNotifications();
                var msg = '';
                if (typeof (error) == 'string') {
                    msg = error;
                } else if (typeof (error) == 'object') {
                    if (error.title) {
                        msg = error.title + ': ';
                    }
                    if (error.message) {
                        msg += error.message;
                    }
                } else {
                    msg = 'There was an error querying the backend';
                }
                self.notify({
                    message: msg,
                    category: 'error',
                    persist: true
                });
            });

            // retrieve basic data like fields etc
            // note this.model and dataset returned are the same
            // TODO: set query state ...?
            this.model.queryState.set(self.state.get('query'), {
                silent: true
            });
            //set height of container
		if($('.dataContent').length>0){
            		var dataHeight = $('.dataContent').height() - 170; //Pivot grand total missing -fixed Sreejith.
            		$('.PanBI-main-data-container').css('overflow', 'hidden');
            		$('.data-view-container-wrap').css('height', dataHeight + 'px');
		}
        },

        setReadOnly: function () {
            this.$el.addClass('recline-read-only');
        },

        _hideSideFilters: function (e) {
            $(".showFilters .filterLegend").toggleClass("active");
            $(e.target).parents('.showFilters').find('.selectedFilters').toggleClass("active");
            if ($(".showFilters .filterLegend").hasClass("active")) {
                $(".showFilters .filterLegend").attr("title", "Hide Filters");
                var slickcontrol = $(document).height() - 350;
                $(".slick-viewport").css({
                    'height': slickcontrol + 'px'
                });
                $(".orb-container.orb-blue").eq(0).addClass('pivotScroll');
                var screenwidth = $(window).height();
                if (screenwidth > 750) {
                    $(".orb-container.orb-blue.pivotScroll").css('max-height', '550px');
                } else {
                    $(".orb-container.orb-blue.pivotScroll").css('max-height', '470px');
                }
            } else {
                $(".showFilters .filterLegend").attr("title", "Show Filters");
                $(".orb-container.orb-blue").eq(0).removeClass('pivotScroll');
            }
            if (!$(".showFilters .filterLegend").hasClass("active")) {
                var slickcontrol = $(".slick-viewport").height() + 10;
                $(".slick-viewport").css({
                    'height': slickcontrol + 'px'
                });
            }
            //added for resizing the pivot grid while toggling sidepanel
            var options = {
                width : $('.data-view-container').width()
            }           
            pgridwidget.resetPivotDOMStructure(options);
        },

        //Comment Icon Click
        commentIconClick: function (e) {
            var self = this;
            $('.commentContainer').show().collaborate({
                catalogId: self.model.get('catalog').CatalogId
            });
            $('.shareContainer').hide();
        },

        //Share Icon Click
        shareIconClick: function (e) {
            var self = this;
            $('.shareContainer').show().shareCollaborate({
                catalogId: self.model.get('catalog').CatalogId,
                reDraw: self.setPermission
            });
            $('.commentContainer').hide();
        },

        setPermission: function (permission) {
            if (!permission) {
                $('.commentShareDiv.shareMainDiv').hide();
            }
        },

        //dropdown change of catalogs
        onChangeDashList: function (e) {
            if (window.location.pathname.indexOf("BI/DataViewer/Index") >= 0)
                window.location = "/BI/DataViewer/Index?CatalogId=" + $(e.target).val();
            else
                window.location = "/BI/MISViewer/Index?misID=" + $(e.target).val() + "&fromCatalogViewer=" + this.model.get('catalog').parentCatalog;
        },

        render: function () {
            var tmplData = this.model.toTemplateJSON();
            tmplData.views = this.pageViews;
            tmplData.sidebarViews = this.sidebarViews;
            var timeState = this.model.timeState.toJSON();
            var interval = _.findWhere(timeState.intervals, {
                id: timeState.interval
            });
            if (interval) interval["selected"] = true;
            $.extend(true, tmplData, {
                title: this.options.title,
                intervals: timeState.intervals,
                timeseiesexist: timeState.enable && timeState.datePicker,
                gotoAnalytics: this.options.onGotoAnalytics != undefined ? this.options.buttons.gotoAnalytics : undefined, //(typeof this.options.onGotoAnalytics === "function"),
                save: this.options.buttons.save,
                export: this.options.buttons.export,
                saveAsDashlet: this.options.buttons.saveAsDashlet,
                navigation: this.options.buttons.navigation,
                pagination: this.options.buttons.pagination,
                search: this.options.buttons.search,
                dashletConfig: {
                    name: this.options.dashlet.name,
                    description: this.options.dashlet.description,
                    tag: this.options.dashlet.tag,
                    category: this.options.dashlet.category,
                    CatalogCategories: this.options.dashlet.CatalogCategories

                }
            });
            if (tmplData.catalog) {
                if (_app.catalogType.Dashlet == tmplData.catalog.CatalogTypeId) {
                    tmplData.catalog.CatalogTypeId = false;
                } else {
                    tmplData.catalog.CatalogTypeId = true;
                }

                if (tmplData.catalog.buttonShow && _app.catalogType.Dashlet == tmplData.catalog.CatalogTypeId)
                    tmplData.catalog.buttonShow = true;
                else
                    tmplData.catalog.CatalogTypeId = true;
            }

            if (tmplData.catalog && tmplData.catalog.dashletList && tmplData.catalog.dashletList.length != 0) {
                $.each(tmplData.catalog.dashletList, function (i, val) {
                    if (tmplData.catalog.CatalogId == val.CatalogId)
                        val["selected"] = true;
                    else val["selected"] = false;
                });

                tmplData.catalog.dashletList = _.groupBy(tmplData.catalog.dashletList, function (obj) {
                    return obj.DashletType;
                });
            }
            if (tmplData.catalog && tmplData.catalog.isSharable == "False") {
                tmplData.catalog.isSharable = false;
            } else {
                tmplData.catalog.isSharable = true;
            }
            if (tmplData.catalog && tmplData.catalog.savedFilters) {
                var tempObj=tmplData.catalog.savedFilters.filter;
                _.each(tempObj, function (filter, key) {
                    if (filter.type == "date-time" || filter.type == "date") {
                        /*formatting selected filter Date Fields*/
                        var format = _.findWhere(tmplData.fields, {
                                        id: filter.fieldName
                        });
                        tmplData.catalog.savedFilters.filter[key].displayValue1 = filter.value1 == "" ? filter.value1.replace(/\|/g, "") : valueFormatter(filter.value1, format).replace(/\|/g, "");
                        tmplData.catalog.savedFilters.filter[key].displayValue2 = filter.value2 == "" ? filter.value2.replace(/\|/g, "") : valueFormatter(filter.value2, format).replace(/\|/g, "");
                    }
                    else
                    {
                        /*Displayvalue to display saved filter in user selected format while value remains same.*/
                        tmplData.catalog.savedFilters.filter[key].displayValue1 = filter.value1;
                        tmplData.catalog.savedFilters.filter[key].displayValue2 = filter.value2;
                    }
                });
                if (tmplData.catalog && tmplData.catalog.asOfDate) {
                    var item = $.grep(tmplData.catalog.savedFilters.filter, function (val, i) {
                        return val.fieldName == "AsOfDate";
                    });
                    if (item.length == 0) {
                        (tmplData.catalog.savedFilters.filter).push({
                            "id": "asOfDate",
                            "fieldName": "AsOfDate",
                            "displayName": "As Of Date",
                            "operator": "Today",
                            "value1": "",
                            "value2": "",
                            "type": "custom",
                            "checked": true,
                            "multiValue": false,
                            "aliasName": "AsOfDate",
                            "title": "As Of Date"
                        });
                    }
                }
                tmplData.catalog.savedFilters["filterLength"] = tmplData.catalog.savedFilters.filter.length == 0 ? false : true;
            }

            var template = Mustache.render(this.template, tmplData);
            this.$el.html(template);
            if (timeState.enable) this._initRageSelector();
            // now create and append other views
            var $dataViewContainer = this.$el.find('.data-view-container');
            var $dataSidebar = this.$el.find('.data-view-sidebar');

            // the main views
            _.each(this.pageViews, function (view, pageName) {
                view.view.render();
                $dataViewContainer.append(view.view.el);
                if (view.view.elSidebar) {
                    $dataSidebar.append(view.view.elSidebar);
                }
            });

            _.each(this.sidebarViews, function (view) {
                this['$' + view.id] = view.view.$el;
                $dataSidebar.append(view.view.el);

            }, this);
            //$dataSidebar.append('<div class="collaborate toggler" style="display: block;">Collaborate</div><div id="jtableSideViewCommentShareTab" class="ui-tabs ui-widget ui-widget-content ui-corner-all" style="display: block;" tab-active="true"><ul class="ui-tabs-nav ui-helper-reset ui-helper-clearfix ui-widget-header ui-corner-all" role="tablist"><li style="background-color: #e1dcd9!important;border: none; " class="ui-state-default ui-corner-top ui-tabs-active ui-state-active" role="tab" tabindex="0" aria-controls="jtableSideViewCommentTab" aria-labelledby="ui-id-12" aria-selected="true"><a href="#jtableSideViewCommentTab" class="filter-CommentTab_comment ui-tabs-anchor" role="presentation" tabindex="-1" id="ui-id-12"><div>Comment</div></a></li><li style="background-color: #e1dcd9!important;border: none;" class="ui-state-default ui-corner-top" role="tab" tabindex="-1" aria-controls="jtableSideViewShareTab" aria-labelledby="ui-id-13" aria-selected="false"><a href="#jtableSideViewShareTab" class="filter-CommentTab_Share ui-tabs-anchor" role="presentation" tabindex="-1" id="ui-id-13"><div>Share</div></a></li></ul><div id="jtableSideViewCommentTab" aria-labelledby="ui-id-12" class="ui-tabs-panel ui-widget-content ui-corner-bottom" role="tabpanel" aria-expanded="true" aria-hidden="false"><div class="jtable-comment-main-container" style="display: block;"><div class="jtable-comment-control-container"><div class="jtable-cmnt-identity-holder"><span class="jtable-cmnt-primary-user-image"></span><div class="jtable-cmnt-username">Assyst</div></div><div class="jtable-cmnt-input-holder"><textarea type="text" class="commentTxtArea"></textarea><button id="" class="ui-widget ui-state-default ui-corner-all ui-button-text-only" title="Comment"><span class="ui-button-text _lngTrans_Translatable">Comment</span></button><button id="" class="ui-widget ui-state-default ui-corner-all ui-button-text-only" title="Clear"><span class="ui-button-text _lngTrans_Translatable">Clear</span></button></div></div><div class="jtable-comment-block-container"></div></div></div><div id="jtableSideViewShareTab" aria-labelledby="ui-id-13" class="ui-tabs-panel ui-widget-content ui-corner-bottom" role="tabpanel" aria-expanded="false" aria-hidden="true" tab-active="false" style="display: none;"><div class="jtable-share-container"><div class="jtable-share-block-sub-title">Access</div><div class="jtable-share-access-block"><div class="jtable-share-access-container"><span class="jtable-share-access-input-sub-holder"><input type="radio" data-item="None" data-accesstype="0" name="ShareAccess" checked="checked"><div>None</div></span><span class="jtable-share-access-input-sub-holder"><input type="radio" data-item="Public" data-accesstype="82001" name="ShareAccess"><div>Public</div></span><span class="jtable-share-access-input-sub-holder"><input type="radio" data-item="Private" data-accesstype="82007" name="ShareAccess"><div>Private</div></span></div><select class="jtable-share-usertype-combo" style="display: none;"><option value="1">Admin</option><option value="2">Author</option><option value="3">Reviewer</option><option value="4">Editor</option></select></div><div class="jtable-share-block-sub-title">Specific User</div><span class="jtable-share-specificuser-add-icon" title="Add more users."></span><div class="jtable-share-user-group-container"><div class="ui-corner-all jtable-share-user-cancel-icon" title="Remove User(s)"></div><div class="jtable-share-user-lov" id="JSELECTOR-UserGroups"><div class="jselector-wrapper-container" data-jselectorid="1"><div class="jselector-multi-select-input-container ui-corner-all"><div class="jselector-multi-select-input-display"></div><input type="text" id="JSELECT-UserGroups" class="ui-autocomplete-input" autocomplete="off"><span role="status" aria-live="polite" class="ui-helper-hidden-accessible"></span><div class="jselector-button-container"></div><input type="hidden" id="Edit-UserGroups" name="UserGroups"></div></div></div><select class="jtable-share-usertype-combo"><option value="1">Admin</option><option value="2">Author</option><option value="3">Reviewer</option><option value="4">Editor</option></select></div><div class="jtable-share-button-panel"><button id="" class="ui-widget ui-state-default ui-corner-all ui-button-text-only" title="Apply"><span class="ui-button-text _lngTrans_Translatable">Apply</span></button><button id="" class="ui-widget ui-state-default ui-corner-all ui-button-text-only" title="Cancel"><span class="ui-button-text _lngTrans_Translatable">Cancel</span></button></div></div></div></div>');
            this.pager = new recline.View.Pager({
                model: this.model,
                onPagingChange: this.options.onPagingChange
            });
            this.$el.find('.recline-results-info').after(this.pager.el);

            this.queryEditor = new recline.View.QueryEditor({
                model: this.model.queryState
            });
            this.$el.find('.query-editor-here').append(this.queryEditor.el);
            this.$el.find("a[data-action='fieldsView'], a[data-action='filterEditor']").hide();
            if (this.$el.find('.parentFilter').length > 0) {
                this.$el.find('.data-view-container-wrap').addClass('Active');
            }
        },

        _hideSidebar: function (e) {
            var self = this;
            if(self.$el.parents('.mis-container').hasClass('with-sidebar')) {
                if(self.state.get('currentView') == "graph" && !self.$el.find(e.currentTarget).hasClass('filterActive')){
                    self.updateNav(self.state.get('currentView'));
                    self.$el.find('.graphEditor').hide();
                    self.$el.find(e.currentTarget).addClass('filterActive');
                    self.$el.find('.recline-filter-editor').show();
                    self.$el.parents('.mis-container').addClass('with-sidebar');
                }
                else {
                    self.$el.parents('.mis-container').removeClass('with-sidebar').width('100%');
                    self.$el.find(e.currentTarget).removeClass('filterActive');
                    self.$el.find('.graph-editor').hide();
                    // call view.view.show after sidebar visibility has been determined so
                    // that views can correctly calculate their maximum width
                    _.each(this.pageViews, function (view, idx) {
                        if (view.id === self.state.get('currentView')) {
                            if (view.view.show) {
                                view.view.show();
                            }
                        }
                    });
                }
            }
            else {
                self.$el.parents('.mis-container').addClass('with-sidebar');
                self.updateNav(self.state.get('currentView'));                
            }
             //Resizing Grid while hide/show visualize/analyze bar
            var contentGrid=_.findWhere(self.pageViews, { id: "grid" }).view.grid;
            if(contentGrid)
            {
                contentGrid.resizeCanvas();
                var totalsPlugin = new TotalsPlugin($.getScrollbarWidth());
                contentGrid.registerPlugin(totalsPlugin);
            }
            if ($('.recline-flot').find('.chartGrid').length != 0) {
                $('.chartGrid').width($('.data-view-container').width());
                chartGrid.resizeCanvas();
                chartGrid.registerPlugin(new TotalsPlugin($.getScrollbarWidth()));
            }
            else
            {
                $('.data-view-container-wrap').removeClass('pivot-active');
            }

            //added for resizing the pivot grid while toggling sidepanel
            var options = {
                width : $('.data-view-container').width()
            }           
            pgridwidget.resetPivotDOMStructure(options);                        
        },

        remove: function () {
            _.each(this.pageViews, function (view) {
                view.view.remove();
            });
            _.each(this.sidebarViews, function (view) {
                view.view.remove();
            });
            this.pager.remove();
            this.queryEditor.remove();
            Backbone.View.prototype.remove.apply(this, arguments);
        },

        _timeIntevalChange: function (e) {
            var self = this;
            var interval = parseInt(this.$el.find('.timeInterval select option:selected').val());
            self.model.timeState.updateInterval(interval);
        },

        _initRageSelector: function () {
            var self = this;
            var timeseries = self.model.timeState.toJSON(); //_.findWhere(self.pageViews, {id: 'timeseries'}).view.state.toJSON();
            var cb = function (start, end, label) {
                self.$el.find('#rangeselector span').html(start.format('MMM D, YYYY') + ' - ' + end.format('MMM D, YYYY'));
            };
            var optionSet = {
                startDate: new Date(timeseries.start),
                endDate: new Date(timeseries.end),
                showDropdowns: true,
                ranges: {},
                opens: 'right',
                applyClass: 'btn-sm btn-primary',
                locale: {
                    applyLabel: 'Submit',
                    cancelLabel: 'Clear',
                    fromLabel: 'From',
                    toLabel: 'To',
                    customRangeLabel: 'Custom',
                    daysOfWeek: ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa'],
                    monthNames: ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'],
                    firstDay: 1
                }
            };
            self.$el.find('#rangeselector').daterangepicker(optionSet, cb);
            self.$el.find('#rangeselector span').html(moment(optionSet["startDate"]).format('MMM D, YYYY') + ' - ' + moment(optionSet["endDate"]).format('MMM D, YYYY'));
            self.$el.find('#rangeselector').on('apply.daterangepicker', function (ev, picker) {
                if (typeof self.options.onDateRangeChange === "function") {
                    var timeseries = _.findWhere(self.pageViews, {
                        id: 'timeseries'
                    }).view.state.toJSON();
                    var data = self.options.onDateRangeChange(timeseries, picker, function (data) {
                        if (typeof data.Records == 'string') {
                            data.Records = JSON.parse(data.Records);
                        }
                        self.model.timeState.updateTime({
                            start: picker.startDate,
                            end: picker.endDate
                        });
                        self.model.pagingState.setPaging(data.TotalRecordCount);
                        self.model.restore(data.Records);
                    });
                }
            });
        },

        // hide the sidebar if empty
        _showHideSidebar: function () {
            var $dataSidebar = this.$el.find('.data-view-sidebar');
            var visibleChildren = $dataSidebar.children().filter(function () {
                return $(this).css("display") != "none";
            }).length;
            if (visibleChildren > 0 && this.state.attributes.currentView == "graph") {
                $dataSidebar.show();
            } else {
                $dataSidebar.hide();
		  $('.maxView').removeClass('with-sidebar');
            }
            
        },

        updateNav: function (pageName) {
            var self = this;
            $('div.nav-wrap .title-wrap .pagination-result').show();
            $('div.nav-wrap .title-wrap .pivot-raw-record-count').hide();
            this.$el.find('.exprt_btn').show();
            this.$el.find('.dashlet').show();
            this.$el.find('.navigation a').removeClass('active');
            var $el = this.$el.find('.navigation a[data-view="' + pageName + '"]');
            $el.addClass('active');
            var elementIndex = $el.index();
            $('.data-view-container > div').removeClass('active').eq(elementIndex).addClass('active').animate({
                'left': '0'
            }, 800, function () {
                $('.data-view-container > div:not(.active)').css('left', '-100%');
            });

            // call view.view.show after sidebar visibility has been determined so
            // that views can correctly calculate their maximum width
            _.each(this.pageViews, function (view, idx) {
                if (view.id === pageName) {
                    if (view.view.show) {
                        view.view.show();
                    }
                }
            });

            this.state.set({
                currentView: pageName
            });

            $('.recline-filter-editor').show();
            $('.pivotEditor,.editor-block').hide();
            $('.Metrics,.metrics-container,.Aggregate,.aggregate-container,.recline-filter-editor .footerSet').show();
            
            self.$el.find('.data-view-sidebar').removeClass('editorClass');
            if (pageName == "graph") {
                 $('.parentFilter').css('width', 'calc(100% - 235px)');
                 $(".dashletList").css("right", "660PX");
                 if ($(".PanBI-main-data-container").hasClass('open'))
                 {
                     $(".dashletList").css("right", "382PX");
                 }
                 self.$el.find('.editor-block').show();
                 self.$el.find('.data-view-sidebar').addClass('editorClass');
                 $('.timeseries-editor,.timeseriesEditor,.pivotEditorSide,.recline-filter-editor').hide();
                 if(self.$el.parents('.mis-container').hasClass('with-sidebar')) {
                    $('.graph-editor,.option-view.visualize-button,.graphEditor,.visualize-buttons').show();
                 }
                 
                 if ($('#divTabViewer').length > 0)
                 {
                     self.$el.parents('.mis-container').find('.data-view-container-wrap').width('calc(100% - 260px)');
                 }
                 else
                 {
                     $('.Metrics,.Aggregate').hide();
                 }
                 $('.header').removeClass('active');
             }
             else if (pageName == "timeseries")
             {
                 $('.graph-editor,.recline-filter-editor,.graphEditor,.pivotEditorSide').hide();
                 if ($('#divTabViewer').length > 0) self.$el.parents('.mis-container').find('.data-view-container-wrap').width('calc(100% - 260px)');
                 $('.timeseries-editor,.option-view.visualize-button,.exprt_btn,.timeseriesEditor,.editor-block').show();
                 $('.data-view-container-wrap').removeClass('pivot-active');
                 $('.header').removeClass('active');
             }
             else if (pageName == "map")
             {
                 $('.timeseries-editor,.graph-editor,.pivotEditorSide,.data-view-sidebar').hide();
                 $('.recline-filter-editor,.option-view.visualize-button,.exprt_btn').show();
                 $('.data-view-container-wrap').removeClass('pivot-active');
                 $('.header').removeClass('active');
             }
             else if (pageName == "grid")
                {      
                 $('.parentFilter').css('width', '98%');
                 $(".dashletList").css("right", "660PX");
                 self.$el.parents('.mis-container').find('.data-view-container-wrap').width('100%');
                 if ($(".PanBI-main-data-container").hasClass('open'))
                 {
                     $(".dashletList").css("right", "382PX");
                 }
                 $('.timeseries-editor,.graph-editor,.editor-block,.pivotEditorSide,.visualize-buttons').hide();
                 $('.timeseries-editor,.graph-editor,.editor-block,.data-view-sidebar').hide();
                 $('.data-view-container-wrap').removeClass('pivot-active');
                 var grid = _.findWhere(self.options.views,
                 {
                     id: "grid"
                 });
                 $('.header').removeClass('active');
                 if (self.model.queryState.get('groupings').selectedGroups == undefined)
                 {
                     grid.view.grid.resizeCanvas();
                     grid.view.grid.registerPlugin(new TotalsPlugin($.getScrollbarWidth()));
                 }
                 //Added by sreejith to show sidebar as default when selecting saved dashlet from catalog.
                 if ((self.model.queryState.get('groupings')['selectedGroups'] != undefined) && ((self.model.queryState.get('groupings').selectedGroups.length > 0) || (self.model.queryState.get('filters').length > 0)))
                 {
                     self.$el.parents('.mis-container').addClass('with-sidebar');
                 }
               }    
              else if (pageName == "pivot") {               
                 /* Added for comment and share icon for both Analyze and Visualize Page */
                 $('.parentFilter').css('width', '98%');
                 $('.dashletList').css('right', '545px');
                 self.$el.parents('.mis-container').find('.data-view-container-wrap').width('100%');
                 $('div.nav-wrap .title-wrap .pagination-result').show();
                 $('div.nav-wrap .title-wrap .pivot-raw-record-count').show();

                 if ($(".title-wrap .pagination-results:visible").length == 0)
                 {
                     $('.dashletList').css('right', '585px');
                 }
                 if ($(".PanBI-main-data-container").hasClass('open'))
                 {
                     $(".dashletList").css("right", "382PX");
                 }
                 if ($(".PanBI-main-data-container").hasClass('open') && $('.pivot').hasClass('active'))
                 {
                     $(".dashletList").css("right", "382PX");
                 }

                 $('.recline-filter-editor,.timeseries-editor,.graph-editor,.editor-block,.option-view.visualize-button,.exprt_btn,.data-view-sidebar').hide();
                 $('.pivotEditorSide,.pivot-raw-record-count').show();
                 $('.data-view-container-wrap').removeClass('pivot-active');
                 $('.header').addClass('active');
                 $('.pivotEditor').show();

                 //added for resizing the pivot grid while toggling sidepanel                 
                 if(pgridwidget){
                     var options = {
                        width : $('.data-view-container').width()
                     }           
                     pgridwidget.resetPivotDOMStructure(options);     
                 }            
             }
        },

        _onMenuClick: function (e) {
            e.preventDefault();
            var action = $(e.target).attr('data-action');
            this['$' + action].toggle();
            this._showHideSidebar();
        },

        onSaveconfigClick: function (e) {
            var self = this;
            if (typeof self.options.onSaveConfig === "function") {
                var data = self.getReclineConfig();
                self.options.onSaveConfig(data);
            }
        },

        getReclineConfig: function () {
            var self = this;
            var fields = [];
            var dasletView = {
                grid: {
                    show: true
                }
            };
            var type = $('.apple_btn.active').attr('data-view');
            _.each(self.pageViews, function (view, idx) {
                var view = self._getView(view.id);
                $.extend(dasletView, view);
            });
            _.each(self.model.fields.toJSON(), function (value, idx) {   
                if(JSON.parse(self.model.get('catalog').schema).computedFields)  {       
                    var item = JSON.parse(self.model.get('catalog').schema).computedFields[value['id']];
                    if(!item) fields.push(value);
                }
                else{
                    fields.push(value);
                }
            });
            var data = {
                fields: fields,
                views: dasletView,
                type: type,
                computedFields: JSON.parse(self.model.get('catalog').schema).computedFields
            };
            return data;
        },

        _onDashletClick: function (e) {
            var self = this;
            self.$el.find('#saveDashlet').attr('title', $(e.target).text());
            var myPos = [($(window).width() / 2), 130];
            if (self.model.recordCount == 0) {
                $("#dialog-confir").html("No record found.");
                $("#dialog-confir").data('object', self).dialog({
                    resizable: false,
                    modal: true,
                    height: "auto",
                    width: "auto",
                    draggable: false,
                    close: function () {
                        $(this).dialog('destroy');
                    },
                    dialogClass: "message error-dialog custom-style no-titlebar"
                });
            } else {
                self.$el.find('#saveDashlet').dialog({
                    position: myPos,
                    modal: true,
                    draggable: false,
                    resizable: false,
                    height: 280,
                    width: 400,
                    buttons: {
                        "Save": function () {
                        
                            var saveOption = $(e.target).text();
                            if (typeof self.options.onSaveAsDashlet === "function") {
                                var data = [],
                                    isDashboard = false;
                                var name = $('#txtName', this).val();
                                var description = $('#txtDescription', this).val();
                                var tag = $('#txtTag', this).val();
                                var category = $('#txtCategory', this).val();
                                var categoryId = $('#ddlCatalogCategories', this).val();
                                var dasletView = self.getActiveViewConfig();
                                
                                if ($.isArray(dasletView)) {
                                    _.each(self.dasletView, function (view, idx) {
                                        var dashTitle = name + '-' + view.id;
                                        if (!_.findWhere(self.options.dashlet.titles, {
                                            "name": dashTitle
                                        })) {
                                            self.options.dashlet.titles.push({
                                                "name": dashTitle
                                            });
                                        }
                                        var computedFields = $.grep(self.model.get('fields'),function(val,i){
                                            return val.formula != null;
                                        });                                

                                        var prow = {};
                                        _.each(computedFields,function(val,i){
                                            prow[val.id] = val;
                                        });

                                        data.push({
                                            name: dashTitle,
                                            description: description,
                                            tag: tag,
                                            category: category,
                                            type: dasletView.type,
                                            computedFields:prow,
                                            view: view.view,
                                            categoryId: categoryId
                                        });
                                    });
                                    isDashboard = true;
                                } else {
                                    if (!_.findWhere(self.options.dashlet.titles, {
                                        "name": name
                                    })) {
                                        self.options.dashlet.titles.push({
                                            "name": name
                                        });
                                    }
                                    var computedFields = $.grep(self.model.get('fields'),function(val,i){
                                        return val.formula != null;
                                    });                                

                                    var prow = {};
                                    _.each(computedFields,function(val,i){
                                        prow[val.id] = val;
                                        val.filter = true;
                                    });

                                    data.push({
                                        name: name,
                                        description: description,
                                        tag: tag,
                                        category: category,
                                        type: dasletView.type,
                                        computedFields:prow,
                                        view: dasletView.view,
                                        categoryId: categoryId
                                    });
                                }
                                self.options.onSaveAsDashlet(data, isDashboard, name, saveOption, self);
                            }
                            $(this).dialog("destroy"); // dialog is destroyed otherwise ul will shown.
                        },
                        "Cancel": function () {
                            $(this).dialog('destroy');
                        }

                    },
                    close: function () {
                        $(this).dialog('destroy');
                    }
                });
                  $('.ui-dialog-buttonset').find('*').each(function(){      //tooltip for buttons
                 $(this).attr('title', $(this).text());
                });
                $('.ui-dialog-buttonset').children().addClass('dashlet');
                $('#saveDashlet').css('height', '180px');
            }
        },

        _onDashletSaveClick: function (e) {
            var self = this;
            $("#dialog-confir").html(g_LangTranslator.Translate('Do you want to overwrite?'));
            $("#dialog-confir").data('object', self).dialog({
                resizable: false,
                modal: true,
                height: "180",
                width: "auto",
                draggable: false,
                close: function () {
                    $(this).dialog('destroy');
                },
                dialogClass: "message info-dialog custom-style no-titlebar",
                buttons: {
                    OK: function () {
                        var saveOption = $(e.target).text();
                        if (typeof self.options.onSaveAsDashlet === "function") {
                            var data = [],
                                isDashboard = false;
                            var name = $('#txtName').val();
                            var description = $('#txtDescription').val();
                            var tag = $('#txtTag').val();
                            var category = $('#txtCategory').val();
                            var categoryId = $('#ddlCatalogCategories').val();
                            var dasletView = self.getActiveViewConfig();
                            if ($.isArray(dasletView)) {
                                _.each(self.dasletView, function (view, idx) {
                                    var dashTitle = name + '-' + view.id;
                                    if (!_.findWhere(self.options.dashlet.titles, {
                                        "name": dashTitle
                                    })) {
                                        self.options.dashlet.titles.push({
                                            "name": dashTitle
                                        });
                                    }
                                    var computedFields = $.grep(self.model.get('fields'),function(val,i){
                                        return val.formula != null;
                                    });                                

                                    var prow = {};
                                    _.each(computedFields,function(val,i){
                                        prow[val.id] = val;
                                    });
                                    data.push({
                                        name: dashTitle,
                                        description: description,
                                        tag: tag,
                                        category: category,
                                        type: dasletView.type,
                                        computedFields:prow,
                                        view: view.view,
                                        categoryId: categoryId
                                    });
                                });
                                isDashboard = true;
                            } else {
                                if (!_.findWhere(self.options.dashlet.titles, {
                                    "name": name
                                })) {
                                    self.options.dashlet.titles.push({
                                        "name": name
                                    });
                                }
                                var computedFields = $.grep(self.model.get('fields'),function(val,i){
                                    return val.formula != null;
                                });                                

                                var prow = {};
                                _.each(computedFields,function(val,i){
                                    prow[val.id] = val;
                                });

                                data.push({
                                    name: name,
                                    description: description,
                                    tag: tag,
                                    category: category,
                                    type: dasletView.type,
                                    view: dasletView.view,
                                    computedFields:prow,
                                    categoryId: categoryId
                                });
                            }
                              $(this).dialog('destroy');
                            self.options.onSaveAsDashlet(data, isDashboard, name, saveOption, self);
                        }
                    },
                    Cancel: function () {
                        $(this).dialog('destroy');
                    }
                }
            });
        },

        reDraw: function (result, dashletList) {
            var self = this;
            self.model.attributes.catalog.CatalogId = result.CatalogId;
            self.model.attributes.catalog.CatalogTypeId = false;
            self.model.attributes.catalog.buttonShow = true;
            if(self.model.attributes.catalog.buttonShow) this.$el.find('.dashletSave').removeClass('disableClass');
            self.model.attributes.catalog.dashletList = [];
            self.model.attributes.catalog.dashletList = dashletList;
            self.model.attributes.catalog.dashletList = _.groupBy(self.model.attributes.catalog.dashletList, function (obj) {
                if (obj.DashletType == "Graph") obj.DashletType = "Chart";
                return obj.DashletType;
            });
            $('.dashletList').remove();
            var dashletListDiv = $('<div class="dashletList"></div>');
            var selectDiv = $('<select class="drpDashList"></select>');
            $.each(self.model.get('catalog').dashletList, function (key, val) {
                var optGrp = $('<optgroup label=' + key + '></optgroup>');
                $.each(val, function (key, value) {
                    var options;
                    if (value.selected)
                        options = $('<option value="' + value.CatalogId + '" selected>' + value.Name + '</option>');
                    else options = $('<option value="' + value.CatalogId + '" >' + value.Name + '</option>');
                    optGrp.append(options);
                });
                $(selectDiv).append(optGrp);
            });
            dashletListDiv.append(selectDiv);
            self.$el.find('.recline-data-explorer .nav-wrap').append(dashletListDiv);

            var commentWidget = $('.commentContainer').data('collaborate');
            if (commentWidget) {
                commentWidget.options.catalogId = result.CatalogId;
                commentWidget._create();
            }

            var shareWidget = $('.shareContainer').data('shareCollaborate');
            if (shareWidget) {
                shareWidget.options.catalogId = result.CatalogId;
                shareWidget._create();
            }
        },

        /*getActiveViewConfig: function () {
            var self = this;
            var dasletView = undefined;
            var view = self._getActiveView();
            switch (view) {
                case 'all':
                    dasletView = [];
                    _.each(self.pageViews, function (view, idx) {
                        dasletView.push({
                            type: view.id,
                            view: self._getView(view.id)
                        });
                    });
                    break;
                default:
                    dasletView = {
                        type: view,
                        view: self._getView(view)
                    };
                    break;
            }

            var preFilters, sort, rows = [],
                cols = [],
                vals = [],
                columnGrandTotal, rowGrandTotal, colSubTotal, rowSubTotal, filterMultiValue;
            var esQueryFilter = $('.pivotEditorSide').aggFunctions('returnData');
            if (config != null) {
                preFilters = pgridwidget.pgrid.filters; //Getting filtered items checkboxes.
                filterMultiValue = pgridwidget.config.filterMultivalue;
                sort = pgridwidget.pgrid.config.sort;
                columnGrandTotal = pgridwidget.pgrid.config.grandTotal.columnsvisible;
                rowGrandTotal = pgridwidget.pgrid.config.grandTotal.rowsvisible;
                colSubTotal = pgridwidget.pgrid.config.columnFields.length == 0 ? undefined : pgridwidget.pgrid.config.columnFields[0].subTotal;
                rowSubTotal = pgridwidget.pgrid.config.rowFields.length == 0 ? undefined : pgridwidget.pgrid.config.rowFields[0].subTotal;

                var rowfields = pgridwidget.pgrid.config.rowFields;
                for (var i = 0; i < rowfields.length; i++) {

                    for (var j = 0; j < pgridwidget.config.fields.length; j++) {
                        if (pgridwidget.config.fields[j].name == rowfields[i].name) { // to get rowfields from all fields

                            if (!rowfields[i].visibleState) { // if  specific field visible state is false make that fields  visible state false in all fields. 
                                pgridwidget.config.fields[j].visibleState = false;
                                break;
                            } else {
                                pgridwidget.config.fields[j].visibleState = true;
                                break;
                            }
                        }

                    }
                    rows.push(rowfields[i].caption); //collect rows.
                }

                var columnFields = pgridwidget.pgrid.config.columnFields
                for (var i = 0; i < columnFields.length; i++) {
                    cols.push(columnFields[i].caption);
                }

                var dataFields = pgridwidget.pgrid.config.dataFields;
                for (var i = 0; i < dataFields.length; i++) {
                    vals.push(dataFields[i].caption);
                }

                var updatedState = {
                    cols: cols,
                    rows: rows,
                    vals: vals,
                    preFilters: preFilters,
                    filterMultiValueKey: filterMultiValue,
                    esQueryFilter: esQueryFilter.filter,
                    pivotFields: pgridwidget.config.fields,
                    totals: {
                        columnGrandTotal: columnGrandTotal,
                        rowGrandTotal: rowGrandTotal,
                        colSubTotal: colSubTotal,
                        rowSubTotal: rowSubTotal
                    },
                    sort: sort
                };

                if (self._getActiveView() == "pivot" && cols.length == 0 && rows.length == 0 && vals.length == 0) {
                    $("#dialog-confir").html("Fields are empty.");
                    $("#dialog-confir").data('object', self).dialog({
                        resizable: false,
                        modal: true,
                        height: "100",
                        width: "auto",
                        draggable: false,
                        close: function () {
                            $(this).dialog('destroy');
                        },
                        dialogClass: "message error-dialog custom-style no-titlebar"
                    });
                } else {
                    dasletView.view.state.queryState['Pivot'] = updatedState;
                }
            }
            return dasletView;
        },*/


         getActiveViewConfig: function () {
                var self = this;
                var dasletView = undefined;
                var view = self._getActiveView();
                var views = {
            grid: {
                    show: true
                  }
            };
            _.each(self.pageViews, function (view, idx) {
                      var type = self._getView(view.id);
                       $.extend(views, type);
                 });
            dasletView = {
                type: view,
                view: views
            };
            return dasletView;
       },

        _showSideviewGraph: function (e) {
            var self = this;
            if ($(e.target).text() == "Chart Editor") {
                self.$el.find('.graphEditor').show();
                self.$el.find('.recline-filter-editor').hide();
                self.$el.parents('.mis-container').addClass('with-sidebar');
            }
        },

        _getView: function (view) {
            var self = this;
            var jsonView = {};
            var activeView = _.findWhere(self.pageViews, {
                id: view
            }).view.state.toJSON();
            switch (view) {
                case 'grid':
                    jsonView = {
                        grid: {
                            show: true
                        }
                    };
                    break;
                case 'graph':
                    jsonView = {
                        graph: {
                            show: true,
                            state: {
                                group: activeView.group,
                                series: activeView.series,
                                graphType: activeView.graphType,
                                transpose: activeView.transpose,
                                dataConvert: activeView.dataConvert,
                                graphState: activeView.graphState
                            }
                        }
                    }
                    break;
                case 'pivot':
                    jsonView = {
                        pivot: {
                            show: true,
                            state: {
                                Pivot: {
                                    rows: [],
                                    cols: [],
                                    vals: []
                                }
                            }
                        }
                    }
                    break;
                case 'timeseries':
                    jsonView = {
                        timeseries: {
                            show: true,
                            state: {
                                group: activeView.group,
                                series: activeView.series,
                                graphType: activeView.graphType,
                                compareField: activeView.compareField,
                                selectedFields: activeView.selectedFields,
                                graphState: activeView.graphState
                            }
                        }
                    }
                    break;
                default:
                    jsonView = {
                        map: {
                            show: true,
                            state: {
                                kpiField: activeView.kpiField,
                                kpiSlabs: activeView.kpiSlabs
                            }
                        }
                    }
                    break;
            }

            var pagingState = {
                pageSize: self.model.pagingState.get("pageSize"),
                pagedList: {
                    pageNumber: self.model.pagingState.get("pagedList").pageNumber
                }
            };
            jsonView["state"] = {
                timeState: self.model.timeState.toJSON(),
                queryState: self.model.queryState.toJSON(),
                pagingState: pagingState
            };

            var preFilters, sort, rows = [],
                cols = [],
                vals = [],
                columnGrandTotal, rowGrandTotal, colSubTotal, rowSubTotal, filterMultiValue,isPivotFieldRequired;

            var esQueryFilter = $('.pivotEditorSide').aggFunctions('returnData');
            if (config != null) {
                preFilters = pgridwidget.pgrid.filters; //Getting filtered items checkboxes.
                filterMultiValue = pgridwidget.config.filterMultivalue;
                sort = pgridwidget.pgrid.config.sort;
                columnGrandTotal = pgridwidget.pgrid.config.grandTotal.columnsvisible;
                rowGrandTotal = pgridwidget.pgrid.config.grandTotal.rowsvisible;
                colSubTotal = pgridwidget.pgrid.config.columnFields.length == 0 ? undefined : pgridwidget.pgrid.config.columnFields[0].subTotal;
                rowSubTotal = pgridwidget.pgrid.config.rowFields.length == 0 ? undefined : pgridwidget.pgrid.config.rowFields[0].subTotal;
                
                //To display pivotfields
                if(pgridwidget.config)
                    isPivotFieldRequired=pgridwidget.config.isPivotFieldRequired;
                else
                    isPivotFieldRequired=true;

                var rowFields = pgridwidget.pgrid.config.rowFields;
                for (var i = 0; i < rowFields.length; i++) {
                    for (var j = 0; j < pgridwidget.config.fields.length; j++) {
                        if (pgridwidget.config.fields[j].name == rowFields[i].name) { // to get rowfields from all fields
                            if (!rowFields[i].visibleState) { // if  specific field visible state is false make that fields  visible state false in all fields. 
                                pgridwidget.config.fields[j].visibleState = false;
                                break;
                            } else {
                                pgridwidget.config.fields[j].visibleState = true;
                                break;
                            }
                        }
                    }
                    rows.push(rowFields[i].caption); //collect rows.
                }

                var columnFields = pgridwidget.pgrid.config.columnFields
                for (var i = 0; i < columnFields.length; i++) {
                    cols.push(columnFields[i].caption);
                }

                var dataFields = pgridwidget.pgrid.config.dataFields;
                for (var i = 0; i < dataFields.length; i++) {
                    vals.push(dataFields[i].caption);
                }

                var updatedState = {
                    cols: cols,
                    rows: rows,
                    vals: vals,
                    preFilters: preFilters,
                    filterMultiValueKey: filterMultiValue,
                    esQueryFilter: esQueryFilter.filter,
                    pivotFields: pgridwidget.config.fields,
                    isPivotFieldRequired:isPivotFieldRequired,
                    totals: {
                        columnGrandTotal: columnGrandTotal,
                        rowGrandTotal: rowGrandTotal,
                        colSubTotal: colSubTotal,
                        rowSubTotal: rowSubTotal
                    },
                    sort: sort
                };
                jsonView.state.queryState['Pivot'] = updatedState;
            }
            return jsonView;
        },

        _onGoToAnalyticClick: function (e) {
            var self = this;
            if (typeof self.options.onGotoAnalytics === "function") {
                self.options.onGotoAnalytics();
            }
        },

        _onExportClick: function (e) {
            var self = this;
            var exportPopup = self._getActiveView(); //self.$el.find('.exportPopup');
            //var view = exportPopup.attr("view");
            var view = exportPopup;

            self._bindExportView(view);
            switch (view) {
                case "grid":
                    var self = this;
                    var jsonExporter = JsonExporter;
                    jsonExporter.options = {
                        reportTitle: self.options.export.title || "MIS", //Give required title dynamically here
                        reportCriteria: self.options.export.criteria, // Structure [{ "fieldName": "StartDate", "operator": "==", "value1": "01-08-2013", "value2": null, "type": "dateTime", "filterType": "mandatory", "displayName": "StartDate", "enabletimeformat": "false", "lookupName": null }, { "fieldName": "EndtDate", "operator": "==", "value1": "31-08-2013", "value2": null, "type": "dateTime", "filterType": "mandatory", "displayName": "EndDate", "enabletimeformat": "false", "lookupName": null }, { "fieldName": "Estate", "operator": "in", "value1": "COTTABETTA", "value2": null, "type": "string", "filterType": "mandatory", "displayName": "Estate", "enabletimeformat": "false", "lookupName": null }, { "fieldName": "Division", "operator": "0", "value1": "", "value2": null, "type": "string", "filterType": "default", "displayName": "Division", "enabletimeformat": "false", "lookupName": null }, { "fieldName": "Gang", "operator": "0", "value1": "", "value2": null, "type": "string", "filterType": "default", "displayName": "Gang", "enabletimeformat": "false", "lookupName": null }]
                        jsonData: function (expOptions) {
                            var _jsonObject = expOptions.options.exportFilteredRecord ? self.model.records.toJSON() : self.model.toJSON().records;
                            var jsonObject = [];
                            $.extend(true, jsonObject, _jsonObject);
                            //?? why we need to remove the geo fields
                            for (var _i = 0; _i < jsonObject.length; _i++) {
                                delete jsonObject[_i]["geo"];
                            }
                            //formatting exporting records
                            var tempJson=jsonObject;
                            _.each(tempJson, function (value, index) {
                               _.each(value, function (val, key) {
                                   var format=_.findWhere(self.model.fields.toJSON(), {
                                                id: key
                                            });
                                   jsonObject[index][key]=valueFormatter(val, format);
                                });
                            });   
                            return jsonObject;
                        },
                        schema: function (expOptions) {
                            var schema = getExportColumns(self.model.fields.toJSON(), expOptions.options.exportAllColumns);
                            return schema;
                        },
                        exportType: self.options.export.type || "PDF", //Default export type(optional)
                        pageSize: self.options.export.pageSize || "A4", //Default page size(optional)
                        promptDialog: self.options.export.promptDialog || true // If set to false, will be exported in default type and page size
                    }
                    jsonExporter.Export();
                    break;
                case "all":
                case "pivot":
                    alert(errorMessage());
                    break;
                case "map":
                case "graph":
                case "timeseries":
                    self.$el[0].scrollIntoView(false);
                    var myPos = [($(window).width() / 2) + 45, 130];
                    $('.exportPopup').dialog({
                        position: myPos,
                        draggable: false,
                        modal: true,
                        resizable: false,
                        'width': '350',
                        'height': '150',
                        buttons: {
                            "Export": function () {
                                var view = $(this).attr("view");
                                var format = $('select.exportSelect :selected', $(this)).val();
                                var cs = new CanvasSaver(self.options.export.url);
                                switch (view) {
                                    case "graph":
                                        var viewer = self.$el.find('.graph');
                                        var svg = $(viewer).find("svg");
                                        svgenie.save(svg[0], {
                                            name: self.options.export.title + "." + format
                                        });
                                        $(this).dialog("close");
                                        break;
                                    case "timeseries":
                                        var viewer = self.$el.find('.timeseries');
                                        var svg = $(viewer).find("svg");
                                        svgenie.save(svg[0], {
                                            name: self.options.export.title + "." + format
                                        });
                                        $(this).dialog("close");
                                        break;
                                    case "map":
                                        var viewer = self.$el.find('.map');
                                        var svg = $(viewer).find("svg");
                                        svgenie.save(svg[0], {
                                            name: self.options.export.title + "." + format
                                        });
                                        $(this).dialog("close");
                                        break;
                                }
                            }
                        },
                        close: function () {
                            $(this).dialog('destroy');
                        }
                    });
                    $('.ui-dialog-buttonset').removeAttr('style');
                    $('.ui-dialog-buttonset').children().addClass('dashlet');
                    $('.ui-dialog-buttonpane').css('margin-top', '-52px');
                    $('.exportPopup').css('height', '70px');
                    $('.exportPopup').css('overflow', 'hidden');
                    break;
            }

            function getExportColumns(fields, showAll) {
                var columnList = [];
                var hiddenType = ['link', 'link-tab', 'link-drill-down'];
                var gridView = self._getPageView('grid');
                if (typeof gridView === 'undefined') return;

                _.each(fields, function (field) {
                    var column = {
                        id: field.id,
                        label: field.label,
                        type: field.type,
                        format:field.format,
                        decimalPoint: field.type === 'number' ? field.decimalPoint : undefined,
                        isHidden: (field.isHidden || $.inArray(field.type, hiddenType) > -1 || (!showAll && gridView.view.grid.getColumnIndex(field.id) == null)) ? 'Y' : 'N',
                        children: field.children
                    };
                    columnList.push(column);
                });
                return columnList;
            }
        },

        _getActiveView: function () {
            var self = this;
            var activBtn = self.$el.find('.navigation a.active');
            var view = activBtn.attr("data-view");
            if ($('#preview').css('display') == "none") {
                view = "pivot";
            }
            return view;
        },

        _getPageView: function (id) {
            var pageView = $.grep(this.pageViews, function (el, i) {
                return el.id == id
            });
            return pageView[0];
        },


        _onCustomSelectorClick: function (e) {
            var action = $(e.target).attr('data-action');
            this['$' + action].toggle();
            this._showHideSidebar();
        },

        _onSwitchView: function (e) {
            e.preventDefault();
            if ($('#btnDataViewerSaveAs').length > 0) {
                $('#btnDataViewerSaveAs').removeClass("disableClass");
            } else {
                $(e.target).parents('.nav-wrap').find('.menu-right .dashlet,.menu-right .exprt_btn').removeClass("disableClass");
            }
            var viewName = $(e.target).attr('data-view');
            if (requestPage != "visualize") {
                if (viewName == "graph" || viewName == "timeseries") {
                    self.$el.parents('.mis-container').find('.data-view-container-wrap').width('calc(100% - 230px)');
                } else {
                    //$('.mis-container .data-view-container-wrap').width('100%')
                }
            }
            self.$el.find(".exportPopup").attr("view", viewName);
            this.updateNav(viewName);
            if (screen.width < 991) {
                $('.visualize-button').removeClass('active');
                $('.mobileToggle').removeClass('mobileToggle');
            }
        },

        _bindExportView: function (viewName) {
            var self = this;
            var exportData = [];
            self.$el.find('.exportGrid').remove();
            if (viewName == 'grid') {
                if ($('.recline-slickgrid').siblings().is(":visible")) {
                    $('.recline-slickgrid').css('border', '0px');
                    $('.recline-slickgrid').css('padding', '0px');
                }
                exportData.push('Excel');
                exportData.push('HTML');
                exportData.push('PDF');
                $('.exportPopup ul').append('<li class="exportGrid" style="float:left;"><input type="checkbox" class="_lngTrans_Translatable"/><label class="_lngTrans_Translatable">Export All Columns</label></li><li class="exportGrid"><input type="checkbox" class="_lngTrans_Translatable"/><label class="_lngTrans_Translatable">Include Gridline</label></li>');
            } else {
                exportData.push('GIF');
                exportData.push('JPG');
                exportData.push('BMP');
            }

            var dropExport = self.$el.find('.exportSelect');
            dropExport.empty();
            for (var i = 0; i < exportData.length; i++) {
                var option = dropExport.append('<option value=' + exportData[i].toLowerCase() + '>' + exportData[i] + '</option>');
            }
        },

        // create a state object for this view and do the job of
        // 
        // a) initializing it from both data passed in and other sources (e.g. hash url)
        //
        // b) ensure the state object is updated in responese to changes in subviews, query etc.
        _setupState: function (initialState) {
            var self = this;
            // get data from the query string / hash url plus some defaults
            var qs = my.parseHashQueryString();
            var query = qs.reclineQuery;
            query = query ? JSON.parse(query) : self.model.queryState.toJSON();
            // backwards compatability (now named view-graph but was named graph)
            var graphState = qs['view-graph'] || qs.graph;
            graphState = graphState ? JSON.parse(graphState) : {};

            // now get default data + hash url plus initial state and initial our state object with it
            var stateData = _.extend({
                query: query,
                'view-graph': graphState,
                backend: this.model.backend.__type__,
                url: this.model.get('url'),
                dataset: this.model.toJSON(),
                currentView: null,
                readOnly: false
            },
                initialState);
            this.state = new recline.Model.ObjectState(stateData);
        },

        _bindStateChanges: function () {
            var self = this;
            // finally ensure we update our state object when state of sub-object changes so that state is always up to date
            this.listenTo(this.model.queryState, 'change', function () {
                self.state.set({
                    query: self.model.queryState.toJSON()
                });
            });
            _.each(this.pageViews, function (pageView) {
                if (pageView.view.state && pageView.view.state.bind) {
                    var update = {};
                    update['view-' + pageView.id] = pageView.view.state.toJSON();
                    self.state.set(update);
                    self.listenTo(pageView.view.state, 'change', function () {
                        var update = {};
                        update['view-' + pageView.id] = pageView.view.state.toJSON();
                        // had problems where change not being triggered for e.g. grid view so let's do it explicitly
                        self.state.set(update, {
                            silent: true
                        });
                        self.state.trigger('change');
                    });
                }
            });
        },

        _bindFlashNotifications: function () {
            var self = this;
            _.each(this.pageViews, function (pageView) {
                self.listenTo(pageView.view, 'recline:flash', function (flash) {
                    self.notify(flash);
                });
            });
        },

        // ### notify
        //
        // Create a notification (a div.alert in div.alert-messsages) using provided
        // flash object. Flash attributes (all are optional):
        //
        // * message: message to show.
        // * category: warning (default), success, error
        // * persist: if true alert is persistent, o/w hidden after 3s (default = false)
        // * loader: if true show loading spinner
        notify: function (flash) {
            var tmplData = _.extend({
                message: 'Loading',
                category: 'warning',
                loader: false
            },
                flash
            );
            var _template;
            if (tmplData.loader) {
                _template = ' \
        <div class="alert alert-info alert-loader _lngTrans_Translatable"> \
          {{message}} \
          <span class="notification-loader">&nbsp;</span> \
        </div>';
            } else {
                _template = ' \
        <div class="alert alert-{{category}} fade in" data-alert="alert"><a class="close" data-dismiss="alert" href="#"></a> \
          {{message}} \
        </div>';
            }
            if ($("#loading").is(":visible")) {
                return;
            }

            var _templated = $(Mustache.render(_template, tmplData));
            _templated = $(_templated).appendTo($('.recline-data-explorer .alert-messages'));
            if (!flash.persist) {
                setTimeout(function () {
                    $(_templated).fadeOut(1000, function () {
                        $(this).remove();
                    });
                }, 1000);
            }
        },

        // ### clearNotifications
        //
        // Clear all existing notifications
        clearNotifications: function () {
            var $notifications = $('.recline-data-explorer .alert-messages .alert');
            $notifications.fadeOut(1500, function () {
                $(this).remove();
            });
        }
    });

    // ### MultiView.restore
    //
    // Restore a MultiView instance from a serialized state including the associated dataset
    //
    // This inverts the state serialization process in Multiview
    my.MultiView.restore = function (state) {
        // hack-y - restoring a memory dataset does not mean much ... (but useful for testing!)
        var datasetInfo;
        if (state.backend === 'memory') {
            datasetInfo = {
                backend: 'memory',
                records: [{
                    stub: 'this is a stub dataset because we do not restore memory datasets'
                }]
            };
        } else {
            datasetInfo = _.extend({
                url: state.url,
                backend: state.backend
            },
                state.dataset
            );
        }
        var dataset = new recline.Model.Dataset(datasetInfo);
        var explorer = new my.MultiView({
            model: dataset,
            state: state
        });
        return explorer;
    };

    // ## Miscellaneous Utilities
    var urlPathRegex = /^([^?]+)(\?.*)?/;

    // Parse the Hash section of a URL into path and query string
    my.parseHashUrl = function (hashUrl) {
        var parsed = urlPathRegex.exec(hashUrl);
        if (parsed === null) {
            return {};
        } else {
            return {
                path: parsed[1],
                query: parsed[2] || ''
            };
        }
    };

    // Parse a URL query string (?xyz=abc...) into a dictionary.
    my.parseQueryString = function (q) {
        if (!q) {
            return {};
        }
        var urlParams = {},
            e, d = function (s) {
                return unescape(s.replace(/\+/g, " "));
            },
            r = /([^&=]+)=?([^&]*)/g;

        if (q && q.length && q[0] === '?') {
            q = q.slice(1);
        }
        while (e = r.exec(q)) {
            // TODO: have values be array as query string allow repetition of keys
            urlParams[d(e[1])] = d(e[2]);
        }
        return urlParams;
    };

    // Parse the query string out of the URL hash
    my.parseHashQueryString = function () {
        var q = my.parseHashUrl(window.location.hash).query;
        return my.parseQueryString(q);
    };

    // Compse a Query String
    my.composeQueryString = function (queryParams) {
        var queryString = '?';
        var items = [];
        $.each(queryParams, function (key, value) {
            if (typeof (value) === 'object') {
                value = JSON.stringify(value);
            }
            items.push(key + '=' + encodeURIComponent(value));
        });
        queryString += items.join('&');
        return queryString;
    };

    my.getNewHashForQueryString = function (queryParams) {
        var queryPart = my.composeQueryString(queryParams);
        if (window.location.hash) {
            // slice(1) to remove # at start
            return window.location.hash.split('?')[0].slice(1) + queryPart;
        } else {
            return queryPart;
        }
    };

    my.setHashQueryString = function (queryParams) {
        window.location.hash = my.getNewHashForQueryString(queryParams);
    };

})(jQuery, recline.View);

/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/

this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";

    my.Options = Backbone.View.extend({
        //className: 'option-view',
        //  template:  ' \
        //    <div class="options"> \
        //        <fieldset> \
        //        <legend class="_lngTrans_Translatable">Options</legend> \
        //         <div class="radio-group"> \
        //          <div class="_lngTrans_Translatable">View:</div><div><label><input type="radio" class="subreport-type" value="tab" name ="{{el}}"  checked="checked" /><span class="_lngTrans_Translatable">Tab</span></label></div> \
        //          <div><label><input type="radio" class="subreport-type" value="drilldown" name ="{{el}}"   /><span class="_lngTrans_Translatable">Drill down</span></label></div> \
        //         </div> \
        //         <div class="checkbox-group"> \
        //            <label class="checkbox"> \
        //                <input type="checkbox" data-action="filterEditor" class="show-filter" value="Filter"/> \
        //              <span class="_lngTrans_Translatable">Filters</span></label> \
        //           </div> \
        //        </fieldset> \
        //    </div> \
        //  ',
        template: ' \
    <div class="options"> \
        <span class="_lngTrans_Translatable" style="line-height: 20px;">Filters</span></label> \
    </div> \
  ',
        events: {
            'click .subreport-type': 'onChangeReportType'
        },
        initialize: function () {
            this.render();
        },

        render: function () {
            var self = this;
            var tmplData = {
                el: self.cid
            };
            var out = Mustache.render(this.template, tmplData);
            //this.$el.html(out);
        },


        onChangeReportType: function (e) {
            //    var self = this;
            //    self.model.queryState.trigger('change');
        }
    });

})(jQuery, recline.View);

/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/


this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";

    my.QueryEditor = Backbone.View.extend({
        className: 'recline-query-editor',
        template: ' \
    <form action="" method="GET" class="form-inline"> \
      <div class="input-prepend text-query"> \
        <span class="add-on"><i class="icon-search"></i></span> \
        <input type="text" name="q" value="{{q}}" class="span2 _lngTrans_Translatable" placeholder="Search data ..." class="search-query" /> \
      </div> \
    </form> \
  ',

        events: {
            'submit form': 'onFormSubmit'
        },

        initialize: function () {
            _.bindAll(this, 'render');
            this.listenTo(this.model, 'change', this.render);
            this.render();
        },
        onFormSubmit: function (e) {
            e.preventDefault();
            var query = this.$el.find('.text-query input').val();
            this.model.set({
                q: query
            });
        },
        render: function () {
            var tmplData = this.model.toJSON();
            var templated = Mustache.render(this.template, tmplData);
            this.$el.html(templated);
        }
    });

})(jQuery, recline.View);

/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/

/*jshint multistr:true */

this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";

    my.ValueFilter = Backbone.View.extend({
        className: 'recline-filter-editor',
        template: ' \
    <div class="filters"> \
      <h3>Filters</h3> \
      <button class="btn js-add-filter add-filter _lngTrans_Translatable">Add filter</button> \
      <form class="form-stacked js-add" style="display: none;"> \
        <fieldset> \
          <label class="_lngTrans_Translatable">Field</label> \
          <select class="fields _lngTrans_Translatable"> \
            {{#fields}} \
            <option value="{{id}}" class="_lngTrans_Translatable">{{label}}</option> \
            {{/fields}} \
          </select> \
          <button type="submit" class="btn _lngTrans_Translatable">Add</button> \
        </fieldset> \
      </form> \
      <form class="form-stacked js-edit _lngTrans_Translatable"> \
        {{#filters}} \
          {{{filterRender}}} \
        {{/filters}} \
        {{#filters.length}} \
        <button type="submit" class="btn update-filter _lngTrans_Translatable">Update</button> \
        {{/filters.length}} \
      </form> \
    </div> \
  ',
        filterTemplates: {
            term: ' \
      <div class="filter-{{type}} filter"> \
        <fieldset> \
          {{field}} \
          <a class="js-remove-filter _lngTrans_Translatable" href="#" title="Remove this filter" data-filter-id="{{id}}">&times;</a> \
          <input type="text" value="{{term}}" name="term" data-filter-field="{{field}}" data-filter-id="{{id}}" data-filter-type="{{type}}" /> \
        </fieldset> \
      </div> \
    '
        },
        events: {
            'click .js-remove-filter': 'onRemoveFilter',
            'click .js-add-filter': 'onAddFilterShow',
            'submit form.js-edit': 'onTermFiltersUpdate',
            'submit form.js-add': 'onAddFilter'
        },
        initialize: function () {
            _.bindAll(this, 'render');
            this.listenTo(this.model.fields, 'all', this.render);
            this.listenTo(this.model.queryState, 'change change:filters:new-blank', this.render);
            this.render();
        },
        render: function () {
            var self = this;
            var tmplData = $.extend(true, {}, this.model.queryState.toJSON());
            // we will use idx in list as the id ...
            tmplData.filters = _.map(tmplData.filters, function (filter, idx) {
                filter.id = idx;
                return filter;
            });
            tmplData.fields = this.model.fields.toJSON();
            tmplData.filterRender = function () {
                return Mustache.render(self.filterTemplates.term, this);
            };
            var out = Mustache.render(this.template, tmplData);
            this.$el.html(out);
        },
        updateFilter: function (input) {
            var self = this;
            var filters = self.model.queryState.get('filters');
            var $input = $(input);
            var filterIndex = parseInt($input.attr('data-filter-id'), 10);
            var value = $input.val();
            filters[filterIndex].term = value;
        },
        onAddFilterShow: function (e) {
            e.preventDefault();
            var $target = $(e.target);
            $target.hide();
            this.$el.find('form.js-add').show();
        },
        onAddFilter: function (e) {
            e.preventDefault();
            var $target = $(e.target);
            $target.hide();
            var field = $target.find('select.fields').val();
            this.model.queryState.addFilter({
                type: 'term',
                field: field
            });
        },
        onRemoveFilter: function (e) {
            e.preventDefault();
            var $target = $(e.target);
            var filterId = $target.attr('data-filter-id');
            this.model.queryState.removeFilter(filterId);
        },
        onTermFiltersUpdate: function (e) {
            var self = this;
            e.preventDefault();
            var filters = self.model.queryState.get('filters');
            var $form = $(e.target);
            _.each($form.find('input'), function (input) {
                self.updateFilter(input);
            });
            self.model.queryState.set({
                filters: filters,
                from: 0
            });
            self.model.queryState.trigger('change');
        }
    });

})(jQuery, recline.View);
/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/

this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";

    my.Pager = Backbone.View.extend({
        className: 'recline-pager',
        template: ' \
    <div class="pagination"> \
    <div class="pagination-wrapper">\
	  <span class="result-count"><span class="doc-count">{{totalRecordCount}}</span> <span class="_lngTrans_Translatable">records</span></span>\
      <ul> \
        <li class="prev action-pagination-update"><a href="">&laquo;</a></li> \
        <li class="active"><a><input name="from" type="text" value="{{from}}" /> &ndash; <input name="to" type="text" value="{{to}}" /> </a></li> \
        <li class="next action-pagination-update"><a href="">&raquo;</a></li> \
      </ul> \
    </div> \
    </div> \
    <div class="paging-container" style="display:none;"></div> \
  ',

        events: {
            'click .action-pagination-update': 'onPaginationUpdate',
            'change input': 'onFormSubmit',
            'click button.page-navigator': 'onPageNavigatorClick',
            'change select.recline-pagination-dropdown': 'onPaginationChange',
            'change select.recline-page-size-dropdown': 'onPageSizeChange'
        },

        initialize: function (options) {
            _.bindAll(this, 'render');
            this.listenTo(this.model.queryState, 'change', this.render);
            this.listenTo(this.model.pagingState, 'change', this.renderPaging);
            this.pagingState = this.model.pagingState;
            this.queryState = this.model.queryState;
            this.render();
        },

        onPageNavigatorClick: function (e) {
            var element = $(e.currentTarget);
            this.pagingState.updatePaging({
                pageNumber: this._tryParseInt(element.attr("pageindex"), 1)
            });
            this.onNavigationSubmit();
        },

        onPaginationChange: function (e) {
            var element = $(e.currentTarget);
            this.pagingState.updatePaging({
                pageNumber: this._tryParseInt(element.val(), 1)
            });
            this.onNavigationSubmit();
        },

        onPageSizeChange: function (e) {
            var element = $(e.currentTarget);
            var pagedList = this.pagingState.get("pagedList");
            var pageSize = this._tryParseInt(element.val(), 5);
            var pageNumber = pagedList.pageNumber;

            if (((pageNumber - 1) * pageSize) > pagedList.itemCount) pageNumber = 1;
            this.pagingState.updatePaging({
                pageNumber: pageNumber
            }, pageSize);
            this.onNavigationSubmit();
        },

        onNavigationSubmit: function () {
            var self = this;
            if (typeof self.options.onPagingChange === "function") {
                var data = self.options.onPagingChange(this.pagingState.toJSON(), function (data) {
                    if (typeof data.Records == 'string') {
                        data.Records = JSON.parse(data.Records);
                    }
                    self.pagingState.setPaging(data.TotalRecordCount);
                    self.model.restore(data.Records);
                });
            }
        },

        onFormSubmit: function (e) {
            e.preventDefault();
            var newFrom = parseInt(this.$el.find('input[name="from"]').val());
            var newSize = (parseInt(this.$el.find('input[name="to"]').val()) - newFrom);
            if(!isNaN(newFrom) && !isNaN(newSize)){
                newFrom = Math.max(newFrom, 0);
                newSize = Math.max(newSize, 1);
                this.queryState.set({
                    size: newSize,
                    from: newFrom
                });
            }
            else {
			$("#dialog-confir").html("Please enter valid numbers.");
                $("#dialog-confir").data('object', self).dialog({
                    resizable: false,
                    modal: true,
                    height: "auto",
                    width: "auto",
                    draggable: false,
                    close: function () {
                        $(this).dialog('destroy');
                    },
                    dialogClass: "message error-dialog custom-style no-titlebar"
                });
            }
        },
        //#166934 Pagination issue
        onPaginationUpdate: function (e) {
            var tot = this.model.recordCount;
            e.preventDefault();
            var $el = $(e.target);
            var newFrom = 0;
            if ($el.parent().hasClass('prev')) {
                newFrom = this.queryState.get('from') - Math.max(0, this.queryState.get('size'));
            } else {
                newFrom = this.queryState.get('from') + this.queryState.get('size');
            }
            newFrom = Math.max(newFrom, 0);
            if (newFrom < tot) {
                this.queryState.set({
                    from: newFrom
                });
            } else {
                $("#dialog-confir").html("No record found.");
                $("#dialog-confir").data('object', self).dialog({
                    resizable: false,
                    modal: true,
                    height: "auto",
                    width: "auto",
                    draggable: false,
                    close: function () {
                        $(this).dialog('destroy');
                    },
                    dialogClass: "message error-dialog custom-style no-titlebar"
                });
            }
        },

        render: function () {
            var tmplData = this.queryState.toJSON();
            tmplData.to = this.queryState.get('from') + this.queryState.get('size');
            tmplData["recordCount"] = this.model.recordCount;
            tmplData["totalRecordCount"] = this.model.recordCount; //TotalRecordCount;        
            var templated = Mustache.render(this.template, tmplData);
            this.$el.html(templated);
            this.renderPaging();
        },

        renderPaging: function () {
            var self = this;
            if (!this.pagingState.get("enable")) return;
            var $container = this.$el.find("div.paging-container");
            var template = '<span class="recline-page-list">' +
                '{{#pageInfo}}' +
                '<span class="recline-page-info">' +
                '<span class="_lngTrans_Translatable">Showing</span>' +
                '<span>&nbsp;{{from}}&nbsp;</span>' +
                '<span class="_lngTrans_Translatable">to</span>' +
                '<span>&nbsp;{{to}}&nbsp;</span>' +
                '<span class="_lngTrans_Translatable">of</span>' +
                '<span>&nbsp;{{itemCount}}&nbsp;</span>' +
                '<span class="_lngTrans_Translatable">records</span>' +
                '</span>' +
                '{{/pageInfo}}' +

                '{{#hasPreviousPage}}' +
                '<span>' +
                '<button class="recline-page-number-first page-navigator" name="first" pageindex="1" >|&lt;</button>' +
                '<button class="recline-page-number-previous page-navigator"  name="previous"  pageindex="{{prevPageIndex}}">&lt;</button>' +
                ' </span>' +
                '{{/hasPreviousPage}}' +

                '<span></span>' +

                '<span class="recline-page-text _lngTrans_Translatable" style="float:left;">Page :</span>' +
                '<select class="recline-pagination-dropdown">' +
                '{{#pages}}' +
                '<option value="{{value}}" class="_lngTrans_Translatable" {{#selected}}selected{{/selected}}>{{value}}</option>' +
                '{{/pages}}' +
                '</select>' +

                '<span class="recline-page-text _lngTrans_Translatable" style="float:left;">Page Size :</span>' +
                '<select class="recline-page-size-dropdown">' +
                '{{#pageSizes}}' +
                '<option value="{{value}}" class="_lngTrans_Translatable" {{#selected}}selected{{/selected}}>{{value}}</option>' +
                '{{/pageSizes}}' +
                '</select>' +

                '{{#hasNextPage}}' +
                '<span>' +
                '<button class="recline-page-number-next page-navigator" pageindex="{{nextPageIndex}}">&gt;</button>' +
                '<button class="recline-page-number-last page-navigator" pageindex="{{pageCount}}">&gt;|</button>' +
                '</span>' +
                '{{/hasNextPage}}' +
                '<span></span>' +
                '</span>';

            var pages = [];
            var pageSizes = [];
            var pagedList = self.pagingState.get("pagedList");
            var pageSize = self.pagingState.get("pageSize");

            $.each(self.pagingState.get("pageSizes"), function (idx, size) {
                pageSizes.push({
                    value: size,
                    selected: (size == pageSize)
                });
            });
            var start = 1;
            var max = pagedList.pageCount;
            if (pagedList.pageCount > 100) {
                if ((pagedList.pageNumber + 100) >= pagedList.pageCount) {
                    start = pagedList.pageCount - 100;
                    max = pagedList.pageCount
                } else if (pagedList.pageNumber > 50) {
                    start = pagedList.pageNumber - 50;
                    max = start + 100;
                } else max = 100;
            }

            var idx = 0;
            for (idx = start; idx <= max; idx++) {
                pages.push({
                    value: idx,
                    selected: (idx == pagedList.pageNumber)
                });
            }
            var pageConfig = {
                pageInfo: (self.pagingState.get("showItemCount") && pagedList.itemCount > 0),
                from: (pagedList.isFirstPage ? 1 : ((pagedList.pageNumber - 1) * pageSize) + 1),
                to: (pagedList.isLastPage ? pagedList.itemCount : (pagedList.pageNumber * pageSize)),
                prevPageIndex: (pagedList.pageNumber - 1),
                nextPageIndex: (pagedList.pageNumber + 1),
                pages: pages,
                pageSizes: pageSizes
            };
            var tmplData = $.extend(true, {}, pagedList, pageConfig);
            var pagingHtml = Mustache.render(template, tmplData);
            $container.empty().html(pagingHtml);
        },

        _tryParseInt: function (value, defaultValue) {
            var retValue = defaultValue;
            var value = $.trim(value);
            if (value.length > 0 && !isNaN(value)) {
                retValue = parseInt(value);
            }
            return retValue;
        }

    });

})(jQuery, recline.View);

//Bug #169342
$(window).load(function () {
    if ($(".filter-form-container").length > 0) {
        $(".slick-viewport").css("overflow-x", "hidden");
    } else {
        $(".slick-viewport").css("overflow-x", "auto")
    }
});