/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   
JQuery Immediately-Invoked Function for recline for Leaflet
Created By      : Rony T S for Assyst International Pvt. Ltd.
Created Date    : 20-Feb-2014
Version         : 1.0.2
Project         : PanAgro

+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/

/*jshint multistr:true */

this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";
    // ## Map view for a Dataset using Leaflet mapping library.
    //
    // This view allows to plot gereferenced records on a map. The location
    // information can be provided in 2 ways:
    //
    // 1. Via a single field. This field must be either a geo_point or 
    // [GeoJSON](http://geojson.org) object
    // 2. Via two fields with latitude and longitude coordinates.
    //
    // Which fields in the data these correspond to can be configured via the state
    // (and are guessed if no info is provided).
    //
    // Initialization arguments are as standard for Dataset Views. State object may
    // have the following (optional) configuration options:
    //
    // <pre>
    //   {
    //     // geomField if specified will be used in preference to lat/lon
    //     geomField: {id of field containing geometry in the dataset}
    //     lonField: {id of field containing longitude in the dataset}
    //     latField: {id of field containing latitude in the dataset}
    //     autoZoom: true,
    //     // use cluster support
    //     // cluster: true = always on
    //     // cluster: false = always off
    //     cluster: false
    //   }
    // </pre>
    //
    // Useful attributes to know about (if e.g. customizing)
    //
    // * map: the Leaflet map (L.Map)
    // * features: Leaflet GeoJSON layer containing all the features (L.GeoJSON)
    var grades = [];
    var gradeColors = [];


    my.Map = Backbone.View.extend({
        template: ' \
    <div class="recline-map"> \
      <div class="panel map"></div> \
    </div> \
',

        // These are the default (case-insensitive) names of field that are used if found.
        // If not found, the user will need to define the fields via the editor.
        latitudeFieldNames: ['lat', 'latitude'],
        longitudeFieldNames: ['lon', 'longitude'],
        geometryFieldNames: ['geojson', 'geom', 'the_geom', 'geometry', 'spatial', 'location', 'geo', 'lonlat'],

        initialize: function (options) {
            var self = this;
            this.visible = true;
            this.mapReady = false;
            // this will be the Leaflet L.Map object (setup below)
            this.map = null;

            var stateData = _.extendOptions({
                type: "Point", //Point, Heat, MultiPolygon (Gobal configuration)
                geomField: null,
                lonField: null,
                latField: null,
                autoZoom: false,
                cluster: false,
                kpiField: null,
                kpiSlabs: [{}],
                kpiRange: {
                    hues: ["#0fff03", "#eb0707"], //["#fdd24e", "#acd03e"],
                    scalesIn: 1000,
                    scales: 10,
                    scalesInView: undefined,
                    note: "Values in thousands"
                }
            },
                options.state
            );

            this.state = new recline.Model.ObjectState(stateData);

            this._clusterOptions = {
                zoomToBoundsOnClick: true,
                //disableClusteringAtZoom: 10,
                maxClusterRadius: 80,
                singleMarkerMode: false,
                skipDuplicateAddTesting: true,
                animateAddingMarkers: false
            };

            // Listen to changes in the fields
            this.listenTo(this.model.fields, 'change', function () {
                self._setupGeometryField();
                self.render();
            });

            // Listen to changes in the records
            this.listenTo(this.model.records, 'add', function (doc) {
                self.redraw('add', doc);
            });
            this.listenTo(this.model.records, 'change', function (doc) {
                self.redraw('remove', doc);
                self.redraw('add', doc);
            });
            this.listenTo(this.model.records, 'remove', function (doc) {
                self.redraw('remove', doc);
            });
            this.listenTo(this.model.records, 'reset', function () {
                self.redraw('reset');
            });

            this.menu = new my.MapMenu({
                model: this.model,
                state: this.state.toJSON()
            });
            this.listenTo(this.menu.state, 'change', function () {
                self.state.set(self.menu.state.toJSON());
                self.redraw();
            });
            this.listenTo(this.state, 'change', function () {
                self.redraw();
                self.legend.update();
            });
            this.elSidebar = this.menu.$el;
        },

        // ## Customization Functions
        //
        // The following methods are designed for overriding in order to customize
        // behaviour

        // ### infobox
        //
        // Function to create infoboxes used in popups. The default behaviour is very simple and just lists all attributes.
        //
        // Users should override this function to customize behaviour i.e.
        //
        //     view = new View({...});
        //     view.infobox = function(record) {
        //       ...
        //     }
        infobox: function (record) {
            var self = this;
            var html = '';
            var type = '';
            var decimalPoint = 2;
            var value = '';
            //for (var key in record.attributes) {
            $.each(record.fields.toJSON(), function( index, item ) {
                var key = item.id;
                var field = self.model.fields.get(key);
                if (!field) return;

                if (!(self.state.get('geomField') && key == self.state.get('geomField')) && !field.get('isHidden')) {
                    // Code by Assyst     //html += '<div><strong>' +  key + '</strong>: '+ record.attributes[key] + '</div>';

                    type = field.get('type');
                    value = record.attributes[key];
                    if (type === 'number' && parseFloat(value) != NaN) {
                        value = numberWithCommas(parseFloat(value).toFixed(field.get('decimalPoint')));
                        html += '<tr class="infobox-tr"><td class="infobox-td-left _lngTrans_Translatable"  defaultkey="' + field.get('name') + '">' + textTranslation(field.get('name')) + '</td><td class="infobox-td-right align-right"> ' + value + '</td></tr>';
                    }
                    else {
                        html += '<tr class="infobox-tr"><td class="infobox-td-left _lngTrans_Translatable"  defaultkey="' + field.get('name') + '">' + textTranslation(field.get('name')) + '</td><td class="infobox-td-right"> ' + value + '</td></tr>';
                    }
                }
            });
            var tableStructure = '<table class="infobox">' + html + '</table>'
            return tableStructure;
        },

        // ## Customization Functions (created by assyst)
        //
        // The following methods are designed for overriding in order to customize
        // behaviour

        // ### getKpiValue
        //
        // Function to get the value for kpi map density
        getKpiValue: function (record) {
            if (this.state.get('kpiField')) {
                return record.attributes[this.state.get('kpiField')]
            }
            return 0;
        },

        // Options to use for the [Leaflet GeoJSON layer](http://leaflet.cloudmade.com/reference.html#geojson)
        // See also <http://leaflet.cloudmade.com/examples/geojson.html>
        //
        // e.g.
        //
        //     pointToLayer: function(feature, latLng)
        //     onEachFeature: function(feature, layer)
        //
        // See defaults for examples
        geoJsonLayerOptions: {
            // pointToLayer function to use when creating points
            //
            // Default behaviour shown here is to create a marker using the
            // popupContent set on the feature properties (created via infobox function
            // during feature generation)
            //
            // NB: inside pointToLayer `this` will be set to point to this map view
            // instance (which allows e.g. this.markers to work in this default case)
            pointToLayer: function (feature, latlng) {
                var marker = ((feature.isHeat && feature.isHeat == 'Y') || this.state.get('type') == "Heat") ? new L.circleMarker(latlng) : new L.Marker(latlng);
                marker.bindPopup(feature.properties.popupContent);
                // this is for cluster case
                this.markers.addLayer(marker);
                return marker;
            },
            // onEachFeature default which adds popup in
            onEachFeature: function (feature, layer) {
                if (feature.properties && feature.properties.popupContent) {
                    layer.bindPopup(feature.properties.popupContent);
                }
            },

            // Modified by Assyst (Rony at 06-Feb-2014)    
            // For map color gradients based on density value
            /******************************************/
               style: function (feature) {
                return {
                    weight: 2,
                    opacity: 1,
                    color: getColor(feature.geometry.properties.kpiValue),
                    dashArray: '3',
                    fillOpacity: 0.7,
                    fillColor: getColor(feature.geometry.properties.kpiValue)
                };
                /****************End*********************/
            }
        },

        // ### Public: Adds the necessary elements to the page.
        //
        // Also sets up the editor fields and the map if necessary.
        render: function () {
            var self = this;
            var htmls = Mustache.render(this.template, this.model.toTemplateJSON());
            this.$el.html(htmls);
            this.$map = this.$el.find('.panel.map');
            // Dashlet Viewer height adjustment
            var parent = this.$el.parents('.content_div');
            if (parent) this.$map.height(parent.height());
            //end
            this.redraw();
            return this;
        },

        // Private: Helper function to remove repeated grouped data
        //
        _groupMapdata: function (data,field1,field2) {
            var groups = _.groupBy(data, function(value){
                return value.attributes[field1] + ',' + value.attributes[field2];
            });

            var mapData = _.map(groups, function(group){
                return group[0]
            });
            return mapData;
        },

        // ### Public: Redraws the features on the map according to the action provided
        //
        // Actions can be:
        //
        // * reset: Clear all features
        // * add: Add one or n features (records)
        // * remove: Remove one or n features (records)
        // * refresh: Clear existing features and add all current records
        redraw: function (action, doc) {
            var self = this;
            action = action || 'refresh';
            // try to set things up if not already
            if (!self._geomReady()) {
                self._setupGeometryField();
            }
            if (!self.mapReady) {
                self._setupMap();
            }

            if (this._geomReady() && this.mapReady) {
                // removing ad re-adding the layer enables faster bulk loading
                this.map.removeLayer(this.features);
                this.map.removeLayer(this.markers);

                var countBefore = 0;
                this.features.eachLayer(function () {
                    countBefore++;
                });

                if (action == 'refresh' || action == 'reset') {
                    this.features.clearLayers();
                    // recreate cluster group because of issues with clearLayer
                    this.map.removeLayer(this.markers);
                    this.markers = new L.MarkerClusterGroup(this._clusterOptions);
                    this._add(self._groupMapdata(this.model.records.models,self.state.attributes.lonField,self.state.attributes.latField));
                } else if (action == 'add' && doc) {
                    this._add(doc);
                } else if (action == 'remove' && doc) {
                    this._remove(doc);
                }

                // this must come before zooming!
                // if not: errors when using e.g. circle markers like
                // "Cannot call method 'project' of undefined"
                if (this.state.get('cluster')) {
                    this.map.addLayer(this.markers);
                } else {
                    this.map.addLayer(this.features);
                }

                if (this.state.get('autoZoom')) {
                    if (this.visible) {
                        this._zoomToFeatures();
                    } else {
                        this._zoomPending = true;
                    }
                }
            }
        },

        show: function () {
            // If the div was hidden, Leaflet needs to recalculate some sizes
            // to display properly
            if (this.map) {
                this.map.invalidateSize();
                if (this._zoomPending && this.state.get('autoZoom')) {
                    this._zoomToFeatures();
                    this._zoomPending = false;
                }
            }
            this.visible = true;
        },

        hide: function () {
            this.visible = false;
        },

        _geomReady: function () {
            return Boolean(this.state.get('geomField') || (this.state.get('latField') && this.state.get('lonField')));
        },

        // Private: Add one or n features to the map
        //
        // For each record passed, a GeoJSON geometry will be extracted and added
        // to the features layer. If an exception is thrown, the process will be
        // stopped and an error notification shown.
        //
        // Each feature will have a popup associated with all the record fields.
        //
        _add: function (docs) {
            var self = this;

            if (!(docs instanceof Array)) docs = [docs];

            var count = 0;
            var wrongSoFar = 0;
            _.every(docs, function (doc) {
                count += 1;
                var feature = self._getGeometryFromRecord(doc);
                if (typeof feature === 'undefined' || feature === null) {
                    // Empty field
                    return true;
                } else if (feature instanceof Object) {
                    feature.properties = {
                        popupContent: self.infobox(doc),
                        // Add a reference to the model id, which will allow us to
                        // link this Leaflet layer to a Recline doc
                        cid: doc.cid,
                        //kpi value for desity on kpi map
                        kpiValue: self.getKpiValue(doc)
                    };

                    try {
                        self.features.addData(feature);
                    } catch (except) {
                        wrongSoFar += 1;
                        var msg = 'Wrong geometry value';
                        if (except.message) msg += ' (' + except.message + ')';
                        if (wrongSoFar <= 10) {
                            self.trigger('recline:flash', {
                                message: msg,
                                category: 'error'
                            });
                        }
                    }
                } else {
                    wrongSoFar += 1;
                    if (wrongSoFar <= 10) {
                        self.trigger('recline:flash', {
                            message: 'Wrong geometry value',
                            category: 'error'
                        });
                    }
                }
                return true;
            });
        },

        // Private: Remove one or n features from the map
        //
        _remove: function (docs) {

            var self = this;

            if (!(docs instanceof Array)) docs = [docs];

            _.each(docs, function (doc) {
                for (var key in self.features._layers) {
                    if (self.features._layers[key].feature.properties.cid == doc.cid) {
                        self.features.removeLayer(self.features._layers[key]);
                    }
                }
            });

        },

        // Private: Return a GeoJSON geomtry extracted from the record fields
        //
        _getGeometryFromRecord: function (doc) {
            if (this.state.get('geomField')) {
                var value = doc.get(this.state.get('geomField'));
                if (typeof (value) === 'string') {
                    // We *may* have a GeoJSON string representation
                    try {
                        value = $.parseJSON(value);
                    } catch (e) { }
                }

                if (typeof (value) === 'string') {
                    value = value.replace('(', '').replace(')', '');
                    var parts = value.split(',');
                    var lat = parseFloat(parts[0]);
                    var lon = parseFloat(parts[1]);
                    if (!isNaN(lon) && !isNaN(parseFloat(lat))) {
                        return {
                            "type": "Point",
                            "coordinates": [lon, lat]
                        };
                    } else {
                        return null;
                    }
                } else if (value && _.isArray(value)) {
                    // [ lon, lat ]
                    return {
                        "type": "Point",
                        "coordinates": [value[0], value[1]]
                    };
                } else if (value && value.lat) {
                    // of form { lat: ..., lon: ...}
                    return {
                        "type": "Point",
                        "coordinates": [value.lon || value.lng, value.lat]
                    };
                }
                // We o/w assume that contents of the field are a valid GeoJSON object
                return value;
            } else if (this.state.get('lonField') && this.state.get('latField')) {
                // We'll create a GeoJSON like 3 object from the two lat/lon fields
                var lon = doc.get(this.state.get('lonField'));
                var lat = doc.get(this.state.get('latField'));
                if (!isNaN(parseFloat(lon)) && !isNaN(parseFloat(lat))) {
                    return {
                        type: 'Point',
                        coordinates: [lon, lat]
                    };
                }
            }
            return null;
        },

        // Private: Check if there is a field with GeoJSON geometries or alternatively,
        // two fields with lat/lon values.
        //
        // If not found, the user can define them via the UI form.
        _setupGeometryField: function () {
            // should not overwrite if we have already set this (e.g. explicitly via state)
            if (!this._geomReady()) {
                this.state.set({
                    geomField: this._checkField(this.geometryFieldNames),
                    latField: this._checkField(this.latitudeFieldNames),
                    lonField: this._checkField(this.longitudeFieldNames),
                    kpiField: this._getkpiField()
                });
                this.menu.state.set(this.state.toJSON());
            }
        },

        // Private: Check if a field in the current model exists in the provided
        // list of names.
        //
        //
        _checkField: function (fieldNames) {
            var field;
            var modelFieldNames = this.model.fields.pluck('id');
            for (var i = 0; i < fieldNames.length; i++) {
                for (var j = 0; j < modelFieldNames.length; j++) {
                    if (modelFieldNames[j].toLowerCase() == fieldNames[i].toLowerCase())
                        return modelFieldNames[j];
                }
            }
            return null;
        },
        // Private: get the default kpi field name
        _getkpiField: function () {
            if (this.state.get('kpiField')) return this.state.get('kpiField');
            var kpiFields = this.model.kpiFields.pluck('id');
            if (kpiFields.length > 0) { return kpiFields[0]; }
            return null;
        },

        // Private: Zoom to map to current features extent if any, or to the full
        // extent if none.
        //
        _zoomToFeatures: function () {
            var bounds = this.features.getBounds();
            if (bounds && bounds.getNorthEast() && bounds.getSouthWest()) {
                this.map.fitBounds(bounds);
            } else {
                this.map.setView([0, 0], 2);
            }
        },

        // Private: Sets up the Leaflet map control and the features layer.
        //
        // The map uses a base layer from [MapQuest](http://www.mapquest.com) based
        // on [OpenStreetMap](http://openstreetmap.org).
        //
        _setupMap: function () {
            var self = this;
            this.map = new L.Map(this.$map.get(0));
            var mapUrl = "https://{s}.tile.thunderforest.com/cycle/{z}/{x}/{y}.png";
            var osmAttribution = ''; // 'Map data &copy; 2011 OpenStreetMap contributors, Tiles Courtesy of <a href="http://www.mapquest.com/" target="_blank">MapQuest</a> <img src="http://developer.mapquest.com/content/osm/mq_logo.png">';
            var bg = new L.TileLayer(mapUrl, {
                maxZoom: 18,
                attribution: osmAttribution
            });

            // Assyst for google plugin(26-feb 2014 amritha)
            // Satellite Map
            if (typeof google === 'object' && typeof google.maps === 'object') {
                var ggl = new L.Google("HYBRID");
                this.map.addControl(new L.Control.Layers({ 'Map': bg, 'Satellite': ggl }, {}));
            }
            this.map.addLayer(bg, ggl);
            /******************* end***********************/

            this.markers = new L.MarkerClusterGroup(this._clusterOptions);

            // rebind this (as needed in e.g. default case above)
            this.geoJsonLayerOptions.pointToLayer = _.bind(
                this.geoJsonLayerOptions.pointToLayer,
                this);

            this.features = new L.GeoJSON(null, this.geoJsonLayerOptions);
            this.map.setView([0, 0], 2);
            this.mapReady = true;

            // Modified by Assyst (Rony at 06-Feb-2014)    
            // For map all color gradients display on right bottom
            /******************************************/
            this.legend = L.control({
                position: 'bottomright'
            });

            this.legend.onAdd = function (map) {
                this.legdiv = L.DomUtil.create('div', 'info legend');
                this.update();
                return this.legdiv;
            };

            this.legend.update = function () {
                grades = self._getKpiGrades();
                gradeColors = self._getKpiColors();

                var kpiField = self.state.get('kpiField');
                if(!kpiField) return;

                var scalesIn = parseInt(self._getKpiState("scalesIn")) || 1000;
                var remarks = self._getKpiState("note") || "Values in thousands";
                var roundup = Math.pow(10, self.model.fields.get(kpiField).get('decimalPoint'));

                var labels = [], from, to, lbfrom, lbto;

                var isInThous = (Math.max.apply(Math, grades) > scalesIn && grades[1] && grades[1] >= scalesIn);
                for (var i = 0; i < grades.length - 1; i++) {
                    from = grades[i];
                    to = grades[i + 1];
                    /*lbfrom = numberWithCommas(parseInt(isInThous ? from/scalesIn : from));
                    lbto = numberWithCommas(parseInt((isInThous &&  to) ? to / scalesIn :  to));*/

                    lbfrom = numberWithCommas(isInThous ? Math.round(from / scalesIn * roundup) / roundup : from);
                    lbto = numberWithCommas((isInThous && to) ? Math.round(to / scalesIn * roundup) / roundup : to);
                    /*lbfrom = numberWithCommas(from);
                    lbto = numberWithCommas( to );*/

                    //                    labels.push(
                    //                        '<div class= "div-legend"><i style="background:' + gradeColors[i] + '"></i> ' +//getColor(from + 1)
                    //                        lbfrom + (to ? '&nbsp;&ndash;&nbsp;' + lbto : '+')+'</div>');

                    labels.push(
                        '<div class= "div-legend"><i style="background:' + gradeColors[i] + '"></i> ' + //getColor(from + 1)
                        lbfrom + '&nbsp;&ndash;&nbsp;' + lbto + ((i == grades.length - 2) ? '+' : '&nbsp;') + '</div>');
                }

                if (isInThous) labels.push("<span style='color:#005580' class='_lngTrans_Translatable' defaultkey='" + remarks + "'>" + textTranslation(remarks) + "</span>");
                this.legdiv.innerHTML = labels.join('');
            };
            this.legend.addTo(this.map);
            /******************End*********************/
        },

        // Private: Helper function to select an option from a select list
        //
        _selectOption: function (id, value) {
            var options = $('.' + id + ' > select > option');
            if (options) {
                options.each(function (opt) {
                    if (this.value == value) {
                        $(this).attr('selected', 'selected');
                        return false;
                    }
                });
            }
        },
        _getKpiGrades: function () {
            var self = this;
            var gradeList = self._getKpiState("grades") || [];
            if (gradeList.length > 1) return gradeList.sort();

            var kpiField = self.state.get('kpiField');
            var kpiScale = self._getKpiState("scalesInView") || self._getKpiState("scales");
            var scalesIn = self._getKpiState("scalesIn");
            
            var field = self.model.fields.get(kpiField);

            var roundup = field ? Math.pow(10, field.get('decimalPoint')) : 2;
            var data = self.model.records.toJSON();

            var maxValue = parseInt(Math.max.apply(Math, data.map(function (o) { return o[kpiField] })));
            var minValue = parseInt(Math.min.apply(Math, data.map(function (o) { return o[kpiField] })));


            /*var modeValue = maxValue > 100 ? scalesIn : 100;
            var incrValue = ((maxValue % modeValue || maxValue == 0) ? (parseInt( maxValue / modeValue) +1) * modeValue : maxValue)/kpiScale;
             
            //var incrValue = Math.pow(10, String(maxValue).length) / kpiScale;
            for (var i = 0, range = 0; i <= kpiScale; i++) {
            gradeList.push((Math.round( range *roundup )/roundup));
            range = range + incrValue;
            }*/

            /*
            var recordCount = data.length;
            var isInThous = (minValue >= scalesIn)
            var incrValue = recordCount > 1 ? (maxValue - minValue)/(recordCount-1) : maxValue
            for (var i = 0, range = minValue; i < recordCount; i++) {   
            var value =  Math.round( range * roundup )/roundup;
            if(isInThous) {
            value = Math.round(value/scalesIn * roundup )/roundup;
            value = value * scalesIn;
            }
            gradeList.push(value);
            range = range + incrValue;
            }*/

            var isInThous = (minValue >= scalesIn)
            var incrValue = kpiScale > 1 ? (maxValue - minValue) / (kpiScale) : maxValue
            for (var i = 0, range = minValue; i <= kpiScale; i++) {
                var value = Math.round(range * roundup) / roundup;
                if (isInThous) {
                    value = Math.round(value / scalesIn * roundup) / roundup;
                    value = value * scalesIn;
                }
                gradeList.push(value);
                range = range + incrValue;
            }
            return gradeList;
        },

        _getKpiColors: function () {

            var self = this;
            var kpiHues = self._getKpiState("hues");
            var kpiHueL = kpiHues[0] || "#acd03e";
            var kpiHueH = kpiHues[1] || "#fdd24e";
            var kpiScale = (grades.length - 2) || 10;

            var huesL = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(kpiHueL);
            var huesH = /^#?([a-f\d]{2})([a-f\d]{2})([a-f\d]{2})$/i.exec(kpiHueH);

            var redL = parseInt(huesL[1], 16);
            var greenL = parseInt(huesL[2], 16);
            var blueL = parseInt(huesL[3], 16);

            var redH = parseInt(huesH[1], 16);
            var greenH = parseInt(huesH[2], 16);
            var blueH = parseInt(huesH[3], 16);

            var redV = (redH - redL) / kpiScale;
            var greenV = (greenH - greenL) / kpiScale;
            var blueV = (blueH - blueL) / kpiScale;

            var colors = [];
            for (var i = 0; i <= kpiScale; i++) {
                var hexColor = "#";
                if (i == 0) {
                    hexColor = kpiHueL;
                }
                else if (i == kpiScale) {
                    hexColor = kpiHueH;
                }
                else {
                    redL = redL + redV;
                    greenL = greenL + greenV;
                    blueL = blueL + blueV;
                    hexColor = "#" + componentToHex(redL) + componentToHex(greenL) + componentToHex(blueL);
                }
                colors.push(hexColor);
            }

            function componentToHex(c) {
                var hex = Math.round(c).toString(16);
                return hex.length == 1 ? "0" + hex : hex;
            }
            return colors;
        },

        _getKpiState: function (name) {
            var self = this;
            var kpiField = self.state.get('kpiField');
            var kpiRange = $.extend(true, {}, self.state.get('kpiRange'));
            var kpiSlabs = $.grep(self.state.get('kpiSlabs'), function (el, i) {
                return el.id == kpiField
            });

            var kpiSlab = (kpiSlabs && kpiSlabs.length >= 1) ? $.extend(true, {}, kpiSlabs[0]) : $.extend(true, {}, kpiSlabs);

            var value = kpiSlab[name] || kpiRange[name]
            if ($.isArray(value) && value.length == 0) value = kpiRange[name];
            return value;
        }
    });

    // Modified by Assyst (Rony at 06-Feb-2014)    
    // For map color gradients
    /******************************************/
    function getColor(d) {
        var lastIndex = 0;
        $.each(grades, function (idx, value) {
            if (d >= value) { lastIndex = idx; }
        });

        if (lastIndex >= gradeColors.length) lastIndex = gradeColors.length - 1;
        var color = gradeColors[lastIndex];
        return color
    }
    /***************End*********************/

    my.MapMenu = Backbone.View.extend({
        className: 'editor kpi',
        template: ' ',
       /* template: ' \
    <form class="form-stacked"> \
      <div class="clearfix" style="display:none"> \
        <div class="editor-field-type"> \
            <label class="radio _lngTrans_Translatable"> \
              <input type="radio" id="editor-field-type-latlon" name="editor-field-type" value="latlon" checked="checked"/> \
              Latitude / Longitude fields</label> \
            <label class="radio _lngTrans_Translatable"> \
              <input type="radio" id="editor-field-type-geom" name="editor-field-type" value="geom" /> \
              GeoJSON field</label> \
        </div> \
        <div class="editor-field-type-latlon"> \
          <label class="_lngTrans_Translatable">Latitude field</label> \
          <div class="input editor-lat-field"> \
            <select> \
            <option value=""></option> \
            {{#geoFields}} \
            <option value="{{id}}" class="_lngTrans_Translatable">{{name}}</option> \
            {{/geoFields}} \
            </select> \
          </div> \
          <label class="_lngTrans_Translatable">Longitude field</label> \
          <div class="input editor-lon-field"> \
            <select> \
            <option value=""></option> \
            {{#geoFields}} \
            <option value="{{id}}" class="_lngTrans_Translatable">{{name}}</option> \
            {{/geoFields}} \
            </select> \
          </div> \
        </div> \
        <div class="editor-field-type-geom" style="display:none"> \
          <label class="_lngTrans_Translatable">Geometry field (GeoJSON)</label> \
          <div class="input editor-geom-field"> \
            <select> \
            <option value=""></option> \
            {{#geoFields}} \
            <option value="{{id}}" class="_lngTrans_Translatable">{{name}}</option> \
            {{/geoFields}} \
            </select> \
          </div> \
        </div> \
      <div class="editor-buttons"> \
        <button type="submit" class="ui-widget ui-state-default ui-corner-all ui-button-text-only ui-btn-general editor-update-map"><span class="ui-button-text _lngTrans_Translatable" title="">Update</span></button> \
      </div> \
      </div> \
      <label title="Key Performance Indicator" class="kpiLabel" class="_lngTrans_Translatable">Key Performance Indicator</label> \
      <span class="kpiSlideDown updownkpi"></span>\
      <div class="kpiDiv">\
          <div class="editor-field-type-kpi" > \
              <div class="input editor-kpi-field"> \
                <select class ="editor-select-kpi"> \
                {{#kpiFields}} \
                <option value="{{id}}" class="_lngTrans_Translatable">{{name}}</option> \
                {{/kpiFields}} \
                </select> \
              </div> \
           </div> \
           <div class="color-picker"> \
                <span class="_lngTrans_Translatable">KPI hues: </span> \
                <span class="color-box hue-start"></span> \
                <span class="_lngTrans_Translatable">-</span> \
                <span class="color-box hue-end"></span> \
                <span class="hue-reset _lngTrans_Translatable">reset</span> \
           </div> \
           <div class="color-slabs editor-scale-field">\
                <label class="_lngTrans_Translatable">Slabs:</label> \
                <select class="select-scale"></select>\
           </div>\
          <div class="editor-options" > \
            <label class="checkbox"> \
              <input type="checkbox" id="editor-auto-zoom" value="autozoom" checked="checked" /> \
              <span class="_lngTrans_Translatable">Auto zoom to features</span></label> \
            <label class="checkbox"  style="display:none;"> \
              <input type="checkbox" id="editor-cluster" value="cluster"/> \
               <span class="_lngTrans_Translatable">Cluster markers</span></label> \
          </div> \
      </div>\
      <input type="hidden" class="editor-id" value="map-1" /> \
    </form> \
  ',*/

        // Define here events for UI elements
        events: {
            'click .editor-update-map': 'onEditorSubmit',
            'change .editor-select-kpi': 'onkpiFieldSubmit',
            'change .editor-field-type': 'onFieldTypeChange',
            'click #editor-auto-zoom': 'onAutoZoomChange',
            'click .updownkpi': 'onHideShowkpi',
            'click #editor-cluster': 'onClusteringChange',
            'change .select-scale': 'onSelectScaleChange'
        },

        initialize: function (options) {
            var self = this;
            _.bindAll(this, 'render');
            this.listenTo(this.model.geoFields, 'change', this.render);
            this.state = new recline.Model.ObjectState(options.state);
            this.listenTo(this.state, 'change', this.render);
            this.render();
        },

        // ### Public: Adds the necessary elements to the page.
        //
        // Also sets up the editor fields and the map if necessary.
        render: function () {
            var self = this;
            var htmls = Mustache.render(this.template, this.model.toTemplateJSON());
            this.$el.html(htmls);

            if (this._geomReady() && this.model.fields.length) {
                if (this.state.get('geomField')) {
                    this._selectOption('editor-geom-field', this.state.get('geomField'));
                    this.$el.find('#editor-field-type-geom').attr('checked', 'checked').change();
                } else {
                    this._selectOption('editor-lon-field', this.state.get('lonField'));
                    this._selectOption('editor-lat-field', this.state.get('latField'));
                    this.$el.find('#editor-field-type-latlon').attr('checked', 'checked').change();
                }
                if (this.state.get('kpiField')) {
                    this._selectOption('editor-kpi-field', this.state.get('kpiField'));
                }
            }
            if (this.state.get('autoZoom')) {
                this.$el.find('#editor-auto-zoom').attr('checked', 'checked');
            } else {
                this.$el.find('#editor-auto-zoom').removeAttr('checked');
            }
            if (this.state.get('cluster')) {
                this.$el.find('#editor-cluster').attr('checked', 'checked');
            } else {
                this.$el.find('#editor-cluster').removeAttr('checked');
            }

            self._huesHandler();
            self._scaleHandler();
            return this;
        },
        //nithin
        _scaleHandler: function () {
            var self = this;
            var kpiScale = self._getKpiState("scales") || 10;
            var dataCount = self.model.records.toJSON().length;
            var slabs = dataCount > kpiScale ? dataCount : kpiScale;

            var scalesInView = self._getKpiState("scalesInView") || self._getKpiState("scales");

            var slabSelect = self.$el.find('.select-scale');
            for (var i = 2; i <= slabs; i++) {
                slabSelect.append('<option value=' + i + '>' + i + '</option>');
            }
            this._selectOption('editor-scale-field', scalesInView);
        },

        _huesHandler: function () {
            var self = this;
            var hues = self._getKpiState('hues');

            self.$el.find('.hue-start').colpick({
                colorScheme: '#E6E6E8',
                layout: 'rgbhex',
                color: hues[0],
                onSubmit: function (hsb, hex, rgb, el) {
                    $(el).css('background-color', '#' + hex).attr("color", '#' + hex);
                    $(el).colpickHide();
                    setHues(['#' + hex, self.$el.find('.hue-end').attr("color")]);
                }
            }).css('background-color', hues[0]).attr("color", hues[0]);

            self.$el.find('.hue-end').colpick({
                colorScheme: '#E6E6E8',
                layout: 'rgbhex',
                color: hues[1],
                onSubmit: function (hsb, hex, rgb, el) {
                    $(el).css('background-color', '#' + hex).attr('color', '#' + hex);
                    $(el).colpickHide();
                    setHues([self.$el.find('.hue-start').attr('color'), '#' + hex])
                }
            }).css('background-color', hues[1]).attr("color", hues[1]);


            var oldHues = self._getKpiState('oldHues');
            var resetBtn = self.$el.find('.hue-reset');
            if (oldHues) resetBtn.show().click(function () {
                resetHues(oldHues);
            });
            else resetBtn.hide();

            function setHues(color) {
                var kpiField = self.state.get('kpiField');
                var defaultHues = $.extend(true, {}, self.state.get('kpiRange'))['hues'];
                var slabs = $.extend(true, [], self.state.get('kpiSlabs'));
                var fslab = $.grep(slabs, function (el, i) { return el.id == kpiField });
                if (!fslab || fslab.length == 0) {
                    slabs.push({
                        "id": kpiField,
                        "hues": color,
                        "oldHues": defaultHues
                    });
                }
                else {
                    if (!fslab[0]["oldHues"]) { fslab[0]["oldHues"] = fslab[0]["hues"] };
                    fslab[0]["hues"] = color;
                }

                self.state.set({ kpiSlabs: slabs });
                return false;
            }

            function resetHues(color) {
                var kpiField = self.state.get('kpiField');
                var slabs = $.extend(true, [], self.state.get('kpiSlabs'));
                var fslab = $.grep(slabs, function (el, i) { return el.id == kpiField });
                if (!fslab || fslab.length == 0) return;

                if (fslab[0]["oldHues"]) { delete fslab[0]["oldHues"]; };
                fslab[0]["hues"] = color;
                self.state.set({ kpiSlabs: slabs });
                return false;
            }
        },

        _geomReady: function () {
            return Boolean(this.state.get('geomField') || (this.state.get('latField') && this.state.get('lonField')));
        },

        onHideShowkpi: function () {
            $('.kpiDiv').slideToggle('fast',function(){
                if($el.find('.updownkpi').hasClass('kpiSlideDown')) $el.find('.updownkpi').addClass('kpiSlideUp').removeClass('kpiSlideDown');
                else $el.find('.updownkpi').addClass('kpiSlideDown').removeClass('kpiSlideUp');
            });
        },

        _getKpiState: function (name) {
            var self = this;
            var kpiField = self.state.get('kpiField');
            var kpiRange = $.extend(true, {}, self.state.get('kpiRange'));
            var kpiSlabs = $.grep(self.state.get('kpiSlabs'), function (el, i) {
                return el.id == kpiField
            });

            var kpiSlab = (kpiSlabs && kpiSlabs.length >= 1) ? $.extend(true, {}, kpiSlabs[0]) : $.extend(true, {}, kpiSlabs);

            var value = kpiSlab[name] || kpiRange[name]
            if ($.isArray(value) && value.length == 0) value = kpiRange[name];
            return value;
        },
        // ## UI Event handlers
        //

        // Public: Update map with user options
        //
        // Right now the only configurable option is what field(s) contains the
        // location information.
        //
        onSelectScaleChange: function () {
            var self = this;
            var scale = this.$el.find('.select-scale > option:selected').val() * 1;
            var kpiField = self.state.get('kpiField');
            var slabs = $.extend(true, [], self.state.get('kpiSlabs'));
            var fslab = $.grep(slabs, function (el, i) { return el.id == kpiField });
            if (!fslab || fslab.length == 0) {
                slabs.push({
                    "id": kpiField,
                    "scalesInView": scale
                });
            }
            else {
                fslab[0]["scalesInView"] = scale;
            }
            self.state.set({ kpiSlabs: slabs });
        },

        onEditorSubmit: function (e) {
       
            e.preventDefault();
            if (this.$el.find('#editor-field-type-geom').attr('checked')) {
                this.state.set({
                    geomField: this.$el.find('.editor-geom-field > select > option:selected').val(),
                    lonField: null,
                    latField: null
                });
            } else {
                this.state.set({
                    geomField: null,
                    lonField: this.$el.find('.editor-lon-field > select > option:selected').val(),
                    latField: this.$el.find('.editor-lat-field > select > option:selected').val()
                });
            }
            return false;
        },

        // Public: Update map with user options
        //        
        onkpiFieldSubmit: function (e) {
            e.preventDefault();
            this.state.set({
                kpiField: this.$el.find('.editor-kpi-field > select > option:selected').val()
            });
            return false;
        },
        // Public: Shows the relevant select lists depending on the location field
        // type selected.
        //
        onFieldTypeChange: function (e) {
            if (e.target.value == 'geom') {
                this.$el.find('.editor-field-type-geom').show();
                this.$el.find('.editor-field-type-latlon').hide();
            } else {
                this.$el.find('.editor-field-type-geom').hide();
                this.$el.find('.editor-field-type-latlon').show();
            }
        },

        onAutoZoomChange: function (e) {
            this.state.set({
                autoZoom: !this.state.get('autoZoom')
            });
        },

        onClusteringChange: function (e) {
            this.state.set({
                cluster: !this.state.get('cluster')
            });
        },

        // Private: Helper function to select an option from a select list
        //
        _selectOption: function (id, value) {
            var options = this.$el.find('.' + id + ' > select > option');
            if (options) {
                options.each(function (opt) {
                    if (this.value == value) {
                        $(this).attr('selected', 'selected');
                        return false;
                    }
                });
            }
        }
    });

})(jQuery, recline.View);


