/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   
JQuery Immediately-Invoked Function for recline NVD3 Graph
Created By      : Rony T S for Assyst International Pvt. Ltd.
Created Date    : 10-Oct-2014
Version         : 1.0.2
Project         : PanAgro

+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/
this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";
    // ## Graph view for a Dataset using Flot graphing library.
    //
    // Initialization arguments (in a hash in first parameter):
    //
    // * model: recline.Model.Dataset
    // * state: (optional) configuration hash of form:
    //
    //        {
    //          group: {column name for x-axis},
    //          series: [{column name for series A}, {column name series B}, ... ],
    //          // options are: lines, points, lines-and-points, bars, columns
    //          graphType: 'lines',
    //          graphOptions: {custom [flot options]}
    //        }
    //
    // NB: should *not* provide an el argument to the view but must let the view
    // generate the element itself (you can then append view.el to the DOM.

    my.NVD3Graph = Backbone.View.extend({
        template: ' \
            <div class="recline-flot"> \
                <div id="nvd3chart_{{viewId}}" class="panel graph" > \
                    {{#controls}}<div title="{{title}}" class="dashContent dashControl {{controlClass}}"></div>{{/controls}}\
                    <div title="Show legend" class="dashContent dashLegend showDashLegend"></div>\
                    <svg  preserveAspectRatio="none"></svg>\
                </div>\
                <div id="nvd3chartData_{{viewId}}" style="width:100%;margin-top: 15px;float: left;"></div>\
                <div class="dashContentLegend"></div>\
            </div>\
        ',
        events: {
            'click .btnLegend': 'onLegendClick',
            'click .dashLegend': 'onDashLegendClick',
            'click .dashControl': 'onDashControlClick'
        },

        initialize: function (options) {
            var self = this;
            //colors for the chart bar.
            this.graphColors = ["#429CD6", "#EBBBD7", "#A1D185", "#F6EB13", "#46C3D1", "#D779B1", "#BCBE32", "#9391C7", "#B2DAF1", "#FFC616"];
            // generating an unique id for the chart
            this.uid = options.id || ("" + new Date().getTime() + Math.floor(Math.random() * 10000));
            this.el = $(this.el);
            _.bindAll(this, 'render', 'redraw');
            this.needToRedraw = true;

            this.listenTo(this.model, 'change', function () {
                this.render();
                this.redraw();
            });

            this.listenTo(this.model.fields, 'reset add', function () {
                this.render();
                this.redraw();
            });

            this.listenTo(this.model.records, 'reset add', function () {
                this.render();
                this.redraw();
            });

            // graph types used
            this.graphTypes = [{
                "key": "multiBar",
                "name": "Multi Bar",
                "xstring": true,
                "sliced": "Multi Bar"
            },
                {
                    "key": "multiBarHorizontal",
                    "name": "Multi Bar Horizontal",
                    "xstring": true,
                    "sliced": "Multi Bar Horizontal"
                },
                {
                    "key": "line",
                    "name": "Line",
                    "xstring": false,
                    "sliced": "Line"
                },
                {
                    "key": "scatter",
                    "name": "Scatter",
                    "xstring": false,
                    "sliced": "Scatter"
                },
                {
                    "key": "scatterPlusLine",
                    "name": "Scatter Plus Line",
                    "xstring": false,
                    "sliced": "Scatter Plus Line"
                },
                {
                    "key": "stackedArea",
                    "name": "Stacked Area",
                    "xstring": false,
                    "sliced": "Stacked Area"
                },
                {
                    "key": "pie",
                    "name": "Pie",
                    "xstring": true,
                    "sliced": "Pie"
                },
                {
                    "key": "donut",
                    "name": "Donut",
                    "xstring": true,
                    "sliced": "Donut"
                }
            ];

            if (options.state.graphTypes && options.state.graphTypes.length > 0) {
                this.graphTypes = $.grep(self.graphTypes, function (value, index) {
                    return ($.inArray(value.key, options.state.graphTypes) > -1);
                });
            }
            if (typeof _.findWhere(this.graphTypes, {
                key: options.state.graphType
            }) === 'undefined') options.state.graphType = "multiBar";
            if (typeof _.findWhere(this.model.toTemplateJSON().fields, {
                id: options.state.group
            }) === 'undefined') {
                var tmplData = this.model.toTemplateJSON();
            }
            var stateData = _.extend({
                group: null,
                // so that at least one series chooser box shows up
                series: [],
                area: [],
                bar: [],
                graphType: 'multiBar',
                seriesType: "multiselect", //[multiselect, singleselect]    
                pivotView: "editor", //[popup, editor]
                showData: true,
                showLegent: self.options.state.enableLegend,
                showControl: true,
                dashboard: true,
                enableLegend: true,
                minLegend: 10,
                dataConvert: {
                    type: 'none',
                    rowField: undefined,
                    columnField: undefined,
                    dataField: undefined,
                    reverse: false,
                    transpose: false
                },
                graphState: undefined
            }, options.state);
            this.state = new recline.Model.ObjectState(stateData);

            this.editor = new my.NVD3Controls({
                model: this.model,
                state: this.state.toJSON(),
                graphTypes: this.graphTypes
            });

            this.listenTo(this.editor.state, 'change', function () {
                var state = $.extend({}, self.editor.state.toJSON(), {
                    showLegent: self.state.get('showLegent')
                });
                self.state.set(state);
            });
            this.listenTo(this.state, 'change', function () {
                self.render();
                self.redraw();
            });

            this.elSidebar = this.editor.$el;
        },

        render: function () {
            var self = this;
            var tmplData = this.model.toTemplateJSON();
            tmplData["viewId"] = this.uid;
            tmplData["enableLegend"] = self.state.get('enableLegend');
            var controls;
            if (self.state.get('dataConvert').type == "pivot" || self.state.get('dataConvert').transpose == true) {
                if (self.state.get('graphType') == "multiBarHorizontal" || self.state.get('graphType') == "multiBar") {
                    controls = true;
                } else controls = false;
            } else {
                if (self.state.get('graphType') == "multiBarHorizontal" || self.state.get('graphType') == "multiBar") {
                    controls = self.state.get('series').length <= 1 ? false : self.state.get('dashboard');
                } else controls = false;
            }

            if (controls) {
                tmplData["controls"] = {
                    show: this.state.attributes.graphState == "Stacked" ? true : false,
                    title: this.state.attributes.graphState == "Stacked" ? "Grouped" : "Stacked",
                    controlClass: this.state.attributes.graphState == "Stacked" ? "showDashControl" : "hideDashControl"
                };
            }
            if (this.state.attributes.width)
                tmplData.width = this.state.attributes.width;

            if (this.state.attributes.height)
                tmplData.height = this.state.attributes.height;

            var htmls = Mustache.render(this.template, tmplData);
            this.$el.empty();
            this.$el.html(htmls);
            this.$graph = this.$el.find('.panel.graph');
            // Dashlet Viewer height adjustment
            var parent = this.$el.parents('.content_div');
            if (parent.context != undefined) {
                this.$graph.height(parent.height());
            } else this.$graph.height(parent.height());
            //end
            if (this.state.get("graphType") == "discreteBar") this.$el.find('.btnLegend').hide();
            else if (!self.state.get('showLegent')) {
                this.$el.find('.btnLegend').show().addClass('hideLegend').removeClass('showLegend').attr("title", "Hide legend");
                this.$el.find('.dashLegend').show().addClass('hideDashLegend').removeClass('showDashLegend').attr("title", "Hide legend");
            } else {
                this.$el.find('.btnLegend').show().addClass('showLegend').removeClass('hideLegend').attr("title", "Show legend");
                this.$el.find('.btnLegend').show().addClass('showDashLegend').removeClass('hideDashLegend').attr("title", "Show legend");
            }
            if (this.state.get("graphType") == "stackedArea") {
                this.$el.find('.dashControl').hide();
            } else {
                this.$el.find('.dashControl').show();
            }
	     $('.modal-content:visible').find('.recline-flot').height($('.modal-content:visible').height() - 100); 
            return this;
        },

        onLegendClick: function (e) {
            var self = this;
            var element = $(e.currentTarget);
            self.state.set({
                showLegent: element.hasClass("showLegend")
            });
        },

        onDashLegendClick: function (e) {
            var self = this;
            var element = $(e.currentTarget);
            self.state.set({
                showLegent: element.hasClass("hideDashLegend")
            });
        },

        onDashControlClick: function (e) {
            var self = this;
            var element = $(e.currentTarget);
            this.state.attributes.graphState = element.hasClass("showDashControl") ? "Grouped" : "Stacked";
            self.state.trigger('change');
        },

        remove: function () {
            this.editor.remove();
            Backbone.View.prototype.remove.apply(this, arguments);
        },

        redraw: function () {
            var self = this;
            // There are issues generating a Flot graph if either:
            // * The relevant div that graph attaches to his hidden at the moment of creating the plot -- Flot will complain with
            //   Uncaught Invalid dimensions for plot, width = 0, height = 0
            // * There is no data for the plot -- either same error or may have issues later with errors like 'non-existent node-value'
            var axisFields = this.model.axisFields.toJSON();
            var dataFields = $.grep(axisFields, function (val, i) {
                if (val.type == "float" || val.type == "int" || val.type == "numeric" || val.type == "number") return true;
            });
            //giving vertical scroll for pivot section
            if(self.state.attributes.dataConvert.type !="none" || self.state.attributes.dataConvert.transpose){
                $('.data-view-container-wrap').addClass('pivot-active');
                var pivotscreen;
                if($('.modal-content').length>0){
                    pivotscreen =   $('.modal-content:visible').height() - 100;
                }
                else {
                    pivotscreen = $(document).height() - 250;
                }
                $(".data-view-container-wrap.pivot-active").css({
                    'height': pivotscreen + 'px'
                });
            }

            // check we have something to plot
            if (this.state.get('group') && this.state.get('series')) {
                var graphType = this.state.get("graphType");
                if(graphType === "pie" || graphType === "donut")
                    self.state.get('dataConvert').transpose = false;
                var series = (graphType === "pie" || graphType === "donut") ? this.createPieSeries() : this.createSeries();
                self.series = series;
                nv.addGraph(function () {
                    var chart = self._getNVD3Graph(graphType, self.state.get('showLegent'));
                    if (self.series && self.series.length == 1 && (graphType != "pie" && graphType != "donut" && graphType != "linePlusBar")) {
                        chart.yAxis.axisLabel(series[0].key);
                    }
                    if (self.state.attributes.dataConvert.type != "pivot") {
                        if ($('#btnDataViewerSaveAs').length > 0) {
                            self.state.get('series').length == 0 ? $('#btnDataViewerSaveAs').addClass("disableClass") : $('#btnDataViewerSaveAs').removeClass("disableClass");
                        } else {
                            var $saveButton = $('#nvd3chart_' + self.uid + ' svg').parents('.recline-data-explorer').find('.header .menu-right .dashlet,.header .menu-right .exprt_btn');
                            self.state.get('series').length == 0 ? $saveButton.addClass("disableClass") : $saveButton.removeClass("disableClass");
                        }
                    }

                    $('#nvd3chart_' + self.uid + ' svg').empty().append('<rect width="100%" height="100%" fill="#fff"></rect>');
                    if(series){
                        d3.select('#nvd3chart_' + self.uid + ' svg')
                            .datum(series)
                            .transition().duration(500)
                            .call(chart);
                    }
                    else {
                        d3.select('#nvd3chart_' + self.uid + ' svg')
                            .datum([])
                            .transition().duration(500)
                            .call(chart);
                    }
                    if (graphType == "multiBarHorizontal") {
                        d3.select(".nv-y>g>g>.nv-axislabel").attr("x", "40%");
                    }

                    if (self.state.get('series').length <= 1) {
                        $('.nv-controlsWrap .nv-series.disabled').remove();
                    }

                    nv.utils.windowResize(chart.update);
                    if (graphType == 'pie' || graphType == 'donut') {
                        d3.selectAll('rect').each(function (d, i) {
                            var group = d3.select(this);
                            group.style('stroke', '')
                                .style('fill', '')
                                .attr("rx", 0)
                                .attr("ry", 0);
                        });
                    }

                    //xvalue tooltip should show the exact value
                    d3.selectAll('title.titleClass').each(function (d, i) {
                        //pie and donut should not have xvalues.
                        if (graphType != 'pie' && graphType != 'donut') {
                            var group = d3.select(this);

                            d = self.xaxisLabels[group.text()];
                            if (d)
                                group.text(d);
                        }
                    });         

                    return chart;
                });
            }
        },

        show: function () {
            // because we cannot redraw when hidden we may need to when becoming visible
            if (this.needToRedraw) {
                this.redraw();
            }
        },

        _getNVD3Graph: function (type, showLegend) {
            var self = this;
            var chart;
            switch (type) {
                case "multiBar":
                    chart = nv.models.multiBarChart();
                    if (self.xvaluesAreString) {
                        chart.reduceXTicks(false);
                        if (this.state.attributes.graphState == "Stacked") chart.stacked(true);
                        else chart.stacked(false);
                        chart.groupSpacing(0.08);
                        if (self.state.get("dashboard")) {
                            if (self.state.get('showLegent')) {
                                chart.margin({
                                    top: 10,
                                    right: 50,
                                    bottom: 80,
                                    left: 30
                                });
                                chart.legend.margin({
                                    top: 0,
                                    right: 0,
                                    bottom: 0,
                                    left: 0
                                });
                            } else {
                                chart.margin({
                                    top: 10,
                                    right: 50,
                                    bottom: 80,
                                    left: 30
                                });
                            }
                        } else chart.margin({
                            top: 10,
                            right: 50,
                            bottom: 80,
                            left: 80
                        });
                        //chart.yAxis.tickFormat(d3.format(',.0f'));
                        chart.gridRender(function (key) {
                            self.createSelectedItemGrid(key.point);
                        });
                        chart.xAxis.rotateLabels(-35);
                    }
                    break;
                case "multiBarHorizontal":
                    chart = nv.models.multiBarHorizontalChart();
                    chart.showValues(true);
                    if (this.state.attributes.graphState == "Stacked") chart.stacked(true);
                    else chart.stacked(false);
                    chart.gridRender(function (key) {
                        self.createSelectedItemGrid(key.point);
                    });
                    if (self.state.get("dashboard")) {
                        if (self.state.get('showLegent')) {
                            chart.margin({
                                top: 20,
                                right: 50,
                                bottom: 60,
                                left: 60
                            });
                            chart.legend.margin({
                                top: 0,
                                right: 0,
                                bottom: 0,
                                left: 0
                            });
                        } else {
                            chart.margin({
                                top: 20,
                                right: 50,
                                bottom: 60,
                                left: 60
                            });
                        }
                    } else chart.margin({
                        top: 20,
                        right: 50,
                        bottom: 60,
                        left: 80
                    });
                    chart.tooltips(true);
                    break;
                case "discreteBar":
                    chart = nv.models.discreteBarChart();
                    chart.tooltips(true);
                    chart.showValues(true);
                    chart.transitionDuration(350);
                    //chart.yAxis.tickFormat(d3.format(',.0f'));
                    chart.xAxis.rotateLabels(-35);
                    break;
                case "line":
                    chart = nv.models.lineChart();
                    // chart.yAxis.tickFormat(d3.format(',.0f'));
                    chart.gridRender(function (key) {
                        self.createSelectedItemGrid(key.point);
                    });
                    chart.xAxis.rotateLabels(-35);
                    if (self.state.get("dashboard")) {
                        if (self.state.get('showLegent')) {
                            chart.margin({
                                top: 10,
                                right: 50,
                                bottom: 80,
                                left: 30
                            });
                            chart.legend.margin({
                                top: 0,
                                right: 0,
                                bottom: 0,
                                left: 0
                            });
                        } else {
                            chart.margin({
                                top: 30,
                                right: 50,
                                bottom: 80,
                                left: 30
                            });
                        }
                    } else chart.margin({
                        top: 10,
                        right: 50,
                        bottom: 80,
                        left: 80
                    });                    
                    chart.xAxis.showMaxMin(false);
                    break;
                case "linePlusBar":
                    chart = nv.models.linePlusBarChart();
                    chart.bars.forceY([0]);
                    chart.xAxis.showMaxMin(false);
                    chart.xAxis.rotateLabels(-35);
                    break;
                case "cumulativeLine":
                    chart = nv.models.cumulativeLineChart();
                    if (this.state.attributes.graphState == "Re-scale y-axis") chart.rescaleY(false);
                    else chart.rescaleY(true);
                    chart.useInteractiveGuideline(true);
                    //chart.yAxis.tickFormat(d3.format(',.0f'));
                    chart.xAxis.rotateLabels(-35);
                    break;
                case "lineWithFocus":
                    chart = nv.models.lineWithFocusChart();
                    //chart.yAxis.tickFormat(d3.format(',.0f'));
                    chart.height(455);
                    // chart.y2Axis.tickFormat(d3.format(',.0f'));
                    chart.xAxis.rotateLabels(-35);
                    chart.x2Axis.rotateLabels(-35);
                    break;
                case "scatter":
                    chart = nv.models.scatterChart();
                    chart.showDistX(true);
                    chart.showDistY(true);
                    //chart.yAxis.tickFormat(d3.format(',.0f'));
                    chart.gridRender(function (key) {
                        self.createSelectedItemGrid(key.point);
                    });
                    if (self.state.get('showLegent')) {
                        chart.margin({
                            top: 10,
                            right: 50,
                            bottom: 80,
                            left: 30
                        });
                        chart.legend.margin({
                            top: 0,
                            right: 0,
                            bottom: 0,
                            left: 0
                        });
                    } else {
                        chart.margin({
                            top: 10,
                            right: 50,
                            bottom: 80,
                            left: 30
                        });
                    }
                    chart.xAxis.rotateLabels(-35);
                    chart.xAxis.showMaxMin(false);
                    break;
                case "scatterPlusLine":
                    chart = nv.models.scatterPlusLineChart();
                    chart.showDistX(true);
                    chart.showDistY(true);
                    //chart.yAxis.tickFormat(d3.format(',.0f'));
                    chart.gridRender(function (key) {
                        self.createSelectedItemGrid(key.point);
                    });
                    if (self.state.get('showLegent')) {
                        chart.margin({
                            top: 10,
                            right: 50,
                            bottom: 80,
                            left: 30
                        });
                        chart.legend.margin({
                            top: 0,
                            right: 0,
                            bottom: 0,
                            left: 0
                        });
                    } else {
                        chart.margin({
                            top: 10,
                            right: 50,
                            bottom: 80,
                            left: 30
                        });
                    }
                    chart.xAxis.rotateLabels(-35);
                    chart.xAxis.showMaxMin(false);
                    break;
                case "stackedArea":
                    chart = nv.models.stackedAreaChart();
                    chart.clipEdge(true);
                    if (this.state.attributes.graphState == undefined) chart.style("stack");
                    else if (this.state.attributes.graphState == "Stacked") chart.style("stack");
                    else if (this.state.attributes.graphState == "Stream") chart.style("stream");
                    else chart.style("expand");
                    if (self.state.get("dashboard")) {
                        if (self.state.get('showLegent')) {
                            chart.margin({
                                top: 10,
                                right: 50,
                                bottom: 80,
                                left: 30
                            });
                            chart.legend.margin({
                                top: 0,
                                right: 0,
                                bottom: 0,
                                left: 0
                            });
                        } else {
                            chart.margin({
                                top: 10,
                                right: 50,
                                bottom: 80,
                                left: 30
                            });
                        }
                    } else chart.margin({
                        top: 10,
                        right: 50,
                        bottom: 80,
                        left: 80
                    });
                    chart.yAxis.width(50);
                    chart.xAxis.rotateLabels(-35);
                    chart.xAxis.showMaxMin(false);
                    break;
                case "pie":
                    chart = nv.models.pieChart();
                    chart.x(function (d) {
                        if (self._groupFieldIsDateTime())
                            return d3.time.format('%x')(new Date(d.x));
                        else
                            return d.x;
                    });
                    chart.showLabels(true);
                    chart.color(d3.scale.ordinal().range(this.graphColors).range());
                    chart.margin({
                        top: 0,
                        right: 0,
                        bottom: 0,
                        left: 30
                    });
                    chart.legend.margin({
                        left: 10
                    });
                    chart.gridRender(function (key) {
                        self.createSelectedItemGrid(key.point);
                    });
                    chart.labelType("percent");
                    chart.pie.pieLabelsOutside(false);
                    //chart.labelThreshold(.05);
                    break;
                case "donut":
                    chart = nv.models.pieChart();
                    chart.x(function (d) {
                        if (self._groupFieldIsDateTime())
                            return d3.time.format('%x')(new Date(d.x));
                        else
                            return d.x;
                    });
                    chart.gridRender(function (key) {
                        self.createSelectedItemGrid(key.point);
                    });
                    chart.showLabels(true);
                    chart.donut(true);
                    chart.legend.margin({
                        left: 10
                    });
                    chart.color(d3.scale.ordinal().range(this.graphColors).range());
                    chart.donutRatio(0.35);
                    //chart.labelThreshold(.05);
                    chart.labelType("percent");
                    // chart.showValues(true);
                    break;
            }

            chart.tooltipContent(function (key, y, e, graph) {
                return self.toolTipContent(type, key, y, e, graph);
            });

            //chart.color(d3.scale.category10().range());
            if (type != "pie" && type != 'donut') {
                chart.yAxis.tickFormat(d3.format('.2s'));
                chart.yAxis.width(50);
                chart.color(d3.scale.ordinal().range(["#429CD6", "#EBBBD7", "#A1D185", "#F6EB13", "#46C3D1", "#D779B1", "#BCBE32", "#9391C7", "#B2DAF1", "#FFC616"]).range());

                var xName = (this.state.attributes.dataConvert.rowField == undefined) ?
                    _.findWhere(self.model.get('fields'), {
                        id: self.state.attributes.group
                    }).name :
                    this.model.fields.get(this.state.attributes.dataConvert.rowField).get('name');

                chart.xAxis.axisLabel(xName);
            }

            if (self.xvaluesAreString) {
                if (type != "pie" && type != 'donut') {
                    chart.xAxis.tickFormat(function (d, idx) {
                        if (self.state.get('dataConvert').reverse && type != "multiBar" && type != "multiBarHorizontal") {
                            self.xaxisLabels.reverse();
                        }
                        var label = self.xvaluesAreIndex ? self.xaxisLabels[d] : d;
                        if (typeof idx != "undefined" && typeof label != "undefined") {
                            if (self.el.parents('.content_div').length > 0) {
                                if ($.trim(label).length > 2) label = $.trim(label).slice(0, 6) + '..';
                            } else {
                                if ($.trim(label).length > 8) label = $.trim(label).slice(0, 8) + '..';
                            }
                        }
                        return label;
                    });
                }
            } else if (self._groupFieldIsDateTime()) {
                if (type != "pie" && type != 'donut') {
                    chart.xAxis.showMaxMin(false).tickFormat(function (d) {
                        return d;
                    });
                    chart.x(function (d) {
                        return d.x;
                    });
                }
            }

            if (type != "discreteBar") {
                var legent = showLegend ? true : self.state.get('showLegent');
                chart.showLegend(legent);
            }
            if (type != "line" && type != "pie" && type != 'donut' && type != "linePlusBar" && type != "scatter" && type != "scatterPlusLine" && type != "stackedArea") {
                chart.showControls(false);
            }
            if (showLegend) {
                self.$el.find('.btnLegend').hide()
            } else this.$el.find('.btnLegend').show();
            if (type == "pie" || type == 'donut') {
                $(self.editor.el).find('.labelsDiv').hide();
            } else $(self.editor.el).find('.labelsDiv').show();
            return chart;
        },

        toolTipContent: function (type, key, y, e, graph) {
            var self = this;
            var item;
            var value;

            if (self.state.get('dataConvert').type == "pivot") {
                if (type == "pie" || type == 'donut') {
                    item = _.findWhere(self.model.get('fields'), {
                        id: self.state.get('dataConvert').dataField
                    }).format;
                    value = e.value;
                    if (item && item.datatype == "Percent") {
                        return '<h3>' + key + '</h3><p>' + _appFun.numberFormatFunction(item, value) + '</p>';
                    } else {
                        return '<h3>' + key + '</h3><p>' + e.value + '</p>';
                    }
                } else {
                    item = _.findWhere(self.model.get('fields'), {
                        id: self.state.get('dataConvert').dataField
                    }).format;
                    if (item && item.datatype == "Percent") {
                        return '<h3>' + key + '</h3><p>' + _appFun.numberFormatFunction(item, e) + " on " + y + '</p>';
                    } else {
                        return '<h3>' + key + '</h3><p>' + e + " on " + y + '</p>';
                    }
                }
            } else if (self.state.get('dataConvert').transpose) {
                item = _.findWhere(self.model.get('fields'), {
                    id: self.state.get('series')[0]
                }).format;
                if (item && item.datatype == "Percent") {
                    return '<h3>' + key + '</h3><p>' + _appFun.numberFormatFunction(item, e) + " on " + y + '</p>';
                } else {
                    return '<h3>' + key + '</h3><p>' + e + " on " + y + '</p>';
                }
            } else {
                if (type == "pie" || type == 'donut') {
                    item = _.findWhere(self.model.get('fields'), {
                        label: e.point.label
                    }).format;
                    value = e.value;
                    if (item && item.datatype == "Percent") {
                        return '<h3>' + key + '</h3><p>' + _appFun.numberFormatFunction(item, value) + '</p>';
                    } else {
                        return '<h3>' + key + '</h3><p>' + e.value + '</p>';
                    }
                } else {
                    item = _.findWhere(self.model.get('fields'), {
                        label: key
                    }).format;
                    key = key;
                    if (item && item.datatype == "Percent") {
                        return '<h3>' + key + '</h3><p>' + _appFun.numberFormatFunction(item, e) + " on " + y + '</p>';
                    } else {
                        return '<h3>' + key + '</h3><p>' + e + " on " + y + '</p>';
                    }
                }
            }
        },

        _groupFieldIsDateTime: function () {
            var self = this;
            if (self.state.get('dataConvert').transpose) {
                return false;
            } else {
                var xfield = _.findWhere(self.model.get('fields'), {
                    id: self.state.get('group')
                });
                var xtype = xfield.type;
                if (self.state.get('dataConvert').type == 'none') {
                    var isDateTime = (xtype === 'date' || xtype === 'date-time' || xtype === 'time');
                    return isDateTime;
                } else if (self.state.get('dataConvert').type == 'pivot') {
                    var item = $.grep(self.model.get('fields'), function (val, i) {
                        return val.id == self.state.get('dataConvert').rowField;
                    });
                    var isDateTime = (item[0].type === 'date' || item[0].type === 'date-time' || item[0].type === 'time');
                    return isDateTime;
                }
            }
        },

        parseRecords: function () {
            var self = this;
            var data = [],
                series = [],
                fields = [],
                xfield, parsed = false,
                rec = [];
            var dataConvert = self.state.attributes.dataConvert;

            var defaults = {
                id: null,
                label: null,
                name: null,
                type: 'string',
                decimalPoint: 2
            };

            if (dataConvert.type == "pivot") {
                xfield = self.model.fields.get(dataConvert.rowField);
               rec = self.model.get('records');
                _.each(rec, function (obj) {
                    _.each(obj, function (value, key) {
                        if (value === null) {
                            obj[key] = 0;
                        }
                        if (key == xfield.get('id') && xfield.get('format') != undefined) {
                            if (xfield.get('format').category == "Date")
                                obj[key] = _appFun.convertToDate(value, xfield.get('format').format);
                        }
                    });
                });
                data = self.model.jsonData.pivot(rec, dataConvert.rowField, dataConvert.columnField, dataConvert.dataField, undefined, self.model.get('fields'));
                var series = _.uniq(_.pluck(rec, dataConvert.columnField));
                var seriesType = $.grep(self.model.get('fields'), function (val, i) {
                    return val.id == dataConvert.columnField;
                });
                series = $.map(series, function (val, i) {
                    if (seriesType[0].type == "date" || seriesType[0].type == "date-time") {
                        if (moment(val).format(seriesType[0].format.format) != "Invalid date") {
                            val = _appFun.convertToDate(val, seriesType[0].format.format);
                        }
                    }
                    return val;
                });
                fields = $.map(series, function (value, index) {
                    return {
                        id: value,
                        label: value,
                        name: value,
                        type: 'float',
                        decimalPoint: 2
                    };
                });

                parsed = true;
            } else {
                xfield = _.findWhere(self.model.get('fields'), {
                    id: self.state.get('group')
                });
                data = $.extend(true, [], self.model.records.toJSON());
                series = self.state.get('series');
            }

            if (dataConvert.transpose) {
                var rowField = xfield.id;
                var records = (parsed) ? data : self.model.records.toJSON();
                var colField = $.extend(true, [], series);
                var fields = self.model.get('fields');
                data = self.model.jsonData.transpose(records, rowField, colField, fields);
                series = _.uniq(_.pluck(records, rowField));
                var seriesType = _.findWhere(fields, {
                    id: rowField
                });
                series = $.map(series, function (val, i) {
                    if (seriesType.type == "date" || seriesType.type == "date-time") {
                        if (moment(val).format(seriesType.format.format) != "Invalid date") {
                            val = _appFun.convertToDate(val, seriesType.format.format);
                        }
                    }
                    return val;
                });

                fields = $.map(series, function (value, index) {
                    return {
                        id: value,
                        label: value,
                        name: value,
                        type: 'float',
                        format: self.state.get('series')[index],
                        decimalPoint: 2
                    };
                });
                parsed = true;
                _.each(data, function (value) {
                    value[rowField] = self.model.fields.get(value[self.state.get('group')]).get('name');
                });
            }

            var recordConfig = {
                parsed: parsed,
                data: data,
                series: series,
                xfield: xfield,
                yfields: fields,
                isDateTime: self._groupFieldIsDateTime(),
                gType: _.findWhere(self.graphTypes, {
                    key: self.state.get('graphType')
                })
            }
            return recordConfig;
        },

        createSeries: function (configs) {
            var self = this;
            self.xvaluesAreString = true;
            self.xvaluesAreIndex = false;
            self.xaxisLabels = [];
            var serieses = [];
            var shapes = ['circle', 'cross', 'triangle-up', 'triangle-down', 'diamond', 'square'];

            var recordConfig = self.parseRecords();
            _.each(recordConfig.series, function (field) {
                var points = [];
                var yfield = recordConfig.parsed ? field : _.findWhere(self.model.get('fields'), {
                    id: field
                });
                var fieldLabel = recordConfig.parsed ? field : yfield.name;

                _.each(recordConfig.data, function (doc, index) {
                    var x = (typeof doc.getFieldValue == 'undefined') ? doc[recordConfig.xfield.id] : doc.getFieldValue(recordConfig.xfield);

                    if (recordConfig.isDateTime) {
                        // cast to string as Date(1990) produces 1970 date but Date('1990') produces 1/1/1990
                        var _date;
                        if (recordConfig.parsed == false) {
                            if (_appFun.convertToDate(String(x), recordConfig.xfield.format.format) != "Invalid date") {
                                _date = moment(_appFun.convertToDate(String(x), recordConfig.xfield.format.format));
                            }
                            x = _appFun.convertToDate(_date._i, recordConfig.xfield.format.format);
                        } else {
                            if (_appFun.convertToDate(String(x), recordConfig.xfield.get('format').format) != "Invalid date") {
                                _date = _appFun.convertToDate(String(x), recordConfig.xfield.get('format').format);
                            }
                            x = _appFun.convertToDate(_date, recordConfig.xfield.get('format').format);
                        }
                        if ($.inArray(x, self.xaxisLabels) < 0) {
                            self.xaxisLabels.push(x);
                        }
                        x = $.inArray(x, self.xaxisLabels);
                        self.xvaluesAreIndex = true;
                    } else if (!recordConfig.gType.xstring) {
                        if ($.isNumeric(x)) x = x.toString();
                        if ($.inArray(x, self.xaxisLabels) < 0) {
                            self.xaxisLabels.push(x);
                        }
                        x = $.inArray(x, self.xaxisLabels);
                        self.xvaluesAreIndex = true;
                    }
                    var y;
                    if (typeof doc.getFieldValue == 'undefined') {
                        if (recordConfig.parsed)
                            y = doc[yfield];
                        else y = doc[yfield.id];
                    } else y = doc.getFieldValue(yfield);
                    if (y == undefined)
                        y = 0;
                    if (!isNaN(parseFloat(y))) {
                        y = parseFloat(y.toString().replace(/,/g, ""));
                    } else {
                        var temp = parseFloat(y.toString().replace(/[,$?]/g, "").replace(/Rs./g, ""));
                        if (!isNaN(temp))
                            y = temp
                    }
                    var ponit = {
                        x: x,
                        y: y,
                        label: fieldLabel
                    };
                    switch (recordConfig.gType.key) {
                        case "scatter":
                        case "scatterPlusLine":
                            $.extend(ponit, {
                                size: Math.random()
                            }); //, shape: shapes[points.length % 6]
                            break;
                    }
                    points.push(ponit);
                });               

                var series = {
                    key: fieldLabel,
                    values: points
                };
                switch (recordConfig.gType.key) {
                    case "line":
                        $.extend(series, {
                            area: ($.inArray(field, self.options.state.area) > -1)
                        });
                        break;
                    case "linePlusBar":
                        $.extend(series, {
                            bar: ($.inArray(field, self.options.state.bar) > -1)
                        });
                        break;
                    case "scatterPlusLine":
                        if (configs) $.extend(series, {
                            slope: configs.slope || -1,
                            intercept: configs.intercept || -1
                        });
                        break;
                }

                //Code for in memory grouping 
                var uniqueList = _.uniq(series.values, function (item, key, x) {
                    return item.x;
                });

                if (self.state.get('dataConvert').reverse) {
                    _.each(uniqueList,function(val,i){
                        val["_sortOrder"] = i;
                    });
                    uniqueList = _appFun.sortByKey('_sortOrder', false, uniqueList);           
                }

                var sum = [];
                var x;
                $.each(uniqueList, function (key, value) {
                    var arr = $.grep(series.values, function (n, i) {
                        return n.x == value.x;
                    });

                    var total = 0;
                    $.each(arr, function (key, data) {
                        total += data.y;
                    });
                    var xValue;
                    if(self.state.get('graphType') != "multiBar" && self.state.get('graphType') != "multiBarHorizontal" && self.state.get('dataConvert').reverse) {
                        xValue = key;
                    }
                    else {
                        xValue = arr[0].x;
                    }
                    sum.push({
                        label: arr[0].label,
                        x: xValue,
                        y: total
                    });
                });

                serieses.push({
                    key: series.key,
                    values: sum
                });
            });

            if (recordConfig.parsed) self._renderGrid(recordConfig.xfield, recordConfig.yfields, recordConfig.data);
            return serieses;
        },

        createPieSeries: function () {
            var self = this;
            self.xvaluesAreIndex = false;
            var series = [];
            var recordConfig = self.parseRecords();

            if (recordConfig.series.length > 0) {
                var field = recordConfig.series[0];
                var points = [];
                var yfield = recordConfig.parsed ? field : self.model.fields.get(field); //.get('name') 
                var fieldLabel = recordConfig.parsed ? field : yfield.get('name');
                _.each(recordConfig.data, function (doc, index) {
                    var x = (typeof doc.getFieldValue == 'undefined') ? doc[(recordConfig.xfield.id == undefined) ? recordConfig.xfield : recordConfig.xfield.id] : doc.getFieldValue(recordConfig.xfield);
                    if (recordConfig.isDateTime) {
                        // cast to string as Date(1990) produces 1970 date but Date('1990') produces 1/1/1990
                        if (recordConfig.parsed == false) {
                            if (moment(String(x)).format(recordConfig.xfield.format.format) != "Invalid date") {
                                var _date = moment(moment(String(x)).format(recordConfig.xfield.format.format));
                            } else {
                                var _date = moment(moment(String(x), recordConfig.xfield.format.format));
                            }
                            x = moment(_date._i, recordConfig.xfield.format.format);
                        } else {
                            if (moment(String(x)).format(recordConfig.xfield.attributes.format.format) != "Invalid date") {
                                var _date = moment(moment(String(x)).format(recordConfig.xfield.attributes.format.format));
                            } else {
                                var _date = moment(moment(String(x), recordConfig.xfield.attributes.format.format));
                            }
                            x = moment(_date._i, recordConfig.xfield.attributes.format.format);
                        }
                    }
                    var y = (typeof doc.getFieldValue == 'undefined') ? doc[(yfield.id == undefined) ? yfield : yfield.id] : doc.getFieldValue(yfield);
                    if (y && parseFloat(y) != NaN) {
                        y = parseFloat(y.toString().replace(/,/g, ""));
                    }

                    series.push({
                        x: x,
                        y: y,
                        label: fieldLabel
                    });
                });
                //Code for in memory grouping 
                var uniqueList = _.uniq(series, function (item, key, x) {
                    return item.x;
                });
                var sum = [];
                var x;
                $.each(uniqueList, function (key, value) {
                    var arr = $.grep(series, function (n, i) {
                        return n.x == value.x;
                    });

                    var total = 0;
                    $.each(arr, function (key, data) {
                        total += data.y;
                    });
                    var xValue = arr[0].x;
                    sum.push({
                        label: arr[0].label,
                        x: xValue,
                        y: total
                    });
                });                
            }
            if (recordConfig.parsed) self._renderGrid(recordConfig.xfield, recordConfig.series, recordConfig.data);
            return sum;
        },

        _getClass: function (type) {
            var className = '';
            switch (type) {
                case 'number':
                case 'numeric':
                case 'int':
                case 'float':
                case 'integer':
                    className = 'align-right'
                    break;
                case 'date':
                case 'datetime':
                case 'date-time':
                case 'timestamp':
                case 'geo_point':
                case 'geo':
                    className = 'align-center'
                    break;
                default:
                    className = 'align-left'
                    break;
            }
            return className;
        },

        createSelectedItemGrid: function (key) {
            var self = this;
            if (self.state.get('dataConvert').type == "none" && self.state.get('dataConvert').transpose == false) {
                var items = $.grep(self.model.get('records'), function (val, i) {
                    var item = $.grep(self.model.get('fields'), function (value, j) {
                        return value.type == "date" || value.type == "date-time" || value.type == "time";
                    });
                    var dFormat = _.findWhere(item, {
                        id: self.state.get('group')
                    });

                    if (dFormat != undefined) {
                        dFormat = dFormat.format.format;
                        if (item.length > 0 && self.state.get('group') == item[0].id) return _appFun.convertToDate(val[self.state.get('group')], dFormat) == _appFun.convertToDate(key.x, dFormat);
                        else return _appFun.convertToDate(val[self.state.get('group')], dFormat) == (self.xvaluesAreIndex ? self.xaxisLabels[key.x] : key.x);
                    } else
                        return val[self.state.get('group')] == (self.xvaluesAreIndex ? self.xaxisLabels[key.x] : key.x);
                });               
                var $container = $("<div id='nvd3DataGrid_" + self.uid + "' class='chartGrid' style='width:100%;'></div>");
                $('#nvd3chartData_' + self.uid).empty().append($container);
                var options = {
                    enableCellNavigation: false,
                    enableColumnReorder: false,
                    forceFitColumns: true
                };

                var columns = getColumns(this.model.fields.toJSON());

                function getColumns(fields, ischild) {
                    var columnList = [];
                    _.each(fields, function (field) {
                        var hasTotal=false;
                        //displays grandtotal only for currency fields
                        if(field.format)
                        {
                            hasTotal=field.format.category=="Currency"?true:false;
                        }
                        var column = {
                            id: field.id,
                            name: field.label,
                            field: field.id,
                            type: field.type,
                            toolTip: field.description,
                            sortable: true,
                            minWidth: 80,
                            format: field.format,
                            hasTotal: hasTotal,
                            width: 80,
                            cssClass: getClass(field.type),
                            formatter: slickFormatter,
                            isHidden: ischild || field.isHidden,
                            headerCssClass: getClass(field.type),
                            children: getChildColumns(field.children)
                        };
                        if (ischild) childcolumns.push(field.id);

                        function getChildColumns(subcolumns) {
                            if (typeof subcolumns === "undefined" || !$.isArray(subcolumns) || subcolumns.length == 0) return undefined;
                            var mapcolumns = $.grep(self.model.fields.toJSON(), function (el, i) {
                                return $.inArray(el.id, subcolumns) > -1;
                            })
                            return (mapcolumns.length > 0) ? getColumns(mapcolumns, true) : undefined;
                        }

                        // Modified by Assyst (Rony at 07-Feb-2014)    
                        // For text align based on data type
                        /******************************************/
                        function getClass(type) {
                            var className = '';
                            switch (type) {
                                case 'number':
                                case 'integer':
                                    className = 'align-right'
                                    break;
                                case 'date':
                                case 'datetime':
                                case 'date-time':
                                case 'timestamp':
                                case 'geo_point':
                                case 'geo':
                                    className = 'align-center'
                                    break;
                                default:
                                    className = 'align-left'
                                    break;
                            }
                            return className;
                        }
                        /***************End*********************/

                        var widthInfo = _.find(self.state.get('columnsWidth'), function (c) {
                            return c.column === field.id;
                        });
                        if (widthInfo) {
                            if (column.minWidth > widthInfo.width) column.minWidth = widthInfo.width;
                            column.width = widthInfo.width;
                        }

                        var editInfo = _.find(self.state.get('columnsEditor'), function (c) {
                            return c.column === field.id;
                        });
                        if (editInfo) {
                            column.editor = editInfo.editor;
                        }
                        columnList.push(column);
                    });
                    return columnList;
                }

                var gridOptions = {
                    currency: true,
                    grandTotal: true
                };

                //If it is dashboard it takes the contentDiv height else default 300px
                if ($('#dashboardContainer').length > 0) {
                    if($('#myModal').length>0){
                        $container.height(300);
                    }
                    else {
                        $container.height($(self.$el).parents('.col').find('.frontFace .contentDrid_div').height());
                    }
                } else {
                    $container.height(300);
                }

                //giving vertical scroll for container section
                $('.data-view-container-wrap').addClass('pivot-active');
                var pivotscreen;
                if($('.modal-content').length>0){
                    pivotscreen =   $('.modal-content:visible').height() - 100;
                }
                else {
                    pivotscreen = $(document).height() - 250;
                }
                $(".data-view-container-wrap.pivot-active").css({
                    'height': pivotscreen + 'px'
                });

                //setting width for the drilldown grid
                setTimeout(function () {
                    if ($('#dashboardContainer').length > 0) {
                        if($('#myModal').length>0){
                            $container.width($('.data-view-container').width());
                        }
                        else {
                            $container.width($(self.$el).parents('.col').find('.frontFace .contentDrid_div').width()); 
                        }            
                    }
                    else{
                        $container.width($('.data-view-container').width());
                    }
                    chartGrid = slickGridCall($container, items, columns, options, gridOptions);
                }, 400);                

                if ($('#dashboardContainer').length > 0) {
                    $(self.$el).parents('.col').find('.btndshGrid').remove();
                    $(self.$el).parents('.col').find('.frontFace .content_div').hide();
                    $(self.$el).parents('.col').find('.frontFace .contentDrid_div').show().empty().append($container);
                    var btnGrid = $('<div class="btndshGrid" title="Chart drilldown Grid"></div>').click(function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        $(self.$el).parents('.col').find('.frontFace .content_div').hide();
                        $(self.$el).parents('.col').find('.frontFace .contentDrid_div').show();
                    });
                    $(self.$el).parents('.col').find('.frontFace .header_div').append(btnGrid);
                }
            }
        },

        _renderGrid: function (xfield, fields, data) {
            var self = this;
            if (self.state.get('showData')) {
                var grid;
                var $container = $("<div id='nvd3DataGrid_" + self.uid + "' class='chartGrid' style='width:100%; min-height:300px;'></div>");
                $('#nvd3chartData_' + self.uid).empty().append($container);
                var columns = [];

                //formatting the pivoted grid of chart with xfield format
                var xformat;
                if (self.state.get('dataConvert').transpose) {
                    xformat = xfield.format;
                } else {
                    xformat = xfield.format == undefined ? xfield.format : xfield.get('format');
                }
                var hastotal;
                if (xformat) {
                    if (xformat.category == "Currency")
                        hastotal = true;
                    else hastotal = false;
                }
                columns.push({
                    id: xfield.id,
                    name: xfield.name,
                    field: xfield.id,
                    type: xfield.type,
                    decimalPoint: xfield.decimalPoint || 2,
                    toolTip: xfield.name,
                    sortable: false,
                    minWidth: 80,
                    format: xformat,
                    hasTotal: hastotal,
                    formatter: slickFormatter,
                    cssClass: self._getClass(xfield.type),
                    headerCssClass: self._getClass(xfield.type)
                });

                _.each(fields, function (field) {
                    if (self.state.get('graphType') == "pie" || self.state.get('graphType') == "donut") {
                        var format = _.findWhere(self.model.get('fields'), {
                            id: self.state.get('dataConvert').dataField
                        }).format;

                        if (format) {
                            hastotal = format.category == "Currency" ? true : false;
                        } else {
                            hastotal = false;
                        }

                        columns.push({
                            id: field,
                            name: field,
                            field: field,
                            type: "numeric",
                            decimalPoint: 2,
                            toolTip: field,
                            format: format,
                            sortable: false,
                            minWidth: 80,
                            hasTotal: hastotal,
                            formatter: slickFormatter,
                            cssClass: self._getClass("numeric"),
                            headerCssClass: self._getClass("numeric")
                        });
                    } else {
                        var format;
                        if (self.state.get('dataConvert').transpose) {
                            format = _.findWhere(self.model.get('fields'), {
                                id: self.state.get('series')[0]
                            }).format;
                        } else {
                            format = _.findWhere(self.model.get('fields'), {
                                id: self.state.get('dataConvert').dataField
                            }).format;
                        }

                        if (format) {
                            hastotal = format.category == "Currency" ? true : false;
                        } else {
                            hastotal = false;
                        }

                        columns.push({
                            id: field.id,
                            name: field.name,
                            field: field.id,
                            type: field.type,
                            decimalPoint: field.decimalPoint || 2,
                            toolTip: field.name,
                            sortable: false,
                            format: format,
                            minWidth: 80,
                            hasTotal: hastotal,
                            formatter: slickFormatter,
                            cssClass: self._getClass(field.type),
                            headerCssClass: self._getClass(field.type)
                        });
                    }
                });

                var options = {
                    enableCellNavigation: false,
                    enableColumnReorder: false,
                    forceFitColumns: true
                };

                var gridOptions = {
                    currency: true,
                    grandTotal: true
                };

                //If it is dashboard it takes the contentDiv height else default 300px
                if ($('#dashboardContainer').length > 0) {
                    if($('#myModal').length>0){
                        $container.height(300);
                    }
                    else {
                        $container.height($(self.$el).parents('.col').find('.frontFace .contentDrid_div').height());
                    }
                } else {
                    $container.height(300);
                }

                //setting width for the pivoted grid
                setTimeout(function () {
                    if ($('#dashboardContainer').length > 0) {
                        if($('#myModal').length>0){
                            $container.width($('.data-view-container').width());
                        }
                        else {
                            $container.width($(self.$el).parents('.col').find('.frontFace .contentDrid_div').width()); 
                        }                   
                    }
                    else{
                        $container.width($('.data-view-container').width());
                    }
                    chartGrid = slickGridCall($container, data, columns, options, gridOptions);
                }, 400);
		if ($('#dashboardContainer').length > 0) {
                    $(self.$el).parents('.col').find('.btndshGrid').remove();
                    //$(self.$el).parents('.col').find('.frontFace .content_div').hide();
                    $(self.$el).parents('.col').find('.frontFace .contentDrid_div').empty().append($container);
                    var btnGrid = $('<div class="btndshGrid" title="Chart drilldown Grid"></div>').click(function (event) {
                        event.preventDefault();
                        event.stopPropagation();
                        $(self.$el).parents('.col').find('.frontFace .content_div').hide();
                        $(self.$el).parents('.col').find('.frontFace .contentDrid_div').show();
                    });
                    $(self.$el).parents('.col').find('.frontFace .header_div').append(btnGrid);
                }
            }
        }
    });
    //edited by nithin for tile view of graph types
    my.NVD3Controls = Backbone.View.extend({
        className: "editor graphEditor",
        template: ' \
    <div class="editor-block"> \
    <form class="form-stacked"> \
      <div class="clearfix"> \
        <div class="outerGraphtype" title="Graph Type"><label class="_lngTrans_Translatable graphType" ></label> \
        <span class="updown slideDown"></span>\
         <div class="input editor-type"> \
              <div class="graphSet"> \
              {{#graphTypes}} \
                <div class="graphIcons"><div value="{{key}}" class="_lngTrans_Translatable tiledGraph {{key}}" name="{{name}}" title="{{name}}"></div><span class="sliced">{{sliced}}</span></div> \
              {{/graphTypes}} \
              </div> \
          </div> \
        </div> \
        <div class="spaceDiv"></div>\
        <div class="series-group"> \
          <label><span class="_lngTrans_Translatable" title="Group Column ">Group Column </span><span class="series-axis _lngTrans_Translatable" title="(Horizontal-Axis)">(Horizontal-Axis)</span></label> \
          <span class="updown slideDown"></span>\
          <div class="input editor-group"> \
              <select> \
              {{#axisFields}} \
              <option value="{{id}}" class="_lngTrans_Translatable" data="{{type}}">{{name}}</option> \
              {{/axisFields}} \
              </select> \
              <div class="dataConvert">\
              </div>\
          </div> \
        </div> \
       <div class="series-editor js-series"> \
          <label><span class="_lngTrans_Translatable" title="Series">Series</span>\
          </label> \
          <span class="updown slideDown"></span>\
          <div class="editor-series-nvd3block"> \
            <div class="editor-series-none"> \
            </div> \
         <div class="editor-series-options" style="display:none"> \
            <label class="checkbox"> \
              <input type="checkbox" id="editor-data-show" value="datashow"  /> \
              <span class="_lngTrans_Translatable">Show Data</span></label> \
          </div> \
        </div>\
       </div> \
       {{^pivotPopupView}}\
       <div class="series-pivot js-series pivotshow"> \
          <label><span class="_lngTrans_Translatable"title="Pivot">Pivot</span>\
          </label> \
          <span class="updown slideDown"></span>\
          <div class="editor-pivot-nvd3block"> \
            <div class="editor-series-pivot pivotDiv"> \
            </div> \
            <div class="pivot-graphButton"> \
                <label class="pivot-ok"><div id="btnPivot" title="Ok" value="ok" class="">OK</div></label> \
                <label class="pivot-reset"><div id="btnPivotReset"title="Reset" value="reset" class="">Reset</div></label> \
            </div> \
        </div>\
       </div> \
       {{/pivotPopupView}}\
       <div class="labelsDiv" ><label title="Switch rows / columns"><input type="checkbox" id="chkTansConvert"  class="chkDataConvert" value= "transpose"  /> Switch rows / columns</label> </div>\
      <div class="labelsDiv" ><label class="checkboxReverseDiv" title="Reverse"><input type="checkbox" id="chkReverseConvert" class="chkDataConvert" value="reverse" / > Reverse</label></div> \
       {{#pivotPopupView}}<label class="pivotLabel"><div id="btnPivotConvert" value="pivot" class="">Pivot Data</div></label> {{/pivotPopupView}}\
      </div> \
      {{#seriesType}} \
      <div class="editor-buttons add-series"> \
        <button type="submit" class="editor-add ui-widget ui-state-default ui-corner-all ui-button-text-only ui-btn-general"><span class="ui-button-text _lngTrans_Translatable" title="">Add Series</span></button> \
      </div> \
      {{/seriesType}} \
      <div class="editor-buttons editor-submit" comment="hidden temporarily" style="display: none;"> \
        <button class="editor-save _lngTrans_Translatable">Save</button> \
        <input type="hidden" class="editor-id" value="chart-1" /> \
      </div> \
    </form> \
  </div> \
',
        templateSeriesEditor: ' \
    <div class="editor-series js-series-{{seriesIndex}}"> \
      <label><span class="_lngTrans_Translatable">Series</span> <span  class="series-name">{{seriesName}}</span> <span class="series-axis _lngTrans_Translatable">(Axis 2)</span>\
      <span class ="remove-series">[<a href="#remove" class="action-remove-series _lngTrans_Translatable">Remove</a>]</span>\
      </label> \
      <div class="input"> \
        <select> \
            {{#axisFields}} \
                <option value="{{id}}" class="_lngTrans_Translatable" data="{{type}}">{{name}}</option> \
            {{/axisFields}} \
        </select> \
      </div> \
    </div> \
  ',
        templateMultiSeriesEditor: ' \
          <div class="inputSelect"> \
            <select> \
            {{#axisFields}} \
            <option value="{{id}}" class="_lngTrans_Translatable" data="{{type}}">{{name}}</option> \
            {{/axisFields}} \
            </select> \
          </div> \
          <div class="chkGroup">\
            {{#axisFields}} \
            <div class="divSeries">\
                <input type="checkbox" value="{{id}}" data="{{type}}" class="_lngTrans_Translatable chkSeries"/>{{name}} \
            </div>\
            {{/axisFields}} \
          </div>\
  ',
        templatePivotPopup: ' \
          <div> \
                <div class="leftblock"> \
                    <div class="addRowsDiv"> \
                        <select name="rowfield"> \
                            <option disabled selected value="-1">Add Row</option> \
                            {{#rowFields}} \
                                <option value={{id}} data-type={{type}}>{{name}}</option> \
                            {{/rowFields}} \
                         </select> \
                        </div> \
                </div> \
                <div class="rightblock"> \
                    <div class="addColDiv"> \
                        <select name="columnfield">\
                            <option disabled selected value="-1">Add Column</option> \
                            {{#columnFields}} \
                                <option value={{id}} data-type={{type}}>{{name}}</option> \
                            {{/columnFields}} \
                        </select> \
                        </div> \
                    <div class="addValDiv"> \
                        <select  name="datafield"> \
                            <option disabled selected value="-1">Add Data</option>\
                            {{#dataFields}}\
                                <option value={{id}} data-type={{type}}>{{name}}</option>\
                            {{/dataFields}}\
                        </select>\
                    </div>\
                </div> \
        ',

        templateSelectedField: '<div class="selDiv"><label class="colText" value={{id}}>{{name}}</label></div>',

        events: {
            'click .tiledGraph': 'onSelectGraphType',
		'change form .editor-group select': 'onEditorSubmit',
            'change form .inputSelect select': 'onEditorSubmit',
            'change .chkSeries': 'onEditorSubmit',
            'click #btnPivotConvert': '_createPopup',
            'change input.chkDataConvert': 'onEditorSubmit',
            'click .updown': 'onSlideToggle',
            'click .editor-add': '_onAddSeries',
            'click .action-remove-series': 'removeSeries',
            'click #editor-data-show': 'onDataShowChange',
            'change .pivotDiv select': 'onPivotSelectChange',
            'click .pivot-graphButton .pivot-ok': 'onPivotOkClick',
            'click .pivot-graphButton .pivot-reset': 'onPivotResetClick'
        },


        initialize: function (options) {
            var self = this;
            _.bindAll(this, 'render');
            this.listenTo(this.model.axisFields, 'reset add', this.render);
            this.state = new recline.Model.ObjectState(options.state);
            this.render();
        },

        pivotConfig: {
            rowField: undefined,
            columnField: undefined,
            dataField: undefined
        },

        render: function () {
            var self = this;
            var tmplData = this.model.toTemplateJSON();
            tmplData["graphTypes"] = self.options.graphTypes;
            tmplData["seriesType"] = (self.state.get('seriesType') != 'multiselect');
            tmplData["pivotPopupView"] = (self.state.get('pivotView') == 'popup');
            var htmls = Mustache.render(this.template, tmplData);
            this.$el.html(htmls);


            //Code added by assyst to remove the options if not specified in type array
            if (this.state.get('types') && this.state.get('types').length > 0) {
                this._removeOption('.editor-type', this.state.get('types'));
            }

            // set up editor from state
            if (this.state.get('graphType')) {
                var graphType = _.findWhere(self.options.graphTypes, {
                    key: this.state.get('graphType')
                });
                this.$el.find('.' + graphType.key).addClass('active');
                this.$el.find('.graphType').html('<span style="margin-right:5px;"> Graph Type:</span><span style="font-weight:bold">' + graphType.name + '</span>');
            }


            var groupName = this.state.get('group');
            if (groupName) {
                this._selectOption('.editor-group', groupName);
            }

            // ensure at least one series box shows up
            var tmpSeries = [""];
            if (this.state.get('series').length > 0) {
                tmpSeries = this.state.get('series');
            }

            var dataConvert = this.state.get('dataConvert');
            //this.$el.find('input.rdoDataConvert[value=' + dataConvert.type + ']').prop('checked', true);   

            if (dataConvert.transpose) this.$el.find('input#chkTansConvert').prop('checked', true);
            if (dataConvert.reverse) this.$el.find('input#chkReverseConvert').prop('checked', true);


            self.$el.find(".editor-series-none").show();


            if (self.state.get('pivotView') != 'popup') {
                self.addPivotBlock();
                self._pivotPopupOption(this.$el, ".addRowsDiv", dataConvert.rowField);
                self._pivotPopupOption(this.$el, ".addColDiv", dataConvert.columnField);
                self._pivotPopupOption(this.$el, ".addValDiv", dataConvert.dataField);
                self.pivotConfig = {
                    rowField: dataConvert.rowField,
                    columnField: dataConvert.columnField,
                    dataField: dataConvert.dataField
                };
            }

            if (dataConvert.type == "pivot") {
                self._hideShowSeries();
            }


            if (self.state.get('seriesType') == 'multiselect') {
                self.addMultiSeries();
                self._multiSelectOption(tmpSeries, groupName);

                if (this.state.get('graphType') == "donut" || this.state.get('graphType') == "pie") {
                    self.$el.find('.dataConvert').hide();
                    self.$el.find('.inputSelect').show();
                } else {
                    self.$el.find('.chkGroup').show();
                }
            } else {
                _.each(tmpSeries, function (series, idx) {
                    self.addSeries(idx);
                    self._selectOption('.editor-series.js-series-' + idx, series, groupName);
                });
            }
            return this;
        },

        _removeOption: function (id, types) {
            var options = this.$el.find(id + ' select > option');
            if (options) {
                options.each(function (opt) {
                    if ($.inArray(this.value, types) == -1) $(this).remove();
                });
            }
        },

        _hideShowSeries: function () {
            var self = this;
            var dataConvert = this.state.get('dataConvert');
            if (dataConvert.type == "pivot") {
                self.$el.find('.series-group,.series-editor').addClass('disabledItems');
                self.$el.find('.labelsDiv #chkTansConvert').parent('label').addClass('disabledItems');
                self.$el.find('.editor-group,.editor-series-nvd3block').hide();
                self.$el.find('.series-group .updown,.series-editor .updown').removeClass('slideUp').addClass('slideDown');
            } else {
                self.$el.find('.series-group,.series-editor').removeClass('disabledItems');
                self.$el.find('.labelsDiv #chkTansConvert').parent('label').removeClass('disabledItems');
            }
        },

        // Private: Helper function to select an option from a select list
        _selectOption: function (id, value, groupName) {
            var options = this.$el.find(id + ' select > option');
            if (options) {
                $.each(options, function (i, val) {
                    if (val.text == groupName) $(val).hide();
                    else $(val).show();
                    if (this.value == value) {
                        $(this).attr('selected', 'selected');
                        return false;
                    }
                });
            }
        },

        _multiSelectOption: function (series, groupName) {
            var self = this;
            var chkOptions = self.$el.find('.series-editor.js-series .divSeries');
            if (chkOptions) {
                $.each(chkOptions, function (i, val) {
                    var value = $(this).find('.chkSeries').val();
                    if (value == groupName) $(val).hide();
                    else $(val).show();
                    if (value != groupName && $.inArray(value, series) > -1) {
                        $(this).find('.chkSeries').attr('checked', true);
                    } else {
                        $(this).find('.chkSeries').attr('checked', false);
                    }
                });
            }

            var options = self.$el.find('.editor-series-nvd3block .inputSelect:visible select > option');
            if (options) {
                $.each(options, function (i, val) {
                    if (this.value == groupName){ 
                        $(val).hide();
                    }
                    else {
                        $(val).show();
                        if(series.length>0){
                            if (this.value != groupName && $.inArray(this.value, series) > -1) {
                                $(this).attr('selected', 'selected');
                            }
                        }
                        else{
                            $(this).attr('selected', 'selected');
                        }
                    }                    
                });
            }
        },

        _pivotPopupOption: function (popup, id, value) {
            var self = this;
            var fields = this.model.toTemplateJSON().axisFields;
            var item = _.findWhere(fields, {
                id: value
            });
            var element = $(id, popup);
            element.find('.selDiv').remove();
            if (item) {
                element.append(Mustache.render(self.templateSelectedField, item));
            }
            element.find('select option:first').prop("selected", true);
        },
        onPivotSelectChange: function (e) {
            var self = this;
            var select = $(e.currentTarget);
            var name = select.attr("name");
            var value = select.val();
            var title = select.find('option:selected').text();
            var element = undefined;
            switch (name) {
                case "rowfield":
                    self.pivotConfig.rowField = value;
                    element = self.$el.find('.addRowsDiv');
                    break;
                case "columnfield":
                    self.pivotConfig.columnField = value;
                    element = self.$el.find('.addColDiv');
                    break;
                case "datafield":
                    self.pivotConfig.dataField = value;
                    element = self.$el.find('.addValDiv');
                    break;
            }
            element.find('.selDiv').remove();
            element.append(Mustache.render(self.templateSelectedField, {
                id: value,
                name: title
            }));
            element.find('select option:first').prop("selected", true);
        },

        onPivotOkClick: function (e) {
            var self = this
            //validation for clicking ok without selecing row/column/data fields
            if (typeof self.pivotConfig.rowField == "undefined" || typeof self.pivotConfig.dataField == "undefined" || typeof self.pivotConfig.columnField == "undefined") {
                $("#dialog-confir").html("Please add Row, Column and Data.");
                $("#dialog-confir").data('object', self).dialog({
                    resizable: false,
                    modal: true,
                    height: "170",
                    width: "auto",
                    draggable: false,
                    close: function () {
                        $(this).dialog('destroy');
                    },
                    dialogClass: "message info-dialog custom-style no-titlebar",
                    buttons: {
                        Ok: function () {
                            $(this).dialog('destroy');
                        }
                    }
                });
            } else {
                var dataConvert = self.state.get("dataConvert");
                var config = $.extend({}, dataConvert, {
                    type: "pivot",
                    columnField: self.pivotConfig.columnField,
                    dataField: self.pivotConfig.dataField,
                    rowField: self.pivotConfig.rowField
                });
                var updatedState = {
                    dataConvert: config
                };

                if ($('#btnDataViewerSaveAs').length > 0) {
                    $('#btnDataViewerSaveAs').removeClass("disableClass");
                } else {
                    $(e.target).parents('.mis-container').find('.nav-wrap .menu-right .dashlet,.nav-wrap .menu-right .exprt_btn').removeClass("disableClass");
                }
                self.state.set(updatedState);
                self._hideShowSeries();
            }
        },

        onPivotResetClick: function (e) {
            var self = this;
            var reset = $("#dialog-confir").html("Pivot table data would be lost. Do you wish to proceed?");

            $("#dialog-confir").data('object', self).dialog({
                resizable: false,
                modal: true,
                height: "auto",
                width: "auto",
                draggable: false,
                close: function () {
                    $(this).dialog('destroy');
                },
                dialogClass: "message warning-dialog custom-style no-titlebar",
                buttons: {
                    Yes: function () {
                        $('.data-view-container-wrap').removeClass('pivot-active');
                        var dataConvert = self.state.get("dataConvert");
                        var config = {
                            type: 'none',
                            columnField: undefined,
                            dataField: undefined,
                            rowField: undefined,
                            reverse: dataConvert.reverse,
                            transpose: dataConvert.transpose
                        };
                        var updatedState = {
                            dataConvert: config
                        };
                        if ($('#btnDataViewerSaveAs').length > 0) {
                            self.state.get('series').length == 0 ? $('#btnDataViewerSaveAs').addClass("disableClass") : $('#btnDataViewerSaveAs').removeClass("disableClass");
                        } else {
                            self.state.get('series').length == 0 ? $(e.target).parents('.mis-container').find('.nav-wrap .menu-right .dashlet,.nav-wrap .menu-right .exprt_btn').addClass("disableClass") : $(e.target).parents('.mis-container').find('.nav-wrap .menu-right .dashlet,.nav-wrap .menu-right .exprt_btn').removeClass("disableClass");
                        }
                        self.state.set(updatedState);
                        self._pivotPopupOption(self.$el, ".addRowsDiv", config.rowField);
                        self._pivotPopupOption(self.$el, ".addColDiv", config.columnField);
                        self._pivotPopupOption(self.$el, ".addValDiv", config.dataField);
                        self._hideShowSeries();
                        self.pivotConfig = {
                            rowField: undefined,
                            columnField: undefined,
                            dataField: undefined
                        }
                        $(this).dialog('destroy');
                    },
                    No: function () {
                        $(this).dialog('destroy');
                    }
                }
            });
        },

        onDataShowChange: function (e) {
            var self = this;
            if (self.$el.find('#editor-data-show').is(":checked")) $('#nvd3chartData_' + self.uid).show();
            else $('#nvd3chartData_' + self.uid).hide();
        },

        onSlideToggle: function (e) {
            var self = this;
            var element = $(e.currentTarget);
            element.next("div").slideToggle('fast', function () {
                if (element.hasClass('slideDown')) {
                    element.addClass('slideUp').removeClass('slideDown');
                    element.parent().addClass('opened');
                } else {
                    element.addClass('slideDown').removeClass('slideUp');
                    element.parent().removeClass('opened');
                }
            });
        },

        onSelectGraphType: function (e) {
            var currentTarget = e.currentTarget;
            this.$el.find('.graphType').html("<span style='margin-right:5px;'> Graph Type:</span><span style='font-weight:bold'>" + currentTarget.getAttribute('name') + "</span>");
            $.each($(currentTarget).parent().siblings(), function (i, val) {
                $(val).find('.tiledGraph').removeClass('active');
            });
            currentTarget.classList.add("active");
            this.onEditorSubmit(e);
        },

        onEditorSubmit: function (e) {
            var self = this;
            var groupName = this.$el.find('.editor-group select').val();
            var grphselect = this.$el.find('.editor-type div.active').attr('value');
            var select = this.$el.find('.editor-group select');
            var $editor = this;
            var $series, series, dataConvert = {};
            if (self.state.get('seriesType') == 'multiselect') {
                if (grphselect === "pie" || grphselect === "donut") {
                    $series = this.$el.find('.inputSelect select');

                    series = $series.map(function () {
                        return groupName != $(this).val() ? $(this).val() : null;
                    });
                    if(series && series.length == 0){
                        series = $.map($series.find('option'), function (val,i) {
                            if ($(val).val() != groupName){
                               return $(val).val();
                            }                    
                        });
                    }
                } else {
                    $series = this.$el.find('.divSeries .chkSeries:checked');
                    series = $series.map(function () {
                        return groupName != $(this).val() ? $(this).val() : null;
                    });
                }
            } else {
                $series = this.$el.find('.editor-series select');
                series = $series.map(function () {
                    return groupName != $(this).val() ? $(this).val() : null;
                });
            }

            if (self.state.get('seriesType') == 'multiselect') {
                self._multiSelectOption(series, groupName);
                if (grphselect == "donut" || grphselect == "pie") {
                    self.$el.find('.dataConvert, .chkGroup').hide();
                    self.$el.find('.inputSelect').show();
                } else {
                    self.$el.find('.inputSelect').hide();
                    self.$el.find('.dataConvert, .chkGroup').show();
                }
            } else {
                if (grphselect === "pie" || grphselect === "donut") {
                    $(".series-axis, .remove-series, .add-series", this.$el).hide();
                    $(".editor-series:not(:first)", this.$el).hide();
                } else {
                    $(".series-axis, .remove-series, .add-series", this.$el).show();
                    $(".editor-series", this.$el).show();
                }
            }

            var transpose = this.$el.find('input#chkTansConvert:checked').is(":checked");
            var reverse = this.$el.find('input#chkReverseConvert:checked').is(":checked");

            if (transpose == true) {
                $('.pivotshow').hide();
            } else {
                $('.pivotshow').show();
                $(".data-view-container-wrap").removeClass("pivot-active");
            }

            dataConvert = self.state.get("dataConvert");
            if (dataConvert.type == 'pivot') {
                dataConvert = $.extend({}, dataConvert, {
                    reverse: reverse,
                    transpose: transpose
                });
            } else {
                dataConvert = {
                    type: 'none',
                    columnField: undefined,
                    dataField: undefined,
                    rowField: undefined,
                    reverse: reverse,
                    transpose: transpose
                }
                //self.$el.find(".editor-series-pivot").hide();     
                self.$el.find(".editor-series-none").show();
            }

            var updatedState = {
                series: $.makeArray(series),
                group: groupName,
                graphType: grphselect,
                dataConvert: dataConvert
            };

            this.state.set(updatedState);
        },


        _createPopup: function () {
            var self = this;
            var data = {
                rowField: undefined,
                columnField: undefined,
                dataField: undefined
            };
            var $popup = $('<div class="pivotDiv" title="Pivot Data"><div>');
            var dataConvert = self.state.get("dataConvert");

            $popup.html(self.addPivotSeries());

            $popup.dialog({
                height: 265,
                width: 450,
                modal: true,
                buttons: {
                    "Ok": function () {
                        if (typeof data.rowField == "undefined" || typeof data.dataField == "undefined")
                            alert("Please add Row, Column and Data.");
                        else {
                            var config = $.extend({}, dataConvert, {
                                type: "pivot",
                                columnField: data.columnField,
                                dataField: data.dataField,
                                rowField: data.rowField
                            });
                            var updatedState = {
                                dataConvert: config
                            };
                            self.state.set(updatedState);
                            $(this).dialog('close');
                        }
                    },
                    "Reset": function () {
                        var reset = confirm("Pivot table data would be lost. Do you wish to proceed?");
                        if (reset == true) {
                            var config = {
                                type: 'none',
                                columnField: undefined,
                                dataField: undefined,
                                rowField: undefined,
                                reverse: dataConvert.reverse,
                                transpose: dataConvert.transpose
                            };
                            var updatedState = {
                                dataConvert: config
                            };
                            self.state.set(updatedState);
                            $(this).dialog('close');
                        }
                    }
                },
                open: function () {
                    var $popelement = $(this);
                    self._pivotPopupOption($popelement, ".addRowsDiv", dataConvert.rowField);
                    self._pivotPopupOption($popelement, ".addColDiv", dataConvert.columnField);
                    self._pivotPopupOption($popelement, ".addValDiv", dataConvert.dataField);

                    $popelement.on('change', 'select', function () {
                        var name = $(this).attr("name");
                        var value = $(this).val();
                        var title = $(this).find('option:selected').text();
                        var element = undefined;
                        switch (name) {
                            case "rowfield":
                                data.rowField = value;
                                element = $popelement.find('.addRowsDiv');
                                break;
                            case "columnfield":
                                data.columnField = value;
                                element = $popelement.find('.addColDiv');
                                break;
                            case "datafield":
                                data.dataField = value;
                                element = $popelement.find('.addValDiv');
                                break;
                        }
                        element.find('.selDiv').remove();
                        element.append(Mustache.render(self.templateSelectedField, {
                            id: value,
                            name: title
                        }));
                        element.find('select option:first').prop("selected", true);
                    });
                },
                close: function () {
                    $(this).dialog('destroy');
                    self._hideShowSeries();
                }
            });
        },

        // Public: Adds a new empty series select box to the editor.
        //
        // @param [int] idx index of this series in the list of series
        //
        // Returns itself.
        addSeries: function (idx) {
            var data = _.extend({
                seriesIndex: idx,
                seriesName: String.fromCharCode(idx + 64 + 1)
            }, this.model.toTemplateJSON());

            var htmls = Mustache.render(this.templateSeriesEditor, data);
            this.$el.find('.editor-series-none').append(htmls);

            //Code by assyst for series hide for pie chart
            if (this.$el.find('.editor-type select').val() === "pie") {
                $(".series-axis", this.$el).hide();
            }

            return this;
        },

        addMultiSeries: function () {
            var axisFields = $.grep(this.model.toTemplateJSON().axisFields, function (val, i) {
                if ((val.type == "float" || val.type == "int" || val.type == "numeric" || val.type == "number")) return true;
            });
            var htmls = Mustache.render(this.templateMultiSeriesEditor, {
                axisFields: axisFields
            });
            this.$el.find('.editor-series-none').append(htmls);
            return this;
        },

        addPivotBlock: function () {
            this.$el.find('.editor-series-pivot').append(this.addPivotSeries());
            return this;
        },

        addPivotSeries: function () {

            var rowFields = this.model.toTemplateJSON().axisFields;
            //var dataFields =   _.where( rowFields, { type : 'number'});


            var dataFields = $.grep(rowFields, function (val, i) {
                if (val.type == "float" || val.type == "int" || val.type == "numeric" || val.type == "number") return true;
            });

            var columnFields = rowFields;
            var htmls = Mustache.render(this.templatePivotPopup, {
                rowFields: rowFields,
                dataFields: dataFields,
                columnFields: columnFields
            });
            return htmls;
        },

        _onAddSeries: function (e) {
            e.preventDefault();
            var count = $('.editor-series', this.$el).length;
            this.addSeries(count);
            //this.addSeries(this.state.get('series').length);
        },

        // Public: Removes a series list item from the editor.
        //
        // Also updates the labels of the remaining series elements.
        removeSeries: function (e) {
            e.preventDefault();
            var $el = $(e.target);
            //$el.parent().parent().remove();
            //Code by assyst for reorde the series while reomve
            $el.closest(".editor-series").remove();
            $(".series-name", this.$el).each(function (idx) {
                $(this).html(String.fromCharCode(idx + 64 + 1));
            });
            this.onEditorSubmit();
        }

    });

})(jQuery, recline.View);

this.recline = this.recline || {};
this.recline.View = this.recline.View || {};
this.recline.View.Graph = this.recline.View.NVD3Graph;
this.recline.View.GraphControls = this.recline.View.NVD3Controls;