/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   
JQuery Immediately-Invoked Function for recline for Slick Grid
Created By      : Rony T S for Assyst International Pvt. Ltd.
Created Date    : 20-Feb-2014
Version         : 1.0.2
Project         : PanAgro

+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/
this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";
    // ## (Data) Grid Dataset View
    //
    // Provides a tabular view on a Dataset.
    //
    // Initialize it with a `recline.Model.Dataset`.
    my.Grid = Backbone.View.extend({
        tagName: "div",
        className: "recline-grid-container",

        initialize: function (modelEtc) {
            var self = this;
            _.bindAll(this, 'render', 'onHorizontalScroll');
            this.listenTo(this.model.records, 'add reset remove', this.render);
            this.tempState = {};
            var state = _.extend({
                hiddenFields: []
            }, modelEtc.state);
            this.state = new recline.Model.ObjectState(state);
        },

        events: {
            // does not work here so done at end of render function
            // 'scroll .recline-grid tbody': 'onHorizontalScroll'
        },

        // ======================================================
        // Column and row menus

        setColumnSort: function (order) {
            var sort = [{}];
            sort[0][this.tempState.currentColumn] = {
                order: order
            };
            this.model.query({
                sort: sort
            });
        },

        hideColumn: function () {
            var hiddenFields = this.state.get('hiddenFields');
            hiddenFields.push(this.tempState.currentColumn);
            this.state.set({
                hiddenFields: hiddenFields
            });
            // change event not being triggered (because it is an array?) so trigger manually
            this.state.trigger('change');
            this.render();
        },

        showColumn: function (e) {
            var hiddenFields = _.without(this.state.get('hiddenFields'), $(e.target).data('column'));
            this.state.set({
                hiddenFields: hiddenFields
            });
            this.render();
        },

        onHorizontalScroll: function (e) {
            var currentScroll = $(e.target).scrollLeft();
            this.$el.find('.recline-grid thead tr').scrollLeft(currentScroll);
        },

        // ======================================================
        // #### Templating
        template: ' \
    <div class="table-container"> \
    <table class="recline-grid table-striped table-condensed" cellspacing="0"> \
      <thead class="fixed-header"> \
        <tr> \
          {{#fields}} \
            <th class="column-header {{#hidden}}hidden{{/hidden}} _lngTrans_Translatable" data-field="{{id}}" style="width: {{width}}px; max-width: {{width}}px; min-width: {{width}}px;" title="{{label}}"> \
              <span class="column-header-name _lngTrans_Translatable">{{label}}</span> \
            </th> \
          {{/fields}} \
          <th class="last-header" style="width: {{lastHeaderWidth}}px; max-width: {{lastHeaderWidth}}px; min-width: {{lastHeaderWidth}}px; padding: 0; margin: 0;"></th> \
        </tr> \
      </thead> \
      <tbody class="scroll-content"></tbody> \
    </table> \
    </div> \
  ',

        toTemplateJSON: function () {
            var self = this;
            var modelData = this.model.toJSON();
            modelData.notEmpty = (this.fields.length > 0);
            // TODO: move this sort of thing into a toTemplateJSON method on Dataset?
            modelData.fields = this.fields.map(function (field) {
                return field.toJSON();
            });
            // last header width = scroll bar - border (2px) */
            modelData.lastHeaderWidth = this.scrollbarDimensions.width - 2;
            return modelData;
        },

        render: function () {
            var self = this;
            this.fields = new recline.Model.FieldList(this.model.fields.filter(function (field) {
                return _.indexOf(self.state.get('hiddenFields'), field.id) == -1;
            }));

            this.scrollbarDimensions = this.scrollbarDimensions || this._scrollbarSize(); // skip measurement if already have dimensions
            var numFields = this.fields.length;
            // compute field widths (-20 for first menu col + 10px for padding on each col and finally 16px for the scrollbar)
            var fullWidth = self.$el.width() - 20 - 10 * numFields - this.scrollbarDimensions.width;
            var width = parseInt(Math.max(50, fullWidth / numFields), 10);
            // if columns extend outside viewport then remainder is 0 
            var remainder = Math.max(fullWidth - numFields * width, 0);
            this.fields.each(function (field, idx) {
                // add the remainder to the first field width so we make up full col
                if (idx === 0) {
                    field.set({
                        width: width + remainder
                    });
                } else {
                    field.set({
                        width: width
                    });
                }
            });
            var htmls = Mustache.render(this.template, this.toTemplateJSON());
            this.$el.html(htmls);
            this.model.records.forEach(function (doc) {
                var tr = $('<tr />');
                self.$el.find('tbody').append(tr);
                var newView = new my.GridRow({
                    model: doc,
                    el: tr,
                    fields: self.fields
                });
                newView.render();
            });
            // hide extra header col if no scrollbar to avoid unsightly overhang
            var $tbody = this.$el.find('tbody')[0];
            if ($tbody.scrollHeight <= $tbody.offsetHeight) {
                this.$el.find('th.last-header').hide();
            }
            this.$el.find('.recline-grid').toggleClass('no-hidden', (self.state.get('hiddenFields').length === 0));
            this.$el.find('.recline-grid tbody').scroll(this.onHorizontalScroll);
            return this;
        },

        // ### _scrollbarSize
        // 
        // Measure width of a vertical scrollbar and height of a horizontal scrollbar.
        //
        // @return: { width: pixelWidth, height: pixelHeight }
        _scrollbarSize: function () {
            var $c = $("<div style='position:absolute; top:-10000px; left:-10000px; width:100px; height:100px; overflow:scroll;'></div>").appendTo("body");
            var dim = {
                width: $c.width() - $c[0].clientWidth + 1,
                height: $c.height() - $c[0].clientHeight
            };
            $c.remove();
            return dim;
        }
    });

    // ## GridRow View for rendering an individual record.
    //
    // Since we want this to update in place it is up to creator to provider the element to attach to.
    //
    // In addition you *must* pass in a FieldList in the constructor options. This should be list of fields for the Grid.
    //
    // Example:
    //
    // <pre>
    // var row = new GridRow({
    //   model: dataset-record,
    //     el: dom-element,
    //     fields: mydatasets.fields // a FieldList object
    //   });
    // </pre>
    my.GridRow = Backbone.View.extend({
        initialize: function (initData) {
            _.bindAll(this, 'render');
            this._fields = initData.fields;
            this.listenTo(this.model, 'change', this.render);
        },

        template: ' \
      {{#cells}} \
      <td data-field="{{field}}" style="width: {{width}}px; max-width: {{width}}px; min-width: {{width}}px;"> \
        <div class="data-table-cell-content"> \
          <a href="javascript:{}" class="data-table-cell-edit _lngTrans_Translatable" title="Edit this cell">&nbsp;</a> \
          <div class="data-table-cell-value">{{{value}}}</div> \
        </div> \
      </td> \
      {{/cells}} \
    ',
        events: {
            'click .data-table-cell-edit': 'onEditClick',
            'click .data-table-cell-editor .okButton': 'onEditorOK',
            'click .data-table-cell-editor .cancelButton': 'onEditorCancel'
        },

        toTemplateJSON: function () {
            var self = this;
            var doc = this.model;
            var cellData = this._fields.map(function (field) {
                return {
                    field: field.id,
                    width: field.get('width'),
                    value: doc.getFieldValue(field)
                };
            });
            return {
                id: this.id,
                cells: cellData
            };
        },

        render: function () {
            this.$el.attr('data-id', this.model.id);
            var html = Mustache.render(this.template, this.toTemplateJSON());
            this.$el.html(html);
            return this;
        },

        // ===================
        // Cell Editor methods

        cellEditorTemplate: ' \
    <div class="menu-container data-table-cell-editor"> \
      <textarea class="data-table-cell-editor-editor" bind="textarea">{{value}}</textarea> \
      <div id="data-table-cell-editor-actions"> \
        <div class="data-table-cell-editor-action"> \
          <button type="submit" class="okButton primary ui-widget ui-state-default ui-corner-all ui-button-text-only ui-btn-general"><span class="ui-button-text _lngTrans_Translatable" title="">Update</span></button> \
          <button class="cancelButton btn danger _lngTrans_Translatable">Cancel</button> \
        </div> \
      </div> \
    </div> \
  ',

        onEditClick: function (e) {
            var editing = this.$el.find('.data-table-cell-editor-editor');
            if (editing.length > 0) {
                editing.parents('.data-table-cell-value').html(editing.text()).siblings('.data-table-cell-edit').removeClass("hidden");
            }
            $(e.target).addClass("hidden");
            var cell = $(e.target).siblings('.data-table-cell-value');
            cell.data("previousContents", cell.text());
            var templated = Mustache.render(this.cellEditorTemplate, {
                value: cell.text()
            });
            cell.html(templated);
        },

        onEditorOK: function (e) {
            var self = this;
            var cell = $(e.target);
            var rowId = cell.parents('tr').attr('data-id');
            var field = cell.parents('td').attr('data-field');
            var newValue = cell.parents('.data-table-cell-editor').find('.data-table-cell-editor-editor').val();
            var newData = {};
            newData[field] = newValue;
            this.model.set(newData);
            this.trigger('recline:flash', {
                message: "Updating row...",
                loader: true
            });
            this.model.save().then(function (response) {
                this.trigger('recline:flash', {
                    message: "Row updated successfully",
                    category: 'success'
                });
            })
                .fail(function () {
                    this.trigger('recline:flash', {
                        message: 'Error saving row',
                        category: 'error',
                        persist: true
                    });
                });
        },

        onEditorCancel: function (e) {
            var cell = $(e.target).parents('.data-table-cell-value');
            cell.html(cell.data('previousContents')).siblings('.data-table-cell-edit').removeClass("hidden");
        }
    });

})(jQuery, recline.View);
/*jshint multistr:true */

/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/


this.recline = this.recline || {};
this.recline.View = this.recline.View || {};

(function ($, my) {
    "use strict";
    // ## SlickGrid Dataset View
    //
    // Provides a tabular view on a Dataset, based on SlickGrid.
    //
    // https://github.com/mleibman/SlickGrid
    //
    // Initialize it with a `recline.Model.Dataset`.
    //
    // Additional options to drive SlickGrid grid can be given through state.
    // The following keys allow for customization:
    // * gridOptions: to add options at grid level
    // * columnsEditor: to add editor for editable columns
    //
    // For example:
    //    var grid = new recline.View.SlickGrid({
    //         model: dataset,
    //         el: $el,
    //         state: {
    //          gridOptions: {editable: true},
    //          columnsEditor: [
    //            {column: 'date', editor: Slick.Editors.Date },
    //            {column: 'title', editor: Slick.Editors.Text}
    //          ]
    //        }
    //      });
    //// NB: you need an explicit height on the element for slickgrid to work
    my.SlickGrid = Backbone.View.extend({
        initialize: function (modelEtc) {
            var self = this;
            this.$el.addClass('recline-slickgrid');
            var windowidth = $(window).height() - 186 + "px";
            if($('.maxView').length>0) {
                this.$el.css("height", $('#myModal').height() - 200);
            }
            else{
                this.$el.css("height", windowidth);
            }
            _.bindAll(this, 'render', 'onRecordChanged');

            this.listenTo(this.model, 'change', this.render);
            this.listenTo(this.model.records, 'add remove reset', this.render);
            this.listenTo(this.model.records, 'change', this.onRecordChanged);
            var state = _.extend({
                hiddenColumns: [],
                columnsOrder: [],
                columnsSort: {},
                columnsWidth: [],
                columnsEditor: [],
                options: {},
                fitColumns: true
            }, modelEtc.state);
            state.hiddenColumns = this.getHiddenColumns();
            state.columnsWidth = this.getColumnsWidth();

            this.state = new recline.Model.ObjectState(state);
            this._slickHandler = new Slick.EventHandler();
        },

        events: {},

        // Assyst need to comment
        getHiddenColumns: function () {
            var self = this;
            var hColumns = [];
            var hFields = $.grep(self.model.fields.toJSON(), function (el, i) {
                return el.isHidden == true
            });
            $.each(hFields, function (key, value) {
                hColumns.push(value.id);
            });
            return hColumns;
        },

        // Assyst need to comment
        getColumnsWidth: function () {
            var self = this;
            var wFields = $.grep(self.model.fields.toJSON(), function (el, i) {
                return $.isNumeric(el.weightage)
            });
            var wColumns = $.map(wFields, function (el, i) {
                return {
                    column: el.id,
                    width: parseFloat(el.weightage)
                }
            });
            return wColumns;
        },

        onRecordChanged: function (record) {
            // Ignore if the grid is not yet drawn
            if (!this.grid) {
                return;
            }

            // Let's find the row corresponding to the index
            var row_index = this.grid.getData().getModelRow(record);
            this.grid.invalidateRow(row_index);
            this.grid.getData().updateItem(record, row_index);
            this.grid.render();
        },

        render: function () {
            var self = this;
            var options = _.extend({
                enableCellNavigation: true,
                enableColumnReorder: true,
                explicitInitialization: true,
                syncColumnCellResize: true,
                forceFitColumns: this.state.get('fitColumns')
            }, self.state.get('gridOptions'));

            // We need all columns, even the hidden ones, to show on the column picker
            var columns = [];
            var childcolumns = [];
            // custom formatter as default one escapes html
            // plus this way we distinguish between rendering/formatting and computed value (so e.g. sort still works ...)
            // row = row index, cell = cell index, value = value, columnDef = column definition, dataContext = full row values           
            var flag = -1;
            var prevCount = 0;
            var sumTotalsFormatter = function (totals, columnDef, cell, typeOp, count) {
                if (prevCount != count) {
                    if (count % typeOp.length == 0) {
                        flag = -1;
                    }
                    prevCount = count;
                }
                if (cell == 0) flag++;
                var selectData = '';
                var type = typeOp[flag];
                if (type) {
                    if (cell == 0) {
                    var fieldproperty=self.model.fields.get(totals.group.field);
                    var fieldvalue=totals.group.value;
                    if(fieldproperty.attributes.format)
                    {
                        if(fieldproperty.attributes.format.datatype=="Date")
                        {
                            fieldvalue=  moment(totals.group.value).format(fieldproperty.attributes.format.format); 
                        }
                    }
                        if (type == "sum") selectData = '<span title="Total for '+ fieldvalue +'" class="coloRed">Total for ' + fieldvalue + '</span>';
                        if (type == "avg") selectData = '<span title="Avg for '+ fieldvalue +'" class="coloBlue">Avg for ' + fieldvalue + '</span>';
                        if (type == "min") selectData = '<span title="Min for '+ fieldvalue +'" class="coloGreen">Min for ' + fieldvalue + '</span>';
                        if (type == "max") selectData = '<span title="Max for '+ fieldvalue +'" class="coloViolet">Max for ' + fieldvalue + '</span>';
                    }

                    var valueT = totals[type][columnDef.field];
                    //formatting the sum based on the schema format
                    if (valueT != null) {
                        var value;
                        if(columnDef.format) 
                            value = _appFun.numberFormatFunction(columnDef.format, (Math.round(parseFloat(valueT) * 100) / 100)); // floating point issue 0.1+0.2 == 0.0000034. For avoid that we can multiply and divide the value with 100 
                        else 
                            value = (Math.round(parseFloat(valueT) * 100) / 100);
                        selectData += ("<span style='margin-left: 5px;'title=" +  value + "><b>" + value + "</b></span>");
                    }
                }
                return selectData;
            }

            columns = getColumns(this.model.fields.toJSON());
            
            function getColumns(fields, ischild) {
                var columnList = [];
                _.each(fields, function (field) {
                    var hastotal;
                    if(field.format) {
                        if(field.format.category=="Currency")
                            hastotal=true;
                        else
                            hastotal=false;
                    }
                    else 
                        hastotal=false;

                    var column = {
                        id: field.id,
                        name: field.label,
                        field: field.id,
                        type: field.type,
                        toolTip: field.description,
                        sortable: true,
                        minWidth: 80,
                        width: 80,
                        hasTotal: hastotal,
                        format:field.id.indexOf("_COUNT")==-1?field.format:null,
                        cssClass: getClass(field.type),
                        formatter: slickFormatter,
                        header: field.formula ? {
                            buttons: [{
                                cssClass: "icon-highlight-off",
                                command: "toggle-highlight",
                                tooltip: "Derived column"
                            }]
                        } : {},
                        formula: field.formula,
                        groupTotalsFormatter: sumTotalsFormatter /*(field.type== "integer"||field.type== "numeric"||field.type== "number")?sumTotalsFormatter:undefined*/,
                        isHidden: ischild || field.isHidden,
                        headerCssClass: getClass(field.type),
                        children: getChildColumns(field.children)
                    };
                    if (ischild) childcolumns.push(field.id);

                    function getChildColumns(subcolumns) {
                        if (typeof subcolumns === "undefined" || !$.isArray(subcolumns) || subcolumns.length == 0) return undefined;
                        var mapcolumns = $.grep(self.model.fields.toJSON(), function (el, i) {
                            return $.inArray(el.id, subcolumns) > -1;
                        })
                        return (mapcolumns.length > 0) ? getColumns(mapcolumns, true) : undefined;
                    }

                    // Modified by Assyst (Rony at 07-Feb-2014)    
                    // For text align based on data type
                    /******************************************/
                    function getClass(type) {
                        var className = '';
                        switch (type) {
                            case 'number':
                            case 'integer':
                            case 'numeric':
                                className = 'align-right'
                                break;
                            case 'date':
                            case 'datetime':
                            case 'date-time':
                            case 'timestamp':
                            case 'geo_point':
                            case 'geo':
                                className = 'align-center'
                                break;
                            default:
                                className = 'align-left'
                                break;
                        }
                        return className;
                    }
                    /***************End*********************/

                    var widthInfo = _.find(self.state.get('columnsWidth'), function (c) {
                        return c.column === field.id;
                    });
                    if (widthInfo) {
                        if (column.minWidth > widthInfo.width) column.minWidth = widthInfo.width;
                        column.width = widthInfo.width;
                    }

                    var editInfo = _.find(self.state.get('columnsEditor'), function (c) {
                        return c.column === field.id;
                    });
                    if (editInfo) {
                        column.editor = editInfo.editor;
                    }
                    columnList.push(column);
                });
                return columnList;
            }

            for (var i = 0; i < columns.length; i++) {
                if ($.inArray(columns[i].id, childcolumns) > -1) {
                    columns[i].isHidden = true;
                    columns[i].sortable = false;
                }
                if (columns[i].children && columns[i].children.length > 0) columns[i].sortable = false;
            }

            // Restrict the visible columns
            var visibleColumns = _.filter(columns, function (column) {
                return (_.indexOf(self.state.get('hiddenColumns'), column.id) === -1 && $.inArray(column.id, childcolumns) == -1);
            });

            // Order them if there is ordering info on the state
            if (this.state.get('columnsOrder') && this.state.get('columnsOrder').length > 0) {
                visibleColumns = visibleColumns.sort(function (a, b) {
                    return _.indexOf(self.state.get('columnsOrder'), a.id) > _.indexOf(self.state.get('columnsOrder'), b.id) ? 1 : -1;
                });
                columns = columns.sort(function (a, b) {
                    return _.indexOf(self.state.get('columnsOrder'), a.id) > _.indexOf(self.state.get('columnsOrder'), b.id) ? 1 : -1;
                });
            }

            // Move hidden columns to the end, so they appear at the bottom of the
            // column picker
            // Transform a model object into a row
            function toRow(m) {
                var row = {};
                self.model.fields.each(function (field) {
                    row[field.id] = m.getFieldValue(field);
                });
                return row;
            }

            function RowSet() {
                var models = [];
                var rows = [];

                this.push = function (model, row) {
                    models.push(model);
                    rows.push(row);
                };

                this.getLength = function () {
                    return rows.length;
                };
                this.getItem = function (index) {
                    return rows[index];
                };
                this.getItemMetadata = function (index) {
                    return {};
                };
                this.getModel = function (index) {
                    return models[index];
                };
                this.getModelRow = function (m) {
                    return _.indexOf(models, m);
                };
                this.updateItem = function (m, i) {
                    rows[i] = toRow(m);
                    models[i] = m;
                };
            }

            var dataViewRecord = [];
            this.model.records.each(function (doc) {
                dataViewRecord.push(doc.attributes);
            });

            _.each(dataViewRecord, function (val, i) {
                val['id'] = i;
            });

            if (dataViewRecord.length == 0) {
                var emptyRow = {};
                emptyRow[visibleColumns[0].field] = "No Records";
                emptyRow["id"] = 'noRecords';
                for (i = 1; i < visibleColumns.length; i++) {
                    emptyRow[visibleColumns[i].field] = "";
                }
                dataViewRecord.push(emptyRow);
            }

            var gridOptions = {
                currency: true,
                grandTotal: true
            };

            var groupItemMetadataProvider = new Slick.Data.GroupItemMetadataProvider();
            self.dataView = new Slick.Data.DataView({
                inlineFilters: true,
                groupItemMetadataProvider: groupItemMetadataProvider
            });
            self.dataView.setItems(dataViewRecord, 'id');
            if (self.model.queryState.get('groupings').selectedGroups && self.model.queryState.get('groupings').selectedGroups.length > 0) {
                gridOptions.grandTotal = false;
                var groupedData = self.groupRowsBy(self.model.queryState.get('groupings').selectedAggs, self.model.queryState.get('groupings').selectedGroups);
                self.dataView.setGrouping(groupedData);
                $(document).on("click", ".grpSpanCollapse span", function (e) {
                    self.dataView.collapseAllGroups();
                });
                $(document).on("click", ".grpSpanExpand span", function (e) {
                    self.dataView.expandAllGroups();
                });
            }

            if ($(this.el).data('slickGridFormula')) {
                $(this.el).slickGridFormula('destroy');
            }

            $(this.el).slickGridFormula({
                data: self.dataView,
                columns: visibleColumns,
                gridOptions: options,
                formulaGrid:self.model.get("formulaGrid"),
                grandTotal: gridOptions.grandTotal,
                contextMenu: true,
                callback: callRecline
            });

            function callRecline(schema, contextmenu, data) {             
                if ($('#dataReclineview').length > 0) {  
                    var catalogSchema = JSON.parse(self.model.get('catalog').schema);
                    var esQuery = self.model.get('catalog').savedFilters;                  
                    switch (contextmenu) {
                        case "delete":
                            delete catalogSchema.computedFields[schema.id];
                            self.model.get('catalog').schema = JSON.stringify(catalogSchema);
                        break;

                        case "edit":
                            var sameItem = _.findWhere(catalogSchema.computedFields, {
                                id: schema.fieldname
                            });
                            if (sameItem) {
                                delete catalogSchema.computedFields[schema.fieldname];
                                self.model.get('catalog').schema = JSON.stringify(catalogSchema);
                                var returnSchema = self.createSchema(schema);
                                if (!catalogSchema["computedFields"])
                                    catalogSchema["computedFields"] = {};
                                catalogSchema.computedFields[returnSchema.id] = returnSchema;
                                self.model.get('catalog').schema = JSON.stringify(catalogSchema);

                            }
                            break;
                        case "create":
                            if (schema) {
                                var returnSchema = self.createSchema(schema);
                                if (!catalogSchema["computedFields"])
                                    catalogSchema["computedFields"] = {};
                                catalogSchema.computedFields[returnSchema.id] = returnSchema;
                                self.model.get('catalog').schema = JSON.stringify(catalogSchema);
                            }
                            break;

                        case "hide":

                            break;
                    }
                    LoadRecline(esQuery, data, [], "#dataReclineview", self.model.get('catalog'),"grid");
                }
                else {
                    //creating an instance of reclineviewer and changing its options and functions.
                    var reclineViewer = $('.reportTabs').data('reclineviewer');
                    var catalogSchema = JSON.parse(reclineViewer.options.catalog.schema);
                    reclineViewer.viewer.setDefault('grid');
                    switch (contextmenu) {
                        case "delete":
                            delete (catalogSchema.computedFields[schema.id]);   
                            reclineViewer.options.catalog.schema = JSON.stringify(catalogSchema);
                            var delFields = [];
                            _.each(reclineViewer.schema.misSchema.fields,function(val, i){
                                if(schema.name != val.name) delFields.push(val);
                            });
                            reclineViewer.schema.misSchema.fields = delFields;                                                  
                        break;
                        case "edit":
                            if (schema) {
                                var sameItem = _.findWhere(catalogSchema.computedFields, {
                                    id: schema.fieldname
                                });
                                if (sameItem) {
                                    delete catalogSchema.computedFields[schema.fieldname];                                 
                                    
                                    var position = self.model.get('fields').getIndexBy("name", sameItem.name); //positon of the selected field
                                    self.model.get('fields').splice(position, 1); //removing the selected item from schema of fields
                                    reclineViewer.options.catalog.schema = JSON.stringify(catalogSchema);                                
                                    var field = {
                                        id: schema.id,
                                        fieldname:schema.name,
                                        label: schema.name,
                                        description: schema.name,
                                        type: schema.type,
                                        isHidden: schema.isHidden,
                                        isFilterable: 'Y',
                                        filter:true,
                                        format: schema.format,
                                        formula: schema.formula,
                                        header: schema.header,
                                        isAxis: "Y",
                                        isKPI: 'N',
                                        isTimeSeries: "N",
                                        isGeo: 'N',
                                        aggType: "matrix",
                                        decimalPoint: schema.decimalPoint
                                    };                       
                                    var newSchema = self.model.get('fields');
                                    var temp;
                                     _.each(newSchema, function (val, i) {
                                        if (field.fieldname == val.id) {
                                            temp = i;
                                        }
                                    });
                                    _.each(newSchema, function (val, i) {
                                        if (field.formula.positionId == val.id) {
                                            if(temp==undefined)
                                                newSchema.push(field);
                                            else
                                                newSchema[temp] = field;
                                            catalogSchema.computedFields[field.id] = self.createSchema(schema);                                    
                                            reclineViewer.options.catalog.schema = JSON.stringify(catalogSchema);
                                        }
                                    });
                                }
                            }
                            break;
                        case "create":
                            if (schema) {
                                var field = {
                                    id: schema.id,
                                    fieldname:schema.fieldname,
                                    label: schema.name,
                                    description: schema.name,
                                    type: schema.type,
                                    isHidden: schema.isHidden,
                                    isFilterable: 'Y',
                                    filter:true,
                                    format: schema.format,
                                    formula: schema.formula,
                                    header: schema.header,
                                    isAxis: "Y",
                                    isKPI: 'N',
                                    isTimeSeries: "N",
                                    isGeo: 'N',
                                    aggType: "matrix",
                                    decimalPoint: schema.decimalPoint
                                };                       
                                var newSchema = self.model.get('fields');
                                var temp;
                                 _.each(newSchema, function (val, i) {
                                    if (field.fieldname == val.id) {
                                        temp=i;
                                    }
                                });
                                _.each(newSchema, function (val, i) {
                                    if (field.formula.positionId == val.id) {
                                        if(temp==undefined)
                                            newSchema.push(field);
                                        else
                                            newSchema[temp] = field;

                                        if (!catalogSchema["computedFields"])
                                            catalogSchema["computedFields"] = {};
                                        catalogSchema.computedFields[field.id] = self.createSchema(schema);                                    
                                        reclineViewer.options.catalog.schema = JSON.stringify(catalogSchema);
                                    }
                                });
                            }
                        break;             
                    }
                    reclineViewer._render();
                }
            }

            this.grid = $(this.el).slickGridFormula('returnGrid');

            // Column sorting
            var sortInfo = this.model.queryState.get('sort');
            if (sortInfo) {
                var column = sortInfo[0].field;
                var sortAsc = sortInfo[0].order !== 'desc';
                this.grid.setSortColumn(column, sortAsc);
            }

            self.dataView.onRowCountChanged.subscribe(function (e, args) {
                self.grid.updateRowCount(args.rows);
                self.grid.render();
                if (self.model.queryState.get('groupings').selectedGroups == undefined) {
                    self.grid.resizeCanvas();
                    totalsPlugin.render();
                }
            });

            self.dataView.onRowsChanged.subscribe(function (e, args) {
                self.grid.invalidateRows(args.rows);
                self.grid.render();
                if (self.model.queryState.get('groupings').selectedGroups == undefined) {
                    var totalsPlugin = new TotalsPlugin($.getScrollbarWidth());
                    self.grid.resizeCanvas();
                    totalsPlugin.render();
                }
            });

            this._slickHandler.subscribe(this.grid.onSort, function (e, args) {
                var order = (args.sortAsc) ? 'asc' : 'desc';
                var sort = [{
                    field: args.sortCol.field,
                    order: order
                }];
                self.grid.invalidateRows(args.rows);
                self.model.query({
                    sort: sort
                });
                if (self.model.queryState.get('groupings').selectedGroups == undefined) {
                    self.grid.resizeCanvas();
                    var totalsPlugin = new TotalsPlugin($.getScrollbarWidth());
                    self.grid.registerPlugin(totalsPlugin);
                }
            });

            this._slickHandler.subscribe(this.grid.onColumnsReordered, function (e, args) {
                self.state.set({
                    columnsOrder: _.pluck(self.grid.getColumns(), 'id')
                });
                if (self.model.queryState.get('groupings').selectedGroups == undefined) {
                    self.grid.resizeCanvas();
                    var totalsPlugin = new TotalsPlugin($.getScrollbarWidth());
                    self.grid.registerPlugin(totalsPlugin);
                }
            });

            this.grid.onColumnsResized.subscribe(function (e, args) {
                var columns = args.grid.getColumns();
                var defaultColumnWidth = args.grid.getOptions().defaultColumnWidth;
                var columnsWidth = [];
                _.each(columns, function (column) {
                    if (column.width != defaultColumnWidth) {
                        columnsWidth.push({
                            column: column.id,
                            width: column.width
                        });
                    }
                });
                self.state.set({
                    columnsWidth: columnsWidth
                });
            });

            this._slickHandler.subscribe(this.grid.onCellChange, function (e, args) {
                // We need to change the model associated value
                //
                var grid = args.grid;
                var model = data.getModel(args.row);
                var field = grid.getColumns()[args.cell].id;
                var v = {};
                v[field] = args.item[field];
                model.set(v);
            });

            if (self.visible) {
                self.grid.init();
                self.rendered = true;                 
                if (self.model.queryState.get('groupings').selectedGroups == undefined || self.model.queryState.get('groupings').selectedGroups.length ==0)
                {
                        self.grid.resizeCanvas();
                        self.grid.registerPlugin(new TotalsPlugin($.getScrollbarWidth()));
                }
                else {
                    // Defer rendering until the view is visible
                    self.rendered = false;
                }
            }

            return this;
        },

        createSchema: function (schema) {
            var self = this;
            if (schema) {
                var currentItem = {
                    aggregate: false,
                    dataType: "numeric",
                    decimalPoint: 2,
                    defaultValue: "",
                    description: schema.name,
                    filter: false,
                    id: schema.id,
                    isAxis: "Y",
                    isGeo: "N",
                    isHidden: "N",
                    isKPI: "N",
                    isTimeSeries: "N",
                    label: schema.name,
                    name: schema.name,
                    metrics: false,
                    title: schema.name,
                    type: "numeric",
                    formula: schema.formula,
                    format: schema.format
                };                
            }
            return currentItem;
        },

        groupRowsBy: function (aggs, groups) {
            var ar = [],
                prow = {};
            var self = this;
            for (var i = 0; i < groups.length; i++) {
                if (aggs.length > 0) {
                    prow["getter"] = groups[i].fieldName;
                    prow["getterName"] = groups[i].fieldName;
                    prow["formatter"] = function (g) {
                        var itemCount = (g.value == "" || g.value == "No Records") ? 0 : g.count;
                        var fields = self._getFields(g.title);
                        var gValue = (fields.type == "date" || fields.type == "date-time") ? moment(g.value).format(fields.format.format) : g.value;
                        return fields.name + ":  " + gValue + "  <span style='color:green'>(" + itemCount + " items)</span>";
                    };
                    prow["aggregateCollapsed"] = true;
                    prow["lazyTotalsCalculation"] = true;
                    var aggregators = [];
                    for (var j = 0; j < aggs.length; j++) {
                        switch (aggs[j].operator.toLowerCase()) {
                            case "sum":
                                aggregators.push(new Slick.Data.Aggregators.Sum(aggs[j].fieldName));
                                break;

                            case "avg":
                                aggregators.push(new Slick.Data.Aggregators.Avg(aggs[j].fieldName));
                                break;

                            case "min":
                                aggregators.push(new Slick.Data.Aggregators.Min(aggs[j].fieldName));
                                break;

                            case "max":
                                aggregators.push(new Slick.Data.Aggregators.Max(aggs[j].fieldName));
                                break;
                        }

                    }
                    prow["aggregators"] = aggregators;
                    ar.push($.extend({}, prow, prow));
                } else {
                    ar[i] = {
                        getter: groups[i].fieldName,
                        getterName: groups[i].fieldName,
                        formatter: function (g) {
                            var itemCount = (g.value == "" || g.value == "No Records") ? 0 : g.count;
                            var fields = self._getFields(g.title);
                            var gValue = (fields.type == "date" || fields.type == "date-time") ? moment(g.value).format(fields.format.format) : g.value;
                            return fields.name + ":  " + gValue + "  <span style='color:green'>(" + itemCount + " items)</span>";
                        },
                        aggregateCollapsed: true,
                        lazyTotalsCalculation: true
                    };
                }
            }
            return ar;
        },

        _getFields: function (id) {
            var self = this;
            return _.findWhere(self.model.fields.toJSON(), {
                id: id
            });
        },

        remove: function () {
            this._slickHandler.unsubscribeAll();
            Backbone.View.prototype.remove.apply(this, arguments);
        },

        show: function () {
            // If the div is hidden, SlickGrid will calculate wrongly some
            // sizes so we must render it explicitly when the view is visible
            if (!this.rendered) {
                if (!this.grid) {
                    this.render();
                }
                this.grid.init();
                this.rendered = true;
            }
            this.visible = true;
        },

        hide: function () {
            this.visible = false;
        }
    });
})(jQuery, recline.View);

Array.prototype.getIndexBy = function (name, value) {
    for (var i = 0; i < this.length; i++) {
        if (this[i][name] == value) {
            return i;
        }
    }
    return -1;
}