/*+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+
   
JQuery Immediately-Invoked Function for recline Timeseries Graph
Created By      : Rony T S for Assyst International Pvt. Ltd.
Created Date    : 10-Oct-2014
Version         : 1.0.2
Project         : PanAgro

+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+-+*/

this.recline = this.recline || {};
this.recline.View = this.recline.View || {};
(function ($, my) {
    "use strict";
    // ## Graph view for a Dataset using Flot graphing library.
    //
    // Initialization arguments (in a hash in first parameter):
    //
    // * model: recline.Model.Dataset
    // * state: (optional) configuration hash of form:
    //
    //        {
    //          group: {column name for x-axis},
    //          series: [{column name for series A}, {column name series B}, ... ],
    //          // options are: lines, points, lines-and-points, bars, columns
    //          defaultGraph: 'lines',
    //          graphOptions: {custom [flot options]}
    //        }
    //
    // NB: should *not* provide an el argument to the view but must let the view
    // generate the element itself (you can then append view.el to the DOM.
   
    my.TimeseriesGraph = Backbone.View.extend({
        template: ' \
    <div class="recline-flot"> \
       <div id="timeserieschart_{{viewId}}" class="panel graph timeseries" >\
        {{#enableLegend}}<div title="Show legend" class="btnLegend showLegend"> <div class="legendDiv">Legend</div></div> {{/enableLegend}}\
        {{#dashboard}}\
           <div title="Show legend" class="dashLegend showDashLegend"></div>\
           <div title="Show Controls" class="dashControl showDashControl"></div>\
       {{/dashboard}}\
       <svg  preserveAspectRatio="none">\
        </svg>\
      </div> \
    </div> \
',
         events: {
            'click .btnLegend': 'onLegendClick',
            'click .dashLegend':'onDashLegendClick',
            'click .dashControl':'onDashControlClick'
          },

        initialize: function (options) {
            var self = this;
            
            //this.graphColors = ["#edc240", "#afd8f8", "#cb4b4b", "#4da74d", "#9440ed"];
            this.graphColors = ["##E5EAEC","#9292C6","#D97AB2","#9869AD","#85B1E0","#3D9EDF","#F89A1C","#FFC718","#BCBF32","#69C079"];
             this.graphTypes =[
            //{ key:'multiBar', name:'Multi Bar', xstring:true,sliced:"Multi Bar" },
            //{ key:'multiBarHorizontal', name:'Multi Bar Horizontal', xstring:true,sliced:"Multi Bar Horizontal" },
            //{ key:'discreteBar', name:'Discrete Bar', xstring:true,sliced:"Discrete Bar" },
            { key:'line', name:'Line', xstring:false,sliced:"Line" },
            //{ key:'linePlusBar', name:'Line Plus Bar', xstring:false,sliced:"Line Plus Bar" },
            //{ key:'cumulativeLine', name:'Cumulative Line', xstring:false,sliced:"Cumulative Line" },
            //{ key:'lineWithFocus', name:'Line With Focus', xstring:false,sliced:"Line With Focus" },
            //{ key:'scatter', name:'Scatter', xstring:false,sliced:"Scatter" },
            //{ key:'scatterPlusLine', name:'Scatter Plus Line', xstring:false,sliced:"Scatter Plus Line" },
            { key:'stackedArea', name:'Stacked Area', xstring:false,sliced:"Stacked Area" }//,
            //{ key:'pie', name:'Pie', xstring:true,sliced:"Pie" },
            //{ key:'donut', name:'Donut', xstring:true,sliced:"Donut" }
            ];
            this.pivotData = [];
            this.uid = options.id || ("" + new Date().getTime() + Math.floor(Math.random() * 10000)); // generating an unique id for the chart
            this.el = $(this.el);     
            _.bindAll(this, 'render', 'redraw');
            this.needToRedraw = true;
            this.listenTo(this.model, 'change', function () { this.render(); this.redraw(); });
            this.listenTo(this.model.timeState, 'change', function () { this.render(); this.redraw(); });            
            this.listenTo(this.model.fields, 'reset add', function () { this.render(); this.redraw(); });
            this.listenTo(this.model.records, 'reset add', function () { this.render(); this.redraw(); });

            this.dateFields = $.grep(this.model.toTemplateJSON().timeSeries, function (el, i) { return (el.type  === 'date' || el.type  === 'date-time' || el.type  === 'time')});
          
           
            if(options.state.graphTypes && options.state.graphTypes.length > 0){
                this.graphTypes =  $.grep(self.graphTypes ,function(value,index){ return ($.inArray( value.key, options.state.graphTypes) > -1 ); }); 
            }
            if( typeof _.findWhere( this.graphTypes, {key : options.state.graphType}) === 'undefined') options.state.graphType = "multiBar" ;
            if( typeof _.findWhere( this.model.toTemplateJSON().fields, {id : options.state.group}) === 'undefined') {
                 var tmplData = this.model.toTemplateJSON();
            }

            var stateData = _.extend({
                group: null,
                // so that at least one series chooser box shows up
                series: [],
                area: [],
                bar: [],
                graphType: 'multiBar',
                showLegent: false,
                showControl: false,
                minLegend: 10,
                dashboard:this.$el.parents('.content_div').length>0?true:false,
                enableLegend: true,
                compareField: [],
                selectedFields:[],
                graphState:[]
            }, options.state);
                        
            this.state = new recline.Model.ObjectState(stateData);           
            this.editor = new my.TimeseriesControls({
                model: this.model,
                state: this.state.toJSON(),
                graphTypes: this.graphTypes,
                dateFields: this.dateFields
            });
           
           
           this.listenTo(this.editor.state, 'change', function () {
                var state = $.extend({}, self.editor.state.toJSON(), { showLegent: self.state.get('showLegent') });
                self.state.set(state);
                //self.render();
                //self.redraw();
            });
            this.listenTo(this.state, 'change', function () {
                self.render();
                self.redraw();
            });

            this.elSidebar = this.editor.$el;
        },

        render: function () {
            var self = this;
            var tmplData = this.model.toTemplateJSON();
            tmplData["viewId"] = this.uid;     
            tmplData["enableLegend"]= self.state.get('enableLegend');
            tmplData["dashboard"]= self.state.get('dashboard');    
                 
            if (this.state.attributes.width)
            	tmplData.width = this.state.attributes.width;

            if (this.state.attributes.height)
            	tmplData.height = this.state.attributes.height;
            var htmls = Mustache.render(this.template, tmplData);
            this.$el.empty();
            this.$el.html(htmls);
            this.$graph = this.$el.find('.panel.graph');
            // Dashlet Viewer height adjustment
            var parent = this.$el.parents('.content_div');
            if (parent.context != undefined) this.$graph.height(parent.height());
            else this.$graph.height(parent.height());
            //end
            if (self.state.get('showLegent'))  {
                this.$el.find('.btnLegend').show().addClass('hideLegend').removeClass('showLegend').attr("title", "Hide legend");
                this.$el.find('.dashLegend').show().addClass('hideDashLegend').removeClass('showDashLegend').attr("title", "Hide legend");
            } else {
                this.$el.find('.btnLegend').show().addClass('showLegend').removeClass('hideLegend').attr("title", "Show legend");
                this.$el.find('.btnLegend').show().addClass('showDashLegend').removeClass('hideDashLegend').attr("title", "Show legend");
            }
            
            if (this.state.get("graphType") == "line")  this.$el.find('.dashControl').hide();
            else if(self.state.get('showControl')){
                this.$el.find('.dashControl').show().addClass('hideDashControl').removeClass('showDashControl').attr("title", "Hide Controls");
            }
            else{
                this.$el.find('.dashControl').show().addClass('showDashControl').removeClass('hideDashControl').attr("title", "Show Controls");
            }

            return this;
        },

        remove: function () {
            this.editor.remove();
            Backbone.View.prototype.remove.apply(this, arguments);
        },

         onLegendClick:function (e) {
            var self= this;
            var element = $(e.currentTarget);
            self.state.set({ showLegent: element.hasClass("showLegend") });
        },

        onDashLegendClick:function (e) {
            var self= this;
            var element = $(e.currentTarget);
            self.state.set({ 
                showLegent: element.hasClass("showDashLegend"),
                showControl:false
            });
        },

        onDashControlClick:function (e) {
            var self= this;
            var element = $(e.currentTarget);
            self.state.set({ 
                showControl: element.hasClass("showDashControl"),
                showLegent:false 
            });
        },

        redraw: function () {
        
            var self = this;
            // There are issues generating a Flot graph if either:
            // * The relevant div that graph attaches to his hidden at the moment of creating the plot -- Flot will complain with
            //   Uncaught Invalid dimensions for plot, width = 0, height = 0
            // * There is no data for the plot -- either same error or may have issues later with errors like 'non-existent node-value'
            var areWeVisible = !jQuery.expr.filters.hidden(this.el);
            if ((!areWeVisible || this.model.records.length === 0)) {
                this.needToRedraw = true;
                return;
            }
            // check we have something to plot
            if (this.state.get('group') && this.state.get('series')) {
                var graphType = this.state.get("graphType");
                var series = (graphType === "pie" || graphType ==="donut" ) ?  this.createPieSeries(): this.createSeries();
                self.series = series;
                if(this.state.get('dashboard')) var showLegend = false;
                else var showLegend = series.length <= this.state.get('minLegend'); 
                nv.addGraph(function() {
                    var chart =self._getTimeseriesGraph(graphType, showLegend);                   
                    if(self.series.length==1)chart.yAxis.axisLabel(series[0].key);
                    if($('#btnDataViewerSaveAs').length>0)
                    {
                        self.series.length==0?$('#btnDataViewerSaveAs').addClass("disableClass"):$('#btnDataViewerSaveAs').removeClass("disableClass");
                    }
                    else{
                        var $saveButton=$('#timeserieschart_' + self.uid + ' svg').parents('.recline-data-explorer').find('.header .menu-right .dashlet,.header .menu-right .exprt_btn');
                        self.series.length==0?$saveButton.addClass("disableClass"):$saveButton.removeClass("disableClass");
                    }
                    //if($('.col').length==0)$('svg').empty();//Nithin's change
                    $('#timeserieschart_' + self.uid+' svg').append('<rect width="100%" height="100%" fill="#fff"></rect>');
                    d3.select('#timeserieschart_' + self.uid+' svg')
                        .datum(series)
                        .transition().duration(500)
                        .call(chart);
                    nv.utils.windowResize(chart.update);
                    return chart;
                });
            }
        },

        show: function () {
            // because we cannot redraw when hidden we may need to when becoming visible
            if (this.needToRedraw) {
                this.redraw();
            }
        },

        _getTimeseriesGraph: function(type, showLegend){
            var self = this;
            var chart;
            switch(type){
                case "multiBar":                
                    chart = nv.models.multiBarChart().groupSpacing(0.5);                    
                    if(self.xvaluesAreString){
                       chart.reduceXTicks(false);
                       chart.color(d3.scale.ordinal().range(["#3A9DD6","#85B1E0","#9869AD","#D97AB2","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                       if(this.$el.parents('.content_div').length <=0)
                       chart.margin({top: 30, right: 20, bottom: 120, left: 100});
                       //chart.staggerLabels(true);
                       //chart.xAxis.rotateLabels(-60);
                       chart.yAxis.tickFormat(d3.format('.2s'));
                       chart.yAxis.width(90);
                       chart.xAxis.rotateLabels(-35);
                     }
                break;
                case "multiBarHorizontal":
                    chart = nv.models.multiBarHorizontalChart();
                    chart.color(d3.scale.ordinal().range(["#3A9DD6","#85B1E0","#9869AD","#D97AB2","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                    if(this.$el.parents('.content_div').length <=0)
                    chart.margin({top: 30, right: 20, bottom: 120, left: 100});
                    //chart.showValues(true);
                    chart.tooltips(true);
                    chart.yAxis.tickFormat(d3.format('.2s'));
                    chart.xAxis.rotateLabels(-35);
                break;                
                case "discreteBar":
                    chart = nv.models.discreteBarChart();
                    chart.staggerLabels(false);
                    chart.color(d3.scale.ordinal().range(["#0A4979","#077ABB","#006D67","#BC4388","#2E3E86","#761E5B","#35A548","#7E7A30","#B6762A","#B94326","#69C079"]).range());
                    if(this.$el.parents('.content_div').length <=0)
                    chart.margin({top: 30, right: 20, bottom: 120, left: 100});
                    chart.tooltips(true);
                    chart.showValues(true);
                    chart.xAxis.rotateLabels(-35);
                break;
                case "historicalBar":
                    chart = nv.models.historicalBarChart();
                    chart.showXAxis(true);
                    chart.showYAxis(true);
                    chart.rightAlignYAxis(true);
                    chart.margin({right: 90});
                    chart.xAxis.rotateLabels(-35);
                break;
                case  "line":
                    chart = nv.models.lineChart();
                    chart.color(d3.scale.ordinal().range(["#3A9DD6","#D97AB2","#9869AD","#85B1E0","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                    if(self.state.get("dashboard")){
                        if(self.state.get('showLegent')){ 
                            chart.margin({top: 10, right: 20, bottom: 58, left: 80});
                            chart.legend.margin({top: 0, right: 0, bottom: 0, left: 0});
                        }
                        else{
                            chart.margin({top: 10, right: 20, bottom: 58, left: 80});
                        }
                    }
                    else chart.margin({top: 10, right: 20, bottom: 80, left: 80});
                    chart.useInteractiveGuideline(true);
                    chart.xAxis.rotateLabels(-35);
                    chart.yAxis.tickFormat(d3.format('.2s'));
                break;
                case "linePlusBar":
                    chart = nv.models.linePlusBarChart();
                    chart.color(d3.scale.ordinal().range(["#3A9DD6","#85B1E0","#9869AD","#D97AB2","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                    if(this.$el.parents('.content_div').length <=0)
                    chart.margin({top: 30, right: 20, bottom: 120, left: 100});
                    chart.xAxis.showMaxMin(false);
                    chart.bars.forceY([0]);
                    chart.xAxis.rotateLabels(-35);
                break;
                case "cumulativeLine":
                    chart = nv.models.cumulativeLineChart();
                    chart.useInteractiveGuideline(true);
                    chart.color(d3.scale.ordinal().range(["#3A9DD6","#85B1E0","#9869AD","#D97AB2","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                    if(this.$el.parents('.content_div').length <=0)
                    chart.margin({top: 30, right: 20, bottom: 120, left: 100});
                    chart.yAxis.tickFormat(d3.format(',.1%'));
                break;
                case "lineWithFocus":
                    chart = nv.models.lineWithFocusChart();
                    chart.yAxis.tickFormat(d3.format(',.2f'));
                    chart.color(d3.scale.ordinal().range(["#3A9DD6","#85B1E0","#9869AD","#D97AB2","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                    if(this.$el.parents('.content_div').length <=0)
                    chart.margin({top: 30, right: 20, bottom: 120, left: 100});
                    chart.y2Axis.tickFormat(d3.format(',.2f'));
                    chart.xAxis.rotateLabels(-35);
                break;
                case "scatter":
                    chart = nv.models.scatterChart();
                    chart.color(d3.scale.ordinal().range(["#3A9DD6","#85B1E0","#9869AD","#D97AB2","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                    if(this.$el.parents('.content_div').length <=0)
                    chart.margin({top: 30, right: 20, bottom: 120, left: 100});
                    chart.showDistX(true);
                    chart.showDistY(true);
                    chart.tooltipContent(function(key) { return "<h3>" + key + "</h3>";});
                    chart.xAxis.rotateLabels(-35);
                break;
                case "scatterPlusLine":
                    chart = nv.models.scatterPlusLineChart();
                    chart.showDistX(true);
                    chart.color(d3.scale.ordinal().range(["#3A9DD6","#85B1E0","#9869AD","#D97AB2","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                    if(this.$el.parents('.content_div').length <=0)
                    chart.margin({top: 30, right: 20, bottom: 120, left: 100});
                    chart.showDistY(true);
                    chart.tooltipContent(function(key) { return "<h3>" + key + "</h3>";});
                    chart.xAxis.rotateLabels(-35);
                break;
                case "stackedArea":
                    chart = nv.models.stackedAreaChart();
                    chart.clipEdge(true);
                    chart.color(d3.scale.ordinal().range(["#3A9DD6","#D97AB2","#9869AD","#85B1E0","#9292C6","#45C3D2","#69C079","#FFC718","#F89A1C","#BCBF32","#69C079"]).range());
                    if(self.state.get("dashboard")){
                        if(self.state.get('showLegent')){ 
                            chart.margin({top: 10, right: 20, bottom: 58, left: 80});
                            chart.legend.margin({top: 0, right: 0, bottom: 0, left: 0});
                        }
                        else{
                            chart.margin({top: 10, right: 20, bottom: 58, left: 80});
                        }
                    }
                    else chart.margin({top: 10, right: 20, bottom: 80, left: 80});
                    if($.inArray( "Stacked", this.state.attributes.graphState )==-1) chart.style("stack");
                    else if($.inArray( "Stream", this.state.attributes.graphState )==-1) chart.style("stream");
                    else chart.style("expand");
                    chart.useInteractiveGuideline(true);
                    chart.xAxis.rotateLabels(-35);
                    chart.yAxis.tickFormat(d3.format('.2s'));
                break;
                case "pie":
                    chart = nv.models.pieChart();
                    chart.showLabels(false);
                    chart.color(d3.scale.ordinal().range(this.graphColors).range());
                    chart.margin({top: 0, right: 0, bottom: 0, left: 30 });
                break;
                case "donut":
                    chart = nv.models.pieChart();
                    chart.showLabels(false);
                    chart.donut(true);
                    chart.color(d3.scale.ordinal().range(this.graphColors).range());
                    chart.donutRatio(0.35);
                break;
            } 
             //chart.color(d3.scale.category10().range());
             
             if(type != "pie" && type != 'donut'){
                var xName = this.model.fields.get(this.state.attributes.group).get('name');
                chart.xAxis.axisLabel(xName);
             }
             if(self.xvaluesAreIndex){
                //chart.showXAxis(false);
                chart.xAxis.tickFormat(function(d) { 
                    return self.xaxisLabels[d];//.replace(/\s/g,"%3Cbr/%3");
                });      
             } 

             if(self.xvaluesAreString){
                 /*if(type != "pie" && type != 'donut'){
                    chart.xAxis.tickFormat(function(d) { 
                        if(d.length>5) return d.slice(0,5)+'..';
                        else return d;
                        //chart.xAxis.rotateLabels(90);
                    });
                 }*/
             }
             
             if(self._groupFieldIsDateTime()){
                    var min_date, max_date;
                    $.map(self.model.records.toJSON(),function(value,idex){                        
                        var parsedDate = new Date(value[self.state.attributes.group]);
                        if(!min_date || parsedDate < min_date) min_date = parsedDate;
                        if(!max_date || parsedDate > max_date) max_date = parsedDate;
                    });

                    chart.xAxis.showMaxMin(true).tickFormat(function(d) { 
                        if(self.el.parents('.content_div').length>0){ 
                            return d3.time.format('%x')(new Date(d)).slice(0,2)+'..';
                        }
                        else{
                            return d3.time.format('%x')(new Date(d));
                        }
                    });

                    var intervalType;
                    intervalType= this.model.timeState.get("interval");
                    switch(intervalType) {
                       case 3:
				            chart.xAxis.tickValues(d3.time.year.range(min_date,max_date,1));
			           break;
			           case 2:
				            chart.xAxis.tickValues(d3.time.month.range(min_date,max_date,1));
			           break;
			           case 1:
				            chart.xAxis.tickValues(d3.time.week.range(min_date,max_date,1));
			           break;
			           default:
				            chart.xAxis.tickValues(d3.time.day.range(min_date,max_date,1));
			           break;
                  }
             }
             //else if(!self.xvaluesAreString ){  chart.xAxis.tickFormat(d3.format(',.2f')); }
             //chart.yAxis.tickFormat(d3.format(',.2f'));

             if (type != "discreteBar") {               
                var legent = showLegend ? true : self.state.get('showLegent');
                chart.showLegend(legent);
             }

             if (type != "line" && self.state.get("dashboard")) {
                var controls = self.state.get('showControl');
                chart.showControls(controls);
             }

             if (showLegend) { self.$el.find('.btnLegend').hide() }
             else self.$el.find('.btnLegend').show();

             if (type == "pie" || type == 'donut') { self.$el.find('.labelsDivTime').hide(); }
             else self.$el.find('.labelsDivTime').show();             

             return chart;
        },

        _groupFieldIsDateTime: function () {
            var xfield = this.model.fields.get(this.state.attributes.group);
            var xtype = xfield.get('type');
            var isDateTime = (xtype === 'date' || xtype === 'date-time' || xtype === 'time');
            return isDateTime;
        },

        createSeries: function (configs) {
            var self = this;
            self.xvaluesAreString = true;
            self.xvaluesAreIndex = false;
            self.xaxisLabels =[];
            var serieses = [];
            var seriesCompare;
            var shapes = ['circle', 'cross', 'triangle-up', 'triangle-down', 'diamond', 'square'];
            var xfield = self.model.fields.get(self.state.attributes.group);
            var isDateTime = self._groupFieldIsDateTime();
            var gType = self.state.attributes.graphType;
            var xstring =  _.findWhere(self.graphTypes, {key: gType}).xstring;
            var cmpField = this.state.attributes.compareField;
            var selectedFields = this.state.attributes.selectedFields;
            var points = [];
            var pointsSelected = [];
            if(this.state.get('multipleCompare')=="true"){                
                var pivserie= this.state.attributes.series[0];
                //Hot Fix by Bony will check it later
                if(typeof(this.state.attributes.series)=="string") {
                    pivserie= this.state.attributes.series;
                }
                var data = $.grep(self.model.records.toJSON(),function(value,index){ //self.model.toJSON().records
                              return ($.inArray( value[cmpField] == undefined?undefined:value[cmpField].toString(), selectedFields) > -1 );
                           });
                var pivotdata = [];
                if(typeof data != "undefined" && data.length > 0 && this.state.attributes.series.length > 0){
                     try{
                        pivotdata =  this.model.jsonData.pivot(data, xfield.id, cmpField, pivserie); 
                        } 
                     catch ( e ) 
                     {
                               pivotdata=[];
                      }
                 }
                 _.each(selectedFields,function(field){                   
                    var series;
                    var pont;
                    _.each(pivotdata, function (doc, index) {
                        var x = doc[xfield.id];
                        if (isDateTime) {
                            var _date = moment(String(x));
                            if (_date.isValid()) {
                                x = _date.toDate().getTime();
                            }
                        } 
                        else if (typeof x === 'string' && !xstring ) {                       
                            if($.isNumeric(x)){
                                 x = parseFloat(x);
                                 self.xvaluesAreString = false;
                            }
                            else {
                                if($.inArray(x, self.xaxisLabels) < 0) {
                                    self.xaxisLabels.push(x);
                                }
                                x = $.inArray(x, self.xaxisLabels);
                                self.xvaluesAreIndex = true;
                            }
                        }
                        //var yfield = self.model.fields.get(field);
                        //var y = doc.getFieldValue(yfield);
                        var y = doc[field]; 
                        if(y == undefined /*&& y != null*/) 
                        {
                            //y = doc.getFieldValue(field); 
                            y=0;
                        }             
                        pont = {x: x, y: y, label:field};
                        switch(gType){
                           case "scatter":
                           case "scatterPlusLine":
                                $.extend(pont, {size: Math.random()});//, shape: shapes[points.length % 6]
                           break;
                        }
                        pointsSelected.push(pont);
                       });
                            series = {key: field, values: pointsSelected};
                            switch(gType){
                                case "line":                           
                                    $.extend(series, { area :  ($.inArray(cmpField, self.options.state.area) > -1) });
                               break;
                               case "linePlusBar":
                                    $.extend(series, { bar : ($.inArray(cmpField, self.options.state.bar) > -1) });
                               break;
                               case "scatterPlusLine":
                                  if(configs)  $.extend(series, {slope: configs.slope || -1, intercept: configs.intercept || -1 });
                               break;
                            } 
                    seriesCompare = series;
                    serieses.push(seriesCompare);
                    pointsSelected=[];
                });
            }

           else{
                _.each(this.state.attributes.series, function (field) {
                    var fieldLabel = self.model.fields.get(field).get('name');
                    _.each(self.model.records.models, function (doc, index) {
                        //var x = doc.getFieldValue(xfield);
                        var x = doc[xfield.id];
                        if(x==undefined) x = doc.getFieldValue(xfield);
                        if (isDateTime) {
                            var _date = moment(String(x));
                            if (_date.isValid()) {
                                x = _date.toDate().getTime();
                            }
                        } else if (typeof x === 'string' && !xstring ) {                       
                            if($.isNumeric(x)){
                                 x = parseFloat(x);
                                 self.xvaluesAreString = false;
                            }
                            else {
                                if($.inArray(x, self.xaxisLabels) < 0) {
                                    self.xaxisLabels.push(x);
                                }
                                x = $.inArray(x, self.xaxisLabels);
                                self.xvaluesAreIndex = true;
                            }
                        }

                        var yfield = self.model.fields.get(field);
                        //var y = doc.getFieldValue(yfield);
                        var y = doc[yfield.id]; 
                        if(y == undefined /*&& y != null*/) 
                        {
                            y = doc.getFieldValue(yfield); 
                        }          
                       // var y = doc.getFieldValue(yfield);     
                        var ponit = {x: x, y: y, label:fieldLabel};
                        switch(gType){
                           case "scatter":
                           case "scatterPlusLine":
                                $.extend(ponit, {size: Math.random()});//, shape: shapes[points.length % 6]
                           break;
                        }
                        points.push(ponit);
                    });  
                 
                    var series = {key: fieldLabel, values: points};
                          
                    switch(gType){
                            case "line":                           
                                $.extend(series, { area :  ($.inArray(field, self.options.state.area) > -1) });
                           break;
                           case "linePlusBar":
                                $.extend(series, { bar : ($.inArray(field, self.options.state.bar) > -1) });
                           break;
                           case "scatterPlusLine":
                              if(configs)  $.extend(series, {slope: configs.slope || -1, intercept: configs.intercept || -1 });
                           break;
                    }
                    serieses.push(series);
                });
            }
            this.pivotData = pivotdata;
            return serieses;

           function cmp(x, y) {
                return x > y ? 1 : x < y ? -1 : 0;
            };

        },
        
        createPieSeries: function () {
            var self = this;
            self.xvaluesAreIndex = false;
            var series = [];
            var xfield = self.model.fields.get(self.state.attributes.group);
            var isDateTime = self._groupFieldIsDateTime();
            //_.each(this.state.attributes.series, function (field) {
            if(this.state.attributes.series.length > 0){
                var field = this.state.attributes.series[0];
                var points = [];
                var fieldLabel = self.model.fields.get(field).get('name');
                _.each(self.model.records.models, function (doc, index) {
                    var x = doc.getFieldValue(xfield);

                    if (isDateTime) {
                        // cast to string as Date(1990) produces 1970 date but Date('1990') produces 1/1/1990
                        var _date = moment(String(x));
                        if (_date.isValid()) {
                            x = _date.toDate().getTime();
                        }
                    } 
                    var yfield = self.model.fields.get(field);
                    var y = doc.getFieldValue(yfield);

                    series.push({
                        x: x,
                        y: y,
                        label:fieldLabel
                   });  
                });   
              }      
            //});
            return series;
        }
    });
    //edited by nithin for tile view of graph types
    my.TimeseriesControls = Backbone.View.extend({
        className: "editor timeseriesEditor",
        template: ' \
  <div class="editor-block"> \
    <form class="form-stacked"> \
      <div class="clearfix"> \
        <div class="outerGraphtype"><label class="_lngTrans_Translatable graphType"></label> \
        <span class="updown slideDown"></span>\
         <div class="input editor-type"> \
              <div class="graphSet"> \
              {{#graphTypes}} \
                <div class="graphIcons timeseriesgraphIcons"><div value="{{key}}" class="_lngTrans_Translatable tiledGraph {{key}}" name="{{name}}" title="{{name}}"></div><span class="sliced">{{sliced}}</span></div> \
              {{/graphTypes}} \
              </div> \
        </div></div> \
        <div class="spaceDiv"></div>\
        <div class="series-group"> \
          <label><span class="_lngTrans_Translatable">Time Column </span></label> \
          <span class="updown slideDown"></span>\
              <div class="input editor-group timesreisGroup"> \
	             <div class="dateFields">\
                      <select> \
                      {{#dateFields}} \
                      <option value="{{id}}" class="_lngTrans_Translatable">{{name}}</option> \
                      {{/dateFields}} \
                      </select> \
                  </div> \
	             <div class="timeInterval">\
	                <select> \
	                     {{#intervals}} \
	                        <option value="{{id}}" class="_lngTrans_Translatable {{name}}" {{#selected}}selected{{/selected}}>{{name}}</option> \
	                     {{/intervals}} \
	                </select> \
	            </div>\
            </div>\
        </div> \
        <div class="editor-series-block"> \
        </div> \
      </div> \
    </form> \
  </div> \
',
  templateSingleSeries: ' \
  <div class="editor-series-group"> \
    <div class="editor-series js-series"> \
      <label><span class="_lngTrans_Translatable">Compare</span>\
      </label> \
      <span class="updown slideDown"></span>\
      <div class="editor-select" >\
          <div class="chkGroup" style="display:block">\
                {{#timeSeries}} \
                <div class="divSeries">\
                    <input type="checkbox" value="{{id}}" data="{{type}}" class="_lngTrans_Translatable chkSeries"/>{{name}} \
                </div>\
                {{/timeSeries}} \
          </div>\
      </div>\
    </div> \
    </div> \
  ',
  templateMultiSeries: ' \
  <div class="editor-series-group"> \
    <div class="editor-series js-series"> \
        <label><span class="_lngTrans_Translatable">Indicators</span>\
        </label> \
        <span class="updown slideDown"></span>\
        <div class="input seriesInput timeseriesIndicator"> \
        <select> \
        {{#timeSeries}} \
            <option value="{{id}}" class="_lngTrans_Translatable">{{name}}</option> \
        {{/timeSeries}} \
        </select> \
        </div> \
    </div> \
   </div> \
   <div class="editor-series-group"> \
    <div class="editor-series js-series"> \
      <label><span class="_lngTrans_Translatable">Compare</span>\
      </label> \
      <span class="updown slideDown"></span>\
      <div class="editor-select" >\
        <div class="compare-fIelds">\
            <select> \
            {{#compareFields}} \
                <option value="{{id}}" class="_lngTrans_Translatable">{{name}}</option> \
            {{/compareFields}} \
            </select> \
        </div> \
        <div class="compare-values chkGroup" style="display:block">\
        </div>\
      </div>\
    </div> \
   </div> \
',

templateCompareSeries: ' \
            {{#compareValues}} \
            <div class="divSeries">\
                <input type="checkbox" value="{{.}}" data="{{.}}" class="_lngTrans_Translatable chkSeries"/>{{.}} \
            </div>\
            {{/compareValues}} \
',

        events: {
            'click .tiledGraph': 'onSelectGraphType',
            'change form select': 'onEditorSubmit',
            'change .chkSeries' : 'onEditorSubmit',
            'click .updown':'onSlideToggle',
           
        },

        initialize: function (options) {
            var self = this;
            _.bindAll(this, 'render');
            this.listenTo(this.model.timeState, 'change', function () {     
                self.redrawIntervels();
            });
            this.state = new recline.Model.ObjectState(options.state);
            this.render();
        },

        redrawIntervels: function(){
                var self = this;
                var template = '<select> {{#intervals}} <option value="{{id}}" class="_lngTrans_Translatable {{name}}" {{#selected}}selected{{/selected}}>{{name}}</option> {{/intervals}}</select> ';
                     
                var timeState = self.model.timeState.toJSON();

                var interval = _.map(timeState.intervals, function(value, idex){ 
                    return {
                        id: value.id,
                        name: value.name,
                        selected : (value.id == timeState.interval)
                    };
                });

                /*var interval = _.findWhere(timeState.intervals, {id: timeState.interval });
                if(interval) interval["selected"]= true;*/
                //var tmplData ={ intervals :timeState.intervals};
                var tmplData ={ intervals :interval};
                var html = Mustache.render(template, tmplData);
                self.$el.find('.timeInterval').empty().html(html);
        },
       
        render: function () {
            var self = this;   

            /*------------ Render Template -----------------*/
            var tmplData = this.model.toTemplateJSON();

            tmplData["graphTypes"]= self.options.graphTypes;   
            tmplData["dateFields"]= self.options.dateFields;  
            tmplData["intervals"]=  self.model.timeState.get("intervals");
            var htmls = Mustache.render(this.template, tmplData);
            this.$el.html(htmls);

            /*------------ Block End -----------------*/
    

            // set up editor from state
            if (this.state.get('graphType')) {
                var graphType = _.findWhere( self.options.graphTypes, {key : this.state.get('graphType')}); 
                this.$el.find('.'+graphType.key).addClass('active');
                this.$el.find('.graphType').html('<span style="margin-right:5px;"> Timeseries Type:</span><span style="font-weight:bold">'+ graphType.name +'</span>');
            }

            if (this.state.get('group')) {
                this._selectOption('.editor-group', this.state.get('group'));
            }

            // ensure at least one series box shows up
            var tmpSeries = [];
            if (this.state.get('series').length > 0) {
                tmpSeries = this.state.get('series');
            }
            var groupName = this.state.get('group');          
            if(this.state.get('multipleCompare')=="true"){
                self.addMultiSeries();
                self.addCompareSeries(self.state.get("compareField"));
                self._selectOption('.compare-fIelds', this.state.get('compareField'));
                self._multiCompareOption(tmpSeries,  this.state.get("selectedFields"), groupName)
            }
            else{
                self.addSingleSeries();
                self._singleCompareOption (tmpSeries, groupName);
            }
            return this;
        },

        
        // Private: Helper function to select an option from a select list
        //
        _selectOption: function (id, value) {
            var options = this.$el.find(id + ' select > option');
            if (options) {
                $.each(options, function (i, val) {
                    if (this.value == value) {
                        $(this).attr('selected', true);
                        return false;
                    }
                });
            }
        },
        
        _singleCompareOption: function (series, groupName) {
            var self = this;                         
            var chkOptions = self.$el.find('.editor-series.js-series .divSeries');
            if (chkOptions) {
                //var checked = document.createDocumentFragment(), unchecked = document.createDocumentFragment();
                $.each(chkOptions, function (i, val) {
                    var value = $(this).find('.chkSeries').val();  
                                     
                    if(value == groupName) $(val).hide(); else $(val).show();
                    if($.inArray(value,series) > -1){
                        $(this).find('.chkSeries').attr('checked', true);
                        //checked.appendChild(chkOptions[i]);
                    }
                    else{
                        $(this).find('.chkSeries').attr('checked', false);
                        //unchecked.appendChild(chkOptions[i]);
                    }
                });
               // self.$el.find(".chkGroup").append(checked).append(unchecked);
            }   
        },
        
        _multiCompareOption: function (series, selectedFieds, groupName) {
            var self = this;                         
            var chkOptions = self.$el.find('.editor-series.js-series .divSeries');
            if (chkOptions) {
                //var checked = document.createDocumentFragment(), unchecked = document.createDocumentFragment();
                $.each(chkOptions, function (i, val) {
                    var value = $(this).find('.chkSeries').val();  
                                     
                    if($.inArray(value, selectedFieds) > -1){
                        $(this).find('.chkSeries').attr('checked', true);
                       // checked.appendChild(chkOptions[i]);
                    }
                    else{
                        $(this).find('.chkSeries').attr('checked', false);
                       // unchecked.appendChild(chkOptions[i]);
                    }
                });
                //self.$el.find(".chkGroup").append(checked).append(unchecked);
            }
                        
            var options = self.$el.find('.timeseriesIndicator select > option');
            if (options) {
                $.each(options, function (i, val) {
                    if(this.value == groupName) $(val).hide();
                    else $(val).show();
                    if ($.inArray(this.value, series) > -1) {
                        $(this).attr('selected', 'selected');
                    }
                });
            }        
        },

        addSingleSeries: function () {        
            var self = this;
            var tmplData = { timeSeries :[] }; 
                         
            //tmplData.timeSeries =  _.where( this.model.toTemplateJSON().timeSeries, { type : 'number'});
            tmplData.timeSeries =  this.model.toTemplateJSON().timeSeries;    
            var htmls = Mustache.render(this.templateSingleSeries, tmplData);
            this.$el.find('.editor-series-block').append(htmls);
            return this;
        },

        addMultiSeries: function () {            
            var self = this;   
           
            var dateType =['date','date-time','time'];
            var tmplData = { compareFields :[], timeSeries:[] };

            tmplData.compareFields = $.grep( this.model.toTemplateJSON().timeSeries, function( value, idx ) { return ($.inArray(value.type, dateType) < 0); });
            tmplData.timeSeries =  _.where( this.model.toTemplateJSON().timeSeries, { type : 'number'});    
                         
            var series = this.state.attributes.series;
            if(series.length == 0 && tmplData.timeSeries.length > 0){ 
                    series.push(tmplData.timeSeries[0].id);
                    this.state.attributes.series=series;
            }  
               
            var htmls = Mustache.render(this.templateMultiSeries, tmplData);
            this.$el.find('.editor-series-block').append(htmls);
            return this;
        },

        addCompareSeries: function (compareField) { 
            var self = this;   
            var tmplData = { compareValues :[] };
            tmplData.compareValues = _.uniq( _.pluck(this.model.toJSON().records, compareField));  
                               
            var selectedFields = this.state.attributes.selectedFields;
            if(selectedFields.length == 0 && tmplData.compareValues.length > 0){ 
                    selectedFields.push(tmplData.compareValues[0]);
                    this.state.attributes.selectedFields=selectedFields;
            }  
                                     
            var htmls = Mustache.render(this.templateCompareSeries, tmplData);
            this.$el.find('.compare-values').empty().append(htmls);
            this.$el.find('.divSeries .chkSeries').first().prop( "checked", true );
            return this;
        },

       onSlideToggle:function (e) {
            var self = this;
            var element = $(e.currentTarget);
            element.next("div").slideToggle('fast',function(){
                if(element.hasClass('slideDown')) {
                    element.addClass('slideUp').removeClass('slideDown');
                    element.parent().addClass('opened');
                }
                else {
                    element.addClass('slideDown').removeClass('slideUp');
                     element.parent().removeClass('opened');
                }
            });
        },

        onSelectGraphType: function (e) {
            var currentTarget = e.currentTarget;
            this.$el.find('.graphType').html("<span style='margin-right:5px;'> Graph Type:</span><span style='font-weight:bold'>"+currentTarget.getAttribute('name')+"</span>");
            $.each($(currentTarget).parent().siblings(),function(i,val){
                $(val).find('.tiledGraph').removeClass('active');
            });
            currentTarget.classList.add("active");
            this.onEditorSubmit(e);
        },

        onEditorSubmit: function (e) {
           
            var self = this; 
            var groupName = this.$el.find('.editor-group select').val();
            var grphselect = this.$el.find('.editor-type div.active').attr('value');
            var interval = parseInt(this.$el.find('.timeInterval select option:selected').val());
            var selectedFields = [], series = [];
            var comapreField = self.state.get("compareField");

            if(this.state.get('multipleCompare')=="true"){
                
                comapreField = this.$el.find('.compare-fIelds select').val();
                if(comapreField != self.state.get("compareField")) {
                    self.addCompareSeries(comapreField, true);
                }

                var $compFields = this.$el.find('.divSeries .chkSeries:checked');
                selectedFields = $compFields.map(function () { return $(this).val(); });

                var seriesSelected = this.$el.find('.timeseriesIndicator select').val();
                if(seriesSelected) series.push(seriesSelected); 
            }
            else{
                var seriesSelected = this.$el.find('.divSeries .chkSeries:checked');
                series = seriesSelected.map(function () { return $(this).val(); });
            }
            
            var updatedState = {
                series: $.makeArray(series),
                group: groupName,
                graphType: grphselect,
                compareField: comapreField,
                selectedFields:$.makeArray(selectedFields)
            };
             //e.preventDefault();       
             $('select').off();
             this.model.timeState.update({
                field: groupName,
                interval: interval
            });
            e.stopPropagation();
            this.state.set(updatedState);
        }
    });

})(jQuery, recline.View);
this.recline = this.recline || {};
this.recline.View = this.recline.View || {};
this.recline.View.Timeseries = this.recline.View.TimeseriesGraph;
this.recline.View.TimeseriesControls = this.recline.View.TimeseriesControls;