<?php

namespace Drupal\taxonomy_import\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\taxonomy\Entity\Term;
use Drupal\taxonomy\Entity\Vocabulary;

/**
 * Contains \Drupal\taxonomy_import\Form\ImportForm.
 */
class ImportForm extends ConfigFormBase {

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'import_taxonomy_form';
  }

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'taxonomy_import.settings',
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('taxonomy_import.settings');
    $form['field_vocabulary_name'] = [
      '#type' => 'textfield',
      '#title' => $this->t('Vocabulary name'),
      '#required' => TRUE,
      '#maxlength_js' => TRUE,
      '#maxlength' => 30,
      '#description' => $this->t('Not more than 30 characters please!'),
      '#default_value' => $config->get('field_vocabulary_name'),
    ];
    $form['taxonomy_file'] = [
      '#type' => 'managed_file',
      '#title' => $this->t('Import file'),
      '#required' => TRUE,
      '#upload_validators'  => [
        'file_validate_extensions' => ['csv xml'],
        'file_validate_size' => [25600000],
      ],
      '#upload_location' => 'public://taxonomy_files/',
      '#description' => $this->t('Upload a file to Import taxonomy!'),
      '#default_value' => $config->get('taxonomy_file'),
    ];
    $form['actions']['#type'] = 'actions';
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => $this->t('Import'),
      '#button_type' => 'primary',
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $field_vocabulary_name = $form_state->getValue(['field_vocabulary_name']);
    $taxonomy_file = $form_state->getValue(['taxonomy_file']);
    // Call to user defined function.
    taxonomy_import_create($field_vocabulary_name, $taxonomy_file);
  }

}

/**
 * Function to implement import taxonomy functionality.
 */
function taxonomy_import_create($voc_name, $taxonomy_file) {

  global $base_url;
  $loc = db_query('SELECT file_managed.uri FROM file_managed WHERE file_managed.fid = :fid', [':fid' => $taxonomy_file[0]]);
  foreach ($loc as $val) {
    // Get location of the file.
    $location = $val->uri;
  }
  $name = $voc_name;
  if (function_exists('mime_content_type')) {
    $mimetype = mime_content_type($location);
  }
  else {
    return 'application/octet-stream';
  }
  // Converting to machine name.
  $machine_readable = strtolower($voc_name);
  // Vocabulary machine name.
  $vid = preg_replace('@[^a-z0-9_]+@', '_', $machine_readable);
  // Creating new vocabulary with the field value.
  $vocabularies = Vocabulary::loadMultiple();
  if (!isset($vocabularies[$vid])) {
    $vocabulary = Vocabulary::create([
      'vid' => $vid,
      'machine_name' => $vid,
      'name' => $name,
    ]);
    $vocabulary->save();
  }
  // Code for fetch and save csv file.
  if ($mimetype == "text/plain") {
    if (($handle = fopen($location, "r")) != FALSE) {
      // Read all data including title.
      $data1 = fgetcsv($handle);
      while ($data1 != FALSE && ($data = fgetcsv($handle)) != FALSE) {
        $termid = 0;
        $term_id = 0;
        // Get tid of term with same name.
        $termid = db_query('SELECT n.tid FROM {taxonomy_term_field_data} n WHERE n.name  = :uid AND n.vid  = :vid', [':uid' => $data[0], ':vid' => $vid]);
        foreach ($termid as $val) {
          // Get tid.
          $term_id = $val->tid;
        }
        // Finding parent of new item.
        $parent = 0;
        if (!empty($data[1])) {
          $parent_id = db_query('SELECT n.tid FROM {taxonomy_term_field_data} n WHERE n.name  = :uid AND n.vid  = :vid', [':uid' => $data[1], ':vid' => $vid]);
          foreach ($parent_id as $val) {
            if (!empty($val)) {
              // Get tid.
              $parent = $val->tid;
            }
            else {
              $parent = 0;
            }
          }
        }
        // Check whether term already exists or not.
        if (empty($term_id)) {
          // Create new term.
          $term = Term::create([
            'parent' => [$parent],
            'name' => $data[0],
            'vid' => $vid,
          ])->save();
        }
        // Code to update existing term field(s).
        else {
          $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->load($term_id);
          $term->parent->setValue($parent);
          $term->Save();
        }
      }
      fclose($handle);
      // Redirecting to taxonomy term overview page.
      $url = $base_url . "/admin/structure/taxonomy/manage/" . $vid . "/overview";
      header('Location:' . $url);exit;
    }
    else {
      drupal_set_message(t('File contains no data'));
    }
  }
  // Code for fetch and save xml file.
  elseif ($mimetype == "application/xml") {
    if (file_exists($location)) {
      $feed = file_get_contents($location);
      $items = simplexml_load_string($feed);
      if (!empty($items)) {
        $item = $items->children();
        foreach ($item as $child) {
          $records = $child;
          $array = (array) $records;
          $j = 0;
          foreach ($array as $val) {
            if ($j == 0) {
              $terms = $val;
            }
            if ($j == 1) {
              $parents = $val;
            }
            $j++;
            if ($j >= 2) {
              break;
            }
          }
          $parent = 0;
          $term_id = 0;
          // Checks if parent tag exists.
          if (isset($parents) && !empty($parents)) {
            $data = $parents;
            $parent_id = db_query('SELECT n.tid FROM {taxonomy_term_field_data} n WHERE n.name  = :uid AND n.vid  = :vid', [':uid' => $data, ':vid' => $vid]);
            foreach ($parent_id as $val) {
              if (!empty($val)) {
                // Get tid.
                $parent = $val->tid;
              }
              else {
                $parent = 0;
              }
            }
          }
          $termid = db_query('SELECT n.tid FROM {taxonomy_term_field_data} n WHERE n.name  = :uid AND n.vid  = :vid', [':uid' => $terms, ':vid' => $vid]);
          foreach ($termid as $val) {
            // Get tid.
            $term_id = $val->tid;
          }
          // Check whether term already exists or not.
          if (empty($term_id)) {
            // Create  new term.
            $term = Term::create([
              'parent' => [$parent],
              'name' => $terms,
              'vid' => $vid,
            ])->save();
          }
          // Code to update existing term field(s).
          else {
            $term = \Drupal::entityTypeManager()->getStorage('taxonomy_term')->load($term_id);
            $term->parent->setValue($parent);
            $term->Save();
          }
        }
        // Redirecting to taxonomy term overview page.
        $url = $base_url . "/admin/structure/taxonomy/manage/" . $vid . "/overview";
        header('Location:' . $url);exit;
      }
      else {
        drupal_set_message(t('File contains no data'));
      }
    }
  }
  elseif ($mimetype == "application/octet-stream") {
    drupal_set_message(t('File contains no data'));
  }
  else {
    drupal_set_message(t('Failed to open the file'));
  }
}
